/*eslint-disable */

/**
 * Created by 19.06.2013-7pm on 04-Nov-15.
 */

(function (window) {

    var ResponsiveBehavior, uiUtils = flexiciousNmsp.UIUtils, flxsConstants = flexiciousNmsp.Constants;

    ResponsiveBehavior = function (grid) {
        if (!grid || !grid.implementsOrExtends("FlexDataGrid"))
            throw "grid reqiured..!";


        /**
         * The behaviour need to apply on.
         * @property grid
         * @type FlexDataGrid
         */
        this.grid = grid;


        /**
         *  There are there modes.
         *
         *  <b><i> scrollBarMode </i></b> - The base HTMLTreeGrid will be extended to provide a
         *  Bootstrap-like API to show or hide the scroll bar on basis of the screen width.
         *  Below a certain pre-configured screen width, a scroll bar shall appear,
         *  and allow the user to scroll through all the columns. Above this screen width,
         *  there shall be no scrollbar.
         *
         *  <b><i> columnPriorityMode </i></b> -  The base HTMLTreeGrid will be extended to expose and
         *  API for a priority model, where each column can be assigned a priority and the columns shall
         *  appear or be hidden on basis of screen width.
         *
         *  <b><i> transposedColumnMode </i></b> - A new column type, TransposedColumn will be added to
         *  accommodate smaller screens, below a certain pre-configured width, all the columns in the grid
         *  shall be replaced by the Transposed Column which shall contain, within a single cell,
         *  all the information that was held in the previous cells in a transposed manner

         * @type {string}
         * @default scrollBarMode
         */
        this.responsiveBehaviorMode = null; //flexiciousNmsp.ResponsiveBehavior.COLUMN_PRIORITY_MODE;

        /**
         * Flag to enable or disable the responsive behavior
         * @type {boolean}
         * @default false
         */
        this.enabled = false; //false; @testing

        /**
         * Pixel width below which the behavior should kick in
         * @type {number}
         * @default 480.
         */
        this.scrollBarWidthTreshhold = 1000;

        /**
         * The width below which columns with priority 1 are not visible.
         * @type {number}
         */
        this.columnPriority1WidthTreshold = 1120;

        /**
         * The width below which columns with priority 2 are not visible.
         * @type {number}
         */
        this.columnPriority2WidthTreshold = 960;

        /**
         * The width below which columns with priority 3 are not visible.
         * @type {number}
         */
        this.columnPriority3WidthTreshold = 800;

        /**
         * The width below which columns with priority 4 are not visible.
         * @type {number}
         */
        this.columnPriority4WidthTreshold = 640;

        /**
         * The width below which columns with priority 5 are not visible.
         * @type {number}
         */
        this.columnPriority5WidthTreshold = 480;

        /**
         * The width below which the transposition mode kicks in.
         * @type {number}
         */
        this.transposedColumnTreshold = 450;

        /**
         * The column responsible for rendering the transposed cells.
         * @type {FlexDataGridColumn}
         */
        this.transposedColumn = null;

        /**
         *  This flag is used to identify whether grid is resized on create.
         *  to prevent us from running again in startup.
         * @type {boolean}
         * @private
         */
        this._gridResized = false;

        /**
         * To identify weather grid resized in the scroll mode.
         * because in the grid scroll responsive mode we need to set the column width initially.
         * @type {boolean}
         * @private
         */
        this._gridResizedInScrollMode = false;

        grid.addEventListener(this, flexiciousNmsp.FlexDataGridEvent.WIDTH_CHANGED, this.onGridWidthChanged);
        grid.addEventListener(this, "columnsChanged", this.onColumnsChanged);

    };

    flexiciousNmsp.ResponsiveBehavior = ResponsiveBehavior; //add to name space
    ResponsiveBehavior.prototype = new flexiciousNmsp.TypedObject(); //setup hierarchy
    ResponsiveBehavior.prototype.typeName = ResponsiveBehavior.typeName = 'ResponsiveBehavior';//for quick inspection
    ResponsiveBehavior.prototype.getClassNames = function () {
        return ["ResponsiveBehavior", "TypedObject"];
    };


    ResponsiveBehavior.prototype.kill = function () {
        this.grid.removeEventListener(flexiciousNmsp.FlexDataGridEvent.WIDTH_CHANGED, this.onGridWidthChanged);
        this.grid.removeEventListener("columnsChanged", this.onColumnsChanged);
    };

    /**
     *
     * @param mode
     * @type string
     */
    ResponsiveBehavior.prototype.setResponsiveBehaviorMode = function(mode){
        if(this.responsiveBehaviorMode == mode)
            return;
        //this._rollBackChanges();
        this.responsiveBehaviorMode = mode;
        this.onGridWidthChanged(null);
    };

    /**
     * function to rollback the changes for the current mode
     * @private
     */
    /* ResponsiveBehavior.prototype._rollBackChanges = function(){
     if (!this.enabled || !this.responsiveBehaviorMode)
     return;
     if (this.responsiveBehaviorMode == flexiciousNmsp.ResponsiveBehavior.SCROLL_BAR_MODE)
     this.grid.setHorizontalScrollPolicy("off"); // however we are move to another mode. another two will not require the scroll bars.
     else if (this.responsiveBehaviorMode == flexiciousNmsp.ResponsiveBehavior.COLUMN_PRIORITY_MODE) {
     for (var level = 5; level >= 1; level--) {
     var colsPriorityMap = flexiciousNmsp.ResponsiveBehavior.getColumnPriorityMap(this.grid.getColumns());
     var colsNeedToShow = colsPriorityMap.getValue(level) || [];
     for (var i = 0; i < colsNeedToShow.length; i++) {
     var col = colsNeedToShow[i];
     if (col.responseVisibleDirty && !col.getVisible()) {
     col.setVisible(true);
     col.responseVisibleDirty = false;
     }
     }
     }
     }
     else if (this.responsiveBehaviorMode == flexiciousNmsp.ResponsiveBehavior.TRANSPOSED_COLUMN_MODE && this.isTransposed) {
     this.reverseTranspose();
     }

     };*/

    /**
     * This is the entry point for the responsive behavior mechanism. When the grid resizes, we apply the responsibe behavior based upon
     * the current grid width.
     * @param event
     */
    ResponsiveBehavior.prototype.onGridWidthChanged = function (event) {
        if (!this.enabled || !this.responsiveBehaviorMode)
            return;
        var cols = this.grid.getColumns();
        if(!cols || !cols.length)
            return;
        if (this.responsiveBehaviorMode == flexiciousNmsp.ResponsiveBehavior.SCROLL_BAR_MODE)
            this.applyScrollBar();
        else if (this.responsiveBehaviorMode == flexiciousNmsp.ResponsiveBehavior.COLUMN_PRIORITY_MODE)
            this.applyColumnPriority();
        else if (this.responsiveBehaviorMode == flexiciousNmsp.ResponsiveBehavior.TRANSPOSED_COLUMN_MODE)
            this.applyTransposedColumn();
        if(!this._gridResized)
            this._gridResized = true;
    };

    ResponsiveBehavior.prototype.onColumnsChanged = function(event){
        this.onGridWidthChanged(null);
    };

    /**
     * This method is responsible for applying the responsive behavior mode scrollbar.
     * @method applyScrollBar
     */
    ResponsiveBehavior.prototype.applyScrollBar = function () {
        var currentPolicy = this.grid.getHorizontalScrollPolicy();
        var requirePolicy = currentPolicy;
        if (this.grid.getWidth() < this.scrollBarWidthTreshhold) {
            requirePolicy = "auto";
            if(!this._gridResizedInScrollMode) {
                var cols = this.grid.getColumns();
                for (var i = 0; i < cols.length; i++) {
                    var col = cols[i];
                    if (col.getColumnWidthMode() == flexiciousNmsp.FlexDataGridColumn.COLUMN_WIDTH_MODE_NONE ||
                        col.getColumnWidthMode() == flexiciousNmsp.FlexDataGridColumn.COLUMN_WIDTH_MODE_PERCENT)
                        col.setWidth(100);
                }
            }
        }
        else
            requirePolicy = "off";
        if (currentPolicy != requirePolicy) {
            this.grid.setHorizontalScrollPolicy(requirePolicy);
            this.grid.reDraw();
        }

        if(!this._gridResizedInScrollMode)
            this._gridResizedInScrollMode = true;
    };

    ResponsiveBehavior.getColumnPriorityMap = function(columns){
        var colsPriorityMap = new flexiciousNmsp.KeyValuePairCollection();
        for (var i = 0; i < columns.length; i++) {
            var col = columns[i];
            if (!colsPriorityMap.getValue(col.responsivePriority))
                colsPriorityMap.addItem(col.responsivePriority, []);
            colsPriorityMap.getValue(col.responsivePriority).push(col);
        }
        return colsPriorityMap;
    };

    /**
     * This method is responsible for applying the responsive behavior mode column priority.
     * @method applyColumnPriority
     */
    ResponsiveBehavior.prototype.applyColumnPriority = function () {

        var i = 0, col;

        var gridWidth = this.grid.getWidth();
        var priorityLevel = this.getPriorityLevel(gridWidth);

        var columns = this.grid.getColumns();
        var colsWidth = 0;
        var colsPriorityMap = new flexiciousNmsp.KeyValuePairCollection();
        for (i = 0; i < columns.length; i++) {
            col = columns[i];
            if (col.getClassNames().indexOf("FlexDataGridColumn") != -1 && col.getVisible() /* && (col.responsivePriority >= priorityLevel || col.responsivePriority == -1) */) {
                colsWidth += col.getWidth();
            }
            if (!colsPriorityMap.getValue(col.responsivePriority))
                colsPriorityMap.addItem(col.responsivePriority, []);
            colsPriorityMap.getValue(col.responsivePriority).push(col);
        }

        if (priorityLevel == -1) {
            this.showColumns(1, colsPriorityMap, colsWidth, gridWidth);// reset all columns
            return;
        }

        if (colsWidth > gridWidth) { // motive to hide the columns
            for (var level = 1; level <= priorityLevel; level++) {
                var colsNeedToHide = colsPriorityMap.getValue(level) || [];
                for (i = 0; i < colsNeedToHide.length; i++) {
                    col = colsNeedToHide[i];
                    if (!col.responseVisibleDirty && col.getVisible()) {
                        col.setVisible(false);
                        col.responseVisibleDirty = true;
                        colsWidth -= col.getWidth();
                        if (colsWidth < gridWidth)       // if columns with comes under grid width, then we dont worry about the priority stuff.
                            break;
                    }
                }
                if (colsWidth < gridWidth)       // if columns with comes under grid width, then we dont worry about the priority stuff.
                    break;
            }
        } else { // here the motive is restore the hidden columns, which is hide by response behaviour.
            this.showColumns(priorityLevel, colsPriorityMap, colsWidth, gridWidth);
        }
        this.ensurePriorityApplied(priorityLevel, colsPriorityMap);
    };

    /**
     * Due to initial load in small window, grid initially creates the column width fit into the grid width.
     * Columns width will be less than grid width. so we cant assume appropriate prority levels are applied.
     * So here a check to ensure all higher prority level columns are hidden with given priority level.
     * for example if in grid initial load the priority level is 4 (i.e grid width comes under columnPriority4WidthTreshold),
     * then we check all other 3, 2 ,1 proirity level columns are hidden.
     * @param priorityLevel
     * @param colsPriorityMap
     * @method ensurePriorityApplied
     */
    ResponsiveBehavior.prototype.ensurePriorityApplied = function(priorityLevel, colsPriorityMap){
        if(!colsPriorityMap)
            return;
        for(var level = priorityLevel-1; level > 0; level--){
            var columnsNeedToHide = colsPriorityMap.getValue(level);
            if(!columnsNeedToHide || !columnsNeedToHide.length)
                continue;
            for(var i = 0; i < columnsNeedToHide.length; i++){
                if(columnsNeedToHide[i].getVisible()){
                    columnsNeedToHide[i].setVisible(false);
                    columnsNeedToHide[i].responseVisibleDirty = true;
                }
            }
        }
    };


    ResponsiveBehavior.prototype.showColumns = function (priorityLevel, colsPriorityMap, colsWidth, gridWidth) {
        for (var level = 5; level >= priorityLevel; level--) {
            var colsNeedToShow = colsPriorityMap.getValue(level) || [];
            for (var i = 0; i < colsNeedToShow.length; i++) {
                var col = colsNeedToShow[i];
                if (col.responseVisibleDirty && !col.getVisible()) {
                    col.setVisible(true);
                    col.responseVisibleDirty = false;
                    colsWidth += col.getWidth();
                    if (colsWidth > gridWidth)
                        break;
                }
            }
            if (colsWidth > gridWidth)
                break;
        }
    };


    /**
     * Calculate the priority level for the current grid width.
     * if grid width is greater than columnPriority1WidthTreshold, then return -1, in which case we just show all columns.
     * else return 1 - 5 based on the grid width,
     * @param width
     * @return {number}
     */
    ResponsiveBehavior.prototype.getPriorityLevel = function (width) {
        if (width >= this.columnPriority1WidthTreshold)
            return -1;
        if (width < this.columnPriority5WidthTreshold)
            return 5;
        else if (width < this.columnPriority4WidthTreshold)
            return 4;
        else if (width < this.columnPriority3WidthTreshold)
            return 3;
        else if (width < this.columnPriority2WidthTreshold)
            return 2;
        else if (width < this.columnPriority1WidthTreshold)
            return 1;
        return -1;
    };

    /**
     * Temporary placeholder properties.
     */
    ResponsiveBehavior.prototype.columnsBackup = null;
    ResponsiveBehavior.prototype.isTransposed = false;
    ResponsiveBehavior.prototype.oldVariableRowHeight = false;
    ResponsiveBehavior.prototype.oldGridHeaderVisible = false;
    ResponsiveBehavior.prototype.oldEnableDefaultDisclosureIcon = false;

    /**
     * This method is responsible for applying the responsive behavior mode transpose.
     * @method applyTransposedColumn
     */
    ResponsiveBehavior.prototype.applyTransposedColumn = function () {
        var gridWidth = this.grid.getWidth();
        this.grid.removeEventListener("columnsChanged", this.onColumnsChanged);
        if(gridWidth < this.transposedColumnTreshold && !this.isTransposed){
            this.oldVariableRowHeight = this.grid.variableRowHeight;
            this.oldEnableDefaultDisclosureIcon = this.grid.enableDefaultDisclosureIcon;
            this.grid.enableDefaultDisclosureIcon = false;
            this.oldGridHeaderVisible = this.grid.getColumnLevel().headerVisible;
            this.grid.getColumnLevel().headerVisible = false;
            this.columnsBackup = this.grid.getColumns();
            if(!this.transposedColumn)
                this.transposedColumn = this.getTransposedColumn();
            this.grid.variableRowHeight = this.grid.variableRowHeightUseRendererForCalculation = true;
            this.grid.setColumns([this.transposedColumn]);
            this.grid.rebuild();
            this.isTransposed = true;
        } else if(gridWidth > this.transposedColumnTreshold && this.isTransposed){
            this.reverseTranspose();
        }
        this.grid.addEventListener(this, "columnsChanged", this.onColumnsChanged);
    };

    ResponsiveBehavior.prototype.reverseTranspose = function(){
        this.isTransposed = false;
        this.grid.variableRowHeight = this.oldVariableRowHeight;
        this.grid.enableDefaultDisclosureIcon = this.oldEnableDefaultDisclosureIcon;
        this.grid.getColumnLevel().headerVisible = this.oldGridHeaderVisible;
        this.grid.setGroupedColumns(this.columnsBackup);
        this.grid.rebuild();
    };

    ResponsiveBehavior.prototype.getTransposedColumn = function(){
        var c = new flexiciousNmsp.FlexDataGridColumn();
        c.itemRenderer = new flexiciousNmsp.ClassFactory(flexiciousNmsp.TransposedColumnItemRenderer);
        c.enableExpandCollapseIcon = c.enableHierarchicalNestIndent = true;
        c.paddingLeft = 25;
        return c;

    };

    // static stuff
    ResponsiveBehavior.SCROLL_BAR_MODE = "scrollBarMode";
    ResponsiveBehavior.COLUMN_PRIORITY_MODE = "columnPriorityMode";
    ResponsiveBehavior.TRANSPOSED_COLUMN_MODE = "transposedColumnMode";
}(window));


/**
 * Created by 19.06.2013-7pm on 04-Nov-15.
 */

(function(window){

    var TransposedColumnItemRenderer = function(){
        flexiciousNmsp.UIComponent.apply(this, ["div"]);
        this.domElement.style.height = "700px";
    };

    flexiciousNmsp.TransposedColumnItemRenderer = TransposedColumnItemRenderer; //add to name space 
    TransposedColumnItemRenderer.prototype.typeName = TransposedColumnItemRenderer.typeName = 'TransposedColumnItemRenderer';//for quick inspection
    TransposedColumnItemRenderer.prototype.getClassNames=function(){
        return ["TransposedColumnItemRenderer","UIComponent"];
    };

    TransposedColumnItemRenderer.prototype.setText = function(txt){

    };


    TransposedColumnItemRenderer.prototype.setData = function(data){
        this.removeAllChildren();
        var columns = [];
        if(this.parent.implementsOrExtends("FlexDataGrid"))
            columns = this.parent.responsiveBehavior.columnsBackup;
        else
            columns = this.parent.level.grid.responsiveBehavior.columnsBackup;

        for(var i = 0; i < columns.length; i++){
            var col = columns[i];
            if(col.excludeFromTransposedMode)
                continue;

            var leftCell = flexiciousNmsp.TransposedColumnItemRenderer.getDataCell();
            var rightCell = flexiciousNmsp.TransposedColumnItemRenderer.getDataCell();

            var row = new flexiciousNmsp.UIComponent({}, "div");
            row.domElement.style.width = "100%";
            row.domElement.className = "flexicious transposedColumnDataRow";

            leftCell.setInnerHTML(col.getHeaderText());
            rightCell.setInnerHTML(col.itemToLabel(data));

            row.addChild(leftCell);
            row.addChild(rightCell);

            this.addChild(row);
        }
    };

    TransposedColumnItemRenderer.prototype.getHeight = function(){
        return this.domElement.offsetHeight;
    };

    TransposedColumnItemRenderer.getDataCell = function(){
        var c = new flexiciousNmsp.UIComponent({}, "span");
        c.domElement.style.width = "50%";
        c.domElement.style.padding = "5px 0";
        c.domElement.style.display = "inline-block";
        c.domElement.className = "flexicious transposedColumnDataCell";
        return c;
    };
}(window));
