/*eslint-disable */
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
  
import TypedObject from '../core/TypedObject'
import Label from '../controls/Label'
import Constants from '../utils/Constants'
/**
 * Attaches the spinner behavior to the owner component.
 * When the startspin method of this behavior is called, the
 * behavior will instantiate a new spinner based on the owners
 * Spinner Factory, and position it in the middle of the owner
 * components display area.
 * When the stop spin method is called, the behavior will remove
 * the spinner from the owner component and stop the spin.
 * @namespace flexiciousNmsp
 * @class SpinnerBehavior
 * 
 * @constructor
 */
export default class SpinnerBehavior extends TypedObject {
    constructor(ownerComponent) {
        super();
        /**
         * The spinner associates itself with this component
         * @property ownerComponent
         * @type UIComponent
         */
        this.ownerComponent = ownerComponent;
        ownerComponent.addEventListener(this, Constants.EVENT_CREATION_COMPLETE, this.onOwnerCreationComplete);
        /**
         * The label that displays below the spinner while it is spinning.
         * @property label
         * @type Label
         */
        this.label;
        /**
         * Whether or not this control is visible.
         * @property spinnerVisible
         * @type Boolean
         * @default false
         */
        this.spinnerVisible = false;
    }

    getClassNames() {
        return ["SpinnerBehavior", "TypedObject"];
    }

    onOwnerCreationComplete(event) {
        const isp = (event.currentTarget);
        if (isp.showSpinnerOnFilterPageSort) {
            isp.showSpinner();
        }
    }

    /**
     * Shows the spinner with default values defined below:<br/>
     * Label: "Loading please wait"<br/>
     * X Position : center X of the grid <br/>
     * Y Position : center Y of the grid<br/>
     * Grid Alpa when the spinner is active : 0.3<br/>
     * Spinner appearance can be modified using styles <br/>.
     * @method showSpinner
     */
    showSpinner(msg) {
        if (typeof msg == "undefined") msg = '';


        const elsToBlur = this.ownerComponent.getElementsToBlur();

        for (const child of elsToBlur) {
            child.attachClass('lessOpacity');
        }

        const rad = this.ownerComponent.getStyle("spinnerRadius") || 10;
        const spinnerColors = this.ownerComponent.getStyle("spinnerColors") || '#000';



        if (msg == '')
            msg = this.ownerComponent.spinnerLabel;



        if (!this.spinner) {
            this.spinner = new flexiciousNmsp.Image();
            this.spinner.attachClass("spinner")
            this.spinner.setAttribute("src", this.ownerComponent.getThemeToolbarIconFolder() + "/spinner.gif");
            this.spinner.grid = this.ownerComponent;
            this.ownerComponent.addChild(this.spinner);
        } else {
            this.ownerComponent.removeChild(this.spinner);
            this.ownerComponent.addChild(this.spinner);
        }
        if (!this.label) {
            this.label = new flexiciousNmsp.Label();
            this.label.attachClass("spinnerLabel");
            this.ownerComponent.addChild(this.label);
        } else {
            this.ownerComponent.removeChild(this.label);
            this.ownerComponent.addChild(this.label);
        }
        this.spinner.setVisible(true);
        this.label.setVisible(true);
        if (msg) {

            this.label.setText(msg);
            const labelWidth = this.label.measureText(msg).width;

            this.spinner.setX((this.ownerComponent.getWidth() / 2) - (labelWidth / 4));
            this.spinner.setY((this.ownerComponent.getHeight() / 2) - 35);

            this.label.setX((this.ownerComponent.getWidth() / 2) - ((labelWidth+40) / 2));
            this.label.setY(this.ownerComponent.getHeight() / 2);



            //UIUtils.positionComponent(this.spinner.el || this.spinner.domElement, this.label, "center top", "center bottom", 0, 30);
        }

        this.spinnerVisible = true;
    }

    showMessage(msg) {
        if (this.spinner)
            this.spinner.setVisible(false);
        if (this.label) {
            this.label.setVisible(false);
        }

        if (!this.label) {
            this.label = new flexiciousNmsp.Label();
            this.label.attachClass("spinnerLabel");
            this.ownerComponent.addChild(this.label);
        } else {
            this.ownerComponent.removeChild(this.label);
            this.ownerComponent.addChild(this.label);
        }

        this.label.setVisible(true);
        this.label.setText(msg);
        const labelWidth = this.label.measureText(msg).width;
        this.label.setX((this.ownerComponent.getWidth() / 2) - ((labelWidth+40) / 2));
        this.label.setY(this.ownerComponent.getHeight() / 2);
    }

    /**
     * Removes the spinner and sets the bodyContainer.alpha back to 1
     * @method hideSpinner
     */
    hideSpinner() {
        if (this.label) {
            this.label.setVisible(false);
        }
        if (this.spinner) {
            this.spinner.setVisible(false);

            const elsToBlur = this.ownerComponent.getElementsToBlur();

            for (const child of elsToBlur) {
                child.detachClass('lessOpacity');
            }
        }
        this.spinnerVisible = false;
    }

    /**
     * @method kill
     */
    kill() {
        this.hideSpinner();
        this.ownerComponent = null;
        if (this.label)
            this.label.kill();
        this.label = null;
        this.spinner = null;
    }
}

flexiciousNmsp.SpinnerBehavior = SpinnerBehavior; //add to name space
SpinnerBehavior.prototype.typeName = SpinnerBehavior.typeName = 'SpinnerBehavior';//for quick inspection