/*eslint-disable */
import React from 'react'
import UIComponent from '../core/UIComponent'
import Filter from '../flexgrid/grids/filters/Filter'

import Constants from '../utils/Constants'
import UIUtils from '../utils/UIUtils' 

/**
 * A ComboBox that implements ISingleSelectFilterControl (IFilterControl) and ISingleSelectDataBoundControl (IDataBoundControl)
 * which enables it to be used within the filtering/binding infrasturcture.
 * @class ComboBox
 * @constructor
 * @namespace flexiciousNmsp
 * 
 */
export default class ComboBox extends UIComponent {
    constructor(tag) {
        super(tag||Constants.COMBO_BOX_BASE_TAG);

        /**
         * Whether or not there is an active search
         * @property hasSearch
         * @type Boolean
         * @default false
         */
        this.hasSearch = false;
        /**
         * Whether or not this control has been registered. This should not be set by your code.
         * @property registered
         * @type Boolean
         * @default false
         */
        this.registered = false;
        /**
         * This is usually automatically set, you don't have to manually set it,
         * unless you're sending strings as Date objects. When set, will attempt
         * to first convert the current value to the type you specified and then
         * do the conversion.
         * Values : auto,string,number,boolean,date
         * @property filterComparisionType
         * @type String
         * @default auto
         */
        this.filterComparisionType = "auto";
        /**
         * The field to search on, usually same as the data field.
         * @property searchField
         * @type String
         * @default null
         */
        this.searchField = null;
        /**
         * The filter operation to apply to the comparison
         * See the FilterExpression class for a list.
         * Please note, for CheckBoxList and MultiSelectComboBox, this field
         * defaults to "InList" and is ignored when set.
         * Valid values are : "Equals,NotEquals,BeginsWith,EndsWith,Contains,DoesNotContain,GreaterThan,LessThan,GreaterThanEquals,LessThanEquals,InList,NotInList,Between"
         * @property filterOperation
         * @type String
         * @default Equals
         */
        this.filterOperation = null;
        /**
         * The event that the filter triggers on. Defaults to "change", or if the
         * filterRenderer supports com.flexicious.controls.interfaces.IDelayedChange, then
         * the delayedChange event.
         * @property filterTriggerEvent
         * @type String
         * @default change
         */
        this.filterTriggerEvent = "change";
        /**
         * The grid that the filter belongs to - can be null
         * if filter is used outside the grid
         * @property grid
         * @type FlexDataGrid
         * @return
         */
        this.grid = null;
        /**
         * The grid column that the filter belongs to - can be null
         * if filter is used outside the grid
         * @return
         */
        this.gridColumn = null;
        /**
         *  Name of the field in the items in the <code>dataProvider</code>
         *  Array to use as the value of drop-down list.
         *  By default, the control uses a property named <code>data</code>
         *  on each Array object and displays it.
         *  <p>However, if the <code>dataProvider</code> items do not contain
         *  a <code>data</code> property, you can set the <code>dataField</code>
         *  property to use a different property.</p>.
         *  This is also used as the search field.
         * @property dataField
         * @type String
         * @default data
         */
        this.dataField = "data";
        /**
         *  Name of the field in the items in the <code>dataProvider</code>
         *  Array to display as the label in the TextInput portion and drop-down list.
         *  By default, the control uses a property named <code>label</code>
         *  on each Array object and displays it.
         *  <p>However, if the <code>dataProvider</code> items do not contain
         *  a <code>label</code> property, you can set the <code>labelField</code>
         *  property to use a different property.</p>
         *
         * @property labelField
         * @type String
         * @default label
         */
        this.labelField = "label";
        /**
         * User-supplied function to run on each item to determine its label.
         *  By default the control uses a property named <code>label</code>
         *  on each <code>dataProvider</code> item to determine its label.
         *  However, some data sets do not have a <code>label</code> property,
         *  or do not have another property that can be used for displaying
         *  as a label.
         *  <p>An example is a data set that has <code>lastName</code> and
         *  <code>firstName</code> fields but you want to display full names.
         *  You use <code>labelFunction</code> to specify a callback function
         *  that uses the appropriate fields and return a displayable String.</p>
         *
         *  <p>The labelFunction takes a single argument which is the item
         *  in the dataProvider and returns a String:</p>
         *  <pre>
         *  myLabelFunction(item:Object):String
         *  </pre>
         * @property labelFunction
         * @type Function
         * @default null
         */
        this.labelFunction = null;

        /**
         * Text of the "All" item. Defaults to "All"
         * @property addAllItemText
         * @type String
         * @default Constants.DEFAULT_ALL_ITEM_TEXT
         */
        this.addAllItemText = flexiciousNmsp.Filter.ALL_ITEM;
        /**
         * used to prevent add all item in the combo
         * @type {boolean}
         */
        this.ignoreAllItem = false;




        //privates

        this._selectedValue = null;

        this._selectedValueDirty = false;

        this._addAllItem = true;

        this._addAllItemDirty = true;

        this._dataProvider = [];

        this._dataProviderDirty = false;

        this._selectedIndex = -1;

        this._selectedIndexDirty = false;

        this._selectedItem = null;

        this._selectedItemDirty = false;

        /**
         * Flag to control whether or not the first optionis disabled (non selectable)
         * @type {boolean}
         * @property allItemDisabled
         */
        this.allItemDisabled = false;

        /**
         * Field on the data provider that can be used to disable the option object associated with the data provider item.
         * @type {string}
         * @property allItemDisabledField
         */
        this.allItemDisabledField = "";


        this.addEventListener(this, Constants.EVENT_CHANGE, this.changeEventHandler);
        this.addEventListener(this, Constants.EVENT_KEY_UP, this.keyUpHandler);
    }

    getClassNames() {
        return ["ComboBox", "UIComponent", "IFilterControl", "ISingleSelectFilterControl", "ISelectFilterControl"];

    }

    changeEventHandler(e) {
        let dp;
        let item;
        let i;
        if (this.domElement.selectedIndex != this.getSelectedIndex()) {
            dp = this.getDataProvider();
            this._selectedIndex = (this.domElement.selectedIndex);

            if (this.dataField) {
                this._selectedValue = (this.domElement.options[this.domElement.selectedIndex].value);
                for (i = 0; i < dp.length; i++) {
                    item = dp[i];
                    if (UIUtils.resolveExpression(item, this.dataField) == this.domElement.options[this.domElement.selectedIndex].value) {
                        this._selectedItem = (item);
                        break;
                    }
                }
            }
        }
    }

    keyUpHandler(e) {
        if (e.keyCode == Constants.KEYBOARD_UP || e.keyCode == Constants.KEYBOARD_DOWN) {
            const cbx = this.domElement;
            this.selectedIndex = cbx.selectedIndex;
            const that = this;
            setTimeout(() => {
                that.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_CHANGE));
                that.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_VALUE_COMMIT));
            }, 100);
        }
    }

    updateDisplayList(unscaledWidth, unscaledHeight) {
        flexiciousNmsp.UIComponent.prototype.updateDisplayList.apply(this, [unscaledWidth, unscaledHeight]);
        this.commitPropertiesAddAllItem();
        this.commitPropertiesSelectedValue();
        this.commitDataProvider();

    }

    //==================IFilterControl Methods===============================/
    clear() {

        //4.ComboBox.prototype.clear() method, attempts to set the selectedIndex. Thus the above method gets called.
        // At this stage, _selectedIndex = 0 causes the selected Item to be the first index of the data provider (list which does not have “ALL” value yet) to be set.
        // This causes the creation of invalid filter condition.
        this.setSelectedIndex(this.getAddAllItem() && (UIUtils.doesArrayContainValue(this.getDataProvider(), this.dataField, this.addAllItemText) != null) ? 0 : -1);

    }

    /**
     * Generic function that returns the value of a IFilterControl
     */
    getValue() {

        return this.getSelectedValue();

    }

    /**
     * Generic function that sets the value of a IFilterControl
     * @param val
     */
    setValue(val) {

        this.setSelectedValue(val);

    }

    //==================IFilterControl Methods===============================/

    commitDataProvider() {
        var cbx = this.domElement;
        var options = cbx ? cbx.options : [];
        var item, i;
        if (this._dataProviderDirty) {
            this._dataProviderDirty = false;
            while (options.length > 0) {
                options.remove(0);
            }
            var dp = this.getDataProvider();
            for (i = 0; i < dp.length; i++) {
                item = dp[i];
                options[options.length] = new Option(this.itemToLabel(item),
                    UIUtils.resolveExpression(item, this.dataField));
                if (i == this.getSelectedIndex()) {
                    options[i].selected = "selected";
                }

                if (this.allItemDisabledField) {
                    options[i].disabled = UIUtils.resolveExpression(item, this.allItemDisabledField);
                }
            }
        }
        if (this._selectedItemDirty) {
            this._selectedIndex = -1;
            this._selectedIndexDirty = true;
            this._selectedItemDirty = false;
            var dp = this.getDataProvider();
            for (i = 0; i < dp.length; i++) {
                item = dp[i];
                if (item == this.getSelectedItem()) {
                    this._selectedIndex = i;
                    break;
                }
            }
        }
        if (this._selectedIndexDirty) {
            this._selectedIndexDirty = false;
            this.setAttribute("defaultValue", this._selectedIndex);
        }
        if (!cbx) {

            this.children = options.map((opt, i) => {
                return <option key={i} value={opt.value} label={opt.label} >{opt.label}</option>;
            });
        }
    }
    componentDidMountCustom() {
        if (this._selectedIndex != this.domElement.selectedIndex) {
            this.domElement.selectedIndex = this._selectedIndex;
        }
    }
    /**
     *
     */
    commitPropertiesSelectedValue() {
        ComboBox.setSelectedItemFromValue(this);
    }

    /**
     *
     * @param val
     *
     */
    setSelectedValue(val) {

        if (!this.dataField || this.dataField.length == 0)
            throw new Error("Cannot set selected value if value is not set");
        this._selectedValue = val;
        this._selectedValueDirty = true;
        this.invalidateDisplayList();
    }

    /**
     * Gets the selected value on our comboboxes. Just like the
     * selected label, but applies to the data property.
     *
     * Needs the datafield property to be set.
     */
    getSelectedValue() {
        return this._selectedValueDirty ? this._selectedValue : (this.getSelectedItem() && this.getSelectedItem().hasOwnProperty(this.dataField))
            ? this.getSelectedItem()[this.dataField] : null;

    }

    commitPropertiesAddAllItem() {
        ComboBox.addAllItemToDataProvider(this);
    }

    /**
     * Flag, when set will cause the associated control to have
     * an Filter.ALL_ITEM Item value, which can then be used by the filtering
     * infrastructure to ignore the column in the search
     * @return
     *
     */
    getAddAllItem() {
        return this._addAllItem;
    }

    /**
     *
     */
    setAddAllItem(value) {
        this._addAllItem = value;
        this._addAllItemDirty = true;
        this.invalidateDisplayList();

    }

    register(container) {
        if (this.registered)
            container.unRegisterIFilterControl(this);
        container.registerIFilterControl(this);
        this.registered = true;
    }

    /**
     *
     * @return {Array}
     */
    getDataProvider() {
        return this._dataProvider;
    }

    /**
     *
     * @param value
     */
    setDataProvider(value) {
        this._dataProvider = value;
        this._dataProviderDirty = true;
        if (this._addAllItem)
            this.setAddAllItem(true);
        this.invalidateDisplayList();
    }

    getSelectedIndex() {
        return this._selectedIndex;
    }

    setSelectedIndex(value) {
        this._selectedIndex = value;
        this._selectedIndexDirty = true;
        if (value >= 0) {
            this._selectedItem = this.getDataProvider()[this._selectedIndex];
        } else {
            this._selectedItem = null;
        }
        this.invalidateDisplayList();
    }

    getSelectedItem() {
        return this._selectedItem;
    }

    setSelectedItem(value) {
        this._selectedItem = value;
        this._selectedItemDirty = true;
        this.invalidateDisplayList();
    }
    render() {
        return super.render();
    }
    /**
     *  Returns the String that the item renderer displays for the given data object.
     */
    itemToLabel(data) {
        return (this.labelFunction != null) ? this.labelFunction(this, data) : ComboBox.itemToLabel(this, data);
    }

    static itemToLabel(control, data) {
        if (!data)
            return "";
        if (control.labelFunction != null)
            return UIUtils.emptyIfNull(control.labelFunction(data));
        if (control.labelField || control.dataField)
            return UIUtils.emptyIfNull(UIUtils.resolveExpression(data, control.labelField || control.dataField));
        return "";
    }

    onChange(e) {
        this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_CHANGE));
        this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_VALUE_COMMIT));
    }

    //static methods:
    static addAllItemToDataProvider(control) {
        if (control.ignoreAllItem)
            return;
        if (control._addAllItemDirty) {
            const array = (control.getDataProvider());
            if (!array) {
                return;
            }
            control._addAllItemDirty = false;
            if (control.getAddAllItem()) {
                if (UIUtils.doesArrayContainValue(array, control.dataField, control.addAllItemText) == null) {
                    const newArray = array.slice();
                    const allItem = new Object();
                    allItem[control.labelField] = control.addAllItemText;
                    allItem[control.dataField] = control.addAllItemText;
                    allItem.selected = true;

                    if (control.allItemDisabledField) {
                        allItem[control.allItemDisabledField] = control.allItemDisabled;
                    }

                    newArray.splice(0, 0, allItem);
                    control.setDataProvider(newArray);
                    if (newArray.length > 0)
                        control.setSelectedItem(newArray[0]);
                }
            }
        }
    }

    static setSelectedItemFromValue(control) {
        let item;
        let i;
        let lhs;
        let rhs;
        let dp;
        if (control._selectedValueDirty) {
            control._selectedValueDirty = false;
            dp = control.getDataProvider();
            for (i = 0; i < dp.length; i++) {
                item = dp[i];
                lhs = (UIUtils.resolveExpression(item, control.dataField)).toString().toLowerCase();
                rhs = control._selectedValue ? control._selectedValue.toString().toLowerCase() : control._selectedValue;
                if (lhs == rhs) {
                    control.setSelectedItem(item);
                    return;
                }
            }
            //we were not able to set - so set to null
            control.setSelectedIndex(-1);
        }
    }

    /**
     * Sets the focus on the textbox
     **/
    focus() {
        if (UIUtils.isIE8()) {
            setTimeout(this.domElement.focus, 100)
        } else {
            flexiciousNmsp.UIComponent.prototype.focus.apply(this);
        }

    }
}

flexiciousNmsp.ComboBox = ComboBox; //add to name space
ComboBox.prototype.typeName = ComboBox.typeName = 'ComboBox';//for quick inspection