/*eslint-disable */
 
import React from 'react'
import DateRange from '../utils/DateRange'
import ComboBox from './ComboBox'
import DateRangePicker from './DateRangePicker'
import FilterExpression from '../flexgrid/grids/filters/FilterExpression'
import FlexDataGridEvent from '../flexgrid/events/FlexDataGridEvent'
import ToolbarAction from '../flexgrid/valueobjects/ToolbarAction'

import Constants from '../utils/Constants'
import UIUtils from '../utils/UIUtils' 
/**
 * A DateComboBox that which can be used within the filtering/binding infrastructure.
 * @class DateComboBox
 * @constructor
 * @namespace flexiciousNmsp
 * 
 */
export default class DateComboBox extends ComboBox {
    constructor(props) {
        super(props);

        this._text = "";

        this.addAllItem = true;

        this.customDateRange = new DateRange(flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM, null, null);

        this.popup = null;
        /**
         * The selected date range
         */
        this.dateRange = null;
        /**
         * An array of DateRange.DATE_RANGE_* constants.
         */
        this._dateRangeOptions = null;
        /**
         * Date format for the custom date range popup
         */
        this.dateFormatString = Constants.DEFAULT_DATE_FORMAT;
        /**
         * One of the DateRange constants, defines the default date range for the when the user clicks on the custom date range. Defaults to last DATE_RANGE_LAST_7_DAYS
         */
        this.defaultDateRangeForDatePicker = flexiciousNmsp.DateRange.DATE_RANGE_LAST_7_DAYS;
        /**
         * Boolean flag to control the visibility of the time picker component when the user chooses to pick a custom date.
         */
        this.showTimePicker = false;


        this.filterComparisionType = flexiciousNmsp.FilterExpression.FILTER_COMPARISION_TYPE_DATE;
        if(this.isChromeAndLinux() || this.isMacAndFirefox() || this.isMacAndSafari())
            this.addEventListener(this, Constants.EVENT_MOUSE_DOWN, this.onClick);
        if(!this.isMacAndFirefox() && !this.isMacAndSafari())
            this.addEventListener(this, Constants.EVENT_MOUSE_CLICK, this.onClick);

        this.setDateRangeOptions([
            flexiciousNmsp.DateRange.DATE_RANGE_LAST_SIXTY_MINTUES,
            flexiciousNmsp.DateRange.DATE_RANGE_LAST_24_HOURS,
            flexiciousNmsp.DateRange.DATE_RANGE_LAST_7_DAYS,
            flexiciousNmsp.DateRange.DATE_RANGE_LASTHOUR, flexiciousNmsp.DateRange.DATE_RANGE_THISHOUR,
            flexiciousNmsp.DateRange.DATE_RANGE_NEXTHOUR, flexiciousNmsp.DateRange.DATE_RANGE_YESTERDAY,
            flexiciousNmsp.DateRange.DATE_RANGE_TODAY, flexiciousNmsp.DateRange.DATE_RANGE_TOMORROW,
            flexiciousNmsp.DateRange.DATE_RANGE_LASTWEEK, flexiciousNmsp.DateRange.DATE_RANGE_THISWEEK,
            flexiciousNmsp.DateRange.DATE_RANGE_NEXTWEEK, flexiciousNmsp.DateRange.DATE_RANGE_LASTMONTH,
            flexiciousNmsp.DateRange.DATE_RANGE_THISMONTH, flexiciousNmsp.DateRange.DATE_RANGE_NEXTMONTH,
            flexiciousNmsp.DateRange.DATE_RANGE_LASTQUARTER, flexiciousNmsp.DateRange.DATE_RANGE_THISQUARTER,
            flexiciousNmsp.DateRange.DATE_RANGE_NEXTQUARTER, flexiciousNmsp.DateRange.DATE_RANGE_LASTYEAR,
            flexiciousNmsp.DateRange.DATE_RANGE_THISYEAR, flexiciousNmsp.DateRange.DATE_RANGE_NEXTYEAR,
            flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM
        ]);

    }

    //handling chrome issue on linux
    isChromeAndLinux(){
        if(navigator.userAgent.indexOf('Chrome') != -1 && navigator.userAgent.indexOf('Linux') != -1)
            return true;
        return false;
    }

    //handling non firefox issue on mac
    isMacAndNotFirefox(){
        if(navigator.userAgent.indexOf('Mac') != -1 && navigator.userAgent.indexOf('Firefox') == -1) {
            return true;
        }
        return false;
    }

    //handling safari issue on mac
    isMacAndSafari(){
        if(navigator.userAgent.indexOf('Mac') != -1 && navigator.userAgent.indexOf('Safari') != -1) {
            return true;
        }
        return false;
    }

    //handling firefox issue on mac
    isMacAndFirefox(){
        if(navigator.userAgent.indexOf('Mac') != -1 && navigator.userAgent.indexOf('Firefox') != -1) {
            return true;
        }
        return false;
    }

    getClassNames() {

        return ["DateComboBox", "ComboBox", "UIComponent", "IFilterControl", "IDateComboBox", "IRangeFilterControl", "ISingleSelectFilterControl", "ISelectFilterControl"];
    }
    /**
     *
     */
    onClick(e) {
        const domTarget = (e.triggerEvent.currentTarget || e.triggerEvent.srcElement);
        //if(domTarget!=this.select)return;
        const si = this.getDataProvider()[this.domElement.selectedIndex];
        if (si && (si[this.dataField] == flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM)) {
            if (e.localX > domTarget.offsetWidth - DateComboBox.DROPDOWN_BUTTON_WIDTH) return;
            // this.popup = new DateRangePicker();
            // this.popup.combo = this;
            // this.popup.grid = this.grid;
            // this.popup.dateFormatString = this.dateFormatString;
            const actions = [ToolbarAction.create(Constants.MCS_BTN_APPLY_LABEL, this.onDatePicker, true),
            ToolbarAction.create(Constants.MCS_BTN_CANCEL_LABEL, this.onDatePickerCancel.bind(this), true),
            ];


            this.popup = <DateRangePicker combo={this} grid={this.grid} dateFormatString={this.dateFormatString} />
            this.popup = UIUtils.addPopUp(this.popup, this.domElement, true, null, "Select Date", actions);
            this.grid.addPopup(this.popup);
            //this.popup.addEventListener(this, Constants.EVENT_CHANGE, this.onDatePicker);
            e.triggerEvent.preventDefault();
            this.domElement.blur();

        }
    }

    onDatePickerCancel(evt) {
        this.grid.removePopup(this.popup);
    }

    onDatePicker(picker) {
        var dateComboBox = picker.domElement.component.props.combo;
        if (DateRangePicker.validate(picker)) {
            dateComboBox.customDateRange.startDate = picker.startDate;
            dateComboBox.customDateRange.endDate = picker.endDate;
            dateComboBox.dispatchEvent(new FlexDataGridEvent(Constants.EVENT_CHANGE));
            dateComboBox.grid.removeChild(dateComboBox.popup);
            return true;
        }else{
            UIUtils.showMessage(Constants.DATE_PICKER_MSG_TXT_ERROR_MSG_TXT);
        }
        return false;
    }

    dispatchEvent(evt) {
        if (evt.type == Constants.EVENT_CHANGE && evt.currentTarget == this) {
            const si = this.getDataProvider()[this.domElement.selectedIndex];
            if (si && (si[this.dataField] == flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM)) {
                if(this.isMacAndNotFirefox()){
                    this.onClick(evt);
                }
                return;
            }
        }
        return super.dispatchEvent(evt);
    }

    /**
     * Generic function that returns the value of a IFilterControl
     */
    getValue() {
        const selectedValue = this.getSelectedValue();

        if (selectedValue == flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM) {
            const dt = new flexiciousNmsp.DateRange(this.defaultDateRangeForDatePicker, null, null);
            return `${flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM}__${this.customDateRange.startDate ? this.customDateRange.startDate.toString() :
                dt.startDate.toString()}__${this.customDateRange.endDate ? this.customDateRange.endDate.toString() : dt.endDate}`;
        }
        else
            return super.getValue();

    }

    /**
     * Generic function that sets the value of a IFilterControl
     * @param val
     */
    setValue(val) {
        if (val.toString().indexOf(flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM) == 0) {
            const split = val.toString().split("__");
            if (split.length == 3) {
                super.setValue(split[0]);
                this.customDateRange.startDate = new Date(Date.parse(split[1].toString()));
                this.customDateRange.endDate = new Date(Date.parse(split[2].toString()));
            } else {
                super.setValue(val);
            }
        }
        else
            super.setValue(val);

    }

    /**
     * The calculated date range start
     */
    getSearchRangeStart() {
        return this.getDateRange() ? this.getDateRange().startDate : null;
    }

    /**
     * The calculated date range end
     */
    getSearchRangeEnd() {
        return this.getDateRange() ? this.getDateRange().endDate : null;

    }

    /**
     * Maximum value for the date picker
     */
    getMaxValue() {
        //I hope the world ends before 2099.
        return new Date(2099, 1, 1);

    }

    /**
     * Minimum value for the date picker
     */
    getMinValue() {
        return new Date(1970, 1, 1);
    }

    getDateRange() {
        const si = this.getSelectedItem();

        if (si != null && si[this.labelField].toString() != this.addAllItemText) {
            if (si[this.dataField] == flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM) {
                return (this.customDateRange.startDate != null &&
                    this.customDateRange.endDate != null && this.customDateRange.startDate.getTime() != this.customDateRange.endDate.getTime()) ? this.customDateRange : null;
            }
            else {
                this.customDateRange = new flexiciousNmsp.DateRange(flexiciousNmsp.DateRange.DATE_RANGE_CUSTOM, null, null);
                return new flexiciousNmsp.DateRange(si[this.dataField], null, null);
            }
        }
        return null;

    }

    /**
     *
     */
    setDateRange(value) {
        if (value == null) return;
        const dp = this.getDataProvider();

        for (const object of dp) {
            if (object.data == value.dateRangeType) {
                this.setSelectedItem(object);
                break;
            }
        }
    }

    /**
     * An array of DateRange.DATE_RANGE_* constants.
     */
    setDateRangeOptions(val) {
        this._dateRangeOptions = val;
        const coll = [];

        for (const dateRange of this._dateRangeOptions) {
            coll.push({ "label": dateRange, "data": dateRange });
        }

        this.setDataProvider(coll);
    }

    /**
     * An array of DateRange.DATE_RANGE_* constants.
     */
    getDateRangeOptions() {
        return this._dateRangeOptions;
    }
}

flexiciousNmsp.DateComboBox = DateComboBox; //add to name space
DateComboBox.prototype.typeName = DateComboBox.typeName = 'DateComboBox';//for quick inspection
/**
 * Width of the dropdown button. Clicking on this button causes the dropdown to open. Clicking outside the button but on the
 * select box when the item is "custom" opens the custom date picker.
 */
DateComboBox.DROPDOWN_BUTTON_WIDTH = 16;