/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import React from 'react'
import Constants from '../utils/Constants'
import UIUtils from '../utils/UIUtils' 
import UIComponent from '../core/UIComponent'
import FlexDataGridEvent from '../flexgrid/events/FlexDataGridEvent'


/**
 * A control composed of two date fields, start date and end date, and an
 * optional DateComboBox, that allows for easy keyboard entry of date ranges.
 * @class DateRangePicker
 * @constructor
 * @namespace flexiciousNmsp
 * 
 */
export default class DateRangePicker extends UIComponent {
    constructor(props) {
        super(props, "div");
        this.startDate = null;
        this.endDate = null;
        /**
         * Date format for the custom date range popup
         */
        this.dateFormatString = Constants.DEFAULT_DATE_FORMAT;
        /**
         * Pointer back to the combobox that launched us
         */
        this.combo = { props };

        this.setWidth(400);
        this.setHeight(150);
    }
    render() {
        const that = this;
        let start = new Date();
        let end = new Date();
        const combo = this.props.combo;
        if (combo) {
            const dt = new flexiciousNmsp.DateRange(combo.defaultDateRangeForDatePicker, null, null);
            start = combo.customDateRange.startDate ? combo.customDateRange.startDate : dt.startDate;
            end = combo.customDateRange.endDate ? combo.customDateRange.endDate : dt.endDate;

        }
        this.startDate = start;
        this.endDate = end;
        //UIUtils.adapter.createDateTimePicker(UIUtils.findElementWithClassName(this.domElement, "datePickerStartDateInput"), this.dateFormatString, start, newDate => { that.startDate = newDate; }, true);
        //UIUtils.adapter.createDateTimePicker(UIUtils.findElementWithClassName(this.domElement, "datePickerEndDateInput"), this.dateFormatString, end, newDate => { that.endDate = newDate; }, true);

        // UIUtils.addDomEventListener(this, UIUtils.findElementWithClassName(this.domElement, "cancelBtn"), "click", this.onCancelClick);
        // UIUtils.addDomEventListener(this, UIUtils.findElementWithClassName(this.domElement, "okBtn"), "click", this.onOkClick);
        // <div className={"datePickerStartDate col-md-offset-1 col-md-5"}><input className={"datePickerStartDateInput"} /></div>
        // <div className={"datePickerEndDate col-md-offset-1 col-md-5"}><input className={"datePickerEndDateInput"} /></div>

        return <div>
            <input style={{ height: 1, width: 1, visibility: "collapse" }} />
            <div className={"flexiciousGrid flexiciousDatePickerPopup"}>
                <div className={"datePickerBar row"}>
                    {UIUtils.adapter.createDateTimePicker(this.dateFormatString, start, Constants.DATE_PICKER_START_DATE, "pickerStartDate")}
                    {UIUtils.adapter.createDateTimePicker(this.dateFormatString, end, Constants.DATE_PICKER_END_DATE, "pickerEndDate")}
                </div>

            </div>
        </div>;
    }
    onCancelClick(evt) {
        closeWindow(evt);
    };

    onOkClick(evt) {
        const target = evt.currentTarget || evt.srcElement;
        const pop = UIUtils.findAncestorByClassName(target, 'flexiciousDatePickerPopup').parentNode;
        if (validate(pop)) {
            pop.component.dispatchEvent(new FlexDataGridEvent(Constants.EVENT_CHANGE));
            closeWindow(evt);
        } else {
            UIUtils.showMessage("Please select start date and end date, and ensure start date is before end date.");
        }
    }
    static validate(pop) {
        const picker = pop.domElement.component;
        picker.startDate = UIUtils.adapter.getDateFromPicker(pop.refs.pickerStartDate);
        picker.startDate.setHours(0);
        picker.startDate.setMinutes(0);
        picker.startDate.setSeconds(0);
        picker.endDate = UIUtils.adapter.getDateFromPicker(pop.refs.pickerEndDate);
        picker.endDate.setHours(23);
        picker.endDate.setMinutes(59);
        picker.endDate.setSeconds(59);
        if (picker.startDate != null && picker.endDate != null && (picker.startDate < picker.endDate)) {
            return true;
        } else {
            return false;
        }
    }
    getClassNames() {
        return ["DateRangePicker", "UIComponent"]
    }
}
flexiciousNmsp.DateRangePicker = DateRangePicker;