/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import MultiSelectComboBox from './MultiSelectComboBox';
import React from 'react'
import UIUtils from '../utils/UIUtils'
import Constants from '../utils/Constants'

export default class MultiSelectComboBoxEx extends MultiSelectComboBox {
    constructor(props, tag, shouldInit) {
        super(props, tag, shouldInit);

        this._dp = null;
        this.allUnchecked = true;
        this.addAllItemText = " ";
        this.onOk = this.onOk.bind(this); 
        this.selectAllItems = this.selectAllItems.bind(this);
        this.createCheckBoxes = this.createCheckBoxes.bind(this);
        this.destroyCheckBoxes = this.destroyCheckBoxes.bind(this);
        this.delayedUpdateCheckBoxes = this.delayedUpdateCheckBoxes.bind(this);
        this.getFilteredDataBySearch = this.getFilteredDataBySearch.bind(this);
    }

    onOk() {
        const wasAllUnchecked = this.allUnchecked;
        this.allUnchecked = false;
        this.onOkButton();
        this.allUnchecked = wasAllUnchecked;
    }

    selectAllItems() {
        let dp = this.getDataProvider();
        for (let i = 0; i < dp.length; i++) {
            const item = dp[i];
            const itemVal = UIUtils.resolveExpression(item, this.dataField);
            this.selectedValues.push(itemVal);
        }
    }

    getFilteredDataBySearch(dp, text) {
        if(text === "") return dp;
        let _items = [];
        for(let key in dp) {
            if(String(dp[key].data).toLowerCase().indexOf(text.toLowerCase()) == 0) {
                _items.push(dp[key]);
            }
        }
        return _items;
    }

    createCheckBoxes(checkBoxContainer, dp) {

        for (let i = 0; i < dp.length; i++) 
        {
            const item = dp[i];
            var row = new flexiciousNmsp.UIComponent("div");
            row.attachClass("checkBoxRow");
            const cb = new flexiciousNmsp.TriStateCheckBox();
            const lbl = flexiciousNmsp.ComboBox.itemToLabel(this, item);
            cb.allowUserToSelectMiddle = false;
            cb.delayDuration = 100;
            cb.children = [lbl];
            
            cb.addEventListener(this, Constants.EVENT_MOUSE_OVER, (e) => {
                this.highlightRow((e.triggerEvent.currentTarget || e.triggerEvent.srcElement));
            });

            cb.addEventListener(this, "delayedChange", (e) => {
                const cbDataProvider = this._dp == null ? this.getDataProvider() : this._dp;
                const cbox = e.target;
                const children = UIUtils.findElementsWithClassName(this.popup.domElement, UIUtils.doLower(flexiciousNmsp.TriStateCheckBox.typeName));
                const cbIndex = children.indexOf(cbox.domElement);
                const cbItem = cbDataProvider[cbIndex];
                let cbVal = UIUtils.resolveExpression(cbItem, this.dataField);
                if(cbVal == null) 
                    cbVal = ""; 
                
                const wasAllUnchecked = this.allUnchecked;
                if (cbox.domElement.innerHTML == this.addAllItemText)
                    this.allUnchecked = cbox.getSelectedState() == flexiciousNmsp.TriStateCheckBox.STATE_UNCHECKED;
                else
                    this.allUnchecked = false;

                if (this.selectedValues.length == 0 && !wasAllUnchecked) {
                    //this means nothing was selected before.
                    if (cbox.domElement.innerHTML == this.addAllItemText) {
                        //cannot do any thing here, since nothing is selected.
                    } else {
                        for (let i = this.getAddAllItem() ? 1 : 0; i < cbDataProvider.length; i++) {
                            const item = cbDataProvider[i];
                            const itemVal = UIUtils.resolveExpression(item, this.dataField);
                            if (itemVal != cbVal) {
                                this.selectedValues.push(itemVal);
                            }
                        }
                    }
                } else {
                    if (cbox.domElement.innerHTML == this.addAllItemText) {
                        this.selectedValues = [];
                    } else {
                        if (cbox.getSelectedState() == flexiciousNmsp.TriStateCheckBox.STATE_CHECKED && !this.selectedValues.includes(cbVal)) {
                            this.selectedValues.push(cbVal);
                        } else if (cbox.getSelectedState() == flexiciousNmsp.TriStateCheckBox.STATE_UNCHECKED && this.selectedValues.includes(cbVal)) {
                            this.selectedValues.splice(this.selectedValues.indexOf(cbVal), 1);
                        }
                    }
                }
                const okB = UIUtils.findElementWithClassName(this.popup.domElement, "okButton");
                if (okB) {
                    okB.style.display = this.allUnchecked ? "none" : "";
                }
                this.updateCheckBoxes();
                this._dp = null;
            });
            
            row.addChild(cb);
            checkBoxContainer.addChild(row);
        }
    }

    delayedUpdateCheckBoxes(that) {
        setTimeout(() => {
            that.updateCheckBoxes();
        }, 100);
    }

    destroyCheckBoxes(checkBoxContainer) {
        checkBoxContainer.removeAllChildren();
    }

    /**
     * @override 
     * @param parent 
     */
    showPopup(parent) {
        if (this.popup) {
            this.destroyPopup();
            return;
        }
        this._previousValue = this.getValue().slice();
        this._previousAllUnchecked = this.allUnchecked;

        //flexiciousNmsp.ComboBox.addAllItemToDataProvider(this);

        let container = new flexiciousNmsp.UIComponent("div");
        container.attachClass("multiSelectComboBoxPopup");
        container.setStyleAttribute("width", "100%");
        //now build from dataprovider
        const dp = this.getDataProvider();
        if (dp && dp.length > 0) {

            var checkBoxContainer = new flexiciousNmsp.UIComponent("div");
            checkBoxContainer.setWidth("100%");
            checkBoxContainer.setStyleAttribute("overflowY", "auto");
            checkBoxContainer.setStyleAttribute("overflowX", "hidden");
            checkBoxContainer.setStyleAttribute("whiteSpace", "nowrap");
            checkBoxContainer.setStyleAttribute("maxHeight", (250 || this.popupHeight) + "px");

            var searchBoxContainer = new flexiciousNmsp.UIComponent("div");
            searchBoxContainer.setStyleAttribute("width", "100%");

            var searchBox = new flexiciousNmsp.TextInput();
            searchBox.setStyleAttribute("width", "100%");
            searchBox.setStyleAttribute("height", "45px");
            searchBoxContainer.addChild(searchBox);

            // to set width of inner TextInput of searchBox
            setTimeout(() => {
                searchBox.setPlaceHolder("Search");

                searchBox.getTextBox().style.cssText = "width: 100%; height: 45px; color: #666666; background-color: #ffffff";
                
                // make popup visible only when width of inner TextInput of searchBox set correctly
                setTimeout(() => {container.setStyleAttribute("visibility", "visible");}, 150);
            }, 1);

            searchBox.addEventListener(this, "delayedChange", event => {
                this._dp = this.getFilteredDataBySearch(this.getDataProvider(), searchBox.getTextBox().value);
                this.destroyCheckBoxes(checkBoxContainer);
                this.createCheckBoxes(checkBoxContainer, this._dp);
            });

            container.addChild(searchBoxContainer);

            this.createCheckBoxes(checkBoxContainer, dp);
            container.addChild(checkBoxContainer);
        }

        const allClearOkCancel = new flexiciousNmsp.UIComponent("div");
        allClearOkCancel.attachClass("allClearOkCancelDiv");
        allClearOkCancel.addChild(
            <div className={'allClearOkCancel'}>
                <span className={'allButton'}>All</span>
                <span className={'clearButton'}>Clear</span>
                <span className={'okButton'}>Ok</span>
                <span className={'cancelButton'}>Cancel</span>
            </div>
        );

        if (!this.alwaysVisible)
            container.addChild(allClearOkCancel);

        let pt = new flexiciousNmsp.Point(0, 0);
        pt = this.localToGlobal(pt);
        {
            if (!parent)
                parent = flexiciousNmsp.DisplayList.instance().documentComponent.domElement.body;
            else {
                pt = parent.globalToLocal(pt);
            }

            //pt=new flexiciousNmsp.Point(pt.x , pt.y);
            const parentContainer = new flexiciousNmsp.UIComponent("div");
            parentContainer.addChild(container);
            container = parentContainer;
        }
        parent.addChild(container);
        container.setWidth((this.dropdownWidth === -1) ? this.getWidth() : this.dropdownWidth);
        //okCancel.setWidth(this.domElement.parentNode.offsetWidth-2);

        container.attachClass("flexiciousGrid");
        container.setStyleAttribute("zIndex", 2000);
        container.setStyleAttribute("visibility", "hidden");

        var filterContainer = this.parent.typeName == "FlexDataGridFilterCell" ? this.parent.parent : this.parent;

        container.move(filterContainer.getX() + this.parent.getX() - (this.grid?this.grid.getHorizontalScrollPosition():0)
        , filterContainer.getY() + this.parent.getY() + this.parent.getHeight())


        //container.move(pt.x,pt.y + this.getHeight());
        //container.domElement.style.position = "fixed";
        this.popup = container;
        this.popup.addEventListener(this, Constants.EVENT_CLICK,
            function (e) {
                if (e.triggerEvent.target.className == "okButton") {
                    this.onOk();
                }
                else if (e.triggerEvent.target.className == "cancelButton") {
                    this.restoreStateAndDestroyPopup();
                }
                else if (e.triggerEvent.target.className == "allButton") {
                    this.selectAllItems();
                    this.delayedUpdateCheckBoxes(this);
                }
                else if (e.triggerEvent.target.className == "clearButton") {
                    this.setValue([]);
                    this.onOk();
                    
                } else if (e.triggerEvent.target == this.getTextBox()) {
                    this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent("insideIconClick"))
                }
            }
        );

        this.delayedUpdateCheckBoxes(this);

        if (!this.alwaysVisible)
            flexiciousNmsp.DisplayList.instance().documentComponent.addEventListener(this, Constants.EVENT_MOUSE_DOWN, this.onDocumentMouseUp);
    }

    /**
     * @override 
     */
    updateCheckBoxes() {
        const cbDataProvider = this._dp == null ? this.getDataProvider() : this._dp;
        const children = UIUtils.findElementsWithClassName(this.popup.domElement, UIUtils.doLower(flexiciousNmsp.TriStateCheckBox.typeName));
        for (let i = 0; i < children.length; i++) {
            const cb = children[i];
            const item = cbDataProvider[i];

            if(this.selectedValues.length>0) {
                let cbVal = UIUtils.resolveExpression(item, this.dataField);
                if(cbVal ==null){
                    cbVal="";
                }
                cb.component.setSelectedState(this.selectedValues.includes(cbVal) ? flexiciousNmsp.TriStateCheckBox.STATE_CHECKED : flexiciousNmsp.TriStateCheckBox.STATE_UNCHECKED)
            }

        }
        this.setLabel();
    }
}

flexiciousNmsp.MultiSelectComboBoxEx = MultiSelectComboBoxEx; //add to name space
MultiSelectComboBoxEx.prototype.typeName = MultiSelectComboBoxEx.typeName = 'MultiSelectComboBoxEx';//for quick 