/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import TextInput from './TextInput';
import React from 'react'
import UIUtils from '../utils/UIUtils'
import Constants from '../utils/Constants'

/**
 * A MultiSelectTextInput that implements IMultiSelectDataBoundControl (IFilterControl) and IMultiSelectDataBoundControl (IDataBoundControl)
 * which enables it to be used within the filtering/binding infrasturcture.
 * @class MultiSelectTextInput
 * @constructor
 * @namespace flexiciousNmsp
 */
export default class MultiSelectTextInput extends TextInput {
    constructor(tag) {
        super(tag);


        /**
         * A List of strings that map to the dataField value of this control,
         * representing the selected items within this control
         * @property selectedValues
         * @type Array
         */
        this.selectedValues = [];
        this.initialRowHeight = -1;
        this.that = this;
        this.createSpanItem = this.createSpanItem.bind(this);
        this.removeSelectedItem = this.removeSelectedItem.bind(this);
        this.getSelectedItemsWidth = this.getSelectedItemsWidth.bind(this);

    }

    componentDidMountCustom() {
        super.componentDidMountCustom();
        this.setLabel();
        this.initialRowHeight = 30;
    }
    addTemplate() {

        const textInputs = <span><span style={{ "marginLeft": "4px" }} className={'selectedItems'}></span><span><input style={{ "width": "80px" }} className={'textBox'} type={'text'} /><img className={'insideIcon inputIcon'} />
            <img className={'outsideIcon inputIcon'} style={{ visibility: 'hidden' }} /></span></span>;
        this.children = [textInputs];
    }

    getClassNames() {
        return ["TextInput", "MultiSelectTextInput", "UIComponent", "IFilterControl", "IMultiSelectFilterControl", "ISelectFilterControl"];
    }

    //==================IFilterControl Methods===============================/
    clear() {
        this.setValue([])
    }

    /**
     * Generic function that returns the value of a IFilterControl
     */
    getValue() {
        return this.selectedValues;
    }

    /**
     * Generic function that sets the value of a IFilterControl
     * @param val
     */
    setValue(val) {
        this.selectedValues = val;
        this.setLabel();
    }

    //==================End IFilterControl Methods===============================/
    removeSelectedItem(evt) {
        if (evt) {
            const valueToRemove = evt.currentTarget.parentElement.childNodes[1].textContent;
            if (this.selectedValues.includes(valueToRemove)) {
                const index = this.selectedValues.indexOf(valueToRemove);
                if (index > -1) {
                    this.selectedValues.splice(index, 1);
                    this.setLabel();
                    super.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE));
                    const evt = new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_KEY_UP)
                    evt.keyCode = Constants.KEYBOARD_ENTER;
                    super.dispatchEvent(evt);
                }

                if (this.that.getSelectedItemsWidth() + this.that.getTextBox().offsetWidth > this.that.getWidth()) {
                    this.that.grid.setFilterRowHeight(this.that.grid.getFilterRowHeight() - this.that.initialRowHeight);
                }
            }
        }
    }
    setLabel() {
        const spans = [];
        if (!this.getTextBox()) {
            return;
        }
        const labels = [];
        if (this.getAddAllItem() && (this.selectedValues.length == 0 || (this.selectedValues.length == 1 && this.selectedValues[0] == this.addAllItemText))
            //||  (this.getDataProvider().length==(this.selectedValues.length+1))
        ) {
        }
        else if (this.getDataProvider() && this.getDataProvider().length > 0) {
            let spanIndex = 0;
            for (const item of this.getDataProvider()) {
                let cbVal = UIUtils.resolveExpression(item, this.autoCompleteLabelField);
                if (cbVal == null) {
                    cbVal = "";
                }
                if (this.selectedValues.includes(cbVal) && (cbVal != this.addAllItemText))
                    labels.push(this.createSpanItem(spanIndex++, cbVal));
            }
        }
        const textInputs = <span><span className={'selectedItems'}>{labels}</span><span style={{ "float": "left" }}><input style={{ "minWidth": "100%", "maxHeight": "25px" }} className={'textBox'} type={'text'} /><img className={'insideIcon inputIcon'} />
            <img className={'outsideIcon inputIcon'} style={{ visibility: 'hidden' }} /></span></span>;
        this.removeAllChildren();
        this.addChild(textInputs);
    }


    createSpanItem(index, value) {
        return (
            <span key={index} style={{ "float": "left", "backgroundColor": "#abcdef", "margin": "2px 3px", "marginBottom": "2px" }} className={"selectedItem"}>
                <span onClick={this.removeSelectedItem} style={{ "fontSize": "16px", "lineHeight": "140%", "fontWeight": "lighter", "color": "#000000", "cursor": "pointer" }}>x</span>
                <span style={{ "marginLeft": "2px" }}>{value}</span>
            </span>
        );
    }

    getSelectedItemsWidth() {
        let totalWidth = 0;
        let elements = document.getElementsByClassName('selectedItems')[0].children;
        for (let i = 0; i < elements.length; i++) {
            totalWidth += elements[i].offsetWidth;
        }
        return totalWidth;
    }

    setAddAllItem() {

    }
    getAddAllItem() {
        return false;
    }
    getDataProvider() {
        return this.autoCompleteSource;
    }
    
    /**
     * Applies the selected item from the list to the autocomplete textbox
     *
     */
    applyAutoCompleteValue() {
        var elem = this.domElement;
        debugger;

        this._lastQuery = this.getText();
        if (this.autoCompleteList.getSelectedItem()) {
            const strToMatch = (UIUtils.isPrimitive(this.autoCompleteList.getSelectedItem())) ?
                this.autoCompleteList.getSelectedItem().toString() :
                UIUtils.toString(UIUtils.resolveExpression(this.autoCompleteList.getSelectedItem(), this.autoCompleteLabelField));
            this.setText("");
            if (!this.selectedValues.includes(strToMatch))
                this.selectedValues.push(strToMatch);

            if (this.that.getSelectedItemsWidth() + this.that.getTextBox().offsetWidth > this.that.getWidth()) {
                this.that.grid.setFilterRowHeight(this.that.grid.getFilterRowHeight() + this.that.initialRowHeight);
            }

            this.setLabel();
            super.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE));
            this.destroyAutoComplete();
        }
    }
    dispatchEvent(evt) {
        if (evt.type === Constants.EVENT_CHANGE) {
            //dont dispatch
        } else {
            super.dispatchEvent(evt);
        }

    }
    setIconVisible(event) {

    }
}

flexiciousNmsp.MultiSelectTextInput = MultiSelectTextInput; //add to name space
MultiSelectTextInput.prototype.typeName = MultiSelectTextInput.typeName = 'MultiSelectTextInput';//for quick inspection
