/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import UIComponent from '../core/UIComponent'
import Constants from '../utils/Constants'
import UIUtils from '../utils/UIUtils' 


/**
 * A NumericRangeBox that which can be used within the filtering/binding infrastructure.	/**
 * An  HBox that contains two textboxes to specify a range.
 * Implements IRangeFilterControl (IFilterControl)
 * @class NumericRangeBox
 * @constructor
 * @namespace flexiciousNmsp
 * 
 */
export default class NumericRangeBox extends UIComponent {
    constructor() {
        super("span");

        /**
         * Whether or not there is an active search
         * @property hasSearch
         * @type Boolean
         * @default false
         */
        this.hasSearch = false;
        /**
         * Whether or not this control has been registered. This should not be set by your code.
         * @property registered
         * @type Boolean
         * @default false
         */
        this.registered = false;
        /**
         * This is usually automatically set, you don't have to manually set it,
         * unless you're sending strings as Date objects. When set, will attempt
         * to first convert the current value to the type you specified and then
         * do the conversion.
         * Values : auto,string,number,boolean,date
         * @property filterComparisionType
         * @type String
         * @default auto
         */
        this.filterComparisionType = "auto";
        /**
         * The field to search on, usually same as the data field.
         * @property searchField
         * @type String
         * @default null
         */
        this.searchField = null;
        /**
         * The filter operation to apply to the comparison
         * See the FilterExpression class for a list.
         * Please note, for CheckBoxList and MultiSelectNumericRangeBox, this field
         * defaults to "InList" and is ignored when set.
         * Valid values are : "Equals,NotEquals,BeginsWith,EndsWith,Contains,DoesNotContain,GreaterThan,LessThan,GreaterThanEquals,LessThanEquals,InList,NotInList,Between"
         * @property filterOperation
         * @type String
         * @default Equals
         */
        this.filterOperation = null;
        /**
         * The event that the filter triggers on. Defaults to "change", or if the
         * filterRenderer supports com.flexicious.controls.interfaces.IDelayedChange, then
         * the delayedChange event.
         * @property filterTriggerEvent
         * @type String
         * @default change
         */
        this.filterTriggerEvent = "change";
        /**
         * The grid that the filter belongs to - can be null
         * if filter is used outside the grid
         * @return
         */
        this.grid = null;
        /**
         * The grid column that the filter belongs to - can be null
         * if filter is used outside the grid
         * @return
         */
        this.gridColumn = null;
        /**
         *  Name of the field in the items in the <code>dataProvider</code>
         *  Array to use as the value of drop-down list.
         *  By default, the control uses a property named <code>data</code>
         *  on each Array object and displays it.
         *  <p>However, if the <code>dataProvider</code> items do not contain
         *  a <code>data</code> property, you can set the <code>dataField</code>
         *  property to use a different property.</p>.
         *  This is also used as the search field.
         * @property dataField
         * @type String
         * @default data
         */
        this.dataField = "data";

        /**
         * The text of the separator separating the two text input boxes.
         * @property separatorText
         * @type String
         * @default "to"
         */
        this.separatorText = "to";

        /**
         * The text input that holds the start value.
         * @property textBoxStart
         * @type TextInput
         */
        this.textBoxStart = this.createTextBox();
        /**
         * The label that separates the two inputs
         * @property separator
         * @type Label
         */
        this.separator = this.createSeparator();;
        /**
         * The text input that holds the end value.
         * @property textBoxEnd
         * @type TextInput
         */
        this.textBoxEnd = this.createTextBox();
        /**
         * The div that holds the error message .
         * @property errorMessageDiv
         * @type UIComponent
         */
        this.errorMessageDiv = new flexiciousNmsp.UIComponent({}, "div");
        /**
         * Text to be used for the error message div.
         * @type {string}
         */
        this.errorMessageText = "";//Constants.NUMERIC_RANGE_BOX_ERROR_MSG_TXT || "Please ensure that the start and end values are numeric, and start value is less than or equals the end value.";

        /**
         * Boolean used to identify the textBoxStart presently contain invalid values
         * @property textBoxStartInvalid
         * @type boolean
         */
        this.textBoxStartInvalid = false;

        /**
         * Boolean used to identify the textBoxEnd presently contain invalid values
         * @property textBoxEndInvalid
         * @type boolean
         */
        this.textBoxEndInvalid = false;

        /**
         * Boolean used to identify if end < start
         * @property rangeInvalid
         * @type boolean
         */
        this.rangeInvalid = false;


        this.addChild(this.textBoxStart);
        this.addChild(this.separator);
        this.addChild(this.textBoxEnd);


        this.textBoxStart.isNumeric = true;
        this.textBoxEnd.isNumeric = true;

        //UIUtils.addChild(document.body, this.errorMessageDiv);
        this.errorMessageDiv.attachClass("errorMessage");
        this.errorMessageDiv.setStyleAttribute("display", "none");

        this.textBoxStart.addEventListener(this, "delayedChange", this.onChange_textBoxStart);
        this.textBoxEnd.addEventListener(this, "delayedChange", this.onChange_textBoxEnd);
        this.textBoxStart.addEventListener(this, Constants.EVENT_MOUSE_OVER, this.onMouseOver);
        this.textBoxEnd.addEventListener(this, Constants.EVENT_MOUSE_OVER, this.onMouseOver);
        this.textBoxStart.addEventListener(this, Constants.EVENT_MOUSE_OUT, this.onMouseOut);
        this.textBoxEnd.addEventListener(this, Constants.EVENT_MOUSE_OUT, this.onMouseOut);
    }

    getClassNames() {
        return ["NumericRangeBox", "UIComponent", "IFilterControl", "IRangeFilterControl"];

    }
    createSeparator(){
        const sep = new flexiciousNmsp.Label();
        sep.setText(this.separatorText);
        return sep;

    }
    createTextBox(){
        return new flexiciousNmsp.TextInput();
    }
    //==================IFilterControl Methods===============================/
    /**
     * Clears out the textboxes
     */
    clear() {
        this.textBoxEnd.clear();
        this.textBoxStart.clear();
    }

    /**
     * Generic function that returns the value of a IFilterControl
     */
    getValue() {
        return this.getRange() == null ? null : this.getRange();
    }

    /**
     * Generic function that sets the value of a IFilterControl
     * @param val
     */
    setValue(val) {
        this.setRange(val);
    }

    //==================IFilterControl Methods===============================/

    onChange_textBoxEnd(evt) {
        this.onChange();
    }

    onChange_textBoxStart(evt) {
        this.onChange();
    }

    onChange() {
        this.errorMessageDiv.setStyleAttribute("display", "none");
        if (this.isRangeValid()) {
            this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE, false, false));
        }
        if (this.textBoxEnd.getValue() == "" && this.textBoxStart.getValue() == "") {
            this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE, false, false));
        }
        this.checkDirty(); // TODO check dirty for each text box validate on edit. if we don't want that behaviour comment out this
    }

    onMouseOver(event) {
        if (event.currentTarget.getValue() != "")
            this.checkDirty();
    }

    onMouseOut(event) {
        if (this.textBoxStartInvalid || this.textBoxEndInvalid || this.rangeInvalid)
            this.hideErrorTip();
    }

    isValidNumber(txtBox) {
        const rgx = /^[0-9]*\.?[0-9]*$/;
        return txtBox && (!txtBox.getValue() || (txtBox.getValue().match(rgx)));
    }

    checkDirty() {
        if (!this.errorMessageText) return; //only kick in if we are explicitly error checking

        this.textBoxStartInvalid = !this.isValidNumber(this.textBoxStart);
        this.textBoxEndInvalid = !this.isValidNumber(this.textBoxEnd);


        if (this.isRangeValid()) {
            this.rangeInvalid = false;
        } else {
            if (this.textBoxEnd.getValue() != "" && this.textBoxStart.getValue() != "") {
                //this means user provided invalid values.
                this.rangeInvalid = true;
            } else {
                this.rangeInvalid = false;
            }
        }

        if (this.textBoxEndInvalid || this.textBoxStartInvalid || this.rangeInvalid) {
            this.textBoxStart.getTextBox().setStyleAttribute("border", NumericRangeBox.prototype.TEXTBOX_DIRTY_STYLE_BORDER);
            this.textBoxEnd.getTextBox().setStyleAttribute("border", NumericRangeBox.prototype.TEXTBOX_DIRTY_STYLE_BORDER);
            if (this.textBoxEndInvalid && this.textBoxEnd.getValue() != "")
                this.showErrorTip(this.textBoxEnd);
            else if (this.textBoxStartInvalid && this.textBoxStart.getValue() != "")
                this.showErrorTip(this.textBoxStart);
            else if (this.rangeInvalid) {
                this.showErrorTip(this.textBoxEnd);
            }
        } else {
            this.textBoxStart.getTextBox().setStyleAttribute("border", "");
            this.textBoxEnd.getTextBox().setStyleAttribute("border", "");
            this.hideErrorTip();
        }
    }

    showErrorTip(txtBox) {
        if (!txtBox || !txtBox.getTextBox())
            return;
        const gap = 10;
        const txtBoxBound = txtBox.getTextBox().getBoundingClientRect();

        UIUtils.addChild(document.body, this.errorMessageDiv);

        this.errorMessageDiv.setStyleAttribute("display", "inline-block");
        this.errorMessageDiv.setText(this.errorMessageText);
        const pos = getErrorTipPos(txtBoxBound, this.errorMessageDiv);
        this.errorMessageDiv.setDtyleAttribute("position", "absolute");
        this.errorMessageDiv.setDtyleAttribute("zIndex", "1111111");
        this.errorMessageDiv.setDtyleAttribute("top", `${(txtBoxBound.top
            + document.body.scrollTop
            - (this.errorMessageDiv.domElement.clientHeight / 2
                - (txtBox.domElement.clientHeight || txtBox.domElement.offsetHeight) / 2)) + 3}px`);
        if (pos == Constants.POS_LEFT) {
            this.errorMessageDiv.attachClass("errorMessage errorTipRight");
            this.errorMessageDiv.setDtyleAttribute("right", `${txtBoxBound.left + gap}px`);
        } else {
            this.errorMessageDiv.attachClass("errorMessage errorTipLeft");
            this.errorMessageDiv.setStyleAttribute("left", `${txtBoxBound.right + gap}px`);
        }
    }

    hideErrorTip() {
        UIUtils.removeChild(document.body, this.errorMessageDiv);
        this.errorMessageDiv.setStyleAttribute("display", "none");
    }

    isRangeValid() {
        const startIndex = parseFloat(this.textBoxStart.getValue());
        const endIndex = parseFloat(this.textBoxEnd.getValue());
        return (!isNaN(startIndex) && !isNaN(endIndex) && startIndex <= endIndex);

    }

    /**
     *Wipes out the text boxes
     */
    reset() {
        this.clear();
    }

    setActualSize(w, h) {
        const spW = 15;
        flexiciousNmsp.UIComponent.prototype.setActualSize.apply(this, [w, h]);
        const remaining = w - spW;
        this.textBoxStart.setActualSize(remaining / 2, h);
        this.textBoxEnd.setActualSize(remaining / 2, h);

    }

    /**
     *Sets the focus on the first checkbox
     */
    focus() {
        this.textBoxStart.focus();
    }

    /**
     * Returns the minimum value of the range
     */
    getMinValue() {
        return this.getRangeStart() != -1 ? this.getRangeStart() : Number.MIN_VALUE;
    }

    /**
     * Returns the maximum value of the range
     */
    getMaxValue() {
        return this.getRangeEnd() != -1 ? this.getRangeEnd() : Number.MAX_VALUE;
    }

    /**
     *If the range is valid, returns the first value of the range.
     */
    getSearchRangeStart() {
        return this.isRangeValid() ? this.getRangeStart() : null;
    }

    /**
     *If the range is invalid, returns the last value of the range.
     */
    getSearchRangeEnd() {
        return this.isRangeValid() ? this.getRangeEnd() : null;
    }

    /**
     * The start of the range
     */
    getRangeStart() {
        const startIndex = parseFloat(this.textBoxStart.getValue());
        return (isNaN(startIndex) ? -1 : startIndex);
    }

    /**
     *
     */
    setRangeStart(o) {
        this.textBoxStart.setValue(o.toString());
    }

    /**
     *Returns an array of [rangeStart,rangeEnd]
     */
    getRange() {
        if (!this.isRangeValid()) return null;
        const a = new Array();
        a.push(this.getRangeStart());
        a.push(this.getRangeEnd());
        return a;
    }

    /**
     *
     */
    setRange(value) {
        if (value == null) return;
        if (value.length != 2) throw new Error("Invalid argument for numeric range box range");
        this.textBoxStart.setValue(value[0].toString());
        this.textBoxEnd.setValue(value[1].toString());
        if (!this.isRangeValid())
            this.reset();
    }

    /**
     * The end of the range
     */
    getRangeEnd() {
        const endIndex = parseFloat(this.textBoxEnd.getValue());
        return (isNaN(endIndex) ? -1 : endIndex);
    }

    /**
     * The start of the range
     */
    setRangeEnd(o) {
        this.textBoxEnd.setValue(o.toString());
    }
}

flexiciousNmsp.NumericRangeBox = NumericRangeBox; //add to name space
NumericRangeBox.prototype.typeName = NumericRangeBox.typeName = 'NumericRangeBox';//for quick inspection
NumericRangeBox.prototype.TEXTBOX_DIRTY_STYLE_BORDER = "1px solid red";

// utils

var getErrorTipPos = (bounds, errTip) => (bounds.right + errTip.domElement.clientWidth) >= getViewportSize().width ?
    Constants.POS_LEFT : Constants.POS_RIGHT;

var getViewportSize = () => {

    let viewPortWidth;
    let viewPortHeight;

    // the more standards compliant browsers (mozilla/netscape/opera/IE7) use window.innerWidth and window.innerHeight
    if (typeof window.innerWidth != 'undefined') {
        viewPortWidth = window.innerWidth,
            viewPortHeight = window.innerHeight
    }

    // IE6 in standards compliant mode (i.e. with a valid doctype as the first line in the document)
    else if (typeof document.documentElement != 'undefined'
        && typeof document.documentElement.clientWidth !=
        'undefined' && document.documentElement.clientWidth != 0) {
        viewPortWidth = document.documentElement.clientWidth,
            viewPortHeight = document.documentElement.clientHeight
    }

    // older versions of IE
    else {
        viewPortWidth = document.getElementsByTagName('body')[0].clientWidth,
            viewPortHeight = document.getElementsByTagName('body')[0].clientHeight
    }
    return { width: viewPortWidth, height: viewPortHeight };
};


Constants.POS_LEFT = "left";
Constants.POS_RIGHT = "right";
Constants.POS_TOP = "top";
Constants.POS_BOTTOM = "bottom";