/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import UIComponent from '../core/UIComponent'
import Timer from '../utils/Timer'

import Constants from '../utils/Constants'
import UIUtils from '../utils/UIUtils'

/**
 * A TextInput  that implements ITextFilterControl (IFilterControl) and ITextDataBoundControl (IDataBoundControl)
 * which enables it to be used within the filtering/binding infrasturcture. It also adds support for a number
 * of features like autocomplete, inside and outside icons, etc.
 * @class TextInput
 * @constructor
 * @namespace flexiciousNmsp
 * 
 */
export default class TextInput extends UIComponent {
    constructor(props, tag, shouldInit) {

        super(props, typeof tag === "string" ? tag : "span", false);
        this.addTemplate();

        //BEGIN FILTER PROPERTIES
        /**
         * Whether or not there is an active search
         * @property hasSearch
         * @type Boolean
         * @default false
         */
        this.hasSearch = false;
        /**
         * Whether or not this control has been registered. This should not be set by your code.
         * @property registered
         * @type Boolean
         * @default false
         */
        this.registered = false;
        /**
         * This is usually automatically set, you don't have to manually set it,
         * unless you're sending strings as Date objects. When set, will attempt
         * to first convert the current value to the type you specified and then
         * do the conversion.
         * Values : auto,string,number,boolean,date
         * @property filterComparisionType
         * @type String
         * @default auto
         */
        this.filterComparisionType = "auto";
        /**
         * The field to search on, usually same as the data field.
         * @property searchField
         * @type String
         * @default null
         */
        this.searchField = null;
        /**
         * The filter operation to apply to the comparison
         * See the FilterExpression class for a list.
         * Please note, for CheckBoxList and MultiSelectComboBox, this field
         * defaults to "InList" and is ignored when set.
         * Valid values are : "Equals,NotEquals,BeginsWith,EndsWith,Contains,DoesNotContain,GreaterThan,LessThan,GreaterThanEquals,LessThanEquals,InList,NotInList,Between"
         * @property filterOperation
         * @type String
         * @default Equals
         */
        this.filterOperation = null;
        /**
         * The event that the filter triggers on. Defaults to "change", or if the
         * filterRenderer supports com.flexicious.controls.interfaces.IDelayedChange, then
         * the delayedChange event.
         * @property filterTriggerEvent
         * @type String
         * @default change
         */
        this.filterTriggerEvent = "enterKeyUp";
        /**
         * The grid that the filter belongs to - can be null
         * if filter is used outside the grid
         * @property grid
         * @type FlexDataGrid
         * @default null
         */
        this.grid = null;
        /**
         * The grid column that the filter belongs to - can be null
         * if filter is used outside the grid
         * @property grid
         * @return
         */
        this.gridColumn = null;
        //END FILTER PROPERTIES

        //BEING DELAYED CHANGE PROPERTIES
        /**
         * The amount of time (in milliseconds) to wait before dispatching the DELAY_CHANGE event.
         * @property delayDuration
         * @type int
         * @default 500
         */
        this.delayDuration = 500; // delay in milliseconds before "delayedChange" event is dispatched.
        /**
         * Whether or not to enable the DELAY_CHANGE event.
         * @property enableDelayChange
         * @type int
         * @default 500
         */
        this.enableDelayChange = true;
        //END DELAYED CHANGE PROPERTIES

        /**
         *  Flag to control whether the icon only shows up when there is text in the box
         * @property showIconWhenHasText
         * @type Boolean
         * @default 500
         */
        this.showIconWhenHasText = false;


        /**
         * If you specify an Icon Style, setting this variable to true will clear the text on icon click.
         * Used for a "clear" icon.
         * @property clearTextOnIconClick
         * @type Boolean
         * @default false
         */
        this.clearTextOnIconClick = false;
        /**
         * Enables auto complete behavior. Please ensure you set the autocompletesource property.
         * @property enableAutoComplete
         * @type Boolean
         * @default false
         */
        this.enableAutoComplete = null;
        /**
         * Plugin specific options for the autocomplete. Depending on the flavor of Flexicious you go with,
         * flexicious will use either the JQuery, Dojo or EXT versions of the autocomplete. This object will
         * be used to pass in additional parameters for autocomplete.
         * @property autoCompleteOptions
         * @type Object
         * @default null
         */
        this.autoCompleteOptions = null;

        /**
         * The source for the autocomplete dropdown (the list of items).
         * @property autoCompleteSource
         * @type Object
         * @default null
         */
        this.autoCompleteSource = null;

        /**
         * Input mask for the text input
         * @property inputMask
         * @type Text
         * @default null
         */
        this.inputMask = null;
        /**
         * Plugin specific options for the input mask. Depending on the flavor of Flexicious you go with,
         * flexicious will use either the JQuery, Dojo or EXT versions of the input mask. This object will
         * be used to pass in additional parameters for input mask.
         * @property inputMaskOptions
         * @type Object
         * @default null
         */
        this.inputMaskOptions = null;

        /**
         * To enable use of the text input in picker type scenarios where we display a label string in the text input,
         * but expose a store for the ID value.
         * @property idValue
         * @type Object
         * @default null
         */
        this.idValue = null;

        /**
         * Dispatched when the user clicks on a the icon specified by the insideIcon property
         * @property onInsideIconClick
         * @type Function
         * @default null
         */
        this.onInsideIconClick = null;
        /**
         * Dispatched when the user clicks on a the icon specified by the outsideIcon property
         * @property onOutsideIconClick
         * @type Function
         * @default null
         */

        this.onOutsideIconClick = null;
        /**
         *  The icon that is displayed inside the textbox, based on the insideIconPosition property.
         * [Style(name="insideIcon", type="Class", inherit="no")]
         * @property insideIcon
         * @type String
         * @default null
         */

        this.insideIcon = null;
        /**
         *  The position of the icon that is displayed inside the textbox.
         *  Default 'right'
         * @property insideIconPosition
         * @type String
         * @default right
         */

        this.insideIconPosition = null;
        /**
         * The icon that is displayed outside the textbox, based on the outsideIconPosition property.
         * [Style(name="outsideIcon", type="Class" , inherit="no")]
         * @property outsideIcon
         * @type String
         * @default null
         */

        this.outsideIcon = null;
        /**
         *  The position of the icon that is displayed outside the textbox.
         *  Default 'right'
         * [Style(name="outsideIconPosition", type="String", enumeration="left,right")]
         * @property outsideIconPosition
         * @type String
         * @default right
         */
        this.outsideIconPosition = null;
        /**
         * The spacing between insideIcon, outsideIcon, and the text input
         * [Style(name="iconGap", type="Number", inherit="no")]
         * @property iconGap
         * @type Integer
         * @default 2
         */
        this.iconGap = 2;
        /**
         *  Style name for the auto complete dropdown
         * [Style(name="autoCompleteStyleName", type="String")]
         * @property autoCompleteStyleName
         * @type String
         * @default null
         */

        this.autoCompleteStyleName = null;
        /**
         * The watermark to show when there is no text entered into the text input.  Depending on the flavor of Flexicious you go with,
         * flexicious will use either the JQuery, Dojo or EXT versions of the watermark.
         * @property watermark
         * @type String
         * @default ""
         */
        this.watermark = "";
        /**
         * Style to apply to the watermark label
         * @property watermarkStyle
         * @type String
         * @default watermarkStyle
         */
        this.watermarkStyle = "watermarkStyle";
        /**
         * Plugin specific options for the watermark. Depending on the flavor of Flexicious you go with,
         * flexicious will use either the JQuery, Dojo or EXT versions of the water mark. This object will
         * be used to pass in additional parameters for water mark.
         * @property watermarkOptions
         * @type Object
         * @default null
         */
        this.watermarkOptions = {};

        /**
         * Width of the inside and outside icons
         * @property iconWidth
         * @type Integer
         * @default 12
         */
        this.iconWidth = 12;
        /**
         * Width of the inside and outside icons
         * @property iconHeight
         * @type Integer
         * @default 12
         */
        this.iconHeight = 12;



        /**
         * Enables autocomplete behavior. Please ensure you set the autocompletesource property.
         * @return
         */
        this.enableAutoComplete = null;
        /**
         * RowCount for the autocomplete dropdown. Defaults to 5, or max length of the dataprovider, which ever is less.
         */
        this.autoCompleteRowCount = 5;
        /**
         * Row height for the autocomplete dropdown. Defaults to 30.
         */
        this.autoCompleteRowHeight = 30;
        /**
         * Width of the autocomplete dropdown. If you do not set it, defaults to the width of the text input
         */
        this.autoCompleteWidth = 0;
        /**
         * The labelFunction for the autocomplete dropdown
         */
        this.autoCompleteLabelFunction = null;
        /**
         * Class factory for the dropdown item renderer.
         */
        this.autoCompleteItemRenderer = null;
        /**
         * The source for the autocomplete dropdown.
         * @return
         */
        this.autoCompleteSource = null;
        /**
         *  Type of the match to filter items. Can be one of: <br/>
         * <ul>
         *     <li>BeginsWith</li>
         *     <li>EndsWith</li>
         *     <li>Contains</li>
         * </ul>
         * @return
         * @default Contains
         */
        this.autoCompleteMatchType = TextInput.AUTO_COMPLETE_MATCH_TYPE_BEGINS_WITH;

        /**
         * The dataField value associated with autocomplete dropdown.
         * @return
         */
        this.autoCompleteDataField = "data";
        /**
         * The labelField value associated with autocomplete dropdown.
         * @return
         */
        this.autoCompleteLabelField = "label";

        /**
         * Property; indicates the set of characters that a user can enter in the text field. The default value is undefined. If this property is null or an empty string (""), a user can enter any character
         * If this property is a string of characters, the user can enter only characters in the string; the string is scanned from left to right. You can specify a range by using a dash (-).
         * If the string begins with ^, all characters that follow the ^ are considered unacceptable characters. If the string does not begin with ^, the characters in the string are considered
         * acceptable. The ^ can also be used as a toggle between acceptable and unacceptable characters.
         * For example, the following code allows A-Z except X and Q:
         * @type {String}
         */
        this.restrict = "";


        this.enableAutoComplete = false;
        this.autoCompleteSource = [];
        this.autoCompleteList = null;
        this.autoCompleteListAlternatingItemColors = [0xffffff, 0xffffff];

        this.maxLength = -1;

        /**
         * a placeholder text for the textinput
         * @type {string}
         */
        this._placeHolder = null;
        this.width = this.width || 100;
        if (this.typeName == 'TextInput')
            this.initFromProps(props);

    }
    componentDidMountCustom() {
        var txtBox = this.getTextBox();
        if (this._valuePending) {
            this.setValue(this._valuePending);
            this._valuePending = "";
        }
        if (this._focusPending) {
            this.focus();
            this._focusPending = "";
        }
        UIUtils.addDomEventListener(this, txtBox, Constants.EVENT_KEY_UP, this.setIconVisible);
        this.addEventListener(this, Constants.EVENT_KEY_UP, function (e) {
            const str = String.fromCharCode(e.keyCode);
            if ((str >= '0' && str <= '9') || (str >= 'a' && str <= 'z') || (str >= 'A' && str <= 'Z') || e.keyCode == Constants.KEYBOARD_DELETE || e.keyCode == Constants.KEYBOARD_BACKSPACE) {
                this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_VALUE_COMMIT));
            }
        });//so delay change is initialized
        this.addEventListener(this, Constants.EVENT_KEY_DOWN, function (e) {
            this.textBoxValue = this.getTextBox().value;
            const str = String.fromCharCode(e.keyCode);
            if ((str >= '0' && str <= '9') || (str >= 'a' && str <= 'z') || (str >= 'A' && str <= 'Z')) {
                this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_CHANGE));
            }
        });//some browsers do not dispatch change on keydown.


        this.addEventListener(this, "delayedChange", this.onTextInputDelayedChange);
        this.addEventListener(this, Constants.EVENT_FOCUS_IN, this.onFocusIn);
        this.addEventListener(this, Constants.EVENT_FOCUS_OUT, this.onFocusOut);
        this.addEventListener(this, Constants.EVENT_KEY_DOWN, this.keyDownHandler);
        this.addEventListener(this, Constants.EVENT_KEY_UP, this.keyUpHandler);
        this.addEventListener(this, Constants.EVENT_KEY_PRESS, this.validate);

        // const txtBox = this.getTextBox();
        // UIUtils.addDomEventListener(this, txtBox, Constants.EVENT_KEY_UP, this.setIconVisible);


        // if (this.enableAutoComplete) {
        //     if (!this.autoCompleteOptions) this.autoCompleteOptions = {};
        //     this.autoCompleteOptions.autoCompleteSource = this.autoCompleteSource;
        //     UIUtils.adapter.setupAutoComplete(txtBox, this.autoCompleteOptions);
        // }

        const insideIconImg = this.getInsideIcon();
        const txt = this.getTextBox();
        const outsideIconImg = this.getInsideIcon();



        this.sizeComponents();




        UIUtils.addDomEventListener(this, txt, Constants.EVENT_KEY_UP, this.setIconVisible);

        if (this.getStyle("insideIcon")) {
            UIUtils.addDomEventListener(this, insideIconImg, Constants.EVENT_CLICK, this.onInsideIcon);
            UIUtils.addDomEventListener(this, insideIconImg, Constants.EVENT_LOAD, this.onInsideIconLoad);
        }

        if (this.getStyle("outsideIcon")) {
            UIUtils.addDomEventListener(this, outsideIconImg, Constants.EVENT_CLICK, this.onOutsideIcon);
            UIUtils.addDomEventListener(this, outsideIconImg, Constants.EVENT_LOAD, this.onOutsideIconLoad);
        }

        if (this.inputMask) {
            if (!this.inputMaskOptions) this.inputMaskOptions = {};
            this.inputMaskOptions.mask = this.inputMask;
            UIUtils.setupInputMask(txt, this.inputMaskOptions);
        }
        if (this.watermark) {
            if (!this.watermarkOptions) this.watermarkOptions = {};
            this.watermarkOptions.watermark = this.watermark;
            this.watermarkOptions.watermarkStyle = this.watermarkStyle;
            UIUtils.setupWaterMark(txt, this.watermarkOptions);
        }
        if (this.maxLength > 0) {
            this.setMaxLength(this.maxLength);
        }
    }
    addTemplate() {
        this.children.push(Constants.TEMPLATE_TEXTINPUT.props.children);
    }
    setWidth(w) {
        super.setWidth(w);
        if (this.domElement) {
            this.sizeComponents();
        }
    }
    sizeComponents() {
        const insideIconImg = this.getInsideIcon();
        const txt = this.getTextBox();
        const outsideIconImg = this.getInsideIcon();
        if (this.getStyle("insideIcon")) {
            insideIconImg.src = (this.getStyle("insideIcon"));
            insideIconImg.style.top = `${Math.ceil((this.getHeight() - this.iconHeight) / 2)}px`;
            insideIconImg.style.width = `${this.iconWidth}px`;
            insideIconImg.style.height = `${this.iconHeight}px`;

            this.setIconVisible(null);
            insideIconImg.style.visibility = "";
        } else if (insideIconImg) {
            //            insideIconImg.style.visibility = "hidden";
        }

        if (this.getStyle("outsideIcon")) {
            outsideIconImg.src = (this.getStyle("outsideIcon"));
            outsideIconImg.style.top = `${Math.ceil((this.getHeight() - this.iconHeight) / 2)}px`;
            outsideIconImg.style.width = `${this.iconWidth}px`;
            outsideIconImg.style.height = `${this.iconHeight}px`;
            outsideIconImg.style.visibility = "";
        } else if (outsideIconImg) {
            //            outsideIconImg.style.visibility = "hidden";
        }

        if (outsideIconImg || insideIconImg) {
            let horizontalGap = this.getStyle("iconGap");
            if ((horizontalGap) == null) {
                horizontalGap = 2;
            }
            if (!this.freeWidth) {
                const tsw = ((this.getWidth() - (this.outsideIcon ? this.iconWidth + horizontalGap : 0)) - 2);
                txt.style.width = `${tsw > 0 ? tsw : 0}px`;
            }
            if (this.getHeight() > 0)
                txt.style.height = `${this.getHeight() - 3}px`;



            const borderX = (this.getStyle("outsideIconPosition") == "left") ? this.iconWidth + horizontalGap : 0;
            if (this.getStyle("outsideIcon")) {
                const borderWidth = this.width - this.iconWidth - horizontalGap;
                outsideIconImg.style.left = `${(this.getStyle("outsideIconPosition") == "left") ? 0 : borderWidth + borderX + horizontalGap}px`;
                //outsideIconImg.style.top=(Math.max(0,(this.height-this.iconHeight+2)/2))+ "px";
                txt.style.left = `${this.getStyle("outsideIconPosition") == "left" ? outsideIconImg.getX() + this.iconWidth + horizontalGap + horizontalGap : horizontalGap}px`;

            }
            if (this.getStyle("insideIcon")) {
                insideIconImg.style.left = `${(this.getStyle("insideIconPosition") == "left") ? horizontalGap + 1 : this.width - this.iconWidth - horizontalGap}px`;
                //insideIconImg.style.top=Math.max(0,(this.getHeight()-insideIconImg.height)/2)+"px";
                txt.style.left = `${(this.getStyle("insideIconPosition") == "left") ? borderX + horizontalGap + this.iconWidth + horizontalGap : borderX + horizontalGap}px`;

            }
        }

    }

    getClassNames() {
        return ["TextInput", "UIComponent", "IFilterControl", "ITextFilterControl", "IDelayedChange"];
    }

    onFocusIn(event) {

        this._textBeforeFocus = this.text;

    }

    onFocusOut(event) {


    }

    onTextInputDelayedChange(event) {


    }

    ///autocomplete support


    /**
     * If the mouse is clicked up, and we dont own the target, kill autocomplete.
     * @param event
     *
     */
    onAutoCompleteMouseUpHandler(event) {
        const that = TextInput.thisVar;
        if (!that.owns(event.target) && !that.autoCompleteList.owns(event.target)) {
            that.destroyAutoComplete();
        }
    }

    /**
     * Apply the list item value.
     */
    onAutoCompleteItemClick(event) {

        if (this.enableAutoComplete) {

            this.applyAutoCompleteValue();
            this.forceFilter();
        }

    }

    /**
     * Applies the selected item from the list to the autocomplete textbox
     *
     */
    applyAutoCompleteValue() {

        if (this.autoCompleteList.getSelectedItem()) {
            const strToMatch = (UIUtils.isPrimitive(this.autoCompleteList.getSelectedItem())) ?
                this.autoCompleteList.getSelectedItem().toString() :
                UIUtils.toString(UIUtils.resolveExpression(this.autoCompleteList.getSelectedItem(), this.autoCompleteLabelField));
            this.setText(strToMatch);
            this.setSelection(this.getText().length, this.getText().length);
            this.destroyAutoComplete();
            this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE));
        }
        this._lastQuery = this.getText();
    }

    keyDownHandler(e) {
        const key = e.keyCode ? e.keyCode : e.which;
        if (this.isNumeric) {
            // Allow: backspace, delete, tab, escape, enter and .
            if ( [46, 8, 9, 27, 13, 110, 190].indexOf(key) !== -1 ||
                // Allow: Ctrl+A
                (key == 65 && e.ctrlKey === true) ||
                // Allow: Ctrl+C
                (key == 67 && e.ctrlKey === true) ||
                // Allow: Ctrl+X
                (key == 88 && e.ctrlKey === true) ||
                // Allow: home, end, left, right
                (key >= 35 && key <= 39)) {
                // let it happen, don't do anything
                return;
            }
            // Ensure that it is a number and stop the keypress
            if ((e.shiftKey ||  (key !== 8 && (key < 48 || key > 57))) && (key < 96 || key > 105)) {
                e.preventDefault();
            }
        }
    }

    /**
 * If the mouse is clicked up, and we dont own the target, kill autocomplete.
 * @param event
 */
    keyUpHandler(event) {


        if (this.enableAutoComplete) {


            if (this.autoCompleteList) {
                if (event.keyCode == Constants.KEYBOARD_ESCAPE || event.keyCode == Constants.KEYBOARD_TAB) {
                    this.destroyAutoComplete();
                    return;
                } else if (event.keyCode == Constants.KEYBOARD_ENTER) {
                    event.keyCode = Constants.KEYBOARD_SPACE;//make it select
                    this.autoCompleteList.keyDownHandler(event);
                    if (this.autoCompleteList && this.autoCompleteList.getSelectedItem()) {
                        this.applyAutoCompleteValue()
                    } else {
                        this.destroyAutoComplete();
                        this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE));
                    }
                    return;
                } else if (event.keyCode == Constants.KEYBOARD_UP ||
                    event.keyCode == Constants.KEYBOARD_DOWN ||
                    event.keyCode == Constants.KEYBOARD_PAGE_DOWN ||
                    event.keyCode == Constants.KEYBOARD_PAGE_UP ||
                    event.keyCode == Constants.KEYBOARD_END ||
                    event.keyCode == Constants.KEYBOARD_HOME) {
                    //autoCompleteList.handleKey(event);
                    this.autoCompleteList.keyDownHandler(event);

                    return;
                }
            }
            const str = String.fromCharCode(event.keyCode);
            if (
                (str >= 'A' && str <= 'Z')
                || (str >= 'a' && str <= 'z')
                || (str >= '0' && str <= '9')
                || event.keyCode == Constants.KEYBOARD_SPACE
                || event.keyCode == Constants.KEYBOARD_DELETE
                || event.keyCode == Constants.KEYBOARD_BACKSPACE
            ) {
                const that = this;
                if (this.autoCompleteList && this.autoCompleteList.getDataProvider().length) {
                    setTimeout(() => {
                        that.refreshAutoComplete();
                    }, 100);
                } else {
                    setTimeout(() => {
                        that.showAutoComplete();
                    }, 100);
                }
            }
        }


    }

    /**
     * @private
     */
    refreshAutoComplete() {

        if (this.autoCompleteList && this.autoCompleteList.getDataProvider()) {
            if (this.getText().length == 0) {
                this.destroyAutoComplete();

                return;
            }

            this.autoCompleteList.rebuildBody();

            if (this.autoCompleteList.getBodyContainer().itemVerticalPositions.length == 0) {
                this.destroyAutoComplete();
            } else {
                this.autoCompleteList.setHeight(Math.max(this.autoCompleteRowCount, Math.min(this.autoCompleteList.getDataProvider().length, this.autoCompleteRowCount)) * this.autoCompleteList.getRowHeight());
            }

        }

    }

    /**
     * Shows the autocomplete.
     * @private
     */
    showAutoComplete() {


        if (this.autoCompleteList) this.destroyAutoComplete();
        if (this.getText().length == 0) {
            return;
        }
        if (this.grid && !this.grid.keyboardListenersPaused) {
            this.grid.keyboardListenersPaused = true;
        }
        this.autoCompleteList = new flexiciousNmsp.FlexDataGrid();
        this.autoCompleteList.setHorizontalScrollPolicy("off");
        this.autoCompleteList.showSpinnerOnFilterPageSort = false;
        this.autoCompleteList.setHeaderVisible(false);
        this.autoCompleteList.noDataMessage = "";
        this.autoCompleteList.alternatingItemColors = this.autoCompleteListAlternatingItemColors;
        const col = new flexiciousNmsp.FlexDataGridColumn();
        col.setDataField(this.autoCompleteLabelField);
        col.paddingRight = 0;
        //TODO
        // col.itemRenderer = new ClassFactory(BoldItemRenderer);
        const list = this.autoCompleteList;
        list.setColumns([col]);
        const autoCompleteStyleName = this.getStyle("autoCompleteStyleName");
        list.owner = this;
        if (this.autoCompleteItemRenderer)
            col.itemRenderer = this.autoCompleteItemRenderer;

        if (autoCompleteStyleName)
            list.styleName = autoCompleteStyleName;

        list.setRowHeight(this.autoCompleteRowHeight)

        //UIUtils.addChild(flexiciousNmsp.DisplayList.instance().documentComponent.domElement.body, list);
        this.grid.addChild(list);

        //list.domElement.style.zIndex = 99999;
        list.setWidth(this.getWidth() || this.autoCompleteWidth || this.domElement.offsetWidth);
        //UIUtils.positionComponent(this.domElement, list.domElement);
        var container = this.parent.typeName == "FlexDataGridFilterCell" ? this.parent.parent : this.parent;
        list.move(container.getX() + this.parent.getX(), container.getY() + this.parent.getY() + this.parent.getHeight())
        //list.setX(list.domElement.offsetLeft+this.domElement.offsetWidth-12);
        TextInput.thisVar = this;
        list.setDataProvider(this.autoCompleteSource);
        list.setFilterFunction(this.filterAutoCompete);

        list.setHeight(Math.max(1, Math.min(this.autoCompleteSource.length, this.autoCompleteRowCount)) * list.getRowHeight());
        if (this.autoCompleteLabelField)
            col.labelField = this.autoCompleteLabelField;
        if (this.autoCompleteLabelFunction != null)
            col.setLabelFunction2(this.autoCompleteLabelFunction);
        list.setWidth(this.autoCompleteWidth || this.getWidth() || this.domElement.offsetWidth);
        //alert()
        //list.height=150;

        //UIUtils.ensureWithinView(list,this)
        list.rebuild();
        list.validateNow();
        const sbRoot = document.body;
        sbRoot.addEventListener(Constants.EVENT_MOUSE_UP, this.onAutoCompleteMouseUpHandler, true);
        list.addEventListener(this, flexiciousNmsp.FlexDataGridEvent.ITEM_CLICK, this.onAutoCompleteItemClick, true);
        if (this.autoCompleteSource.length == 0) {
            this.destroyAutoComplete();//nothing to show..
        }
        this.grid.forceUpdate();

    }

    /**
     * Destroys the autocomplete list.
     *
     */
    destroyAutoComplete() {

        if (!this.autoCompleteList) return;
        if (this.grid) {
            this.grid.keyboardListenersPaused = false;
        }
        this.grid.removeChild(this.autoCompleteList);

        //UIUtils.removePopUp(this.autoCompleteList);
        const sbRoot = document.body;
        sbRoot.removeEventListener(Constants.EVENT_MOUSE_UP, this.onAutoCompleteMouseUpHandler, true);
        this.autoCompleteList = null;

    }

    /**
     * Based on the autoCompleteMatchType, filter the autocomplete dataprovider that
     * @param item
     * @return
     *
     */
    filterAutoCompete(item) {

        let strToMatch = (UIUtils.isPrimitive(item)) ?
            item.toString() :
            UIUtils.toString(UIUtils.resolveExpression(item, TextInput.thisVar.autoCompleteLabelField));

        strToMatch = strToMatch.toLowerCase();
        const txtToMatch = TextInput.thisVar.getText().toLowerCase();
        if (!strToMatch || !txtToMatch) return true;
        const strIndex = strToMatch.indexOf(txtToMatch);
        if (TextInput.thisVar.autoCompleteMatchType == TextInput.AUTO_COMPLETE_MATCH_TYPE_BEGINS_WITH) {
            return strIndex == 0;
        } else if (TextInput.thisVar.autoCompleteMatchType == TextInput.AUTO_COMPLETE_MATCH_TYPE_CONTAINS) {
            return strIndex >= 0;
        }
        else if (TextInput.thisVar.autoCompleteMatchType == TextInput.AUTO_COMPLETE_MATCH_TYPE_CONTAINS) {
            return (strIndex >= 0 &&
                (strIndex + TextInput.thisVar.getText().length) == strToMatch.length)
                ;
        }
        return true;

    }

    //===========DELAYED CHANGE INCLUDE======================//
    dispatchEvent(evt) {
        if (this.enableDelayChange) {
            if ((evt.type == Constants.EVENT_KEY_UP && (this.textBoxValue != this.getTextBox().value)) || evt.keyCode == 8) {
                // if change event, intercept
                if (this.timerInstance == null) {
                    this.timerInstance = new Timer(this.delayDuration, 1);
                    this.timerInstance.addEventListener(this, Constants.EVENT_TIMER_COMPLETE, this.onTimerComplete, false, 0, true);
                }
                this.timerInstance.reset(); // reset if already set...
                this.timerInstance.repeatCount = 1;
                // starts the timer ticking
                this.timerInstance.start();
            }
        }
        return flexiciousNmsp.UIComponent.prototype.dispatchEvent.apply(this, [evt]);
    }

    onTimerComplete(event) {
        this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent("delayedChange"));
        if (this.timerInstance) {
            this.timerInstance.stop();
            this.timerInstance = null;
        }
    }

    //===========END DELAYED CHANGE INCLUDE======================//


    //==================IFilterControl Methods===============================/
    clear() {
        if (this.getTextBox()) {
            this.getTextBox().value = "";
        }
    }

    /**
     * Generic function that returns the value of a IFilterControl
     */
    getValue() {
        return this.getTextBox() ? (this.getTextBox().value == this.watermark ? "" : this.getTextBox().value) : this._valuePending;
    }

    /**
     * Generic function that sets the value of a IFilterControl
     * @param val
     */
    setValue(val) {
        if (!this.getTextBox()) {
            this._valuePending = val;

        } else {
            this.getTextBox().value = val != null ? val.toString() : "";

        }
    }

    //==================End IFilterControl Methods===============================/
    /**
     * Returns the inner text input
     */
    getTextBox() {
        return UIUtils.findElementWithClassName(this.domElement, 'textBox');
    }

    /**
     * Returns the icon within the text input (This is usually the clear icon)
     */
    getInsideIcon() {
        return UIUtils.findElementWithClassName(this.domElement, 'insideIcon');
    }

    /**
     * Returns the icon outside the span (This is usually the picker icon)
     */
    getOutsideIcon() {
        return UIUtils.findElementWithClassName(this.domElement, 'outsideIcon');
    }

    /**
     * Sets the visibility of the icons when there is text in the box on basis of the showIconWhenHasText flag.
     */
    setIconVisible(event) {
        const othis = event ? (event.srcElement || event.currentTarget).parentNode.component : this;
        if (othis.showIconWhenHasText && othis.getInsideIcon()) {
            othis.getInsideIcon().style.visibility = (othis.getTextBox().value.length > 0 && othis.getTextBox().value != othis.watermark) ? "visible" : "hidden";
        }
    }

    /**
     * Sets the provided value as the source for the auto complete options.
     */
    setDataProvider(val) {
        this.autoCompleteSource = val;
    }

    /**
     * When icon is enabled, handles the click event of the icon. If clearTextOnIconClick is set to true, clears the text.
     * Dispatches the INSIDE_ICON_CLICK event.
     * @param evt
     **/
    onInsideIcon(evt) {
        const target = evt.currentTarget || evt.srcElement;
        const oThis = target.parentNode.component;
        oThis.dispatchEvent(new flexiciousNmsp.BaseEvent(TextInput.INSIDE_ICON_CLICK, true, true));
        if (oThis.clearTextOnIconClick) {
            oThis.clear();
            oThis.forceFilter();
        }

    }

    onInsideIconLoad(evt) {
        //        var target = evt.currentTarget || evt.srcElement;
        //        var oThis=target.parentNode.component;
        //
        //        if(oThis.getInsideIcon().width && oThis.getInsideIcon().width!= oThis.iconWidth){
        //            oThis.iconWidth =  oThis.getInsideIcon().width || oThis.iconWidth;
        //            oThis.invalidateDisplayList();
        //        }
        //        if(oThis.getInsideIcon().height && oThis.getInsideIcon().height!= oThis.iconHeight){
        //            oThis.iconHeight =  oThis.getInsideIcon().height || oThis.iconHeight;
        //            oThis.invalidateDisplayList();
        //        }
    }

    onOutsideIconLoad(evt) {
        //        var target = evt.currentTarget || evt.srcElement;
        //        var oThis=target.parentNode.component;
        //        if(oThis.getOutsideIcon().width && oThis.getOutsideIcon().width!= oThis.iconWidth){
        //            oThis.iconWidth =  oThis.getOutsideIcon().width || oThis.iconWidth;
        //            oThis.invalidateDisplayList();
        //        }
        //        if(oThis.getOutsideIcon().height && oThis.getOutsideIcon().height!= oThis.iconHeight){
        //            oThis.iconHeight =  oThis.getOutsideIcon().height || oThis.iconHeight;
        //            oThis.invalidateDisplayList();
        //        }
    }

    /**
     * When icon is enabled, handles the click event of the icon. If clearTextOnIconClick is set to true, clears the text.
     * Dispatches the INSIDE_ICON_CLICK event.
     * @param evt
     **/
    onOutsideIcon(evt) {
        const target = evt.currentTarget || evt.srcElement;
        const oThis = target.parentNode.component;
        oThis.dispatchEvent(new flexiciousNmsp.BaseEvent(TextInput.OUTSIDE_ICON_CLICK, true, true));

    }

    /**
     * Dispatches the filter programmatic.
     **/
    forceFilter() {
        this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CHANGE, true, true));
        const kb = new flexiciousNmsp.BaseEvent(Constants.EVENT_KEY_UP, true, true);
        kb.keyCode = Constants.KEYBOARD_ENTER;
        this.dispatchEvent(kb);//to trigger filter
    }

    /**
     * Sets the focus on the textbox
     **/
    focus() {
        const txtBox = this.getTextBox();
        if (!txtBox) {
            this._focusPending = true;
        }
        try {
            txtBox.focus();
        } catch (e) {
            //ignore for external date pickers.
        }
    }

    getText() {
        return this.getTextBox() ? this.getTextBox().value : "";
    }

    setText(val) {
        if (this.getTextBox()) {
            this.getTextBox().value = val;
        }
    }

    /**
     * Sets the selection on the text box.
     **/
    setSelection(begin, end) {

        const text = this.getTextBox().value;
        const txtBox = this.getTextBox();
        if (typeof begin == 'undefined') begin = 0;
        if (typeof end == 'undefined') end = text.length;

        if (text.length == 0) return;
        if (typeof begin == 'number') {
            end = (typeof end == 'number') ? end : begin;
            if (txtBox.setSelectionRange) {
                txtBox.setSelectionRange(begin, end);
            } else if (txtBox.createTextRange) {
                const range = txtBox.createTextRange();
                range.collapse(true);
                range.moveEnd('character', end);
                range.moveStart('character', begin);
                range.select();
            }
        }
    }

    setMaxLength(len) {
        const txtBox = this.getTextBox();
        if (txtBox && len) {
            txtBox.maxLength = len;
        }
    }

    validate(evt) {
        if (!this.restrict) return;
        const theEvent = evt || window.event;
        let key = theEvent.keyCode || theEvent.which;
        key = String.fromCharCode(key);
        const regex = /[0-9]|\./;
        if (!regex.test(key)) {
            theEvent.returnValue = false;
            if (theEvent.preventDefault) theEvent.preventDefault();
        }
    }

    setPlaceHolder(placeHolder) {
        this._placeHolder = placeHolder;
        this.getTextBox().placeholder = placeHolder;
    }

    getPlaceHolder() {
        return this._placeHolder;
    }
}

flexiciousNmsp.TextInput = TextInput; //add to name space 
TextInput.prototype.typeName = TextInput.typeName = 'TextInput';//for quick inspection
/**
 * Match all items in the autocomplete dataprovider that start with the value of the text property
 */
TextInput.AUTO_COMPLETE_MATCH_TYPE_BEGINS_WITH = "BeginsWith";
/**
 * Match all items in the autocomplete dataprovider that end with the value of the text property
 */
TextInput.AUTO_COMPLETE_MATCH_TYPE_ENDS_WITH = "EndsWith";
/**
 * Match all items in the autocomplete dataprovider that contain the value of the text property
 */
TextInput.AUTO_COMPLETE_MATCH_TYPE_CONTAINS = "Contains";

TextInput.INSIDE_ICON_CLICK = "insideIconClick";
TextInput.OUTSIDE_ICON_CLICK = "outsideIconClick";
TextInput.AUTO_COMPLETE_MATCH_TYPE_BEGINS_WITH = "BeginsWith";
TextInput.AUTO_COMPLETE_MATCH_TYPE_ENDS_WITH = "EndsWith";
TextInput.AUTO_COMPLETE_MATCH_TYPE_CONTAINS = "Contains";