/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../utils/Constants'
import UIUtils from '../utils/UIUtils' 
import UIComponent from '../core/UIComponent'

/**
 * A Checkbox  that implements ITriStateCheckBoxFilterControl (IFilterControl)
 * which enables it to be used within the filtering/binding infrasturcture.
 * In addition to the on and off states supported by the regular check box, the Tristate checkbox supports
 * an additional middle state, which is depicted by a checkbox with a fill, an is designed to indicate
 * that the checkbox is neither on or off, just in a undefined/null state.
 * @class TriStateCheckBox
 * @constructor
 * @namespace flexiciousNmsp
 * 
 */
export default class TriStateCheckBox extends UIComponent {
    constructor() {
        super();
        //this.setInnerHTML("");
        //BEGIN FILTER PROPERTIES
        /**
         * Whether or not there is an active search
         * @property hasSearch
         * @type Boolean
         * @default false
         */
        this.hasSearch = false;
        /**
         * Whether or not this control has been registered. This should not be set by your code.
         * @property registered
         * @type Boolean
         * @default false
         */
        this.registered = false;
        /**
         * This is usually automatically set, you don't have to manually set it,
         * unless you're sending strings as Date objects. When set, will attempt
         * to first convert the current value to the type you specified and then
         * do the conversion.
         * Values : auto,string,number,boolean,date
         * @property filterComparisionType
         * @type String
         * @default auto
         */
        this.filterComparisionType = "auto";
        /**
         * The field to search on, usually same as the data field.
         * @property searchField
         * @type String
         * @default null
         */
        this.searchField = null;
        /**
         * The filter operation to apply to the comparison
         * See the FilterExpression class for a list.
         * Please note, for CheckBoxList and MultiSelectComboBox, this field
         * defaults to "InList" and is ignored when set.
         * Valid values are : "Equals,NotEquals,BeginsWith,EndsWith,Contains,DoesNotContain,GreaterThan,LessThan,GreaterThanEquals,LessThanEquals,InList,NotInList,Between"
         * @property filterOperation
         * @type String
         * @default Equals
         */
        this.filterOperation = null;
        /**
         * The event that the filter triggers on. Defaults to "change", or if the
         * filterRenderer supports com.flexicious.controls.interfaces.IDelayedChange, then
         * the delayedChange event.
         * @property filterTriggerEvent
         * @type String
         * @default change
         */
        this.filterTriggerEvent = "change";
        /**
         * The grid that the filter belongs to - can be null
         * if filter is used outside the grid
         * @property grid
         */
        this.grid = null;
        /**
         * The grid column that the filter belongs to - can be null
         * if filter is used outside the grid
         * @property gridColumn
         * @return
         */
        this.gridColumn = null;
        //END FILTER PROPERTIES

        //BEING DELAYED CHANGE PROPERTIES
        /**
         * The amount of time (in milliseconds) to wait before dispatching the DELAY_CHANGE event.
         * @property delayDuration
         * @type int
         * @default 500
         */
        this.delayDuration= 500; // delay in milliseconds before "delayedChange" event is dispatched.
        /**
         * Whether or not to enable the DELAY_CHANGE event.
         * @property enableDelayChange
         * @type int
         * @default 500
         */
        this.enableDelayChange= true;
        //END DELAYED CHANGE PROPERTIES



        /**
         * In single select datagrids, set this to true for the icon to
         * appear as if it is a radio button
         * @property radioButtonMode
         * @type Boolean
         * @default false
         */
        this.radioButtonMode=false;



        /**
         * Allows user to select the middle mode.
         * @property allowUserToSelectMiddle
         * @type Boolean
         * @default false
         */
        this.allowUserToSelectMiddle=false;


        this._selected=false;

        this._toggle=true;

        this._middle=false;

        this._selectedSet=false;

        this.addEventListener(this,"change",e => {});//so delay change is initialized

        this.addEventListener(this,Constants.EVENT_CLICK,(e) => {
            this.clickHandler(e);
            this.invalidateDisplayList();
        });

        this.cbClass="";

        this.attachClass("unSelectableText");


    }

    getClassNames() {
        return ["TriStateCheckBox","UIComponent","IFilterControl","ITriStateCheckBoxFilterControl","IDelayedChange"];
    }

    //===========DELAYED CHANGE INCLUDE======================//
    dispatchEvent(event) {
        if(this.enableDelayChange)
        {
            if(event.type == Constants.EVENT_CHANGE)
            {
                // if change event, intercept
                if (this.timerInstance == null)
                {
                    this.timerInstance = new flexiciousNmsp.Timer(this.delayDuration);
                    this.timerInstance.addEventListener(this,Constants.EVENT_TIMER_COMPLETE, this.onTimerComplete,false,0,true);
                }
                this.timerInstance.reset(); // reset if already set...
                this.timerInstance.repeatCount = 1;
                // starts the timer ticking
                this.timerInstance.start();
            }
        }
        return flexiciousNmsp.UIComponent.prototype.dispatchEvent.apply(this,[event]);
    }

    onTimerComplete(event) {
        this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent("delayedChange"));
        this.timerInstance.stop();
        this.timerInstance=null;
    }

    //===========END DELAYED CHANGE INCLUDE======================//


    //==================IFilterControl Methods===============================/
    clear() {
        this.setSelectedState(flexiciousNmsp.TriStateCheckBox.STATE_MIDDLE);
    }

    /**
     * Generic function that returns the value of a IFilterControl
     */
    getValue() {
        return this.getSelectedState();
    }

    /**
     * Generic function that sets the value of a IFilterControl
     * @param val
     */
    setValue(val) {
        this.setSelectedState(val);
    }

    //==================End IFilterControl Methods===============================/

    /**
     * Returns one of STATE_MIDDLE|STATE_CHECKED|STATE_UNCHECKED
     * on basis of what the user has selected.
     */
    getSelectedState() {
        if(this.getMiddle())return flexiciousNmsp.TriStateCheckBox.STATE_MIDDLE;
        return (this.getSelected()?flexiciousNmsp.TriStateCheckBox.STATE_CHECKED:flexiciousNmsp.TriStateCheckBox.STATE_UNCHECKED);

    }

    /**
     * Switches the state to middle state.
     */
    setMiddle(val) {
        this._selectedSet=true;
        if (this._middle != val)
        {
            this._selected = false;
            this._middle = val;
            this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_CHANGE));
            this.invalidateDisplayList();
        }
    }

    /**
     * Returns if the current state is middle state.
     */
    getMiddle() {
        return this._middle;

    }

    setSelectedState(val) {
        this._selectedSet = true;
        if(val==flexiciousNmsp.TriStateCheckBox.STATE_MIDDLE)
        {
            this._middle=true;
            this._selected=false;
        }
        else if(val==flexiciousNmsp.TriStateCheckBox.STATE_CHECKED){
            this._selected=true;
            this._middle=false;
        }
        else if(val==flexiciousNmsp.TriStateCheckBox.STATE_UNCHECKED)
        {
            this._selected=false;
            this._middle=false;
        }
        this.invalidateDisplayList();
    }

    setActualSize(w, h) {
        flexiciousNmsp.UIComponent.prototype.setActualSize.apply(this,[w,h]);
        this.setStyleAttribute("left" , `${(this.getWidth()-12)/2}px`);
        this.setStyleAttribute("top" , `${(this.getHeight()-12)/2}px`);

    }

    /**
     * The width of this label
     * @return
     */
    getWidth() {
        return this.width ;
    }

    /**
     * The height of this label
     * @return
     */
    getHeight() {
        return this.height;
    }

    /**
     * Default click handler
     * @param event
     **/
    clickHandler(event) {
        if (!this.getEnabled())
        {
            event.stopPropagation();
            return;
        }

        if (this._toggle)
        {
            if(this.allowUserToSelectMiddle)
            {
                if(!this.getMiddle())
                {
                    if(!this.getSelected())
                    {
                        this.setMiddle(true);
                    } else {
                        this.setSelected(false);
                    }
                } else {

                    if(!this.getSelected())
                    {
                        this.setMiddle(false);
                        this.setSelected(true);
                    }
                    else
                    {
                        this.setMiddle(false);
                    }
                }
            } else {
                this.setMiddle(false);
                this.setSelected(!this.getSelected());
            }
        }
        this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_CHANGE))
    }

    /**
     * On basis of the selectedState, sets the icon
     **/
    updateDisplayList(w, h) {
        flexiciousNmsp.UIComponent.prototype.updateDisplayList.apply(this,[w,h]);
        //radioDisabled
        //radio
        //radioSelected
        //radioOver
        //radioSelectedOver
        //radioDown
        //radioSelectedDown

        //checkBoxDisabled
        //checkBox
        //checkBoxSelected
        //checkBoxMiddle
        //checkBoxOver
        //checkBoxSelectedOver
        //checkBoxMiddleOver
        //checkBoxDown
        //checkBoxSelectedDown
        //checkBoxMiddleDown
        if(this.cbClass){
            this.detachClass(this.cbClass);
        }
        if(this.radioButtonMode){
            this.cbClass= this.getEnabled()?this.getSelected()?"radioSelected":"radio":this.getSelected()?"radioSelectedDisabled":"radioDisabled";
        }else{
            this.cbClass= this.getMiddle()?this.getEnabled()?"checkBoxMiddle":"checkBoxMiddleDisabled":this.getEnabled()?this.getSelected()?"checkBoxSelected":"checkBox":this.getSelected()?"checkBoxSelectedDisabled":"checkBoxDisabled";
        }
        this.attachClass(this.cbClass);
    }

    setSelected(val) {
        this._selectedSet=true;
        if (this._selected != val)
        {
            this._selected = val;
            this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_CHANGE));
            this.invalidateDisplayList();
        }
    }

    getSelected() {
        return this._selected;

    }

    kill() {
        flexiciousNmsp.UIComponent.prototype.kill.apply(this);
        this.grid=null;
        this.gridColumn=null;
        if(this.timerInstance)
            this.timerInstance.stop();
        this.timerInstance=null;
    }
}

flexiciousNmsp.TriStateCheckBox = TriStateCheckBox; //add to name space
TriStateCheckBox.prototype.typeName = TriStateCheckBox.typeName = 'TriStateCheckBox';//for quick inspection
TriStateCheckBox.STATE_CHECKED = "checked";
TriStateCheckBox.STATE_UNCHECKED = "unchecked";
TriStateCheckBox.STATE_MIDDLE = "middle";