/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import EventDispatcher from './EventDispatcher';
import UIComponent from './UIComponent';

const isLeftButton = (evt) => {
    evt = evt || window.event;
    if ("buttons" in evt) {
        return evt.buttons == 1;
    }
    var button = evt.which || evt.button;
    return button == 1;
}
/**
 * This class keeps track of all display list objects that are in need of validation.
 * This class supports deferred rendering, to prevent the UI from having to refresh on each
 * property change. Instead, once all property changes are committed, the UI can refresh.
 * This class is supposed to be used as a singleton. Do NOT instantiate new DisplayList objects
 * instead, use DisplayList.instance().
 * @constructor
 * @class DisplayList
 * @namespace flexiciousNmsp
 * 
 */
export default class DisplayList extends EventDispatcher {
    constructor() {
        super();
        /**
        * A list of UIComponents that are pending validation.
        * @type {Array}
        */
        this.invalidObjects = [];
        /**
         * The ID of the timer object associated with this display list.
         * @type {int}
         */
        this.updatePending = -1;
        /**
         * A pointer to the document object.
         * @type {UIComponent}
         */
        this.documentComponent = new UIComponent();

        this.documentComponent.setDomElement(document);
        if (document) {
            document.addEventListener("mousedown", (evt) => {
                if (isLeftButton(evt)) {
                    DisplayList.isMouseDown++;
                }
                //console.log('mousedown')
            }
            );
            document.addEventListener("mouseup", () => {
                DisplayList.isMouseDown = 0;
                //console.log('mouseup')
            });
            document.addEventListener("mousemove", event => {
                DisplayList.pageX = event.pageX;
                DisplayList.pageY = event.pageY;
            });

        }

    }
    getClassNames() {
        return ["TypedObject", "EventDispatcher", this.typeName];
    }

    /**
     * Singleton DisplayList Instance.
     */
    static instance() {
        return DisplayList.singletonInstance;
    }

    /**
     * Removes all event listeners
     */
    kill() {
        flexiciousNmsp.EventDispatcher.prototype.kill.apply(this);
    }

    /**
     * Queues the provided object for a validate call. Once the validateObjects method is called, all the queued objects
     * get their render method called on them.
     * @param obj Object to invalidate
     */
    addInvalidObject(obj) {
        if (this.updatePending === -1) {
            const that = this;
            this.updatePending = setTimeout(() => { that.validateObjects(); }, 1);
        }
        this.invalidObjects.push(obj);
    }

    /**
     * If you wish to remove the object from a validation queue prior to it being rendered.
     * @param obj Object that was previously invalidated.
     */
    removeInvalidObject(obj) {
        const myIdx = this.invalidObjects.indexOf(obj);
        if (myIdx >= 0) {
            this.invalidObjects.splice(myIdx, 1);
        }
    }

    /**
     * Instantly validates all objects, rather than waiting for them to validate.
     */
    validateObjects() {
        if (this.updatePending !== -1) {
            window.clearTimeout(this.updatePending);
        }
        this.updatePending = -1;
        while (this.invalidObjects.length > 0) {
            const obj = this.invalidObjects.shift();
            if (obj.domElement) {
                obj.updateDisplayList(obj.getWidth(), obj.getHeight());
            }
        }
    }

    /**
     * An object to validate now, rather than waiting for the invalidation timer
     * @param obj   An object to validate now
     */
    validateNow(obj) {
        const myIdx = this.invalidObjects.indexOf(obj);
        if (myIdx >= 0) {
            obj.updateDisplayList(obj.getWidth(), obj.getHeight());
            this.invalidObjects.splice(myIdx, 1);
        }
    }
}

flexiciousNmsp.DisplayList = DisplayList;
DisplayList.prototype.typeName = DisplayList.typeName = "DisplayList";//for quick inspection
DisplayList.isMouseDown = 0;