/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import UIUtils from '../utils/UIUtils'
import Constants from '../utils/Constants'
import EventDispatcher from './EventDispatcher'
import BaseEvent from '../flexgrid/events/BaseEvent'
import FlexDataGridEvent from '../flexgrid/events/FlexDataGridEvent'
import Point from '../utils/Point'
import ClassFactory from '../utils/ClassFactory'
import DisplayList from './DisplayList'
import React from 'react'
import ReactDataGridCellContainer from './ReactDataGridCellContainer'
import ReactDOM from 'react-dom';

const uiUtil = UIUtils;
const flxConstants = Constants;;
const domEvents = [Constants.EVENT_CHANGE, Constants.EVENT_CLICK, Constants.EVENT_DOUBLE_CLICK, Constants.EVENT_FOCUS_CHANGE,
Constants.EVENT_FOCUS_IN, Constants.EVENT_FOCUS_OUT, Constants.EVENT_KEY_DOWN, Constants.EVENT_KEY_UP,
Constants.EVENT_MOUSE_CLICK, Constants.EVENT_MOUSE_DOWN, Constants.EVENT_MOUSE_MOVE, Constants.EVENT_MOUSE_OUT,
Constants.EVENT_MOUSE_OVER, Constants.EVENT_MOUSE_UP, Constants.EVENT_MOUSE_WHEEL,
Constants.EVENT_SCROLL];

let reactEventMap = {};
reactEventMap[Constants.EVENT_CHANGE] = "onChange";
reactEventMap[Constants.EVENT_CLICK] = "onClick";
reactEventMap[Constants.EVENT_DOUBLE_CLICK] = "onDoubleClick";
reactEventMap[Constants.EVENT_FOCUS_CHANGE] = "onFocusChange";
reactEventMap[Constants.EVENT_FOCUS_IN] = "onFocusIn";
reactEventMap[Constants.EVENT_FOCUS_OUT] = "onBlur";
reactEventMap[Constants.EVENT_KEY_DOWN] = "onKeyDown";
reactEventMap[Constants.EVENT_KEY_UP] = "onKeyUp";
reactEventMap[Constants.EVENT_MOUSE_CLICK] = "onClick";
reactEventMap[Constants.EVENT_MOUSE_DOWN] = "onMouseDown";
reactEventMap[Constants.EVENT_MOUSE_MOVE] = "onMouseMove";
reactEventMap[Constants.EVENT_MOUSE_OUT] = "onMouseOut";
reactEventMap[Constants.EVENT_MOUSE_OVER] = "onMouseOver";
reactEventMap[Constants.EVENT_MOUSE_UP] = "onMouseUp";
reactEventMap[Constants.EVENT_MOUSE_WHEEL] = "onWheel";
reactEventMap[Constants.EVENT_SCROLL] = "onScroll";
let pendingUpdates = [];

// Returns a function, that, as long as it continues to be invoked, will not
// be triggered. The function will be called after it stops being called for
// N milliseconds. If `immediate` is passed, trigger the function on the
// leading edge, instead of the trailing.
const debounce = function (func, wait, immediate) {
    var timeout;
    return function () {
        var context = this, args = arguments;
        var later = function () {
            timeout = null;
            if (!immediate) func.apply(context, args);
        };
        var callNow = immediate && !timeout;
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
        if (callNow) func.apply(context, args);
    };
};
/**
 * A Utility class that contains generic functions needed in multiple
 * places.
 * @constructor
 * @class UIComponent
 * @namespace flexiciousNmsp
 * 
 */
export default class UIComponent extends EventDispatcher {
    static clearPendingUpdates() {
        pendingUpdates = [];
    }
    static flushPendingUpdates() {
        // for (const comp of pendingUpdates) {
            //comp.component.setStateInternal(comp.state);
        // }
        pendingUpdates = [];
        UIComponent.statePaused = false;
    }
    constructor(propsOrTagName, tagName, shouldInit) {
        let props = {};
        if (typeof propsOrTagName === 'string') {
            //console.error('Props cannot be string')
            tagName = propsOrTagName;
            props = {};
        } else {
            props = propsOrTagName;
        }
        if (!props) {
            props = {};
        }
        super(props, tagName, shouldInit);
        this.props = props;
        if (typeof tagName === "undefined") {
            tagName = "span";
        }
        this.tagName = tagName;
        this.height = 0;
        this.width = 0;
        this._text = "";
        this.includeInLayout = true;
        this.innerHTML = "";
        /**
         * A list of event listeners that are added to the domElement
         * (these create html dom event listeners as opposed to js only event listeners)
         * @type {Array}
         */
        this.domListeners = [];

        /**
         * The dom element associated with this UIComponent.
         * @type {HTMLElement}
         */
        this.domElement = null;
        /**
        * The react element associated with this UIComponent. It will only be initialized in the render method?
        * @type {ReactElement}
        */
        this.reactElement = null;

        /**
         * Whether or not this control is enabled.
         */
        this.enabled = true;

        /**
         * Whether or not this control is enabled.
         */
        this.visible = true;


        /**
         * This is a getter/setter for the data property. When the cell is created, it belongs to a row
         * The data property points to the item in the grids dataprovider that is being rendered by this cell.
         * @type {*}
         */
        this.data = null;

        this.children = [];
        this.childrenWithProps = [];

        /**
         * A flag to store if this object has been marked for invalidation.
         * @type {Boolean}
         */
        this.invalid = false;

        this.propertyBag = {};
        for (var attr in props) {
            if (attr.indexOf('on') == 0) {
                this.addEventListener(this, attr.substring(2), props[attr]);
            }
        }

        this.propertyBag.className = this.propertyBag.className || (props ? props.className : "") || "";
        this.propertyBag.style = {};
        if (shouldInit) {
            this.initFromProps(props);
        }
        this.attachClass(UIUtils.doLower(this.typeName));

        //this.invalidateDisplayList();
    }

    initFromProps(props) {
        for (const key in props) {
            if (key == "state") {
                this.state = props.state;
            }
            else if (key !== "children") {
                //console.log(key);
                if (key.indexOf("on") == 0) {
                    this.applyAttribute(this, key, props[key], true)
                } else {
                    UIUtils.checkSetterAndApply(this, key, props[key]);
                }
            }
        }
    }
    componentWillReceiveProps(nextProps) {
        this.initFromProps(nextProps);
    }
    componentWillMount() {
        this.validateNow();
    }
    componentWillUpdate() {
        this.validateNow();
    }
    getAllChildren() {
        return this.children.concat(this.props.children || []);
    }
    render() {

        if (this.typeName == "FlexDataGrid" || this.typeName == "ReactDataGrid" || this.typeName == "ReactDataGridPsuedoScroll") {
            //console.log("rendering " + this.typeName)
            let allChildren = this.getAllChildren();
            allChildren = allChildren.map((child, idx) => {
                if (typeof child["implementsOrExtends"] == "function") {
                    if (!child.key) {
                        child.key = idx;
                    } return child.render();
                } else {
                    return child;
                }
            });
            this.reactElement = React.createElement(this.tagName, this.propertyBag, allChildren);
            return this.reactElement;

        } else {
            return <ReactDataGridCellContainer component={this} key={this.key} />
        }

    }
    setStateIfDomPresent(state, db) {
        if (this.domElement) {
            this.setState(state);
        }
    }
    removeComponentAttribute(attr) {
        delete this.propertyBag[attr];
        if (this.domElement) {//I am mounted
            //this.setState({ propertyBag: this.propertyBag });
            this.setState({ timeStamp: new Date() });
        }
    }
    getStyleAttribute(attr) {
        if (this.domElement) {//I am mounted
            return this.domElement.style[attr];
        }
        return this.propertyBag.style[attr];
    }
    getAttribute(attr) {
        if (this.domElement) {//I am mounted
            return this.domElement[attr];
        }
        return this.propertyBag[attr] || this.propertyBag[attr.toLowerCase()];
    }
    setComponentStyleAttribute(attr, val) {
        this.setStyleAttribute(attr, val);
    }
    mergeObjects(obj1, obj2) {
        let newObject = {};
        for (var key in obj1) {
            newObject[key] = obj1[key]
        }
        newObject[obj2[0]] = obj2[1];
        return newObject;
    }
    setStyleAttribute(attr, val) {
        if (this.domElement) {//I am mounted
            this.domElement.style[attr] = val;
            const newStyle = this.mergeObjects(this.propertyBag.style, [attr, val]);
            this.propertyBag.style = newStyle;
            return;
        }
        const newStyle1 = this.mergeObjects(this.propertyBag.style, [attr, val]);
        this.propertyBag.style = newStyle1;
        //this.setStateIfDomPresent({ propertyBag: this.propertyBag });
        this.setStateIfDomPresent({ timeStamp: new Date() });

    }
    setAttribute(attr, val) {
        if (this.domElement) {//I am mounted
            this.domElement[attr] = val;
            this.propertyBag[attr] = val;
            return;
        }

        if(attr === 'scrollLeft' || attr === 'scrollTop') {
            attr = attr.toLowerCase();
        }

        this.propertyBag[attr] = val;
        //this.setStateIfDomPresent({ propertyBag: this.propertyBag });
        this.setStateIfDomPresent({ timeStamp: new Date() });

    }
    getComponentStyleAttribute(attr) {
        return this.propertyBag.style[attr];
    }
    componentDidMount() {
        const domElement = ReactDOM.findDOMNode(this);
        this.setDomElement(domElement);
    }
    componentWillUnmount() {
        this.kill();
    }


    getClassNames() {
        return ["TypedObject", "EventDispatcher", "UIComponent", this.typeName];
    }

    /**
     * Sets the domElement of this UIComponent to the specified element, and creates a property on the dom element
     * called component to this.
     * @param elem  The domElement to associate with the current UIComponent.
     */
    setDomElement(elem) {
        this.domElement = elem;
        if (this.domElement !== null)
            this.domElement.component = this;
    }

    /**
     * Recursively iterates through the parent hierarchy of the given element to check to see if the current node is
     * in its parent hierarchy.
     * @param elem  The element to check
     * @return {Boolean}    If the current node is in the hierarchy of the given element
     */
    owns(elem) {
        if (!elem) {
            return false;
        }
        while (elem) {
            if (elem === this.domElement) {
                return true;
            }
            elem = elem.parentNode;
        }
        return false;
    }

    /**
     * Called as a part of render phase.
     * @param w
     * @param h
     */
    updateDisplayList(w, h) {
        if (!this.initialized) {
            this.initialize();
            this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_CREATION_COMPLETE));
        }
        if(this._stateInvalid){
            this._stateInvalid = false;
            this.setState({ timeStamp: new Date() });
        }
        this.invalid = false;
    }

    /**
     * This method will queue the component so the display list will render it once the current frame of execution is over.
     */
    invalidateDisplayList() {
        if (this.invalid) {
            return;
        }
        this.invalid = true;
        if (typeof flexiciousNmsp.DisplayList.instance != 'undefined') {
            flexiciousNmsp.DisplayList.instance().addInvalidObject(this);
        }
    }

    /**
     * Any initialization code is supposed to go in this method.
     */
    initialize() {
        this.initialized = true;
    }

    /**
     * This methods takes another UIComponent object, and adds it to the dom as a child of the domElement of this UIComponent.
     * @param child Can be either another UIComponent or a DomElement.
     */
    addChild(child) {
        this.childAddedOrRemoved=true;
        this.addChildNoState(child);
        if(!this.statePaused)
        this.setStateIfDomPresent({ numChildren: this.children.length }, true);
    }
    pauseState(){
        this.statePaused = true;
    }
    resumeState(){
        this.statePaused = false;
        this.setStateIfDomPresent({ numChildren: this.children.length }, true);
    }
    addChildNoState(child){
        if (!this.childIndex) {
            this.childIndex = 0;
        }
        if (child.typeName) {
            
            child.key = ++this.childIndex;
        }
        if (child.typeName =="FlexDataGrid") {
            child.propertyBag.key = child.key;
        }
        UIUtils.addChild(this, child);
    }
    /**
     * This methods takes another UIComponent object, and adds it to the dom as a child of the domElement of this UIComponent.
     * @param child Can be either another UIComponent or a DomElement.
     * @param idx to add the child at
     */
    addChildAt(child, idx) {
        const refChild = this.getChildAt(idx);
        if (refChild)
            UIUtils.addChild(this, child, refChild);
        else
            this.addChild(child);
    }
    /**
     * This methods takes another UIComponent object, and removes it from the dom.
     * @param child Can be either another UIComponent or a DomElement.
     */
    removeChild(child) {
        if (this.children.indexOf(child) >= 0) {
            this.childAddedOrRemoved=true;
            UIUtils.removeChild(this, child);
            if (child.reactComponent)
                child.reactComponent = null;
            if (child.domElement)
                child.domElement = null;
            //this.setStateIfDomPresent(this.state);
            if(!this.statePaused)
            this.setStateIfDomPresent({ numChildren: this.children.length }, true);
        }

    }

    removeAllChildren(child) {
        while (this.numChildren()) {
            this.removeChildAt(0);
        }
        this.childIndex = 0;
    }

    /**
     * This methods takes another UIComponent object, and removes it from the dom.
     */
    removeChildAt(idx) {
        const child = this.getChildAt(idx);
        this.removeChild(child);
    }

    /**
     * Converts the given local Point to global co-ordinates.
     * @param ptIn {flexiciousNmsp.Point}
     * @return {flexiciousNmsp.Point}
     */
    localToGlobal(ptIn) {
        const offset = UIUtils.offset(this.domElement);
        return new flexiciousNmsp.Point(offset.left + ptIn.x, offset.top + ptIn.y);
    }

    /**
     * Converts the given global Point to local co-ordinates.
     * @param ptIn {flexiciousNmsp.Point}
     * @return {flexiciousNmsp.Point}
     */
    globalToLocal(ptIn) {
        const offset = UIUtils.offset(this.domElement);
        return new flexiciousNmsp.Point(ptIn.x - (offset ? offset.left : 0) + this.domElement.scrollLeft, ptIn.y - (offset ? offset.top : 0) + this.domElement.scrollTop);
    }

    /**
     * Gets the number of children of the dom element associated with this UIComponent.
     * @return {*}
     */
    numChildren() {
        return this.children.length;
    }

    /**
     * Returns the child of the domElement at the specified index.
     * @param idx
     * @return {*}
     */
    getChildAt(idx) {
        return this.children[idx];
    }

    /**
     * First, calls super.kill(), which takes out all event listeners. Then, removes all the dom event listeners
     * that we add. Then, wipes out the reference on the domElement. Then, wipes out the domElement reference.
     * Finally, removes itself from the display list if its pending validation.
     */
    kill() {
        var i;
        let arr;
        let ev;
        let domElement;
        let f;
        for (i = 0; i < this.domListeners.length; i += 1) {
            arr = this.domListeners[i];
            domElement = arr[0];
            ev = arr[1];
            f = arr[2];
            this.removeDomEventListener(ev, f);
        }
        this.domListeners = [];
        flexiciousNmsp.DisplayList.instance().removeInvalidObject(this);
        for (var i = 0; i < this.children.length; i++) {
            if (this.children[i].kill)
                this.children[i].kill();
        }
        super.kill();;

        if (this.domElement) {
            // if (this.domElement.parentNode)
            //     UIUtils.removeChild(this.domElement.parentNode, this.domElement);
            this.domElement.component = null;
            this.domElement = null;
        }
        this.parent = null;

        //this.children = [];
        this.setState({ numChildren: this.children.length });
    }

    /**
     * When you call addEventListener on a UIComponent, certain events, defined by the domEvents array are also associated
     * with the corresponding domElement. This method is used to add such event listeners.
     * @param evt   Event to add
     * @param f     Event listener to add.
     */
    addDomEventListener(evt, f) {
        if (!this.domElement) {
            this.propertyBag[reactEventMap[evt]] = f.bind(this);
        } else {
            UIUtils.addDomEventListener(this, this.domElement, evt, f);
        }
    }

    /**
     * Removes a previously attached DOM event listener.
     * @param evt   Event to add
     * @param f     Event listener to add.
     */
    removeDomEventListener(evt, f) {
        if (this.domElement) {
            UIUtils.removeDomEventListener(this.domElement, evt, f);
        }
    }

    /**
     * Calls super.addEventListener, and for dom based events, also adds a HTML Event listener
     */
    addEventListener(context, type, listener) {
        flexiciousNmsp.EventDispatcher.prototype.addEventListener.apply(this, [context, type, listener]);
        if (domEvents.includes(type)) {
            this.addDomEventListener(type, this.domEventFired);
        }
    }

    /**
     * Calls super.addEventListener, and for dom based events, also removes the previously added HTML Event listener
     */
    removeEventListener(type, listener) {
        flexiciousNmsp.EventDispatcher.prototype.removeEventListener.apply(this, [type, listener]);
        if (domEvents.includes(type)) {
            this.removeDomEventListener(type, this.domEventFired);
        }
    }

    /**
     * Calls super.addEventListener, and for dom based events, also removes the previously added HTML Event listener
     */
    removeEventListenerKeepDomListener(type, listener) {
        flexiciousNmsp.EventDispatcher.prototype.removeEventListener.apply(this, [type, listener]);

    }

    /**
     * The event handler that gets triggered from the HTML DOM.
     * @param event Event from the HTML DOM.
     */
    domEventFired(event) {
        let evt;
        let target;
        let ptLocal;
        target = event.currentTarget || event.srcElement;

        if (!event.target && !event.srcElement) {
            return;
        }
        evt = new flexiciousNmsp.FlexDataGridEvent(event.type);
        evt.triggerEvent = event;
        if (!target.component) {
            evt.currentTarget = evt.target = this;
            this.setDomElement(target);
        } else {
            evt.currentTarget = evt.target = target.component;
        }

        if (event.type === Constants.EVENT_SCROLL || event.type == "wheel") {
            evt.scrollTop = target.scrollTop;
            evt.scrollLeft = target.scrollLeft;
            if (event.type == "wheel") {
                evt.type = Constants.EVENT_MOUSE_WHEEL;
                evt.wheelDelta = (event.deltaY);
                if (Constants.EVENT_MOUSE_WHEEL == "DOMMouseScroll") {
                    evt.wheelDelta *= 10;
                }
            }
        } else if (event.type.indexOf("mouse") === 0) {
            evt.pageX = event.pageX;
            evt.pageY = event.pageY;
            ptLocal = evt.currentTarget.globalToLocal(new flexiciousNmsp.Point(evt.pageX, evt.pageY));
            evt.localX = ptLocal.x;
            evt.localY = ptLocal.y;
        } else if (event.type.indexOf("click") === 0) {
            if (!evt.currentTarget.domElement) {
                return;
            }
            evt.pageX = event.pageX;
            evt.pageY = event.pageY;
            ptLocal = evt.currentTarget.globalToLocal(new flexiciousNmsp.Point(evt.pageX, evt.pageY));
            evt.localX = ptLocal.x;
            evt.localY = ptLocal.y;
            evt.shiftKey = event.shiftKey;
            evt.ctrlKey = event.ctrlKey;
            evt.altKey = event.altKey;
        } else if (event.type.indexOf("key") === 0) {
            if (event.which === null) {
                event.which = event.charCode !== null ? event.charCode : event.keyCode;
            }
            evt.keyCode = event.which;
            evt.shiftKey = event.shiftKey;
            evt.ctrlKey = event.ctrlKey;
            evt.altKey = event.altKey;
        }
        evt.currentTarget.dispatchEvent(evt);
    }

    validateNow() {
        this.updateDisplayList(this.getWidth(), this.getHeight());
    }

    setActualSize(w, h) {
        this.setWidth(w);
        this.setHeight(h);
        this.invalidateDisplayList();
    }
    validateNowIfValid() {
        if (this.invalid) {
            this.validateNow();
        }
    }
    invalidateState(){
        if(!this._stateInvalid){
            this._stateInvalid=true;
            this.invalidateDisplayList();    
        }
    }
    requestRender(){
        this.setState({ timeStamp: new Date() });
    }
    getWidth() {

        return this.width;
    }

    setWidth(val) {
        if (typeof val == "string") {
            this.setStyleAttribute("width", val);
            this.width = parseInt(val);
        }
        else if (val >= 0) {
            this.width = val;
            this.setStyleAttribute("width", `${val}px`);
        }
        return val;
    }
    getHeight() {

        return this.height;
    }

    getPixelHeight() {

        return this.domElement.offsetHeight;
    }

    setHeight(val) {
        if (typeof val == "string") {
            //this.domElement.style.height = val;
            this.setStyleAttribute("height", val);
            this.height = parseInt(val);
        }
        else if (val >= 0) {
            this.height = val;
            //this.domElement.style.height = `${val}px`;
            this.setStyleAttribute("height", `${val}px`);
        }
        return val;
    }

    setActualSizeFromDomElement() {
        this.setActualSize(this.getWidth(), this.getHeight());
    }

    /**
     * Returns the width of the given text.
     */
    measureText(textToMeasure) {
        if (!UIComponent.measurer) {
            UIComponent.measurer = document.createElement("div");
            UIComponent.measurer.style.position = "absolute";
        }
        UIUtils.addChild(document.body, UIComponent.measurer);
        UIComponent.measurer.innerHTML = (textToMeasure);
        const w = UIComponent.measurer.offsetWidth;
        document.body.removeChild(UIComponent.measurer);
        UIComponent.measurer = null;
        return { width: w };
    }

    /**
     * Getter for the text property
     */
    getText() {
        return this._text;
    }

    /**
     * Setter for the text property
     * @param val
     */
    setText(val) {
        this._text = val;
        if (val && typeof val === "object" && ("$$typeof" in val)) {
            //this is a react element
            this.children = [val];
            if (this.domElement) {
                this.domElement.textContent = val;
            }
        } else {
            this.setInnerHTML(val != null ? val.toString() : "");
        }
    }

    /**
     * Getter for the innerHTML property
     */
    getInnerHTML() {
        return this.innerHTML;
    }

    /**
     * Setter for the innerHTML property
     * @param val
     */
    setInnerHTML(val) {
        this.innerHTML = val;
        //UIUtils.setHtml(this.domElement, val);
        if (val && val.indexOf("<") >= 0) {
            this.children = [React.createElement("div", { dangerouslySetInnerHTML: { __html: val } })]
            if (this.domElement) {
                this.domElement.innerHTML = val;
            }
        } else {
            this.children = [val];
            if (this.domElement) {
                this.domElement.textContent = val;
            }
        }
    }

    focus() {
        if (this.domElement) {
            this.domElement.focus();
        } else {
            this.setAttribute("autoFocus", true)
        }
    }

    /**
     * Sets the X and Y co ordinates of the current dom element
     */
    move(x, y) {
        this.setX(x);
        this.setY(y);
    }

    /**
     * Gets the X co ordinate
     */
    getX() {
        return this.x;
    }

    /**
     * Gets the Y co ordinate
     */
    getY() {

        return this.y;
    }

    /**
     * Sets the X co ordinate
     */
    setX(val) {
        if (val < 0 || isNaN(val)) return val;
        this.x = val;
        if (this.domElement) {
            this.domElement.style.left = `${val}px`;
        }
        this.setStyleAttribute("left", `${val}px`);
        return val;
    }

    /**
     * Sets the Y co ordinate
     */
    setY(val) {
        if (isNaN(val)) return val;
        this.y = val;
        if (this.domElement) {
            this.domElement.style.top = `${val}px`;
        }
        this.setStyleAttribute("top", `${val}px`);
        return val;
    }

    /**
     * Sets the visible flag
     */
    setVisible(val) {
        if (this.visible != val) {
            this.visible = val;
            //this.domElement.style.display = val ? "" : "none";
            this.setStyleAttribute("display", val ? "" : "none");
        }
    }

    /**
     * Sets the visible flag
     */
    getVisible(val) {
        return this.visible;
    }

    /**
     * Sets the enabled flag
     */
    setEnabled(val) {
        this.enabled = val;
        //this.domElement.disabled = !val;
        this.setAttribute("disabled", !val);
    }

    /**
     * Sets the enabled flag
     */
    getEnabled() {
        return this.enabled;
    }

    /**
     * Sets the visible flag
     */
    setChildIndex(child, index) {
        if (child.domElement) {
            child.domElement.style.zIndex = index;
            child.propertyBag.style.zIndex = index;
        } else {
            child.setStyleAttribute("zIndex", index);
        }
    }

    getHorizontalScrollPolicy() {
        return this.horizontalScrollPolicy;
    }

    getVerticalScrollPolicy() {
        return this.verticalScrollPolicy;
    }

    setHorizontalScrollPolicy(policy) {
        this.horizontalScrollPolicy = policy;
        //this.domElement.style.overflow = (policy!=Constants.SCROLL_POLICY_OFF)?"auto":"hidden";
        const overflowX = ((policy == Constants.SCROLL_POLICY_AUTO) ? "auto" :
            (policy == Constants.SCROLL_POLICY_ON) ? "scroll" : "hidden");
        //this.domElement.style.overflowX=overflowX;
        this.setStyleAttribute("overflowX", overflowX);
    }

    setVerticalScrollPolicy(policy) {
        this.verticalScrollPolicy = policy;
        //this.domElement.style.overflow = (policy!=Constants.SCROLL_POLICY_OFF)?"auto":"hidden";
        const overflowY = ((policy == Constants.SCROLL_POLICY_AUTO) ? "auto" : (policy == Constants.SCROLL_POLICY_ON) ? "scroll" : "hidden");
        //this.domElement.style.overflowY=overflowY;
        this.setStyleAttribute("overflowY", overflowY);
    }

    /**
     *
     */
    getVerticalScrollPosition() {
        return this.domElement ? this.domElement.scrollTop : 0;
    }

    /**
     *
     */
    setVerticalScrollPosition(val) {
        if (this.domElement) {
            return this.domElement.scrollTop = val;
        } else {
            this.setAttribute("scrollTop", val);
        }
        return val;
    }

    /**
     *
     */
    getHorizontalScrollPosition() {
        return this.domElement ? this.domElement.scrollLeft : 0;
    }

    /**
     *
     */
    setHorizontalScrollPosition(val) {
        if (this.domElement) {
            return this.domElement.scrollLeft = val;
        } else {
            this.setAttribute("scrollLeft", val);
        }
        return val;
    }

    /**
     *
     */
    getAutomationName() {
        return this.domElement.id;
    }

    /**
     *
     */
    setAutomationName(val) {
        //return this.domElement.id = val;
        this.setAttribute("id", val);
    }

    /**
     * This is a getter/setter for the data property. When the cell is created, it belongs to a row
     * The data property points to the item in the grids dataprovider that is being rendered by this cell.
     */
    getData() {
        return this.data;
    }

    /**
     * This is important, because the grid looks for a "setData" method on the renderer.
     * @param val
     */
    setData(val) {
        this.data = val;
    }

    setErrorString(val, errorBorderColor) {
        this.errorString = val;
        if (this.errorString) {
            this._prevBorder = this.domElement ? this.domElement.style.border : this.getStyleAttribute('border');
            //this.domElement.style.border = "solid 1px red";
            this.setStyleAttribute("border", `solid 1px ${errorBorderColor ? errorBorderColor : "red"}`);
        } else if (!this.errorString) {
            //this.domElement.style.border = this._prevBorder;
            this.setStyleAttribute("border", this._prevBorder);
        }
    }

    getErrorString(val) {
        return this.errorString;
    }

    setHandCursor(val) {
        this.handCursor = val;
        if (val)
            this.attachClass("handCursor");
        else
            this.detachClass("handCursor");

    }
    attachClass(newClass) {

        const className = this.propertyBag.className;
        UIUtils.attachClass(this.propertyBag, newClass);
        if (className !== this.propertyBag.className) {
            this.setAttribute("className", this.propertyBag.className);
        }
    }
    detachClass(newClass) {
        const className = this.propertyBag.className;
        UIUtils.detachClass(this.propertyBag, newClass);
        if (className !== this.propertyBag.className) {
            this.setAttribute("className", this.propertyBag.className);
        }
    }

    getHandCursor() {
        return this.handCursor;
    }

    setToolTip(val) {
        this.toolTip = val;
        this.setAttribute("title", val);

    }

    getHandCursor() {
        return this.toolTip;
    }

    ownsPoint(pt) {
        pt = this.globalToLocal(pt);
        if (pt.x > (this.getWidth() || this.domElement.offsetWidth) || pt.y > (this.getHeight() || this.domElement.offsetHeight) || pt.x < 0 || pt.y < 0) return false;
        return true;
    }

    getIncludeInLayout(val) {
        return this.includeInLayout;
    }

    setIncludeInLayout(val) {
        this.includeInLayout = val;
        let myStyle = this.propertyBag.style;
        if (!val && (myStyle.display != "none")) {
            this.oldDisplay = myStyle.display;
            //myStyle.display = "none";
            this.setStyleAttribute("display", "none");
        } else if (val && (myStyle.display == "none")) {
            if (this.oldDisplay) {
                //myStyle.display = this.oldDisplay;
                this.setStyleAttribute("display", this.oldDisplay);
            } else if (myStyle.display == "none") {
                //myStyle.display = "";
                this.setStyleAttribute("display", "");
            }
        }
    }
    setState(newState) {
        if (UIComponent.statePaused) {
            pendingUpdates.push({ component: this, state: newState });
            return;
        }
        this.setStateInternal(newState);
    }
    setStateInternal(newState) {
        if (this.reactComponent) {
            this.reactComponent.setState(newState);
        } else if (this.domElement) {
            super.setState(newState);
        }

    }
    getElementByTagName(tag) {
        for (var i = 0; i < this.children.length; i++) {
            var child = this.children[i];
            if (child.tagName == tag) {
                return child;
            }
        }
        return null;
    }
}

flexiciousNmsp.UIComponent = UIComponent;
UIUtils.UIComponentFactory = new flexiciousNmsp.ClassFactory(flexiciousNmsp.UIComponent);
UIComponent.prototype.typeName = UIComponent.typeName = "UIComponent";
UIComponent.measurer = null;

DisplayList.singletonInstance = new DisplayList();