/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils'
import UIComponent from '../../core/UIComponent'
/**
 * A Class responsible for rendering the expand collapse icon inside a cell.
 * @class ExpandCollapseIcon
 * @constructor
 */
export default class ExpandCollapseIcon extends UIComponent {
    constructor() {
        super();

        /**
         * @property hasChildren
         * @type {Boolean}
         * @edefault = false
         */
        this.hasChildren = false;

        /**
         * @property open
         * @type {Boolean}
         * @default = false;
         */
        this.open = false;

        /**
         *
         * @type {null}
         * @default = null
         */
        this.onExpandCollapse = null;
    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["ExpandCollapseIcon", "UIComponent", "IExpandCollapseComponent"];
    }

    /**
     * refreshing the contents of a cell.
     */
    refreshCell() {

        if (this.getCell())
            this.getCell().refreshCell();

    }

    /**
     *When icon is enabled, handles the click event of the icon.
     */
    doClick() {

        ExpandCollapseIcon.doClick(this);

    }

    /**
     *
     * @param iexp
     */
    static refreshCell(iexp) {
        if (iexp.parent == null || !iexp.getRowInfo()) return;
        if (iexp.getRowInfo().rowPositionInfo.getIsDataRow()) {
            const item = iexp.getRowInfo().getData();
            if (!item) return;
            iexp.open = iexp.getLevel().isItemOpen(item);
            if (iexp.getLevel().grid.hasChildrenFunction != null) {
                iexp.hasChildren = iexp.getLevel().grid.hasChildrenFunction(item, this, iexp);
            }
            else {
                iexp.hasChildren = iexp.getLevel().nextLevelRenderer || (iexp.getLevel().getIsClientItemLoadMode() ? iexp.getLevel().nextLevel
                    && (iexp.getLevel().grid.getLength(iexp.getLevel().getChildren(item, true, false, false)) > 0) :
                    iexp.getLevel().childrenCountField ? (item[iexp.getLevel().childrenCountField] > 0) : true);
            }
            iexp.setToolTip(iexp.getLevel().grid.expandCollapseTooltipFunction(iexp.getCell()));
        }
    }

    /**
     *
     * @param expandCollapseComponent
     */
    static doClick(expandCollapseComponent) {

        if (expandCollapseComponent.parent == null) return;
        if (!expandCollapseComponent.getLevel().grid.allowInteractivity || !expandCollapseComponent.getRowInfo() || expandCollapseComponent.getRowInfo().getIsFillRow()) return;
        expandCollapseComponent.open = !expandCollapseComponent.open;
        expandCollapseComponent.getCell().dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridExpandCollapseCell.EVENT_EXPAND_COLLAPSE));
        const fdgE = expandCollapseComponent.implementsOrExtends("FlexDataGridExpandCollapseCell") ? expandCollapseComponent : null;
        if (fdgE)
            fdgE.invalidateBackground();
        else
            this.drawIcon(expandCollapseComponent);
        if (expandCollapseComponent.getLevel())
            expandCollapseComponent.setToolTip(expandCollapseComponent.getLevel().grid.expandCollapseTooltipFunction(expandCollapseComponent.getCell()));


    }

    /**
     *
     * @param iexp
     */
    static drawIcon(iexp) {

        if (iexp.parent == null || !iexp.getLevel()) return;
        const cell = iexp.getCell();
        if (cell.getRowInfo() && cell.getRowInfo().getIsFillRow()) {
            return
        }
        if (iexp.hasChildren) {
            const isExp = (iexp.typeName == "ExpandCollapseIcon")
            var img = iexp.tagName == "img" ? iexp : iexp.getElementByTagName("img");
            if (!img) {
                img = new flexiciousNmsp.UIComponent({}, "img");
                iexp.addChild(img)
            } else {
                //img = img.component
            }
            img.attachClass("disclosureIcon");
            if (iexp.open)
                img.setAttribute("src", flexiciousNmsp.Constants.IMAGE_PATH + cell.getStyleValue("disclosureOpenIcon"));
            else
                img.setAttribute("src", flexiciousNmsp.Constants.IMAGE_PATH + cell.getStyleValue("disclosureClosedIcon"));

            img.move(cell.getLevel().grid.paddingLeft, isExp ? 6 : ((cell.getRowInfo().height - 12) / 2));
            if( cell.getColumn() && cell.getColumn().expandCollapseIconPlacementFunction != null ) {
                cell.getColumn().expandCollapseIconPlacementFunction(img);
            }
            img.setStyleAttribute("visibility", "visible");

        } else {
            var img = iexp.tagName == "img" ? iexp : iexp.getElementByTagName("img");
            if (img)
                img.setStyleAttribute("visibility", "hidden");
        }
    }

    /**
     *
     * @return {Level}
     */
    getLevel() {
        return this.getCell() ? this.getCell().getLevel() : null;
    }

    /**
     *
     * @return {RowInfo}
     */
    getRowInfo() {
        return this.getCell() ? this.getCell().getRowInfo() : null;
    }

    /**
     *
     * @return {window}
     */
    getCell() {
        return this.parent;
    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.ExpandCollapseIcon = ExpandCollapseIcon; //add to name space
ExpandCollapseIcon.prototype.typeName = ExpandCollapseIcon.typeName = 'ExpandCollapseIcon';//for quick inspection