/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import FlexDataGridCell from './FlexDataGridCell'
/**
 * A container class for all column group cells. In addition to the functionality it inherits from FlexDataGridCell,
 * it adds support for the following feautres:
 * <ul>
 * <li>Ability to span across multiple columns to visually appear as a single column header</li>
 * </ul>
 * FlexDataGridColumnGroupCell inherits from FlexDataGridCell, which provides a bulk of the functionality to it.
 * @inheritDoc
 * @constructor
 * @class FlexDataGridColumnGroupCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridColumnGroupCell extends FlexDataGridCell {
    constructor() {
        super();
        /**
         * @property columnGroup
         * @type { FlexDataGridColumnGroup}
         * @default null
         */
        this.columnGroup = null;

        /**
         * @property _listenersAdded
         * @type {Boolean}
         * @private
         * @default false
         */
        this._listenersAdded = false;

        /**
         * @property background
         * @type {Boolean}
         * @default false
         */
        this.background = false;

        /**
         * @property  _wxInvalid
         * @type {Boolean}
         * @private
         * @default false
         */
        this._wxInvalid = false;

    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["FlexDataGridColumnGroupCell", "FlexDataGridCell", "UIComponent", "IFlexDataGridCell"];
    }

    /**
     * Destroy function.
      */
    destroy() {

        this.setText("");
        this.background = false;
        if (this.level) {
            this._listenersAdded = false;
            this.level.grid.removeEventListener(flexiciousNmsp.FlexDataGridEvent.COMPONENTS_CREATED, this.invalidateWX);
            this.level.removeEventListener(flexiciousNmsp.FlexDataGridEvent.COLUMNS_RESIZED, this.invalidateWX);
            if (this.getColumn()) {
                this.getColumn().removeEventListener(flexiciousNmsp.FlexDataGridEvent.COLUMN_X_CHANGED, this.invalidateWX);
            }
        }
        flexiciousNmsp.FlexDataGridCell.prototype.destroy.apply(this, []);
    }

    /**
     * handles columns Resize event.
     * @param event
     */
    onColumnsResized(event) {

        if (this.columnGroup) {
            const result = this.columnGroup.getWX();
            this.setWidth(result[0]);
            this.setX(result[1]);
            if (this.componentInfo)
                this.componentInfo.setX(this.getX());
            //this.setY(this.columnGroup.getY());
        }
        this.invalidateDisplayList();

    }

    /**
     *  handles the refreshing the contents of a cell.
     */
    refreshCell() {

        flexiciousNmsp.FlexDataGridCell.prototype.refreshCell.apply(this, []);
        this.onColumnsResized(null);
        if (!this._listenersAdded) {
            this._listenersAdded = true;
            this.level.grid.addEventListener(this, flexiciousNmsp.FlexDataGridEvent.COMPONENTS_CREATED, this.invalidateWX);
            this.level.addEventListener(this, flexiciousNmsp.FlexDataGridEvent.COLUMNS_RESIZED, this.invalidateWX);
            if (this.getColumn()) {
                this.getColumn().addEventListener(this, flexiciousNmsp.FlexDataGridEvent.COLUMN_X_CHANGED, this.invalidateWX);
            }
        }
        this.setText(this.columnGroup.getHeaderText(this));
        this.setToolTip(this.columnGroup.expandCollapseTooltipFunction(this));
        this.setSelectable(false);
    }

    /**
     * handle the invalidateWX.
     * @param event
     */
    invalidateWX(event) {

        this._wxInvalid = true;
        this.invalidateBackground();
        this.invalidateDisplayList();

    }

    /**
     *
     * @param unscaledWidth
     * @param unscaledHeight
     */
    updateDisplayList(unscaledWidth, unscaledHeight) {

        flexiciousNmsp.FlexDataGridCell.prototype.updateDisplayList.apply(this, [unscaledWidth, unscaledHeight]);
        if (this._wxInvalid) {
            this.onColumnsResized(null);
            this._wxInvalid = false;
        }

        if (this.level && this.columnGroup && this.columnGroup.enableExpandCollapse) {


            //var bitmapData=ExpandCollapseIcon.getBitmapData(cls);
            const point = this.columnGroup.expandCollapsePositionFunction(this);
            const paddingLeft = UIUtils.nanToZero(point.getX());
            const paddingTop = UIUtils.nanToZero(point.getY());



            //this.getRenderer().setX(UIUtils.nanToZero(this.getStyle("paddingLeft")));
            this.getRenderer().setWidth(this.getWidth - UIUtils.nanToZero(this.getStyle("paddingLeft")) - UIUtils.nanToZero(this.getStyle("paddingRight")));
        }

    }

    /**
     *
     * @return {DisplayObject}
     */
    getColumnGroupIcon() {
        if (!this._columnGroupIcon) {
            this._columnGroupIcon = new flexiciousNmsp.Image();
            this.addChild(this._columnGroupIcon);
        }
        let cls = this.level.grid.getStyle("columnGroupOpenIcon");
        if (this.columnGroup.isClosed())
            cls = this.level.grid.getStyle("columnGroupClosedIcon");
        this._columnGroupIcon.setSource(this.level.grid.imagesRoot + cls);

        return this._columnGroupIcon;
    }

    /**
     *
     * @return {String}
     */
    getRolloverColor() {

        return this.getStyleValue("columnGroupRollOverColors");

    }

    /**
     *
     * @return {String}
     */
    getBackgroundColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getBackgroundColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.currentBackgroundColors) return this.currentBackgroundColors;
        return this.getStyleValue("columnGroupColors");

    }

    /**
     *
     * @return {String}
     */
    getTextColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getTextColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.currentTextColors) return this.currentTextColors;
        return this.getStyle("color");

    }

    /**
     *
     * @return {*|Boolean|Boolean|Boolean|*}
     */
    getDrawTopBorder() {
        //if we're at a nested this.level, grid does not draw horizontal lines,
        //and there is no filter row, we draw a line above us to seperate ourselves from the data item above.
        return (this.level && this.level.getNestDepth() > 1 && !this.level.grid.getStyle("horizontalGridLines")
            && (this.level.getFilterRowHeight() == 0)) ||
            this.getStyleValue("columnGroupDrawTopBorder");
    }

    /**
     *
      * @return {String}
     */
    getPrefix() {
        return "columnGroup";
    }

    /**
     * If there is a columnGroup associated, returns its startColumn property. Else returns
     * the default column associated with this cell.
     */
    getColumn() {
        return this.columnGroup ? this.columnGroup.getStartColumn() : flexiciousNmsp.FlexDataGridCell.prototype.getColumn.apply(this, []);
    }
}

flexiciousNmsp.FlexDataGridColumnGroupCell = FlexDataGridColumnGroupCell; //add to name space
FlexDataGridColumnGroupCell.prototype.typeName = FlexDataGridColumnGroupCell.typeName = 'FlexDataGridColumnGroupCell';//for quick inspection