/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import ReactDataGridCellContainer from '../../core/ReactDataGridCellContainer'
import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils'
import FlexDataGridCell from './FlexDataGridCell'
/**
 * @inheritDoc
 * @constructor
 * @class FlexDataGridDataCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridDataCell extends FlexDataGridCell {
    constructor(props, tagName) {
        super(props, tagName)

        /**
         * The colSpan associated with this cell.
         * @property colSpan
         * @type int
         * @default 1
         */
        this.colSpan = 1;

        /**
         * The rowSpan associated with this cell.
         * @property rowSpan
         * @type int
         * @default 1
         */
        this.rowSpan = 1;

    }
    invalidateDisplayList() {
        if (ReactDataGridCellContainer.isScrolling) {
            return;
        }
        super.invalidateDisplayList();
    }

    /**
     *
     */
    getClassNames() {
        return ["FlexDataGridDataCell", "FlexDataGridCell", "UIComponent", "IFlexDataGridDataCell", "IFlexDataGridCell"];
    }

    /**
     * This method is responsible for preparing the cell for display. It gets triggered both the first
     * time the cell is intantiated, and again when the cell is pulled out from cache, and also when the
     * cell is assigned to another row data object during vertical scrolling, or another column during
     * horizontal scrolling.
     *
     * For all cells, It performns the following steps:
     * <ul>
     * <li>It sets the doubleClickEnabled flag on the cell on basis of grid.enableDoubleClickEdit||grid.doubleClickEnabled</li>
     * <li>It sets the automationName to friendly name</li>
     * <li>It sets the destroyed flag to false</li>
     * <li>It sets the currentBackgroundColors and currentTextColors to null</li>
     * <li>It sets the styleName property on the renderer to the level</li>
     * <li>It dispatches the RENDERER_INITIALIZED event, if one is wired up</li>
     * <li>For checkbox based cells, it calls the initializeCheckBoxRenderer method</li>
     * <li>If enableExpandCollapse is set to true, it initializes the expand collapse icon component</li>
     * <li>It calls the placeComponent method</li>
     * <li>If enableIcon method is set to true, it initializes the icon component</li>
     * </ul>
     * For data cells, It performns the following additional steps:
     * <ul>
     * <li>It sets the textDecoration, useHandCursor, selectable, truncateToFit properties</li>
     * <li>For checkbox based cells, it sets the enabled flag based upon the checkRowSelectable method of the level</li>
     * </ul>
     * For header cells, It performns the following additional steps:
     * <ul>
     * <li>It creates the sort icon, if one is necessary, and hides the renderer, if the hideHeaderText flag is set to true</li>
     * </ul>
     * For footer cells, It performns the following additional steps:
     * <ul>
     * <li>Calculates, formats and displays the footer value based upton the footerOperation or footerLabelFunction and other footer related properties</li>
     * </ul>
     */
    refreshCell() {

        if (this.rowInfo.getIsFillRow()) return;
        if (ReactDataGridCellContainer.isScrolling) {
            const column = this.getColumn();
            const rowInfo = this.rowInfo;
            if (column) {
                this.setText(column.itemToLabel(rowInfo.getData(), this));
            }
            return;
        }
        super.refreshCell();
        flexiciousNmsp.CellUtils.refreshCell(this);
        const mine = this.getRenderer();

        if (mine.implementsOrExtends && mine.implementsOrExtends("UIComponent") && this.getColumn() && !this.getColumn().hasCustomItemRenderer()) {

            const rendererComponent = mine;
            this.setSelectable(this.getColumn().selectable);
            if (this.getColumn().useUnderLine && rendererComponent.getStyleAttribute("textDecoration") != "underline")
                rendererComponent.setStyleAttribute("textDecoration", "underline");
            else if (!this.getColumn().useUnderLine && rendererComponent.getStyleAttribute("textDecoration") == "underline")
                rendererComponent.setStyleAttribute("textDecoration", "");
            if (this.getColumn().useHandCursor) {
                rendererComponent.setStyleAttribute("cursor", "pointer");
            } else
                rendererComponent.setStyleAttribute("cursor", "");

            if (this.getColumn().truncateToFit) {
                rendererComponent.setStyleAttribute("textOverflow", "ellipsis");
                rendererComponent.setAttribute("title", rendererComponent.innerHTML);
            }
            else {
                rendererComponent.setStyleAttribute("textOverflow", "clip");
                rendererComponent.setAttribute("title", "");
            }
            const ta = this.getStyleValue("textAlign") || "";
            rendererComponent.setStyleAttribute("textAlign", ta);

        }
        if (this.getColumn() && this.getColumn().implementsOrExtends("FlexDataGridCheckBoxColumn")) {
            this.setEnabled(this.level.checkRowSelectable(this, this.rowInfo.getData()));
        }
    }

    /**
     * In case of column.enableHierarchicalNestIndent, returns maxPaddingCellWidth else returns zero
     */
    getLeftPadding() {
        return this.getColumn() && this.getColumn().enableHierarchicalNestIndent ? this.level.getMaxPaddingCellWidth() + this.getColumn().getStyle("paddingLeft") : flexiciousNmsp.FlexDataGridCell.prototype.getLeftPadding.apply(this);
    }

    highlight(forceHighlight) {
        const hpdg = this.getLevel().grid;
        this.invalidateDisplayList();

        for (const textField of this.getTextFields()) {
            //if we're inside a HP datagrid, which has a globalHightlightString specified, and our column
            //is in globalHighlightColumns or globalHighlightColumns is empty, and our text contains the
            //globalHightlightString (case insensitive), then we highlight.
            const txt = textField;//textField.owner instanceof Label?(textField.owner).text:textField.text;


            if (hpdg && hpdg.getGlobalHightlightString() && (hpdg.getGlobalHightlightAll() || forceHighlight)
                && (hpdg.getGlobalHighlightColumns().length == 0)
                && txt.innerText.toLowerCase().includes(hpdg.getGlobalHightlightString().toLowerCase())) {

                /*var beginIndex=txt.toLocaleLowerCase().indexOf(hpdg.globalHightlightString.toLocaleLowerCase());
                 var endIndex=beginIndex+hpdg.globalHightlightString.length;
                 textField.alwaysShowSelection=true;

                 textField.setSelection(beginIndex,endIndex);*/

                const pattern = new RegExp(UIUtils.escapeRegExp(hpdg.getGlobalHightlightString()), 'gi');

                txt.innerHTML = txt.innerHTML.replace(pattern, `<span style='background-color:yellow;'>${hpdg.getGlobalHightlightString()}</span>`);
            } else {
                //textField.setSelection(0,0);

                txt.innerHTML.replace(`<span style='background-color:yellow;'>${hpdg.getGlobalHightlightString()}</span>`, hpdg.getGlobalHightlightString());
            }
        }
    }

    removeHighlight() {
        for (const textField of this.getTextFields()) {
            textField.innerHTML = textField.innerHTML.replace('<span style="background-color:yellow;">', "");
            textField.innerHTML = textField.innerHTML.replace("</span>", "");
        }
    }

    getTextFields() {

        /*if(this.renderer.domElement){
         return [this.getRenderer()];
         }*/
        return [this.getRenderer().domElement];

    }
}

flexiciousNmsp.FlexDataGridDataCell = FlexDataGridDataCell; //add to name space
FlexDataGridDataCell.prototype.typeName = FlexDataGridDataCell.typeName = 'FlexDataGridDataCell';//for quick inspection