/*eslint-disable */
import ExpandCollapseIcon from './ExpandCollapseIcon'
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils'
import FlexDataGridCell from './FlexDataGridCell'

/**
 * @inheritDoc
 * @constructor
 * @class FlexDataGridExpandCollapseCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridExpandCollapseCell extends FlexDataGridCell {
    constructor() {
        super();
        /**
         *
         * @type {Boolean}
         * @property  hasChildren
         * @default false
         */
        this.hasChildren = false;

        /**
         * Boolean
         * @type {Boolean}
         * @property  open
         * @default false
         */
        this.open = false;

        /**
         *When we need right locked padding, set this
         * @type {Boolean}
         * @property  forceRightLock
         * @default false
         */
        this.forceRightLock = false;

        /**
         *The colSpan associated with this cell.
         * @type {int}
         * @property colSpan
         * @default 1
         */
        this.colSpan = 1;

        /**
         *The rowSpan associated with this cell.
         * @type {int}
         * @property rowSpan
         * @default 1
         */
        this.rowSpan = 1;

    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["FlexDataGridExpandCollapseCell", "FlexDataGridCell", "UIComponent",
            "IFlexDataGridDataCell", "IExpandCollapseComponent", "IFlexDataGridCell"];
    }

    /**
     * handles the Destroy of this cell.
     */
    destroy() {
        flexiciousNmsp.FlexDataGridCell.prototype.destroy.apply(this, []);
        this.open = false;
        this.forceRightLock = false;
    }

    /**
     * Refreshing the contenets of this cell.
     */
    refreshCell() {

        flexiciousNmsp.FlexDataGridCell.prototype.refreshCell.apply(this, []);
        this.currentBackgroundColors = null;
        this.invalidateBackground();
        flexiciousNmsp.ExpandCollapseIcon.refreshCell(this);
        flexiciousNmsp.ExpandCollapseIcon.drawIcon(this);

    }

    /**
     * handle doClick
     */
    doClick() {

        this.onClick(null);

    }

    /**
     * handles the onClick event.
     * @param event
     */
    onClick(event) {

        flexiciousNmsp.ExpandCollapseIcon.doClick(this);

    }

    /**
     *
     * @return {String}
     */
    getBackgroundColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getBackgroundColorFromGrid(this);
        if (fromGrid != null) return fromGrid;

        if (this.rowInfo && this.rowInfo.getIsDataRow()) return flexiciousNmsp.FlexDataGridCell.prototype.getBackgroundColors.apply(this, []);
        if (this.rowInfo && this.rowInfo.getIsFillRow()) return this.getStyleValue("backgroundColor");
        if (this.rowInfo) {
            if (this.rowInfo.getIsHeaderRow()) {
                return this.getStyleValue("headerColors");
            }
            else if (this.rowInfo.getIsPagerRow()) {
                return this.getStyleValue("pagerColors");
            }
            else if (this.rowInfo.getIsFooterRow()) {
                return this.getStyleValue("footerColors");
            }
            else if (this.rowInfo.getIsFilterRow()) {
                return this.getStyleValue("filterColors");
            }
            else {
                const odd = ((this.rowInfo && this.rowInfo.rowPositionInfo.getRowIndex() % 2) == 0);
                return this.getStyleValue("alternatingItemColors")[odd ? 0 : 1];
            }
        }
        else {
            return this.getStyleValue("backgroundColor");

        }

    }

    /**
     *
     * @return {String}
     */
    getTextColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getTextColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.currentTextColors) return this.currentTextColors;
        return this.getStyle("color");

    }

    /**
     *
     * @param w
     * @param h
     */
    setActualSize(w, h) {

        flexiciousNmsp.FlexDataGridCell.prototype.setActualSize.apply(this, [w, h]);

    }

    /**
     *
     * @return {IFlexDataGridCell}
     */
    getCell() {
        return this;
    }

    /**
     *
     * @return {Boolean}
     */
    getDrawTopBorder() {
        return this.getStyleValue(`${this.prefix}DrawTopBorder`)
    }

    /**
     *
     * @return {Boolean}
     */
    getIsExpandCollapseCell() {
        return true;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLocked() {

        return this.getColumn() ? this.getColumn().getIsLocked() : this.forceRightLock || this.level.grid.lockDisclosureCell;
    }

    /**
     *
     * @return {String}
     */
    getPrefix() {
        return this.rowInfo ? this.rowInfo.getIsHeaderRow() ? ("header")
            : this.rowInfo.getIsPagerRow() ? ("pager")
                : this.rowInfo.getIsFooterRow() ? ("footer")
                    : this.rowInfo.getIsFilterRow() ? ("filter")
                        : this.rowInfo.getIsRendererRow() ? ("renderer")
                            : "" : "";
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLeftLocked() {
        //return column?column.isLeftLocked:!forceRightLock&&level.grid.lockDisclosureCell;
        return this.getColumn() ? this.getColumn().getIsLeftLocked() : !this.forceRightLock && this.level.grid.lockDisclosureCell;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsRightLocked() {
        //return column?column.isRightLocked:forceRightLock||super.isRightLocked;
        return this.getColumn() ? this.getColumn().getIsRightLocked() : this.forceRightLock || flexiciousNmsp.FlexDataGridCell.prototype.getIsRightLocked.apply(this, []);
    }

    /**
     *
     * @return {IExpandCollapseComponent}
     */
    getIExpandCollapseComponent() {
        return this;
    }
}

flexiciousNmsp.FlexDataGridExpandCollapseCell = FlexDataGridExpandCollapseCell; //add to name space
FlexDataGridExpandCollapseCell.prototype.typeName = FlexDataGridExpandCollapseCell.typeName = 'FlexDataGridExpandCollapseCell';//for quick inspection

/**
 *
 * @type {String}
 */
FlexDataGridExpandCollapseCell.EVENT_EXPAND_COLLAPSE = "expandCollapse";