/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import FlexDataGridCell from './FlexDataGridCell'
/**
 * @inheritDoc
 * @constructor
 * @class FlexDataGridFilterCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridFilterCell extends FlexDataGridCell {
    /**
     *
      * @return {Array}
     */
    getClassNames() {
        return ["FlexDataGridFilterCell","FlexDataGridCell","UIComponent","IFlexDataGridCell"];
    }

    /**
     *
     */
    getBackgroundColors() {

        const fromGrid= flexiciousNmsp.CellUtils.getBackgroundColorFromGrid(this);
        if(fromGrid!=null)return fromGrid;
        if(this.currentBackgroundColors)return this.currentBackgroundColors;
        return this.level.getStyleValue("filterColors");

    }

    /**
     *
     */
    getIconUrl(over) {
        if(typeof over=="undefined")over=false;

        return null;

    }

    /**
     *
     */
    getTextColors() {

        const fromGrid= flexiciousNmsp.CellUtils.getTextColorFromGrid(this);
        if(fromGrid!=null)return fromGrid;
        if(this.currentTextColors)return this.currentTextColors;
        return this.getStyle("color");

    }

    /**
     *
     */
    initializeCheckBoxRenderer(renderer) {


    }

    /**
     *
     */
    getRolloverColor() {

        if(this.getColumn()) return this.getColumn().getStyleValue("filterRollOverColors");
        return this.level.getStyleValue("filterRollOverColors");

    }

    /**
     * handles destroy of a cell
     * @method destroy
     */
    destroy() {

        const iFilter=this.getRenderer().implementsOrExtends && this.getRenderer().implementsOrExtends("IFilterControl")?this.getRenderer():null;
        if(iFilter)
            iFilter.clear();
        //const iSelectFilter = this.getRenderer().implementsOrExtends("ISelectFilterControl")?this.getRenderer():null;
        flexiciousNmsp.FlexDataGridCell.prototype.destroy.apply(this,[]);


    }

    /**
     * Refreshing the contens of this cell.
     * @method refreshCell
     */
    refreshCell() {

        flexiciousNmsp.FlexDataGridCell.prototype.refreshCell.apply(this,[]);
        if(this.getRenderer().hasOwnProperty("enabled"))
            this.getRenderer()["enabled"]=this.level.grid.allowInteractivity;

        if(this.level.getNestDepth()>1){
            const filter=this.level.grid.itemFilters.getValue(this.getRowInfo().getData());
            if(!filter){
                const filterControl = this.getRenderer().implementsOrExtends("IFilterControl")?this.getRenderer():null;
                if(filterControl)
                filterControl.clear();
            }else{
                //we need to refresh ourselves.
                this.level.grid.getFilterContainer().initializeFilterRenderer(this.getRenderer(),this.getColumn(),this.getRowInfo().getData(),[],this.getLevel());
            }
        }
    }

    /**
     * Returns one of the following values on basis of what type of cell this is:
     * <ul>
     * <li>Blank string for data cells</li>
     * <li>header</li>
     * <li>footer</li>
     * <li>filter</li>
     * <li>columnGroup</li>
     * <li>pager</li>
     * </ul>
     * @return {String}
     * @method getPrefix
     */
    getPrefix() {
        return "filter";
    }

    /**
     * Calculated property that controls whether or not to draw the top border. Each cell in Ultimate will draw its own background
     * and border. In order to prevent cells from drawing double borders (cell above me drew bottom border, so I should not need
     * to draw my top border), this flag evaluates to false. But in certain cases, you may want to draw both top and bottom border.
     * The bottom border is always controlled by the horizontalGridLines style property. The top border is controlled by the drawTopBorder
     * style property. In order to provide fine grained control over border drawing functionality, there are the following style
     * properties available.
     * <ul>
     * <li>drawTopBorder</li>
     * <li>headerDrawTopBorder</li>
     * <li>footerDrawTopBorder</li>
     * <li>pagerDrawTopBorder</li>
     * <li>filterDrawTopBorder</li>
     * <li>columnGroupDrawTopBorder</li>
     * </ul>
     * To control the bottom border, the following properties are available:
     * <ul>
     * <li>horizontalGridLines</li>
     * <li>headerHorizontalGridLines</li>
     * <li>footerHorizontalGridLines</li>
     * <li>pagerHorizontalGridLines</li>
     * <li>filterHorizontalGridLines</li>
     * <li>columnGroupHorizontalGridLines</li>
     * </ul>
     * @return {*|Boolean|Boolean|*}
     * @method getDrawTopBorder
     */
    getDrawTopBorder() {
        //if we're at a nested level, grid does not draw horizontal lines,
        //and there is no filter row, we draw a line above us to seperate ourselves from the data item above.
        return (this.level && this.level.getNestDepth()>1 && !this.level.grid.getStyle("horizontalGridLines"))
            || this.getStyleValue("filterDrawTopBorder")
    }


    
}

flexiciousNmsp.FlexDataGridFilterCell = FlexDataGridFilterCell; //add to name space
FlexDataGridFilterCell.prototype.typeName = FlexDataGridFilterCell.typeName = 'FlexDataGridFilterCell';//for quick inspection
/**
 * @inheritDoc
 * Event Type:com.flexicious.nestedtreedatagrid.events.ExtendedFilterPageSortChangeEvent
 */

FlexDataGridFilterCell.FILTER_CHANGE="filterChange";

/**
 *
 * @type {String}
 * Event Type:com.flexicious.nestedtreedatagrid.events.ExtendedFilterPageSortChangeEvent
 */
FlexDataGridFilterCell.EVENT_FILTERCHANGE='filterChange';