/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils'
import FlexDataGridCell from './FlexDataGridCell'

/**
 * @inheritDoc
 * @constructor
 * @class FlexDataGridFooterCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridFooterCell extends FlexDataGridCell {
    constructor() {
        super();
        this.dataProvider = [];

    }

    getClassNames() {
        return ["FlexDataGridFooterCell", "FlexDataGridCell", "UIComponent", "IFlexDataGridCell"];
    }

    /**
     * Refreshing the contents of this cell.
     */
    refreshCell() {

        flexiciousNmsp.FlexDataGridCell.prototype.refreshCell.apply(this, []);
        if (this.getColumn() && (this.getColumn().footerLabelFunction != null || this.getColumn().footerLabelFunction2 != null || this.getColumn().footerOperation != null || this.getColumn().footerLabel != null)) {

            if (this.level.getNestDepth() == 1)
                this.calculateValue(UIUtils.filterArray(this.level.grid.getRootFlat(), this.level.grid.getRootFilter(), this.level.grid, this.level, false));
            else
                this.calculateValue(this.level.getParentLevel().getChildren(this.rowInfo.getData(), true, false, true))
        }
        else {
            this.setText("");
        }

    }

    /**
     * a.	The default text of the footer is calculated on basis of the following logic: <br/>
     * i.	First we check to see if a footerLabelFunction2 is specified on the column. If so, we use it.<br/>
     * ii.	Second, we check to see if there is a footerLabelFunction specified on the column. If so, we use it.<br/>
     * iii.	Finally, if neither of these is specified, we use a default footer label function, which performs the following logic:<br/>
     * 1.	Checks to see the value of the columns footerOperation. The valid values for this property are sum, min, max, count and average. On basis of this , it computes the value.<br/>
     * 2.	It calls toFixed method passing in the footerOperationPrecision value to give it the appropriate number of decimal places<br/>
     * 3.	If there is a footerFormater specified, it calls the format method of the footerFormatter passing in the value and displays the result.<br/>
     * 4.	If there is a footerLabel specified, it will concatenate the value of the footer label to the result of the calculated value in step 3.<br/>
     * @method calculateValue
     * @return {String}
     */
    calculateValue(flat) {


        this.dataProvider = flat;
        this.setText(this.getColumn().calculateFooterValue(flat, this));

        // if (this.getColumn().footerLabelFunction2 != null)
        //     this.setText(this.getColumn().footerLabelFunction2(this));
        // else if (this.getColumn().footerLabelFunction != null)
        //     this.setText(this.getColumn().footerLabelFunction(this.getColumn()));
        // else
        //     this.setText(this.defaultLabelFunction(this.getColumn()));

    }

    /**
     * 1.	Checks to see the value of the columns footerOperation. The valid values for this property are sum, min, max, count and average. On basis of this , it computes the value.<br/>
     * 2.	It calls toFixed method passing in the footerOperationPrecision value to give it the appropriate number of decimal places<br/>
     * 3.	If there is a footerFormater specified, it calls the format method of the footerFormatter passing in the value and displays the result.<br/>
     * 4.	If there is a footerLabel specified, it will concatenate the value of the footer label to the result of the calculated value in step 3.<br/>
     * @param col
     * @method defaultLabelFunction
     * @return {String}
     */
    defaultLabelFunction(col) {

        let value = 0;

        switch (col.footerOperation) {
            case FlexDataGridFooterCell.SUM:
                value = UIUtils.sum(this.dataProvider, col.getDataField());
                break;
            case FlexDataGridFooterCell.AVERAGE:
                value = UIUtils.average(this.dataProvider, col.getDataField());
                break;
            case FlexDataGridFooterCell.MIN:
                value = UIUtils.min(this.dataProvider, col.getDataField());
                break;
            case FlexDataGridFooterCell.MAX:
                value = UIUtils.max(this.dataProvider, col.getDataField());
                break;
            case FlexDataGridFooterCell.COUNT:
                value = UIUtils.getLength(this.dataProvider);
                break;
        }

        let val = (value) ? (typeof value == 'number') ?
            value.toFixed(col.footerOperationPrecision) : value.toString() : '';

        if (col.footerFormatter != null) {
            if (!col.footerFormatter["format"]) {
                col.footerFormatter = new col.footerFormatter();
            }
            col.footerFormatter.precision = col.footerOperationPrecision;
            val = col.footerFormatter.format(val);
        } else {
            val = col.doFormat(val);
        }
        return (col.footerLabel != null ? col.footerLabel : "") + val;

    }

    /**
     *
     * @return {String}
     */
    getBackgroundColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getBackgroundColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.currentBackgroundColors) return this.currentBackgroundColors;
        return this.level.getStyleValue("footerColors");

    }

    /**
     *
     * @return {String}
     */
    getTextColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getTextColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.currentTextColors) return this.currentTextColors;
        return this.getStyle("color");

    }

    /**
     *
     * @param renderer
     * @type DisplayObject
     */
    initializeCheckBoxRenderer(renderer) {


    }

    /**
     *
     * @return {String}
     */
    getRolloverColor() {

        if (this.getColumn()) return this.getColumn().getStyleValue("footerRollOverColors");
        return this.level.getStyleValue("footerRollOverColors");

    }

    /**
     *
     * @return {String}
     */
    getPrefix() {
        return "footer";
    }

    /**
     *
     * @return {Boolean}
     */
    getDrawTopBorder() {
        return this.getStyleValue("footerDrawTopBorder")
    }

}

flexiciousNmsp.FlexDataGridFooterCell = FlexDataGridFooterCell; //add to name space
FlexDataGridFooterCell.prototype.typeName = FlexDataGridFooterCell.typeName = 'FlexDataGridFooterCell';//for quick inspection

FlexDataGridFooterCell.SUM = 'sum';
FlexDataGridFooterCell.MIN = 'min';
FlexDataGridFooterCell.MAX = 'max';
FlexDataGridFooterCell.AVERAGE = 'average';
FlexDataGridFooterCell.COUNT = 'count';