/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import Label from '../../controls/Label';
import Image from '../../controls/Image';
import FlexDataGridHeaderCell from './FlexDataGridHeaderCell';

/**
 * A container class for all header cells. In addition to the functionality it inherits from FlexDataGridCell,
 * it adds support for the following feautres:
 * <ul>
 * <li>Ability to trigger a column resize</li>
 * <li>Ability to render sort icons and multiColumnSort labels</li>
 * <li>Ability to render and initialize the checkbox header</li>
 * </ul>
 *
 * FlexDataGridHeaderCell inherits from FlexDataGridCell, which provides a bulk of the functionality to it.
 * @inheritDoc
 * @constructor
 * @class FlexDataGridHeaderCellHS
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridHeaderCellHS extends FlexDataGridHeaderCell {
    constructor() {
        super();
        this.sortLabel = new Label();
        this.icon = new Image();
        this.sortLabel.setVisible(false);
        this.icon.setVisible(false);
        this.addChild(this.sortLabel);
        this.addChild(this.icon);

    }

    
    /**
     * Creates the sort icon. Depending whether the enableMultiColumnSort
     * flag is set to true, this method will create a sort label as well, which
     * is responsible for initializing the icon.
     */
    createSortIcon(container) {

        let ic;
        if (this.icon.getVisible()) {
            this.destroySortIcon();
        }
        if (this.getColumn() && !this.getColumn().sortable) return;
        const grid = this.level.grid;
        if (this.level.getEnableMultiColumnSort()) {
            var sl = this.sortLabel;
            sl.setAttribute("className", "sortLabel");
            sl.useHandCursor = !grid.getEnableSplitHeader();
            sl.buttonMode = !grid.getEnableSplitHeader();
            sl.mouseChildren = grid.getEnableSplitHeader();
            sl.setToolTip(grid.multiColumnSortGetTooltip(this));
            sl.addEventListener(this, Constants.EVENT_CLICK, this.onSortLabelClick, false);
            sl.setText(this.level.getSortIndex(this.getColumn()).toString());
        }

        const p = this;
        if (this.sortAscending) {
            this.icon = this.level.createAscendingSortIcon(this.icon);
        }
        else {
            this.icon = this.level.createDescendingSortIcon(this.icon);
        }

        this.icon.setVisible(true);
        if (this.level.getEnableMultiColumnSort())
            this.sortLabel.setVisible(true);
        
        if (this.domElement) {
            this.placeSortIcon();
        } else {
            this.placementPending = true;
        }

        if(this.icon) {
            this.icon.addEventListener(this, Constants.EVENT_CLICK,this.onSortLabelClick,false);
        }
    }
    refreshCell(){
        this.destroySortIcon();
        super.refreshCell();
    }

    /**
     * Removes the sort icon and the sort label
     */
    destroySortIcon() {
        this.icon.setVisible(false);
        this.sortLabel.setVisible(this.sortLabel);
        this.sortLabel.removeEventListener(Constants.EVENT_CLICK, this.onSortLabelClick, false);
        if (this.domElement && this.level) {
            this.placeComponent(this.getRenderer(), this.getWidth(), this.getHeight(), undefined);
        }
    }
    removeSortIcon(){
        this.icon.setVisible(false);
    }

    setRendererSize(cellRenderer, w, h) {


        if (this.level && this.level.grid.getEnableSplitHeader() && this.getColumn() && this.getColumn().sortable) {
            w = w - this.level.grid.headerSortSeparatorRight;
        } else if (this.icon.getVisible()) {
            w = w - 14;
        }
        flexiciousNmsp.FlexDataGridCell.prototype.setRendererSize.apply(this, [cellRenderer, w, h]);

    }
    
    /**
     * Places the sort icon after it is created. By default the column is placed 3 pixels from the right, half way from the top.
     * If the enableMultiColumnSort flag is set to true, the label is places above or below the icon based on if it is ascending or
     * descending respectively.
     */
    placeSortIcon() {

        if (this.icon) {
            let pt = new flexiciousNmsp.Point(this.getWidth() - 18, (this.getHeight() / 2) - 10); 
            this.icon.move(pt.getX(), pt.getY());


            if (this.sortLabel && this.level.getEnableMultiColumnSort()) {
                const right = this.level.grid.headerSortSeparatorRight;
                this.sortLabel.setWidth(this.level.grid.getStyle("multiColumnSortNumberWidth"));
                this.sortLabel.setHeight(this.level.grid.getStyle("multiColumnSortNumberHeight"));
                //pt=new flexiciousNmsp.Point(this.getWidth() - right+2, Math.max(0,((this.getHeight()-this.sortLabel.getHeight)/2)-1));
                //pt= this.icon.parent.globalToLocal(this.localToGlobal(pt));
                this.sortLabel.move(pt.getX() + 5, pt.getY() + 16);
                if (this.level && this.level.getSortIndex(this.getColumn()) > 9) {
                    this.sortLabel.move(pt.getX() - 5, pt.getY() + 4);
                }

                if(this.level && this.level && this.level.grid && this.level.grid.sortIconPlacementFunction){
                    this.level.grid.sortIconPlacementFunction(this);
                }
            }
        }
    } 
    
    /**
     * Returns icon!=null
     */
    getIsSorted() {
        return this.icon.getVisible();
    }
}