/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import FlexDataGridCell from './FlexDataGridCell'
/**
 * @inheritDoc
 * @constructor
 * @class FlexDataGridPaddingCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridPaddingCell extends FlexDataGridCell {
    constructor() {
        super();
        this.forceRightLock = false;

        this.scrollBarPad = null;

    }

    getClassNames() {
        return ["FlexDataGridPaddingCell", "FlexDataGridCell", "UIComponent", "IFixedWidth", "IFlexDataGridCell"];
    }

    /**
     * handles destroy of this cell.
     */
    destroy() {
        flexiciousNmsp.FlexDataGridCell.prototype.destroy.apply(this, []);
        this.scrollBarPad = false;
    }

    /**
     *
     */
    setActualSize(w, h) {
        flexiciousNmsp.FlexDataGridCell.prototype.setActualSize.apply(this, [w, h]);
    }

    /**
     *
     * @return {String}
     */
    getBackgroundColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getBackgroundColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.rowInfo && this.rowInfo.getIsDataRow()) return flexiciousNmsp.FlexDataGridCell.prototype.getBackgroundColors.apply(this, []);
        if (this.rowInfo && this.rowInfo.getIsFillRow()) return this.getStyleValue("backgroundColor");
        if (this.rowInfo) {
            if (this.rowInfo.getIsHeaderRow()) {
                return this.getStyleValue("headerColors");
            }
            else if (this.rowInfo.getIsPagerRow()) {
                return this.getStyleValue("pagerColors");
            }
            else if (this.rowInfo.getIsFooterRow()) {
                return this.getStyleValue("footerColors");
            }
            else if (this.rowInfo.getIsFilterRow()) {
                return this.getStyleValue("filterColors");
            }
            else {
                const odd = ((this.rowInfo && this.rowInfo.rowPositionInfo.getRowIndex() % 2) == 0);
                return this.getStyleValue("alternatingItemColors")[odd ? 0 : 1];
            }
        }
        else {
            return this.getStyleValue("backgroundColor");
        }

    }

    /**
     *
     * @return {String}
     */
    getTextColors() {
        return 0x000000;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLocked() {
        return this.scrollBarPad ? false : this.forceRightLock || this.level.grid.lockDisclosureCell;
    }

    /**
     *
     * @return {String}
     */
    getPrefix() {
        return this.rowInfo ? this.rowInfo.getIsHeaderRow() ? ("header")
            : this.rowInfo.getIsPagerRow() ? ("pager")
                : this.rowInfo.getIsFooterRow() ? ("footer")
                    : this.rowInfo.getIsFilterRow() ? ("filter")
                        : this.rowInfo.getIsRendererRow() ? ("renderer")
                            : "" : "";
    }

    /**
     * Refreshing the contenets of this cell.
     */
    refreshCell() {

        flexiciousNmsp.FlexDataGridCell.prototype.refreshCell.apply(this, []);
        this.currentBackgroundColors = null;
        this.invalidateBackground();

    }

    /**
     * @return {*|Boolean|Boolean|Boolean|*|Boolean|Boolean}
     */
    getDrawTopBorder() {
        //if we're at a nested level, grid does not draw horizontal lines,
        //and there is no filter row, we draw a line above us to seperate ourselves from the data item above.

        return this.getStyleValue(`${this.prefix}DrawTopBorder`) ||
            ((this.rowInfo.getIsHeaderRow() && this.level.getNestDepth() > 1 && !this.level.grid.getStyle("horizontalGridLines")
                && (this.level.getFilterRowHeight() == 0))
                || (this.rowInfo.getIsFilterRow() && this.level.getNestDepth() > 1 && !this.level.grid.getStyle("horizontalGridLines"))
            )
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLeftLocked() {
        return this.scrollBarPad ? false : !this.forceRightLock && this.level.grid.lockDisclosureCell;
    }

    /**
     *
     */
    getIsRightLocked() {
        flexiciousNmsp.FlexDataGridCell.prototype.getIsRightLocked.apply(this, []);
    }
}

flexiciousNmsp.FlexDataGridPaddingCell = FlexDataGridPaddingCell; //add to name space
FlexDataGridPaddingCell.prototype.typeName = FlexDataGridPaddingCell.typeName = 'FlexDataGridPaddingCell';//for quick inspection


