/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import FlexDataGridCell from './FlexDataGridCell'
/**
 * @inheritDoc
 * @constructor
 * @class FlexDataGridPagerCell
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridPagerCell extends FlexDataGridCell {

    getClassNames() {
        return ["FlexDataGridPagerCell", "FlexDataGridCell", "UIComponent", "IFixedWidth", "IFlexDataGridCell"];
    }

    /**
     *
     * @return {String}
     */
    getBackgroundColors() {

        //if(currentBackgroundColors)return currentBackgroundColors;
        const fromGrid = flexiciousNmsp.CellUtils.getBackgroundColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        return this.getStyleValue("pagerColors");

    }

    /**
     *
     * @return {String}
     */
    getTextColors() {

        const fromGrid = flexiciousNmsp.CellUtils.getTextColorFromGrid(this);
        if (fromGrid != null) return fromGrid;
        if (this.currentTextColors) return this.currentTextColors;
        return this.getStyle("color");

    }

    /**
     *
     * @return {String}
     */
    getRolloverColor() {

        return this.getStyleValue("pagerRollOverColors");

    }

    /**
     *
     * @param w
     * @param h
     */
    setActualSize(w, h) {

        flexiciousNmsp.FlexDataGridCell.prototype.setActualSize.apply(this, [w, h]);
        //setRendererSize(renderer,w,h);
        this.invalidateDisplayList();

    }

    /**
     *
     * @param cellRenderer
     * @param w
     * @param h
     */
    setRendererSize(cellRenderer, w, h) {

        cellRenderer.setActualSize(w, h);

    }

    /**
     * handles destroy of this cell.
     */
    destroy() {

        flexiciousNmsp.FlexDataGridCell.prototype.destroy.apply(this, []);

    }

    /**
     * Refreshing the contents of this cell.
     */
    refreshCell() {
        flexiciousNmsp.FlexDataGridCell.prototype.refreshCell.apply(this);

        const iPager = this.getRenderer();
        iPager.doDispatchEvents = false;
        const grid = this.level.grid;
        iPager.setPageSize(this.level.pageSize);
        const item = this.rowInfo.getData();
        const filter = grid.getItemFilter(this.level, item);
        iPager.setPageIndex(-1);//to ensure we redraw
        iPager.setTotalRecords(-1);//to ensure we redraw
        if (this.level.enablePaging) {
            if (this.level.getNestDepth() == 1) {
                iPager.setTotalRecords(grid.getTotalRecords());
                iPager.setPageIndex(filter && filter.pageIndex >= 0 ? filter.pageIndex : 0);

            }
            else if (item && !this.level.getIsClientFilterPageSortMode()) {
                const container = grid.getContainerForCell(this);
                const loadInfo = container.findLoadingInfo(item, this.level.getParentLevel(), this.level.childrenField.length > 0);
                if (loadInfo) {
                    iPager.setTotalRecords(loadInfo.totalRecords);
                    iPager.setPageIndex(loadInfo.pageIndex);
                }
            }
            else if (item && item.hasOwnProperty("length")) {
                iPager.setTotalRecords(item.length);
                iPager.setPageIndex(filter && filter.pageIndex >= 0 ? filter.pageIndex : 0);
            }
            else if (item) {
                const result = this.level.getParentLevel().getChildren(item, true, false, true);
                if (grid.getLength(result))
                    iPager.setTotalRecords(result.length);
                iPager.setPageIndex(filter && filter.pageIndex >= 0 ? filter.pageIndex : 0);
            }
        }
        iPager.doDispatchEvents = true;
        iPager.updateDisplayList(iPager.getWidth(), iPager.getHeight());

    }

    /**
     *
     * @return {String}
     */
    getPrefix() {
        return "pager";
    }

    /**
     *
     * @return {Boolean}
     */
    getDrawTopBorder() {
        return this.getStyleValue("pagerDrawTopBorder")
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLocked() {
        return true;
    }

    /**
     *
     * @param cellRenderer
     * @param w
     * @param h
     */
    setRendererSize(cellRenderer, w, h) {

        flexiciousNmsp.CellUtils.setRendererSize(cellRenderer, w, h);
        //cellRenderer.setWidth("100%");
    }

    setWidth(value) {
        flexiciousNmsp.FlexDataGridCell.prototype.setWidth.apply(this, [value]);
    }
}

flexiciousNmsp.FlexDataGridPagerCell = FlexDataGridPagerCell; //add to name space
FlexDataGridPagerCell.prototype.typeName = FlexDataGridPagerCell.typeName = 'FlexDataGridPagerCell';//for quick inspection
