/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import TypedObject from '../../core/TypedObject'
/**
 * Base class for all events using the Flexicious Event Dispatcher mechanism.
 * @constructor
 * @class BaseEvent
 * @namespace flexiciousNmsp
 * 
 * @param type
 * @param [bubbles]
 * @param [cancelable]
 * @constructor
 */
export default class BaseEvent extends TypedObject {
    constructor(type, bubbles, cancelable) {

        super();
        /**
         * Type of this event.
         * @type {String}
         * @property type
         * @default type
         */
        this.type = type;
        /**
         * Ignored property
         * @type {Boolean}
         * @property bubbles
         * @default bubbles
         */
        this.bubbles = bubbles;
        /**
         * Whether or not this event can be cancelled.
         * @type {Boolean}
         * @property cancelable
         * @default cancelable
         */
        this.cancelable = cancelable;
        /**
         * Has the default for this event been prevented
         * @type {Boolean}
         * @property defaultPrevented
         * @default false
         */
        this.defaultPrevented = false;
    }

    getClassNames() { //for support of "is" keyword
        return ["TypedObject", this.typeName];
    }

    /**
     * Has the default for this event been prevented. This is true if the preventDefault method has been called.
     * @return {Boolean}
     * @method isDefaultPrevented
     */
    isDefaultPrevented() {
        return this.defaultPrevented;
    }

    /**
     * If this event is cancelable, then the default prevented flag is set to true.
     * @return {Boolean}
     * @method preventDefault
     */
    preventDefault() {
        this.defaultPrevented = true;
        if(this.triggerEvent && this.triggerEvent!=this){
            this.triggerEvent.preventDefault();
        }

    }

    /**
     * If this event is cancelable, then the default prevented flag is set to true.
     * @return {Boolean}
     * @method stopPropagation
     */
    stopPropagation() {
        if(this.triggerEvent && this.triggerEvent!=this){
            this.triggerEvent.stopPropagation();
        }

    }
}

flexiciousNmsp.BaseEvent = BaseEvent; //add to name space
BaseEvent.prototype.typeName = BaseEvent.typeName = "BaseEvent";//for quick inspection
