/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
 
import BaseEvent from './BaseEvent'


/**
 * A version of the FilterPageSortChangeEvent, however, this one has a advanced filter which
 * contains the level information in addition to the regular filter.
 * @constructor
 * @class ExtendedFilterPageSortChangeEvent
 * @namespace
 * 
 */

export default class ExtendedFilterPageSortChangeEvent extends BaseEvent {
    constructor(type, filter, bubbles, cancelable) {
        super(type, bubbles, cancelable);
        if (typeof filter == "undefined") filter = null;
        /**
         * Event that caused this event to happen. Usually the change event from the filter control
         * @property  triggerEvent
         * @type Event
         * @default null
         */
        this.triggerEvent = null;
        /**
         * The filter object that contains all the information about the filter arguments, sorts, etc.
         * @property  filter
         * @type AdvancedFilter
         * @default null
         */
        this.filter = null;
        /**
         * Could be one of three values : filterChange, sortChange, or pageChange.
         * @property  cause
         * @type String
         * @default null
         */
        this.cause = null;
        /**
         * @property  filter
         * @type AdvancedFilter
         * @default filter
         */
        this.filter = filter;

        this.triggerEvent = this.triggerEvent;

    }

    getClassNames() {
        return ["ExtendedFilterPageSortChangeEvent", "Event"];
    }

    /**
     * Creates a new instance of this event.
     */
    clone() {
        return new flexiciousNmsp.ExtendedFilterPageSortChangeEvent(this.type,
            this.filter, this.bubbles, this.cancelable);

    }
}

flexiciousNmsp.ExtendedFilterPageSortChangeEvent = ExtendedFilterPageSortChangeEvent; //add to name space
ExtendedFilterPageSortChangeEvent.prototype.typeName = ExtendedFilterPageSortChangeEvent.typeName = 'ExtendedFilterPageSortChangeEvent';//for quick inspection
ExtendedFilterPageSortChangeEvent.FILTER_PAGE_SORT_CHANGE = "filterPageSortChange";
ExtendedFilterPageSortChangeEvent.FILTER_CHANGE = "filterChange";
ExtendedFilterPageSortChangeEvent.PAGE_CHANGE = "pageChange";
ExtendedFilterPageSortChangeEvent.SORT_CHANGE = "sortChange";
ExtendedFilterPageSortChangeEvent.ITEM_LOAD = "itemLoad";