/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import BaseEvent from './BaseEvent'
/**
 * Class that holds information about all events that the grid will dispatch.
 * @constructor
 * @class FlexDataGridEvent
 * @namespace flexiciousNmsp
 * 
 *
 * @param [type]
 * @param [grid]
 * @param [level]
 * @param [column]
 * @param [cell]
 * @param [item]
 * @param [triggerEvent]
 * @param [bubbles]
 * @param [cancelable]
 * @constructor
 */

export default class FlexDataGridEvent extends BaseEvent {
    constructor(type, grid, level, column, cell, item, triggerEvent, bubbles, cancelable) {
        if (typeof grid === "undefined") {
            grid = null;
        }
        if (typeof level === "undefined") {
            level = null;
        }
        super(type, bubbles, cancelable);
        /**
         * The FlexDataGrid associated with the cell that triggered this event
         * @type {FlexDataGrid}
         * @property grid
         * @default grid
         */
        this.grid = grid;
        /**
         * The FlexDataGridColumnLevel associated with the cell that triggered this event
         * @type {FlexDataGridColumnLevel}
         * @property level
         * @default level
         */
        this.level = level;
        /**
         * The FlexDataGridColumn associated with the cell that triggered this event
         * @type {FlexDataGridColumn}
         * @property column
         * @default column
         */
        this.column = column;
        /**
         * The IFlexDataGridCell associated with the cell that triggered this event
         * @type {IFlexDataGridCell}
         */
        this.cell = cell;
        /**
         * The data associated with the cell that triggered this event
         * @type {Object}
         * @property item
         * @default item
         */
        this.item = item;
        /**
         * The event that triggered this event. For example, a mouse click on a cell triggers an Item Click on the grid.
         * @type {Event}
         * @property triggerEvent
         * @default triggerEvent
         */
        this.triggerEvent = triggerEvent;

    }

    getClassNames() { //for support of "is" keyword
        return ["BaseEvent", this.typeName];
    }
}

flexiciousNmsp.FlexDataGridEvent = FlexDataGridEvent; //add to name space
FlexDataGridEvent.prototype.typeName = FlexDataGridEvent.typeName = "FlexDataGridEvent";//for quick inspection
/**
 *  Dispatched when row selection or cell selection changes.
 * Event Type:com.flexicious.nestedtreedatagrid.events.FlexDataGridEvent
 * @event change
 */
FlexDataGridEvent.CHANGE = "change";
FlexDataGridEvent.AUTO_REFRESH = "autoRefresh";
FlexDataGridEvent.DATA_PROVIDER_CHANGE = "dataProviderChange";
FlexDataGridEvent.COLUMNS_RESIZED = "columnsResized";
FlexDataGridEvent.COLUMNS_SHIFT = "columnsShift";
FlexDataGridEvent.COMPONENTS_CREATED = "componentsCreated";
FlexDataGridEvent.COLUMN_RESIZED = "columnResized";
FlexDataGridEvent.COLUMN_X_CHANGED = "columnXChanged";
FlexDataGridEvent.COLUMN_VISIBLE_CHANGED="columnVisibleChanged";
FlexDataGridEvent.HEADER_CLICKED = "headerClicked";
FlexDataGridEvent.SELECT_ALL_CHECKBOX_CHANGED = "selectAllCheckBoxChanged";
FlexDataGridEvent.SELECT_CHECKBOX_CHANGED = "selectCheckBoxChanged";
FlexDataGridEvent.ITEM_OPEN = "itemOpen";
FlexDataGridEvent.ITEM_OPENING = "itemOpening";
FlexDataGridEvent.ITEM_CLOSE = "itemClose";
FlexDataGridEvent.ITEM_CLOSING = "itemClosing";
FlexDataGridEvent.ITEM_EDIT_CANCEL = "itemEditCancel";
FlexDataGridEvent.ITEM_EDIT_VALUE_COMMIT = "itemEditValueCommit";
FlexDataGridEvent.ITEM_EDIT_END = "itemEditEnd";
FlexDataGridEvent.ITEM_EDIT_BEGINNING = "itemEditBeginning";
FlexDataGridEvent.ITEM_EDIT_BEGIN = "itemEditBegin";
FlexDataGridEvent.ITEM_EDITOR_CREATED = "itemEditorCreated";
FlexDataGridEvent.ITEM_FOCUS_IN = "itemFocusIn";
FlexDataGridEvent.ITEM_ROLL_OVER = "itemRollOver";
FlexDataGridEvent.ITEM_ROLL_OUT = "itemRollOut";
FlexDataGridEvent.ITEM_CLICK = "itemClick";
FlexDataGridEvent.ITEM_RIGHT_CLICK = "itemRightClick";
FlexDataGridEvent.ITEM_DOUBLE_CLICK = "itemDoubleClick";
FlexDataGridEvent.DYNAMIC_LEVEL_CREATED = "dynamicLevelCreated";
FlexDataGridEvent.DYNAMIC_ALL_LEVELS_CREATED = "dynamicAllLevelsCreated";
FlexDataGridEvent.ICON_MOUSE_OVER = "iconMouseOver";
FlexDataGridEvent.ICON_CLICK = "iconClick";
FlexDataGridEvent.ICON_MOUSE_OUT = "iconMouseOut";
FlexDataGridEvent.PREBUILT_FILTER_RUN = "prebuiltFilterRun";
FlexDataGridEvent.RENDERER_INITIALIZED = "rendererInitialized";
FlexDataGridEvent.CELL_RENDERED = "cellRendered";
FlexDataGridEvent.CELL_CREATED = "cellCreated";
FlexDataGridEvent.COLUMN_STRETCH = "columnStretch";
FlexDataGridEvent.PLACING_SECTIONS = "placingSections";
FlexDataGridEvent.WIDTH_CHANGED = "widthChanged";
FlexDataGridEvent.BODY_CONTAINER_MOUNTED = "bodyContainerMounted";
