/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
 
import BaseEvent from './BaseEvent'
/**
 * When grid.enableVirtualScroll=true, the grid dispatches a virtualScroll event when the grid scrolls.
 * This event contains a recordsToLoad array, which contains an series of VirtualScrollLoadInfo records.
 * This provides the application a list of record indexes to load, and which levels to load them at.
 *
 * In flat grids, the FlexDataGridVirtualScrollEvent.recordsToLoad will always contain a list of VirtualScrollLoadInfo records were level=1.
 * For nested grids, depending on where the user scrolls and which items are open, you could get a list of records from multiple levels,
 * for example. Records 1-3 from level 1, All children of record 3 from level 2, and then Record 4-10 from level 1.
 *
 * Setting enableVirtualScroll requires that you specify childrenCountField as well as selectedKeyField.
 * Additionally, enableVirtualScroll also requires that you set filterPageSortMode=server and
 * itemLoadMode=server. Finally, virtual scroll does not work with variableRowHeights.
 * @constructor
 * @class FlexDataGridVirtualScrollEvent
 * @namespace flexiciousNmsp
 * 
 */
export default class FlexDataGridVirtualScrollEvent extends BaseEvent {
    constructor(type, grid, recordsToLoad, bubbles, cancelable) {
        super(type, bubbles, cancelable);
        /**
         * The FlexDataGrid object
         * @type {FlexDataGrid}
         * @property grid
         * @default grid
         */
        this.grid = grid;
        /**
         * This event contains a recordsToLoad array, which contains an series of VirtualScrollLoadInfo records.
         * This provides the application a list of record indexes to load, and which levels to load them at.
         * @type {Array}
         * @property recordsToLoad
         * @default recordsToLoad
         */
        this.recordsToLoad = recordsToLoad;
    }

    getClassNames() { //for support of "is" keyword
        return ["BaseEvent", this.typeName];
    }
}

flexiciousNmsp.FlexDataGridVirtualScrollEvent = FlexDataGridVirtualScrollEvent; //add to name space
FlexDataGridVirtualScrollEvent.prototype.typeName = FlexDataGridVirtualScrollEvent.typeName = "FlexDataGridVirtualScrollEvent";//for quick inspection
FlexDataGridVirtualScrollEvent.VIRTUAL_SCROLL = "virtualScroll";