/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
 
import BaseEvent from './BaseEvent'
/**
 * Dispatched when the grids' preferencePersistenceMode='server' and:
 * 1) The Grid needs to retrieve the preference data from the server
 * 2) The Grid needs to persist the  preference data to the server
 * 3) The Grid needs to clear out its preference data.
 *
 * @constructor
 * @class PreferencePersistenceEvent
 * @namespace flexiciousNmsp
 * 
 * @param type
 * @param preferenceKey
 * @param preferenceXml
 * @param [bubbles]
 * @param [cancelable]
 * @constructor
 */
export default class PreferencePersistenceEvent extends BaseEvent {
    constructor(type, preferenceKey, preferenceXml, bubbles, cancelable) {

        super(type, bubbles, cancelable);
        /**
         *A key that uniquely identifies the grid whose preference data
         * needs to be persisted on the server
         * @property preferenceKey
         * @type String
         * @default null
         */
        this.preferenceKey = null;
        /**
         *An xml string that contains all the preference data for this grid.
         * @property preferenceXml
         * @type String
         * @default null
         */
        this.preferenceXml = null;
        /**
         * If you want to save any custom data into the preferences string, populate this property in the
         * savePreferences event. (This event will be dispatched JUST before saving the preferences. And it
         * will be available to you in the loadPreferences event. (This event is called right after the preferences
         * are applied to the IPersistable)
         * @property customData
         * @type String
         * @default null
         */
        this.customData = null;
        /**
         * Name of the preference to persist. Added in 2.9 to support multiple preferences
         * @property preferenceName
         * @type String
         * @default null
         */
        this.preferenceName = null;
        /**
         * Whether the preference to add is the default. Added in 2.9 to support multiple preferences
         * @property isDefault
         * @type boolean
         * @default null
         */
        this.isDefault = null;



        this.preferenceKey = preferenceKey;
        this.preferenceXml = preferenceXml;


    }

    getClassNames() {
        return ["PreferencePersistenceEvent", "Event"];
    }
}

flexiciousNmsp.PreferencePersistenceEvent = PreferencePersistenceEvent; //add to name space 
PreferencePersistenceEvent.prototype.typeName = PreferencePersistenceEvent.typeName = 'PreferencePersistenceEvent';//for quick inspection
PreferencePersistenceEvent.PREFERENCES_LOADING = "preferencesLoading";
PreferencePersistenceEvent.LOAD_PREFERENCES = "loadPreferencesEvent";
PreferencePersistenceEvent.PERSIST_PREFERENCES = "persistPreferencesEvent";
PreferencePersistenceEvent.CLEAR_PREFERENCES = "clearPreferencesEvent";