/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import Exporter from './Exporter';

/**
 * Exports the grid in CSV format
 * @constructor
 * @class CsvExporter
 * @namespace flexiciousNmsp
 */
export default class CsvExporter extends Exporter {
    
    getClassNames() {
        return ["CsvExporter", "Exporter"];
    }

    /**
     * Writes the header of the grid (columns) in csv format
     * @param grid
     * @return {String}
     * @method writeHeader
     */
    writeHeader(grid) {

        return `${this.buildHeader(grid)}\r\n`;

    }

    /**
     * Builds the CSV representation of a header row.
     * @param grid
     * @return {String}
     * @method buildHeader
     */
    buildHeader(grid) {
        let str = "";
        let colIndex = 0;
        let i = 0;
        while (i++ < this.getNestDepth())
            str += " ,";
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col))continue;
            str += `${this.escapeCsv(flexiciousNmsp.Exporter.getColumnHeader(col, colIndex++))},`;
        }

        if (str.indexOf("ID") == 0) {
            //http://support.microsoft.com/kb/323626
            str = ` ${str}`;
        }
        if (str.length > 0) {
            str = str.substring(0, str.length - 1);
        }
        return str;
    }

    /**
     * Escapes CSV characters to make valid CSV string
     * @param val
     * @return {String|XML}
     * @method escapeCsv
     */
    escapeCsv(val) {

        return val ? val.toString().replace("\"\n", "\"").replace(/"/g, "\"\"") : "";

    }

    /**
     * Writes an individual record in csv format
     * @param grid
     * @param record
     * @return {String}
     * @method writeRecord
     */
    writeRecord(grid, record) {
        let str = "";
        let i = 0;
        if (!this.reusePreviousLevelColumns) {
            while (i++ < this.getNestDepth()) {
                str += " ,";
            }
        }
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col))continue;
            let itemVal = col.itemToLabel(record);
            if (/^\d+$/.test(itemVal)) {
                if(parseFloat(itemVal)>CsvExporter.SCIENTIFIC_NOTATION_THRESHOLD)
                {
                    itemVal = `\t${itemVal}`;
                }
            }
            str += `"${this.getSpaces(col)}${this.escapeCsv(itemVal)}",`;
        }

        if (str.length > 0)
            str = str.substr(0, str.length - 1);
        str += "\r\n";
        return str;
    }

    /**
     * CsvExporter.SCIENTIFIC_NOTATION_THRESHOLD=1000000;
     * Writes the footer in CSV format
     * @method writeFooter
     * @param grid
     * @param dataProvider
     * @return {String}
     *
     */
    writeFooter(grid, dataProvider) {
        if(this.exportOptions.includeFooters){
            let str ="";
            let i = 0;
            if (!this.reusePreviousLevelColumns) {
                while (i++ < this.getNestDepth()) {
                    str += " ,";
                }
            }
            const items = grid.getExportableColumns();

            for (const col of items) {
                if(!this.isIncludedInExport(col))continue;
                str += `"${this.getSpaces(col)}${this.escapeCsv(col.calculateFooterValue(dataProvider))}",`;
            }

            if(str.length>0)
                str=str.substr(0,str.length-1);

            str += "\r\n";
            return str;
        }
        return "";

    }

    /**
     * Extension of the download file.
     * @return {String}
     * @method getExtension
     */
    getExtension() {
        return "csv";
    }

    /**
     * Returns the content type so MS Excel launches
     * when the exporter is run.
     * @return {String}
     * @method getContentType
     *
     */
    getContentType() {
        return "text/csv"
    }

    /**
     * Name of the exporter
     * @return {String}
     * @method getName
     *
     */
    getName() {
        return "Excel"
    }
}

flexiciousNmsp.CsvExporter = CsvExporter; //add to name space
CsvExporter.prototype.typeName = CsvExporter.typeName = 'CsvExporter';//for quick inspection
CsvExporter.SCIENTIFIC_NOTATION_THRESHOLD=99999999999;
