/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 

/**
 * Exports the grid in CSV format
 * @constructor
 * @namespace
 */
export default class Excel2007Exporter extends Exporter {
    constructor() {
        
        super();
        /**
         * Writes the header of the grid (columns) in csv format
         * @param grid
         * @return
            *
         */
        this.strTable = "";

    }

    getClassNames() {
        return ["Excel2007Exporter","Exporter"];
    }

    /**
     * Writes the header of the grid (columns) in Excel 2007 format
     * @param {flexiciousNmsp.FlexDataGrid} grid 
     */
    writeHeader(grid) {

        this.buildHeader(grid);
        return  "";

    }

    /**
     * @private
     * @param grid
     * @return
     */
    buildHeader(grid) {
        let colIndex=0;

        this.strTable += "<Row ss:StyleID='s68'>";
        while(colIndex++<this.nestDepth)
            this.strTable += "<Cell><Data ss:Type='String'></Data></Cell>";

        for (const col of grid.getExportableColumns()) {
            if(!this.isIncludedInExport(col))
                continue;

            this.strTable += `<Cell><Data ss:Type='String'>${flexiciousNmsp.Exporter.getColumnHeader(col,colIndex)}</Data></Cell>`;
            colIndex++;
        }

        this.strTable += "</Row>";
    }

    /**
     * Just in case in the client wants to customize the upload
     * behavior of their exporters.
     * @param body
     * @param exportOptions
     * @method uploadForEcho
     */
    uploadForEcho(body, exportOptions) {
        let strWorkbook = "<?xml version='1.0' encoding='ISO-8859-1'?><?mso-application progid='Excel.Sheet'?>" +
            "<Workbook xmlns='urn:schemas-microsoft-com:office:spreadsheet' " +
            "xmlns:ss='urn:schemas-microsoft-com:office:spreadsheet' xmlns:x='urn:schemas-microsoft-com:office:excel' " +
            "xmlns:o='urn:schemas-microsoft-com:office:office' xmlns:html='http://www.w3.org/TR/REC-html40'>" +
            "<Styles>" +
            "<Style ss:Name='Normal' ss:ID='Default'>" +
            "<Alignment ss:Vertical='Bottom' />" +
            "<Borders />" +
            "<Font />" +
            "<Interior />" +
            "<NumberFormat />" +
            "<Protection />" +
            "</Style>" +
            "<Style ss:ID='s68'>" +
            "<Borders>" +
            "<Border ss:Position='Bottom' ss:LineStyle='Continuous' ss:Weight='2'/>" +
            "<Border ss:Position='Top' ss:LineStyle='Continuous' ss:Weight='2'/>" +
            "</Borders>" +
            "<Interior ss:Color='#4F81BD' ss:Pattern='Solid'/>" +
            "</Style>" +
            "</Styles>" +
            "<Worksheet ss:Name='Sheet1'>" +
            "<Table>";
        strWorkbook += `${this.strTable}</Table></Worksheet></Workbook>`;
        flexiciousNmsp.Exporter.prototype.uploadForEcho.apply(this,[strWorkbook, this.exportOptions]);
        this.strTable="";
    }

    /**
     * Writes an individual record in csv format
     * @param grid
     * @param record
     * @return
        *
     */
    writeRecord(grid, record) {
        let colIndex=0;
        this.strTable += "<Row>";
        if(!this.reusePreviousLevelColumns){
            while(colIndex++<this.nestDepth)
                this.strTable += "<Cell><Data ss:Type='String'></Data></Cell>";
        }

        for (const col of grid.getExportableColumns()) {
            if(!this.isIncludedInExport(col))
                continue;
            const str = col.itemToLabel(record);
            this.strTable += `<Cell><Data ss:Type='String'>${str}</Data></Cell>`;
        }

        this.strTable += "</Row>";
        return "";
    }

    /**
     * Writes the footer in CSV format
     * @param grid
     * @param dataProvider
     */
    writeFooter(grid, dataProvider) {
//
//        if(this.exportOptions.includeFooters){
//            var colIndex=0;
//            this.strTable += "<Row ss:StyleID='s68'>";
//            if(!this.reusePreviousLevelColumns){
//                while(colIndex++<this.nestDepth)
//                    this.strTable += "<Cell><Data ss:Type='String'></Data></Cell>";
//            }
//            for(var i=0;i<grid.getExportableColumns().length;i++){
//                var col=grid.getExportableColumns()[i];
//                if(!this.isIncludedInExport(col))
//                    continue;
//                var str = col.calculateFooterValue(dataProvider);
//                this.strTable += "<Cell><Data ss:Type='String'>" + str + "</Data></Cell>";
//            }
//            this.strTable += "</Row>";
//
//        }
        return "";

    }

    /**
     * Extension of the download file.
     * @return
     */
    getExtension() {
        return "xml";
    }

    /**
     * Returns the content type so MS Excel launches
     * when the exporter is run.
     * @return
     */
    getContentType() {
        return "text/xml"
    }

    /**
     * Name of the exporter
     * @return
        *
     */
    getName() {
        return "Excel 2007";
    }
}

flexiciousNmsp.Excel2007Exporter = Excel2007Exporter; //add to name space
Excel2007Exporter.prototype.typeName = Excel2007Exporter.typeName = 'Excel2007Exporter';//for quick inspection