/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import UIUtils from '../../utils/UIUtils'
import Constants from '../../utils/Constants'
import PrintExportOptions from '../utils/PrintExportOptions'

/**
 * Class to export the datagrid, either one page at a time
 * or specific pages, or all pages.
 * @constructor
 * @class ExportOptions
 * @namespace flexiciousNmsp
 */
export default class ExportOptions extends PrintExportOptions {
 constructor() {
     super();
     /**
      * A URL that the export infrastructure uses to trigger a file
      * download.
      * The various exporters (Doc,CSV,TXT,HTML) basically build a string to
      * export and post it
      * to this url along with the content type and the file extension
      * This is the url to a server that simply writes back the content to
      * the client along with the mime type and content disposition headers.
      *
      * C# code to do this follows (Java and PHP would do the same thing).
      * Response.AddHeader("content-disposition", string.Format("attachment; filename=Download.{0}",Request["extension"]));
      * Response.ContentType = Request["contentType"];
      * Response.Write(Request["body"]);
      * Response.End();
      * Defaults to:
      * It is recommended you provide your own echo URL.
      * @property echoUrl
      * @type {String}
      * @default Blank
      */

     this.echoUrl = "";
     /**
      *A list of exporters that are responsible for handling each of the export
      * formats. Defaults to an array of CsvExporter, DocExporter, HtmlExporter
      * and TxtExporter.
      * @default
      * @property exporters
      * @type Array
      */

     this.exporters = [new flexiciousNmsp.CsvExporter(), new flexiciousNmsp.DocExporter(), new flexiciousNmsp.HtmlExporter(), new flexiciousNmsp.TxtExporter(), new flexiciousNmsp.XmlExporter()];

     if(flexiciousNmsp.PdfExporter){
         this.exporters.push(new flexiciousNmsp.PdfExporter());
     }
     /**
      * The view responsible for rendering the export options
      * Default implementation is provided by com.flexicious.export.ExportOptionsView
      * @property exportOptionsRenderer
      * @type ClassFactory
      */

     this.exportOptionsRenderer = new flexiciousNmsp.ClassFactory(flexiciousNmsp.ExportOptionsView);
     /**
      * An instance of the exportOptionsRenderer. Please note, this is only instantiated when you call the export method on the controller.
      * @property exportOptionsView
      * @type UIComponent
      * @default null
      */
     this.exportOptionsView = null;

     /**
      *
      * @type {flexiciousNmsp.CsvExporter}
      * @property exporter
      * @default null
      */
     this.exporter = new flexiciousNmsp.CsvExporter();
     /**
      * Flag to control the download message. Defaults to true.
      * @property useSaveFileMessage
      * @type Boolean
      * @default false
      */

     this.useSaveFileMessage = false;
     /**
      * Flag to enable local file persistence. Defaults to true for Flex 4.
      * @property enableLocalFilePersistence
      * @type Boolean
      * @default true
      */
     this.enableLocalFilePersistence = true;
     /**
      * Flag to copy the buffer to clipboard, instead of sending it to the server/disk.
      * @property copyToClipboard
      * @type Boolean
      * @default false
      */
     this.copyToClipboard = false;
     /**
      * File name for the download. Defaults to download.xxx where xxx is the extension.
      * @property exportFileName
      * @type String
      * @default download
      */
     this.exportFileName = "download";
     /**
      * Any custom data that is passed into the print/export mechanism.<br/>
      * This will then be available on Print/Export Options when the Print or Export is run.<br/>
      * Please note, this is not persisted in preferences.
      * @property printExportParameters
      * @type Object
      * @default null
      */
     this.printExportParameters = null;
     /**
      * Flag, which defaults to false, launches the preview and options screen in modal mode.
      * @property modalWindows
      * @type Boolean
      * @default null
      */
     this.modalWindows = null;
     /**
      * For HTML/DOC export, we specify this so that the nested
      * export does not look jagged
      * @property tableWidth
      * @type Number
      * @default 0
      */
     this.tableWidth = 0;
     /**
      * If set to true, exports all the records, irrespective of the filter and paging functions.
      * Please note, for this to work, the dataprovider for the grid MUST be an Array Collection
      * @property exportAllRecords
      * @type Boolean
      * @default false
      */
     this.exportAllRecords = false;
     /**
      * Instead of using the built in csv exporter, instead exports HTML that
      * Microsoft Excel is able to render. Generates a file with an XLS extension.
      * @property useExcelExporter
      * @type Boolean
      * @default false
      */
     this.useExcelExporter = false;
     /**
      * A list of exportable columns
      * @property availableColumns
      * @type ArrayCollection
      * @default []
      */
     this.availableColumns = [];
     /**
      * A list of columns to export
      * @property columnsToExport
      * @type ArrayCollection
      * @default []
      */
     this.columnsToExport = [];
     /**
      * The grid being exported
      * @property grid
      * @type IExtendedDataGrid
      * @default null
      */
     this.grid = null;

     /**
      * A property that will launch the resultant export into a new window.
      * Used for the print option.
      * @property openNewWindow
      * @type Boolean
      * @default false
      */
     this.openNewWindow = false;
     /**
      * Title for the print options dialog
      * @property printOptionsWindowTitle
      * @type String
      * @default Print Options
      */
     this.exportOptionsWindowTitle = Constants.EXPORT_OPTIONS_TITLE;



     /**
      * Whether or not to include footer cells in the export
      */
     this.includeFooters=true;
 }

 getClassNames() {
     return ["ExportOptions", "PrintExportOptions", "TypedObject"];
 }

 /**
  * Convenience method to create an exporter of a specified type
  * @param exporterIndex
  * @return {ExportOptions} 
  */
 static create(exporterIndex) {
     if (typeof exporterIndex == "undefined")exporterIndex = 0;

     const myExportOptions = new ExportOptions();
     myExportOptions.exporter = myExportOptions.exporters[exporterIndex];
     return myExportOptions;

 }

 /**
  * Name of the current exporter.
  */
 getExporterName() {
     return this.exporter.getName();
 }
}

flexiciousNmsp.ExportOptions = ExportOptions; //add to name space
ExportOptions.prototype.typeName = ExportOptions.typeName = 'ExportOptions';//for quick inspection
ExportOptions.CSV_EXPORT = 0;
ExportOptions.DOC_EXPORT = 1;
ExportOptions.HTML_EXPORT = 2;
ExportOptions.TXT_EXPORT = 3;
ExportOptions.XML_EXPORT = 4;
ExportOptions.PDF_EXPORT = 5;