/*eslint-disable */
/* FileSaver.js
 * A saveAs() FileSaver implementation.
 * 1.3.2
 * 2016-06-16 18:25:19
 *
 * By Eli Grey, http://eligrey.com
 * License: MIT
 *   See https://github.com/eligrey/FileSaver.js/blob/master/LICENSE.md
 */

/*global self */
/*jslint bitwise: true, indent: 4, laxbreak: true, laxcomma: true, smarttabs: true, plusplus: true */

/*! @source http://purl.eligrey.com/github/FileSaver.js/blob/master/FileSaver.js */

var saveAs = saveAs || (function (view) {
    "use strict";
    // IE <10 is explicitly unsupported
    if (typeof view === "undefined" || typeof navigator !== "undefined" && /MSIE [1-9]\./.test(navigator.userAgent)) {
        return;
    }
    var
        doc = view.document
        // only get URL when necessary in case Blob.js hasn't overridden it yet
        , get_URL = function () {
            return view.URL || view.webkitURL || view;
        }
        , save_link = doc.createElementNS("http://www.w3.org/1999/xhtml", "a")
        , can_use_save_link = "download" in save_link
        , click = function (node) {
            var event = new MouseEvent("click");
            node.dispatchEvent(event);
        }
        , is_safari = /constructor/i.test(view.HTMLElement) || view.safari
        , is_chrome_ios = /CriOS\/[\d]+/.test(navigator.userAgent)
        , throw_outside = function (ex) {
            (view.setImmediate || view.setTimeout)(function () {
                throw ex;
            }, 0);
        }
        , force_saveable_type = "application/octet-stream"
        // the Blob API is fundamentally broken as there is no "downloadfinished" event to subscribe to
        , arbitrary_revoke_timeout = 1000 * 40 // in ms
        , revoke = function (file) {
            var revoker = function () {
                if (typeof file === "string") { // file is an object URL
                    get_URL().revokeObjectURL(file);
                } else { // file is a File
                    file.remove();
                }
            };
            setTimeout(revoker, arbitrary_revoke_timeout);
        }
        , dispatch = function (filesaver, event_types, event) {
            event_types = [].concat(event_types);
            var i = event_types.length;
            while (i--) {
                var listener = filesaver["on" + event_types[i]];
                if (typeof listener === "function") {
                    try {
                        listener.call(filesaver, event || filesaver);
                    } catch (ex) {
                        throw_outside(ex);
                    }
                }
            }
        }
        , auto_bom = function (blob) {
            // prepend BOM for UTF-8 XML and text/* types (including HTML)
            // note: your browser will automatically convert UTF-16 U+FEFF to EF BB BF
            if (/^\s*(?:text\/\S*|application\/xml|\S*\/\S*\+xml)\s*;.*charset\s*=\s*utf-8/i.test(blob.type)) {
                return new Blob([String.fromCharCode(0xFEFF), blob], { type: blob.type });
            }
            return blob;
        }
        , FileSaver = function (blob, name, no_auto_bom) {
            if (!no_auto_bom) {
                blob = auto_bom(blob);
            }
            // First try a.download, then web filesystem, then object URLs
            var
                filesaver = this
                , type = blob.type
                , force = type === force_saveable_type
                , object_url
                , dispatch_all = function () {
                    dispatch(filesaver, "writestart progress write writeend".split(" "));
                }
                // on any filesys errors revert to saving with object URLs
                , fs_error = function () {
                    if ((is_chrome_ios || (force && is_safari)) && view.FileReader) {
                        // Safari doesn't allow downloading of blob urls
                        var reader = new FileReader();
                        reader.onloadend = function () {
                            var url = is_chrome_ios ? reader.result : reader.result.replace(/^data:[^;]*;/, 'data:attachment/file;');
                            var popup = view.open(url, '_blank');
                            if (!popup) view.location.href = url;
                            url = undefined; // release reference before dispatching
                            filesaver.readyState = filesaver.DONE;
                            dispatch_all();
                        };
                        reader.readAsDataURL(blob);
                        filesaver.readyState = filesaver.INIT;
                        return;
                    }
                    // don't create more object URLs than needed
                    if (!object_url) {
                        object_url = get_URL().createObjectURL(blob);
                    }
                    if (force) {
                        view.location.href = object_url;
                    } else {
                        var opened = view.open(object_url, "_blank");
                        if (!opened) {
                            // Apple does not allow window.open, see https://developer.apple.com/library/safari/documentation/Tools/Conceptual/SafariExtensionGuide/WorkingwithWindowsandTabs/WorkingwithWindowsandTabs.html
                            view.location.href = object_url;
                        }
                    }
                    filesaver.readyState = filesaver.DONE;
                    dispatch_all();
                    revoke(object_url);
                }
                ;
            filesaver.readyState = filesaver.INIT;

            if (can_use_save_link) {
                object_url = get_URL().createObjectURL(blob);
                setTimeout(function () {
                    save_link.href = object_url;
                    save_link.download = name;
                    click(save_link);
                    dispatch_all();
                    revoke(object_url);
                    filesaver.readyState = filesaver.DONE;
                });
                return;
            }

            fs_error();
        }
        , FS_proto = FileSaver.prototype
        , saveAs = function (blob, name, no_auto_bom) {
            return new FileSaver(blob, name || blob.name || "download", no_auto_bom);
        }
        ;
    // IE 10+ (native saveAs)
    if (typeof navigator !== "undefined" && navigator.msSaveOrOpenBlob) {
        return function (blob, name, no_auto_bom) {
            name = name || blob.name || "download";

            if (!no_auto_bom) {
                blob = auto_bom(blob);
            }
            return navigator.msSaveOrOpenBlob(blob, name);
        };
    }

    FS_proto.abort = function () { };
    FS_proto.readyState = FS_proto.INIT = 0;
    FS_proto.WRITING = 1;
    FS_proto.DONE = 2;

    FS_proto.error =
        FS_proto.onwritestart =
        FS_proto.onprogress =
        FS_proto.onwrite =
        FS_proto.onabort =
        FS_proto.onerror =
        FS_proto.onwriteend =
        null;

    return saveAs;
} (
    typeof self !== "undefined" && self
    || typeof window !== "undefined" && window
    || this.content
    ));
// `self` is undefined in Firefox for Android content script context
// while `this` is nsIContentFrameMessageManager
// with an attribute `content` that corresponds to the window

if (typeof module !== "undefined" && module.exports) {
    module.exports.saveAs = saveAs;
} else if ((typeof define !== "undefined" && define !== null) && (define.amd !== null)) {
    define("FileSaver.js", function () {
        return saveAs;
    });
}


/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import UIUtils from '../../utils/UIUtils'
import Constants from '../../utils/Constants'
import TypedObject from '../../core/TypedObject'
/**
 * psuedo abstract class responsible for most export functionality.
 * @constructor
 * @class Exporter
 * @namespace flexiciousNmsp
 * 
 */
export default class Exporter extends TypedObject {
    constructor() {
        super();

        this.nestIndent = 15;
        this.exportOptions = null;

        this.allRecords = [];

        this.nestDepth = 0;

        this.reusePreviousLevelColumns = false;
    }

    getClassNames() {
        return ["Exporter", "TypedObject"];
    }

    /**
     * Writes out the header for the row
     * @param grid
     * @return {String}
     * @method writeHeader
     */
    writeHeader(grid) {

        return "";

    }

    /**
     * Writes out the record for the row
     * @param grid
     * @param record
     * @return {String}
     * @method writeRecord
     */
    writeRecord(grid, record) {

        return "";

    }

    /**
     * Writes out the footer for the row
     * @method writeFooter
     * @param grid
     * @param dataProvider
     * @return {String}
     */
    writeFooter(grid, dataProvider) {

        return "";

    }

    /**
     * Gets the header for the row
     * @param col
     * @param colIndex
     * @return {*}
     * @method getColumnHeader
     */
    static getColumnHeader(col, colIndex) {

        const adg = col.getLevel().grid;
        const headerText = col.getHeaderText() ? col.getHeaderText() : col.getDataField() ? col.getDataField() : (`Column ${colIndex + 1}`);
        if (adg) {
            if (col.columnGroup) {
                return `${col.columnGroup.getConcatenatedHeaderText()} - ${headerText}`;
            }
            const cols = adg.getGroupedColumns().slice();
            UIUtils.sortOn(cols, "_depth", true);
            cols.reverse();

            for (const grp of cols) {
                if (grp.implementsOrExtends("FlexDataGridColumnGroup")) {
                    if (grp.getAllColumns().includes(col)) {
                        return `${grp.getConcatenatedHeaderText()} - ${headerText}`;
                    }
                }
            }
        }
        return headerText;

    }

    /**
     * Saves the file to disk, or sends it to a remote server to buffer back.
     * @param body
     * @param fileName
     * @param exportOptions
     * @method saveFile
     */
    saveFile(body, fileName, exportOptions) {
        //this.uploadToServer(body,exportOptions);
        if (exportOptions.openNewWindow) {
            const win = UIUtils.openBrowserPopup('about:blank');

            if (!win) {
                UIUtils.showMessage(Constants.DISABLE_POPUP_BLOCKERS);
                return;
            }
            UIUtils.setHtml(win.document.body, body);
            win.document.title = exportOptions.exportFileName;
            win.print();
            win.close();
            if (exportOptions.showColumnPicker)
                UIUtils.removePopUp(exportOptions.exportOptionsView);
        }
        else if (typeof (swfobject) != 'undefined' && swfobject.hasFlashPlayerVersion("9.0.18")) {

            const div = document.createElement('DIV');
            div.className = "bottomButtonBarSave";
            div.innerHTML = "You need Flash 10 for this to work!";
            UIUtils.addChild(exportOptions.exportOptionsView.domElement, div);
            Downloadify.create(div, {
                filename() {
                    return fileName;
                },
                data() {
                    return body;
                },
                onComplete() { div.parentNode.removeChild(div); UIUtils.removePopUp(exportOptions.exportOptionsView); },
                onCancel() { div.parentNode.removeChild(div); UIUtils.removePopUp(exportOptions.exportOptionsView); },
                onError() { div.parentNode.removeChild(div); UIUtils.removePopUp(exportOptions.exportOptionsView); },

                swf: `${Constants.IMAGE_PATH}/media/downloadify.swf`,
                downloadImage: `${Constants.IMAGE_PATH}/download.png`,
                width: 100,
                height: 30,
                transparent: true,
                append: false
            });
        }
        else {
            this.uploadToServer(body, exportOptions);
            //            flexiciousNmsp.saveAs(
            //                new flexiciousNmsp.Blob(
            //                    [body]
            //                    , {type: "text/plain;charset="+ document.characterSet}
            //                )
            //                , param1
            //            );
        }

    }

    /**
     * Just in case in the client wants to customize the upload
     * behavior of their exporters.
     * @param body
     * @param exportOptions
     * @method uploadForEcho
     */
    uploadForEcho(body, exportOptions) {

        if (exportOptions.grid) {
            const expEvent = new flexiciousNmsp.ExportEvent(flexiciousNmsp.ExportEvent.AFTER_EXPORT, false, false);
            expEvent.exportOptions = exportOptions;
            expEvent.textWritten = body;
            exportOptions.grid.dispatchEvent(expEvent);
            if (expEvent.textWritten != body) {
                body = expEvent.textWritten;
            }
        }

        let hasFlash = false;
        try {
            hasFlash = Boolean(new ActiveXObject('ShockwaveFlash.ShockwaveFlash'));
        } catch (exception) {
            hasFlash = ('undefined' != typeof navigator.mimeTypes['application/x-shockwave-flash']);
        }
        try {
            var isFileSaverSupported = !!new Blob;
        } catch (e) { }
        if (exportOptions.openNewWindow) {
            const win = UIUtils.openBrowserPopup('about:blank');

            if (!win) {
                UIUtils.showMessage(Constants.DISABLE_POPUP_BLOCKERS);
                return;
            }
            UIUtils.setHtml(win.document.body, body);
            win.document.title = exportOptions.exportFileName;
            win.print();
            win.close();
            if (exportOptions.showColumnPicker)
                UIUtils.removePopUp(exportOptions.exportOptionsView);
        }
        else if (exportOptions.copyToClipboard) {
            UIUtils.pasteToClipBoard(body);
        }
        else if (exportOptions.enableLocalFilePersistence && isFileSaverSupported && typeof (saveAs) != 'undefined') {
            const blob = new Blob([body], { type: "text/plain;charset=utf-8" });
            saveAs(blob, `${exportOptions.exportFileName}.${this.getExtension()}`);
            if (exportOptions.showColumnPicker)
                UIUtils.removePopUp(exportOptions.exportOptionsView);
        }
        else if (exportOptions.enableLocalFilePersistence && hasFlash) {
            this.saveFile(body, `${exportOptions.exportFileName}.${this.getExtension()}`, exportOptions);
        }
        else {
            if (exportOptions.echoUrl) {
                this.uploadToServer(body, exportOptions);
            } else {
                alert("Unable to export - please include filesaver, downloadify or specify an echoURL [Please read http://blog.htmltreegrid.com/post/Excel-Word-and-Html-Export-and-the-Echo-URL.aspx]");
            }
            if (exportOptions.showColumnPicker)
                UIUtils.removePopUp(exportOptions.exportOptionsView);
            /* //now that we have the doucment bits, push it to the server
             //and ask it to buffer it back to us with the proper content
             //type so our browser nicely displays a file download dialog.
             var variables=new flexiciousNmsp.URLVariables();
             variables.extension=this.extension;
             variables.contentType=this.contentType;
             variables.body=body;
             //this variable defaults to http://www.flexicious.com/Home/Echo
             //but it is recommended you change it to an internal server to
             //boost performance. Please read the as doc for ExportOptions
             var u=new flexiciousNmsp.URLRequest(exportOptions.echoUrl);
             u.data=variables;
              u.method=URLRequestMethod.POST;
             flexiciousNmsp.navigateToURL(u, "_self");*/
        }

    }

    /**
     * Not used
     * @method uploadToServer
     */
    uploadToServer(body, exportOptions) {
        const div = document.createElement('DIV');
        div.innerHTML = `<form id="flxsexportform" action="${exportOptions.echoUrl}" method="post" target="_blank"><input type="hidden" id="flxsbody" name="body" /><input type="hidden" id="flxscontentType" name="contentType" /><input type="hidden" id="flxsextension" name="extension" /></form>`;
        window.document.body.appendChild(div);
        document.getElementById('flxsbody').value = body;
        document.getElementById('flxscontentType').value = this.getContentType();
        document.getElementById('flxsextension').value = this.getExtension();
        document.getElementById('flxsexportform').submit();
        div.parentNode.removeChild(div);
    }

    /**
     * Checks whether this column should be exported
     * @param col
     * @return {Boolean|*}
     * @method isIncludedInExport
     */
    isIncludedInExport(col) {

        return !this.exportOptions || this.exportOptions.columnsToExport.length == 0 ||
            UIUtils.doesArrayContainValue(this.exportOptions.columnsToExport, "name", col.getUniqueIdentifier());

    }

    /**
     * Depending on the level, gets indentation spaces
     * @param col
     * @param spChar
     * @return {String}
     * @method getSpaces
     */
    getSpaces(col, spChar) {
        if (typeof spChar == "undefined") spChar = " ";

        if (!this.reusePreviousLevelColumns)
            return "";
        if (!col.hasOwnProperty("enableHierarchicalNestIndent"))
            return "";
        if (!col["enableHierarchicalNestIndent"])
            return "";
        let result = "";
        let i = 0;
        while (i < (this.nestIndent * this.getNestDepth())) {
            result += spChar;
            i++;
        }
        return result;

    }

    /**
     * Name of the exporter
     * @return {String}
     * @method getName
     */
    getName() {
        return "";
    }

    /**
     * Extension of the download file.
     * @return {String}
     * @method getExtension
     */
    getExtension() {
        return "";
    }

    /**
     * Returns the content type so the appropriate application launches
     * when the exporter is run.
     * @return {String}
     * @method getContentType
     *
     */
    getContentType() {
        return "";
    }

    /**
     * The current nest depth of the record being exported
     * @return {Number}
     * @method getNestDepth
     */
    getNestDepth() {

        return this.nestDepth;

    }

    /**
     * The nest indent on basis of the nest depth
     * @return {Number}
     * @method getNestIndent
     */
    getNestIndent() {

        return this.nestIndent;

    }
}

flexiciousNmsp.Exporter = Exporter; //add to name space
Exporter.prototype.typeName = Exporter.typeName = 'Exporter';//for quick inspection