/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import UIUtils from '../../utils/UIUtils'
import Constants from '../../utils/Constants'
import ExportController from './ExportController'
/**
 * The nested grid does not show an export options window.
 * We directly run the export based on the data that is currently being
 * displayed in the grid.
 * @constructor
 * @class ExtendedExportController
 * @namespace flexiciousNmsp.export
 */
export default class ExtendedExportController extends ExportController {
    constructor() {
        super();
        /**
         * For HTML/DOC export, we specify this so that the nested
         * export does not look jagged
         */
        this.DEFAULT_TABLE_WIDTH = 1000;
    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["ExtendedExportController", "ExportController", "TypedObject"];
    }

    static instance() {
        return ExtendedExportController._instance;
    }

    /**
     * Run the actual export.
     */
    doExport() {

        const exportOptions = this._exportOptions;
        const grid = this._grid.implementsOrExtends("FlexDataGrid") ? this._grid : null;
        if (grid.getFilterPageSortMode() == "server" &&
            (exportOptions.printExportOption == flexiciousNmsp.PrintExportOptions.PRINT_EXPORT_ALL_PAGES ||
                exportOptions.printExportOption == flexiciousNmsp.PrintExportOptions.PRINT_EXPORT_SELECTED_PAGES)
        ) {
            this.dispatchDataRequest();
            return;
        }
        this.runExport(grid.getDataForPrintExport(exportOptions));

    }

    /**
     *
     * @param [iCollectionView]
     * @param [allOrSelectedPages]
     *
     */
    runExport(iCollectionView, allOrSelectedPages) {
        if (typeof allOrSelectedPages == "undefined") allOrSelectedPages = false;
        var grid = this._grid;
        grid.inExport = true;
        this.runNestedExport(iCollectionView);
        grid.inExport = false;
        grid.recordBeingExported=null;
    }

    /**
     *
     * @param iCollectionView
     * @param level
     */
    runNestedExport(iCollectionView, level) {
        if (typeof level == "undefined") level = null;

        const grid = this._grid;

        if (!level) level = grid.getColumnLevel();
        const exportOptions = this._exportOptions;
        exportOptions.exporter.exportOptions = exportOptions;

        for (const obj of iCollectionView) {
            if (!exportOptions.exporter.allRecords)
                exportOptions.exporter.allRecords = [];

            exportOptions.exporter.allRecords.push(obj);
        }

        if (!exportOptions.tableWidth)
            exportOptions.tableWidth = Constants.DEFAULT_TABLE_WIDTH;
        let body = exportOptions.exporter.writeHeader(this._grid);

        for (const obj2 of iCollectionView) {
            body += this.writeRecord(obj2, level);
        }

        this.setExportLevel(level);
        body += exportOptions.exporter.writeFooter(this._grid, iCollectionView);
        exportOptions.exporter.uploadForEcho(body, exportOptions);
        this.setExportLevel(null);
    }

    /**
     *
     * @param obj
     * @param level
     * @return {String}
     */
    writeRecord(obj, level) {

        const grid = this._grid.implementsOrExtends("FlexDataGrid") ? this._grid : null;
        const exportOptions = this._exportOptions;
        let body = "";
		grid.recordBeingExported=obj;
        this.setExportLevel(level);
        body += exportOptions.exporter.writeRecord(this._grid, obj);
        if (level.nextLevel) {
            this.setExportLevel(level.nextLevel);
            const result = level.getChildren(obj, true, false, true);
            if (grid.getLength(result) > 0 && !level.nextLevel.excludeFromExport) {
                if (!grid.currentExportLevel.reusePreviousLevelColumns)
                    body += exportOptions.exporter.writeHeader(grid);

                for (const child of result) {
                    body += this.writeRecord(child, level.nextLevel);
                }

                if (!grid.currentExportLevel.reusePreviousLevelColumns)
                    body += exportOptions.exporter.writeFooter(grid, result);
            }
        }
        return body;

    }

    /**
     *
     * @param level
     */
    setExportLevel(level) {

        const grid = this._grid.implementsOrExtends("FlexDataGrid") ? this._grid : null;
        this.getExportOptions().exporter.nestDepth = level ? level.getNestDepth() - 1 : 0;
        this.getExportOptions().exporter.reusePreviousLevelColumns = level ? level.reusePreviousLevelColumns : false;
        grid.currentExportLevel = level;

    }

    /**
     * The current export options
     * @return {ExportOptions}
     */
    getExportOptions() {
        return this._exportOptions;
    }

    /**
     * The current grid being exported
     * @return {IExtendedDataGrid}
     */
    getGrid() {
        return this._grid;
    }
}

flexiciousNmsp.ExtendedExportController = ExtendedExportController; //add to name space
ExtendedExportController.prototype.typeName = ExtendedExportController.typeName = 'ExtendedExportController';//for quick inspection

/**
 * For HTML/DOC export, we specify this so that the nested
 * export does not look jagged
 */
ExtendedExportController._instance = new ExtendedExportController();