/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import Exporter from './Exporter'
/**
 * Exports the grid in HTML format
 * @constructor
 * @class HtmlExporter
 * @namespace flexiciousNmsp
 */
export default class HtmlExporter extends Exporter {
    constructor() {
        super();
        this.index = 0;
    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["HtmlExporter", "Exporter"];
    }

    /**
     *
     * @param grid
     * @type IExtendedDataGrid
     * @return {String}
     */
    writeHeader(grid) {

        let str = "";
        if (this.getNestDepth() > 0)
            str += `<tr><td colspan='${grid['currentExportLevel'].getParentLevel().getExportableColumns(null, false, this.exportOptions).length + 1}'>`;
        str += `<table ${(this.exportOptions && this.exportOptions.tableWidth) ? "width='" + this.exportOptions.tableWidth + "'" : ""}><tr>${this.buildTh(grid)}</tr>\r\n`;
        if (this.getNestDepth() > 0)
            str = `${str}</td></tr>`;
        return str;

    }

    /**
     *
     * @param grid
     *  @type IExtendedDataGrid
     * @return {String}
     */
    buildTh(grid) {
        let str = "";
        let colIndex = 0;
        if (this.getNestDepth() > 0)
            str += `<th width='${this.getNestDepth() * this.getNestIndent()}'></th>`;
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col)) continue;
            str += `<th>${flexiciousNmsp.Exporter.getColumnHeader(col, colIndex++)}</th>\r\n`;
        }

        return str;
    }

    startDocument(grid) {
        flexiciousNmsp.Exporter.prototype.startDocument.apply(this, [grid]);
        this.index = 0;
    }

    /**
     * Builds the html representation of a header row.
     * @param {IExtendedDataGrid} grid
     * @param record
     * @return {String}
     */
    writeRecord(grid, record) {
        let str = `<tr class='${this.index % 2 == 0 ? 'even' : 'odd'}'>\r\n`;
        if (this.getNestDepth() > 0 && !this.reusePreviousLevelColumns)
            str += `<td width='${this.getNestDepth() * this.getNestIndent()}'></td>`;
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col)) continue;
            str += `<td>${this.getSpaces(col, "&nbsp;")}${this.escapeHtml(col.itemToLabel(record))}</td>\r\n`;
        }

        str += "</tr>\r\n";
        this.index++;
        return str;
    }

    /**
     * Escapes html characters to make valid html string
     * @param val
     * @return {String|XML}
     */
    escapeHtml(val) {

        return val ? val.toString().replace("\"\n", "\"").replace(/"/g, "\"\"") : "";

    }

    /**
     * Writes the footer in HTML format
     * @param grid
     * @param dataProvider
     * @return {String}
     */
    writeFooter(grid, dataProvider) {
        if (this.exportOptions.includeFooters) {
            let str = "<tr>\r\n";
            if (this.getNestIndent() > 0 && !this.reusePreviousLevelColumns)
                str += `<td width='${this.getNestDepth() * this.getNestIndent()}'></td>`;
            const items = grid.getExportableColumns();

            for (const col of items) {
                if (!this.isIncludedInExport(col)) continue;
                str += `<td>${this.getSpaces(col, "&nbsp;")}${this.escapeHtml(col.calculateFooterValue(dataProvider))}</td>\r\n`;
            }

            str += "</tr>\r\n";
            return `${str}</table>\r\n`
        }
        return "</table>\r\n"
    }

    /**
     * Extension of the download file.
     @return {String}
     */

    getExtension() {
        return "html";
    }

    /**
     * Returns the content type so the browser launches
     * when the exporter is run.
     * @return {String}
     */
    getContentType() {
        return "text/html"
    }

    /**
     * Name of the exporter
     * @return {String}
     */
    getName() {
        return "Html"
    }

    uploadForEcho(body, exportOptions) {
        if (this.implementsOrExtends('DocExporter')) {
            body = Constants.WORD_EXPORT_CSS + body;
        } else {
            body = Constants.HTML_EXPORT_CSS + body;
        }
        flexiciousNmsp.Exporter.prototype.uploadForEcho.apply(this, [body, exportOptions]);

    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.HtmlExporter = HtmlExporter; //add to name space

/**
 *
 * @type {String}
 */
HtmlExporter.prototype.typeName = HtmlExporter.typeName = 'HtmlExporter';//for quick inspection