/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import Exporter from './Exporter'
/**
 * Exports the grid in CSV format
 * @constructor
 * @namespace
 */
export default class PdfExporter extends Exporter {
    constructor() {
        super();
        /**
         * object representing the columns
         */
        this.columns = [];

        /**
         * object representing the data
         */
        this.data = [];
    }

    getClassNames() {
        return ["PdfExporter", "Exporter"];
    }

    /**
     * Builds the PDF representation of a header row.
     * @param grid
     * @method writeHeader
     */
    writeHeader(grid) {

        this.buildHeader(grid);
        return "";

    }

    /**
     * @private
     * @param grid
     * @return
     *
     */
    buildHeader(grid) {
        let colIndex = 0;

        //nested tables not supported for now
        //while(colIndex++<this.nestDepth)
        //    this.strTable += "<Cell><Data ss:Type='String'></Data></Cell>";
        for (const col of grid.getExportableColumns()) {
            if (!this.isIncludedInExport(col))
                continue;
            this.columns.push({ title: flexiciousNmsp.Exporter.getColumnHeader(col, colIndex), key: col.getDataField() });
            colIndex++;
        }
    }


    /**
     * Just in case in the client wants to customize the upload
     * behavior of their exporters.
     * @param body
     * @param exportOptions
     * @method uploadForEcho
     */
    uploadForEcho(body, exportOptions) {
        const strWorkbook = "";


        const doc = new jsPDF('l', 'pt');
        doc.autoTable(this.columns, this.data);
        doc.save(`${exportOptions.exportFileName}.${this.getExtension()}`);

        //flexiciousNmsp.Exporter.prototype.uploadForEcho.apply(this,[strWorkbook, this.exportOptions]);
        this.columns = [];
        this.data = [];

        UIUtils.removePopUp(exportOptions.exportOptionsView);
    }

    /**
     * Writes an individual record in PDF format
     * @param grid
     * @param record
     * @return
     *
     */
    writeRecord(grid, record) {
        const colIndex = 0;
        //        this.strTable += "<Row>";
        //        if(!this.reusePreviousLevelColumns){
        //            while(colIndex++<this.nestDepth)
        //                this.strTable += "<Cell><Data ss:Type='String'></Data></Cell>";
        //        }
        const item = {};

        for (const col of grid.getExportableColumns()) {
            if (!this.isIncludedInExport(col))
                continue;
            item[col.getDataField()] = col.itemToLabel(record);
        }

        this.data.push(item);
        //this.strTable += "</Row>";
        return "";
    }

    /**
     * Writes the footer in PDF format
     * @param grid
     * @param dataProvider
     */
    writeFooter(grid, dataProvider) {
        //
        //        if(this.exportOptions.includeFooters){
        //            var colIndex=0;
        //            this.strTable += "<Row ss:StyleID='s68'>";
        //            if(!this.reusePreviousLevelColumns){
        //                while(colIndex++<this.nestDepth)
        //                    this.strTable += "<Cell><Data ss:Type='String'></Data></Cell>";
        //            }
        //            for(var i=0;i<grid.getExportableColumns().length;i++){
        //                var col=grid.getExportableColumns()[i];
        //                if(!this.isIncludedInExport(col))
        //                    continue;
        //                var str = col.calculateFooterValue(dataProvider);
        //                this.strTable += "<Cell><Data ss:Type='String'>" + str + "</Data></Cell>";
        //            }
        //            this.strTable += "</Row>";
        //
        //        }
        return "";

    }

    /**
     * Extension of the download file.
     * @return
     *
     */
    getExtension() {
        return "pdf";
    }

    /**
     * Returns the content type so MS Excel launches
     * when the exporter is run.
     * @return
     *
     */
    getContentType() {
        return "application/pdf"
    }

    /**
     * Name of the exporter
     * @return
     *
     */
    getName() {
        return "PDF";
    }
}

flexiciousNmsp.PdfExporter = PdfExporter; //add to name space
PdfExporter.prototype.typeName = PdfExporter.typeName = 'PdfExporter';//for quick inspection