/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import Exporter from './Exporter'

/**
 * Exports the grid in text format
 * @constructor
 * @class TxtExporter
 * @namespace flexiciousNmsp
 */
export default class TxtExporter extends Exporter {

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["TxtExporter", "Exporter"];
    }

    /**
     *
     * @param grid
     * @return {String}
     */
    writeHeader(grid) {

        return `${this.buildHeader(grid)}\r\n`;

    }

    /**
     * Builds the text representation of a header row.
     * @param grid
     * @return {String}
     */
    buildHeader(grid) {
        let str = "";
        let colIndex = 0;
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col)) continue;
            str += `${flexiciousNmsp.Exporter.getColumnHeader(col, colIndex++)}\t`;
        }

        return str;
    }

    /**
     * Writes an individual record in text format
     * @param grid
     * @param record
     * @return {String}
     */
    writeRecord(grid, record) {
        let str = "";
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col)) continue;
            str += `${col.itemToLabel(record)}\t`;
        }

        if (str.length > 0)
            str = str.substr(0, str.length - 1);
        str += "\r\n";
        return str;
    }

    /**
     * Writes the footer in plain text format
     * @param grid
     * @param dataProvider
     * @return {String}
     */
    writeFooter(grid, dataProvider) {

        if (this.exportOptions.includeFooters) {
            let str = "";
            const items = grid.getExportableColumns();

            for (const col of items) {
                if (!this.isIncludedInExport(col)) continue;
                str += `${col.calculateFooterValue(dataProvider)}\t`;
            }

            if (str.length > 0)
                str = str.substr(0, str.length - 1);
            str += "\r\n";
        }
        return "";

    }

    /**
     * Extension of the download file.
     @return {String}
     */
    getExtension() {
        return "txt";
    }

    /**
     * Returns the content type so the text editor launches
     * when the exporter is run.
     @return {String}
     */
    getContentType() {
        return "text/plain"
    }

    /**
     * Name of the exporter
     * @return {String}
     */
    getName() {
        return "Text"
    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.TxtExporter = TxtExporter; //add to name space
TxtExporter.prototype.typeName = TxtExporter.typeName = 'TxtExporter';//for quick inspection