/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import Exporter from './Exporter'

/**
 * Exports the grid in XML format
 * @constructor
 * @class XmlExporter
 * @namespace flexiciousNmsp
 */
export default class XmlExporter extends Exporter {
    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["XmlExporter", "Exporter"];
    }

    /**
     * 
     * Writes the header of the grid (columns) in Xml format
     * @param grid
     * @return {String}
     */
    writeHeader(grid) {

        let str = "";
        str += "<records>\r\n";
        return str;

    }

    /**
     * Writes an individual record in xml format
     * @param grid
     * @param record
     * @return {String}
     */
    writeRecord(grid, record) {
        let str = "<record\r\n ";
        const items = grid.getExportableColumns();

        for (const col of items) {
            if (!this.isIncludedInExport(col)) continue;
            str += `${col.getDataField().replace(/\./g, "")}='${this.escapeXml(col.itemToLabel(record))}'\r\n`;
        }

        str += "/>\r\n";
        return str;
    }

    /**
     * Escapes XML characters to make valid xml string
     * @param val
     * @return {String|XML}
     */
    escapeXml(val) {
        val += ""; // Subash :  some times val may be a number. that time it prevent from glitch

        return val ? val.replace("\"\n", "\"")
            .replace(/'/g, "&quot;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/&/g, "&amp;")
            : "";

    }

    /**
     * Writes the footer in xml format
     * @param grid
     * @return {String}
     */
    writeFooter(grid) {

        return "</records>\r\n"

    }

    /**
     * Extension of the download file.
     * @return {String}
     */
    getExtension() {
        return "xml";
    }

    /**
     * Returns the content type so the browser launches
     * when the exporter is run.
     * @return {String}
     *
     */
    getContentType() {
        return "text/xml"
    }

    /**
     * Name of the exporter
     * @return {String}
     */
    getName() {
        return "Xml"
    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.XmlExporter = XmlExporter; //add to name space
XmlExporter.prototype.typeName = XmlExporter.typeName = 'XmlExporter';//for quick inspection