/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import UIComponent from '../../core/UIComponent'

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
/**
 * @author Flexicious
 * Attaches to the owner component (which is the bodyContainer) and scrolls vertically along with it.
 * The horizontal scroll of this component is set to off)
 * @constructor
 * @class ElasticContainer
 * @namespace flexiciousNmsp
 */
export default class ElasticContainer extends UIComponent {
    constructor(grid) {
        super({}, "div");
        this.boundContainer = null;

        /**
         * The grid that we belong to 
         * @type {flexiciousNmsp.FlexDataGrid}
         * @property grid
         * @default flexiciousNmsp.FlexDataGrid
         */
        this.grid = grid;
        /**
         *
         * @type {flexiciousNmsp.UIComponent}
         * @property backgroundForFillerRows
         * @default flexiciousNmsp.UIComponent()
         */
        this.backgroundForFillerRows = new flexiciousNmsp.UIComponent("div");

        
        if (this.grid)
            this.backgroundForFillerRows.addEventListener(this, Constants.EVENT_MOUSE_OVER, this.onGridMouseOut);

        if (this.grid.enableFillerRows) {
            this.backgroundForFillerRows.attachClass("backgroundForFillerRows");
            this.addChild(this.backgroundForFillerRows);
            this.setChildIndex(this.backgroundForFillerRows, 0);
        }

    }
    //todo
    componentDidMount() {
        super.componentDidMount();

        if (this.grid.implementsOrExtends("PrintFlexDataGrid")) {
            UIUtils.addDomEventListener(this, this.domElement, "mousewheel", evt => {

                evt.preventDefault();
                evt.stopPropagation();
            });
        }

    }
    /**
     *
     * @method onGridMouseOut
     * @param event
     */
    onGridMouseOut(event) {

        this.grid.onGridMouseOut();

    }

    /**
     *
     * @method getClassNames
     * @return {Array}
     */
    getClassNames() {
        return ["ElasticContainer", "UIComponent"];
    }

    /**
     * @method setBackgroudFillerSize
     */
    setBackgroudFillerSize() {

        let ht = this.getHeight() - this.grid.getBodyContainer().getCalculatedTotalHeight();
        if (ht < 0) ht = 0;
        this.backgroundForFillerRows.setHeight(ht);
        this.backgroundForFillerRows.setWidth(this.getWidth());
        //TODO
        //this.backgroundForFillerRows.domElement.width = this.backgroundForFillerRows.getWidth();
        //this.backgroundForFillerRows.domElement.height = this.backgroundForFillerRows.getHeight();
        if (ht > 0) {
            this.backgroundForFillerRows.move(0, this.grid.getBodyContainer().getCalculatedTotalHeight());
        }
    }

    /**
     * @method getScrollableRect
     * @return {flexiciousNmsp.Rectangle}
     */
    getScrollableRect() {
        const bounds = new flexiciousNmsp.Rectangle();
        bounds.setHeight(this.boundContainer.getScrollableRect().getHeight());
        if (this.boundContainer.getHorizontalScrollBar())
            bounds.setHeight(bounds.getHeight() + this.boundContainer.getHorizontalScrollBar().getHeight());
        return bounds;

    }

    /**
     * @method kill
     */
    kill() {
        flexiciousNmsp.UIComponent.prototype.kill.apply(this);
        this.boundContainer = null;
        this.grid = null;
        this.backgroundForFillerRows = null;
    }
}


/**
 *
 * @type {Function}
 */
flexiciousNmsp.ElasticContainer = ElasticContainer; //add to name space
ElasticContainer.prototype.typeName = ElasticContainer.typeName = 'ElasticContainer';//for quick inspection