/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import UIComponent from '../../core/UIComponent'
import FactoryKey from './cache/FactoryKey'

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 

import FlexDataGridContainerBase from './FlexDataGridContainerBase'
import FlexDataGridEvent from '../events/FlexDataGridEvent'
import ItemLoadInfo from '../valueobjects/ItemLoadInfo'
/**
 * FlexDataGridBodyContainer is the container for the unlocked section of the datagrid. This is the only container in the entire
 * component tree that scrolls.
 * In addition to all the base functionality that it inherits from FlexDataGridContainerBase, it adds the following pieces of functionality:
 *
 * <ul>
 * <li>Vertical Scroll Renderer Recycling</li>
 * </ul>
 *
 * @inheritDoc
 * @constructor
 * @class FlexDataGridBodyContainer
 * @namespace flexiciousNmsp
 */
export default class FlexDataGridBodyContainer extends FlexDataGridContainerBase {
    constructor(grid) {
        super(grid);
        ///added for sticky headers
        /**
         * When you have sticky headers, this will be the one floating row.
         */
        this.floatingHeaderRowInfo = null;

        /**
         *
         * @type {FlexDataGridContainerBase}
         * @property grid
         * @default grid
         */
        this.grid = grid;
        /**
         *
         * @type {Array}
         * @property matchSticks
         * @default []
         */
        this.matchSticks = [];
        /**
         * A list of items that the user has opened. Only applicable grids with nested column levels.
         * @type {Array}
         * @property openItems
         * @default []
         */
        this.openItems = [];
        /**
         *
         * @type {flexiciousNmsp.UIComponent}
         * @property backgroundForFillerRows
         * @default flexiciousNmsp.UIComponent()
         */
        this.backgroundForFillerRows = new flexiciousNmsp.UIComponent("div");
        if (this.grid)
            this.backgroundForFillerRows.addEventListener(this, Constants.EVENT_MOUSE_OVER, this.gridMouseOut);
        /**
         *
         * @type {null}
         * @private
         * @property _calculatedTotalHeight
         * @default null
         */

        this._calculatedTotalHeight = 0;
        /**
         *
         * @type {Boolean}
         * @private
         * @property _heightCalculated
         * @default false
         */

        this._heightCalculated = false;
        /**
         *
         * @type {Boolean}
         * @private
         * @property _processing
         * @default false
         */
        this._processing = false;
        /**
         *
         * @type {Boolean}
         * @private
         * @property _drawDirty
         * @default false
         */
        this._drawDirty = false;
        /**
         *
         * @type {Boolean}
         * @private
         * @property _recreateRows
         * @default false
         */
        this._recreateRows = false;
        /**
         *
         * @type {Object}
         * @private
         * @property _rowCache
         * @default {}
         */
        this._rowCache = {};
        /**
         *
         * @type {null}
         * @property currentRowPointer
         * @default null
         */
        this.currentRowPointer = null;
        /**
         *
         * @type {Array}
         * @private
         * @property _visibleRange
         * @default []
         */
        this._visibleRange = [];
        /**
         *
         * @type {Array}
         * @private
         * @property _visibleRangeH
         * @default []
         */
        this._visibleRangeH = [];
        /**
         *
         * @type {Array}
         * @property fillerRows
         * @default []
         */
        this.fillerRows = [];
        /**
         *
         * @type {Boolean}
         * @property onDisabledCell
         * @default false
         */

        this.onDisabledCell = false;
        /**
         *
         * @type {Number}
         * @private
         * @property _verticalScrollPending
         * @default -1
         */
        this._verticalScrollPending = -1;
        /**
         *
         * @type {Object}
         * @property parentMap
         * @default {}
         */
        this.parentMap = {};

        /**
         *
         * @type {Number}
         * @property horizontalMask
         * @default 0
         */
        this.horizontalMask = 0;

        /**
         *
         * @type {Number}
         * @property verticalMask
         * @default 0
         */
        this.verticalMask = 0;
        /**
         * Active only when the body containers calculateRowHeight method is running.
         * This dictionary will be purged as soon as calculateRowHeight is done running.
         * @type {null}
         * @property variableRowHeightRenderers
         * @default null
         */
        this.variableRowHeightRenderers = null;

        if (this.grid.enableFillerRows) {
            this.backgroundForFillerRows.attachClass("backgroundForFillerRows");
            this.addChild(this.backgroundForFillerRows);
            this.setChildIndex(this.backgroundForFillerRows, 0);
        }
    }

    /**
     * @method getClassNames
     * @return Array
     */ 
    getClassNames() {
        return ["FlexDataGridBodyContainer", "FlexDataGridContainerBase", "Container", "UIComponent"];
    }

    /**
     * @method updateDisplayList
     * @param unscaledWidth
     * @param unscaledHeight
     */
    updateDisplayList(unscaledWidth, unscaledHeight) {

        this.grid.traceEvent(`begin section list${this._drawDirty}${this._verticalScrollPending}`);
        if (this._verticalScrollPending != -1) {
            this.gotoVerticalPosition(this._verticalScrollPending);
            this._verticalScrollPending = -1;
        }
        if (this._drawDirty) {
            this.pauseState();
            this._drawDirty = false;
            this.drawRows(false);
            this._recreateRows = false;
            this.grid.synchronizeLockedVerticalScroll();
            this.grid.synchronizeHorizontalScroll();
            this.setStyleAttribute("display", "none");
            const v = this.getHeight();
            const v1 = this.getWidth();
            this.setStyleAttribute("display", "block");
            this.resumeState();
        }
        flexiciousNmsp.FlexDataGridContainerBase.prototype.updateDisplayList.apply(this, [unscaledWidth, unscaledHeight]);
        this.grid.traceEvent("end section list")

    }

    /**
     * handle getScrollableRect
     * @method getScrollableRect
     * @return {flexiciousNmsp.Rectangle}
     * @method getScrollableRect
     */
    getScrollableRect() {

        const bounds = new flexiciousNmsp.Rectangle();
        bounds.setHeight(this.calculateTotalHeight());
        bounds.setWidth(this.grid.getColumnLevel().getSumOfColumnWidths([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_NONE]) - 1);
        return bounds;

    }

    /**
     * handle filterchange
     * @method onFilterChange
     * @param event
     */
    onFilterChange(event) {

        flexiciousNmsp.FlexDataGridContainerBase.prototype.onFilterChange.apply(this, [event]);
        const trigger = event.triggerEvent.currentTarget;
        const treeCell = trigger.parent;
        const rowInfo = treeCell.rowInfo;
        if (!this.grid.itemFilters.getValue(rowInfo.getData() == null ? Constants.TOP_LEVEL_FILTER : rowInfo.getData())) {
            this.grid.itemFilters.addItem(rowInfo.getData() == null ? Constants.TOP_LEVEL_FILTER : rowInfo.getData(), new flexiciousNmsp.AdvancedFilter());
        }
        const col = treeCell.getColumn();
        const odata = treeCell.rowInfo.getData();
        const lvl = treeCell.getLevel();


        this.grid.itemFilters.getValue(rowInfo.getData() == null ? Constants.TOP_LEVEL_FILTER : rowInfo.getData()).pageIndex = 0;
        this.expandCollapse(rowInfo.getData());
        this.expandCollapse(rowInfo.getData());

        if (lvl.nestDepth > 1) {
            for (const row of this.rows) {
                if (row.getIsFillRow() && row.getData() == odata) {
                    const cell = row.getCellForColumn(col);
                    const fm = ((cell.component).getRenderer());
                    if (fm) {
                        fm.focus();
                    }
                    this.grid.currentCell = (cell.component);
                    this.grid.invalidateCells();
                    break;
                }
            }
        }
    }

    /**
     *
     * @method onPageChange
     * @param event
     */
    onPageChange(event) {

        flexiciousNmsp.FlexDataGridContainerBase.prototype.onPageChange.apply(this, [event]);
        if ((event.target).level.getIsClientFilterPageSortMode()) {
            const pager = event.target;
            if (!this.grid.itemFilters.getValue(pager.rowInfo.getData() == null ? Constants.TOP_LEVEL_FILTER : pager.rowInfo.getData())) {
                this.grid.itemFilters.addItem(pager.rowInfo.getData() == null ? Constants.TOP_LEVEL_FILTER : pager.rowInfo.getData(), new flexiciousNmsp.AdvancedFilter());
            }
            this.grid.itemFilters.getValue(pager.rowInfo.getData() == null ? Constants.TOP_LEVEL_FILTER : pager.rowInfo.getData()).pageIndex = pager.getPageIndex();
            const item = pager.rowInfo.getData();
            this.expandCollapse(item);
            this.expandCollapse(item);
        }

    }

    /**
     * @method getRowPositionInfo
     * @param {Object} item
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} lvl
     * @return {*}
     */
    getRowPositionInfo(item, lvl) {
        if (typeof lvl == "undefined") lvl = null;

        const items = this.itemVerticalPositions;

        for (const dataitem of items) {
            if (lvl != null) {
                if (lvl.areItemsEqual(item, dataitem.rowData)) {
                    return dataitem;
                }
            }
            else if (dataitem.rowData == item)
                return dataitem;
        }

        return null;
    }

    /**
     * @method getRowPositionInfoFromRows
     * @param {Object} item
     * @return {*}
     */
    getRowPositionInfoFromRows(item) {
        const items = this.rows;

        for (const row of items) {
            if (row.getData() == item)
                return row.rowPositionInfo;
        }

        return null;
    }

    /**
     * On basis of the vertical scroll position provided, does a quick binary lookup of the record
     * to be shown at that row.
     * 
     * @method binarySearch
     * @param {Array} arr 
     * @param {Number} verticalPos
     * @param {Number} low
     * @param {Number} high
     * @return {flexiciousNmsp.RowPositionInfo} 
     * 
     */
    binarySearch(arr, verticalPos, low, high) {

        if (high < low)
            return null;// not found
        const mid = parseInt((low + high) / 2);
        if (arr.length == 0)
            return null;
        if (arr.length > 0 && arr[arr.length - 1].getVerticalPosition() < verticalPos)
            return arr[arr.length - 1];
        else if (((verticalPos - arr[mid].getVerticalPosition()) >= 0) && ((verticalPos - arr[mid].getVerticalPosition()) <= arr[mid].getRowHeight()))
            return arr[mid];
        else if (arr[mid].getVerticalPosition() > verticalPos)
            return this.binarySearch(arr, verticalPos, low, mid - 1);
        else if (arr[mid].getVerticalPosition() < verticalPos)
            return this.binarySearch(arr, verticalPos, mid + 1, high);
        else
            return null;

    }

    /**
     * Gets a copy of the calculated item vertical positions
     * @method getItemVerticalPositions
     * @return {Array}
     */
    getItemVerticalPositions() {

        return this.itemVerticalPositions.slice();

    }

    /**
     * Returns the RowPosition Info object for the item at the given vertical scroll position
     * 
     * @param {Number} position
     * @return {flexiciousNmsp.RowPositionInfo}
     */
    getItemAtPosition(position) {

        return this.binarySearch(this.itemVerticalPositions, position, 0, this.itemVerticalPositions.length);

    }

    /**
     * Sets the records in the visible range.
     * @method setVisibleRange
     */
    setVisibleRange() {


        const currrentPosition = this.getVerticalScrollPosition();
        const upperBound = Math.max(currrentPosition - this.grid.getColumnLevel().getRowHeight(), 0);
        const lowerBound = Math.min(currrentPosition + this.getHeight() + this.grid.getColumnLevel().getRowHeight(), this.calculateTotalHeight());
        this._visibleRange = [this.getItemAtPosition(upperBound), this.getItemAtPosition(lowerBound)];
        this._visibleRangeH = [this.getHorizontalScrollPosition() - (this.horizontalMask) > 0 ? this.getHorizontalScrollPosition() - (this.horizontalMask) : 0
            , this.getHorizontalScrollPosition() + (this.getWidth() + this.horizontalMask)];

    }

    /**
     * @method createComponents
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Number} currentScroll
     * 
     */
    createComponents(level, currentScroll) {
        if (typeof currentScroll == "undefined") currentScroll = 0;

        if (!this._heightCalculated) this.calculateTotalHeight();
        if (this.getVerticalScrollPosition() > this._calculatedTotalHeight) {
            this.setVerticalScrollPosition(0);
        }


        flexiciousNmsp.FlexDataGridContainerBase.prototype.createComponents.apply(this, [level, currentScroll]);

        if (this.currentRowPointer && this.currentRowPointer.rowData) {
            this.grid.currentPoint.contentY = this.currentRowPointer.getVerticalPosition();
        }
        this._drawDirty = true;
        this._recreateRows = true;
        this.doInvalidate();
        //drawRows();

    }

    /**
     * @method getRowsForRecycling
     * @return {Boolean}
     */
    getRowsForRecycling() {
        return this.getAllRows();
    }

    /**
     * @method getRowsForSnapping
     * @return {*}
     */
    getRowsForSnapping() {

        const rowsForSnapping = this.getAllRows();
        for (const key in this._rowCache) {
            rowsForSnapping.concat((this._rowCache[key]));
        }
        return rowsForSnapping;

    }

    /**
     * @method recycleH
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Boolean} scrollRight
     */
    recycleH(level, scrollRight) {
        if (typeof scrollRight == "undefined") scrollRight = true;

        this.setVisibleRange();
        this.endEdit(this.getEditor());
        if(this.grid.enableHorizontalScrollOptimizationsBETA){
            this.childAddedOrRemoved=false;
            this.pauseState();
        }
        flexiciousNmsp.FlexDataGridContainerBase.prototype.recycleH.apply(this, [level, scrollRight]);

        if (this.fillerRows.length > 0) {
            this.resizeFillerCells();
            this.fillerRows[0] = this.processItem(this.fillerRows[0].rowPositionInfo, true, this.fillerRows[0]);
        }
        if (this.grid.enableHorizontalScrollOptimizationsBETA) {
            if (this.childAddedOrRemoved) {
                this.resumeState();
            } else {
                this.statePaused = false;
                this.grid.getLeftLockedContent().statePaused = false;
                this.grid.getRightLockedContent().statePaused = false;
            }
        }
    }

    /**
     * @method getRowsForRecycling
     * @return {Object} returns new row
     */
    getRowsForRecycling() {
        const newRows = (this.rows.slice());
        const items = this.fillerRows;

        for (const fillerRow of items) {
            newRows.push(fillerRow);
        }

        return newRows;
    }

    /**
     * @method recycle
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Boolean} scrollDown
     * @param {Number} scrollDelta
     * @param {Boolean} scrolling
     * 
     */
    recycle(level, scrollDown, scrollDelta, scrolling) {
        if (typeof scrollDelta == "undefined") scrollDelta = 0;
        if (typeof scrolling == "undefined") scrolling = true;

        if ((this.getVerticalScrollPosition() + scrollDelta) > this.getMaxVerticalScrollPosition()) {
            this.setVerticalScrollPosition(this.getMaxVerticalScrollPosition());
        }
        if ((this.getVerticalScrollPosition() + scrollDelta) < 0) {
            this.setVerticalScrollPosition(0);
        }
        this.setVisibleRange();
        let row;
        let i;
        const offPage = scrollDown ? (scrollDelta) >= this.getHeight() : (0 - scrollDelta) >= this.getHeight();

        if (this.rows.length == 0 && !this.grid.recalculateSeedOnEachScroll) return;
        const toRemove = [];
        for (i = 0; i < this.rows.length; i++) {
            row = this.rows[i];
            if (offPage || ((row.getY() < this._visibleRange[0].getVerticalPosition())) || (row.getY() > this._visibleRange[1].getVerticalPosition())) {
                if (this.grid.getHasRowSpanOrColSpan() && !row.getIsFillRow()) {
                    const maxCellHt = row.getMaxCellHeight();
                    if ((maxCellHt > row.getHeight()) && ((maxCellHt + row.rowPositionInfo.getVerticalPosition()) > this.getVerticalScrollPosition())) {
                        //this means we'd still be in view
                        scrolling = false;//cannot use scrolling optimization anymore.
                        continue;
                    }
                }
                toRemove.push(row);
            }
        }
        for (let j = 0; j < toRemove.length; j++) {
            row = toRemove[j];
            this.rows.splice(this.rows.indexOf(row), 1);
            this.saveRowInCache(row);
        }

        let start = this._visibleRange[0];
        let end = this._visibleRange[1];
        if(start && end && ((start.getRowHeight()> this.getHeight()) || (end.getRowHeight()> this.getHeight())) ){

        }
        else if (scrolling) {
            if (!scrollDown) {
                if (start == null) return;
                if (!offPage && this.rows.length > 0 && this.rows[0].rowPositionInfo.getRowIndex() >= start.getRowIndex()) {
                    end = this.rows[0].rowPositionInfo;
                }
                if (this.grid.recalculateSeedOnEachScroll) {

                }
                else if (end && (end.getRowIndex() > 0))
                    end = this.itemVerticalPositions[end.getRowIndex() - 1];
            }
            else {
                if (end == null) return;
                if (!offPage && this.rows.length > 0 && this.rows[this.rows.length - 1].rowPositionInfo.getRowIndex() <= end.getRowIndex()) {
                    start = this.rows[this.rows.length - 1].rowPositionInfo;
                }
                if (start == null) return;
                if (this.itemVerticalPositions.length > (start.getRowIndex() + 1)) {
                    if (this.grid.recalculateSeedOnEachScroll && start.getRowIndex() == end.getRowIndex()) {

                    }
                    else
                        start = this.itemVerticalPositions[start.getRowIndex() + 1];
                }
            }
        }
        if (start == null || end == null) return;
        
        if(this.grid.enableHorizontalScrollOptimizationsBETA){
            this.childAddedOrRemoved=false;
            this.pauseState();
        }

        this.drawRowsUsingCache(start, end, scrollDown, offPage);
        if (this.grid.enableHorizontalScrollOptimizationsBETA) {
            if (this.childAddedOrRemoved) {
                this.resumeState();
            } else {
                this.statePaused = false;
                this.grid.getLeftLockedContent().statePaused = false;
                this.grid.getRightLockedContent().statePaused = false;
            }
        }
        //checkInconsistency()
    }

    /**
     * @method drawRowsUsingCache
     * @param {flexiciousNmsp.RowPositionInfo} start
     * @param {flexiciousNmsp.RowPositionInfo} end
     * @param {Boolean} scrollDown
     * @param {Boolean} offPage
     * 
     */
    drawRowsUsingCache(start, end, scrollDown, offPage) {

        let i;
        if (this.grid.getHasRowSpanOrColSpan()) {
            if (this.getVerticalScrollPosition() > 0 && !scrollDown) {
                const items = this.itemVerticalPositions;
                for (i = 0; i < items.length; i++) {
                    const rowPos = items[i];
                    if (rowPos.getRowIndex() > start.getRowIndex())
                        break;
                    if (rowPos.rowSpan > 1) {
                        if ((rowPos.rowSpan + rowPos.getRowIndex()) >= start.getRowIndex()) {
                            //this.implementsOrExtends('a undrawn row, th')){
                            if (!this.rowExists(rowPos)) {
                                this.processItemPositionInfoUsingCache(rowPos, 0, true);
                            }
                        }
                    }
                }
            }
        }

        //trace("drawing " + start.getRowIndex() + " to " + end.getRowIndex())
        for (let rowIndex = start.getRowIndex(); rowIndex <= end.getRowIndex(); rowIndex++) {
            const seed = this.itemVerticalPositions[rowIndex];

            if (!offPage) {
                let exists = false;
                for (i = 0; i < this.rows.length; i++) {
                    const row = this.rows[i];
                    if (row.rowPositionInfo == seed) {
                        exists = true;
                        break;
                    }
                }
                if (exists)
                    continue;
            }
            let insertionPoint = this.rows.length;
            if (!scrollDown)
                insertionPoint = rowIndex - start.getRowIndex();
            if (seed)
                this.processItemPositionInfoUsingCache(seed, insertionPoint, scrollDown);

        }
        if (this.grid.getHasRowSpanOrColSpan()) {
            this.snapToColumnWidths();
        }
        this.checkStickyHeader();
    }

    /**
     * @method processItemPositionInfoUsingCache
     * @param {flexiciousNmsp.RowPositionInfo} seed
     * @param {Number} insertionPoint
     * @param {Boolean} scrollDown
     * 
     */
    processItemPositionInfoUsingCache(seed, insertionPoint, scrollDown) {
        if (typeof scrollDown == "undefined") scrollDown = true;

        let row, newRow;
        let found = false;
        const index = 0;

        const seedKey = `${seed.getLevelNestDepth()}${seed.getRowType()}`;

        if (this._rowCache[seedKey] && this._rowCache[seedKey].length > 0) {
            row = this._rowCache[seedKey].pop();
            found = true;
        }
        if (found) {
            row.showHide(true);
            row.invalidateCells();
            row.setRowPositionInfo(seed, this.grid.variableRowHeight);
            this.rows.splice(insertionPoint, 0, row);
            if (this._recreateRows) {
                const existing = row.cells;
                row.cells = [];
                newRow = this.processRowPositionInfo(seed, scrollDown, row, existing);
                //newRow.forceUpdate()
                while (existing.length > 0) {
                    this.removeComponent(existing[0]);
                    existing.splice(0, 1);
                }
            }
            this.grid.placeComponents(row);
        }
        else {
            this.grid.currentPoint.contentY = seed.getVerticalPosition();
            newRow = this.processRowPositionInfo(seed, scrollDown);
        }

    }

    /**
     * @method checkInconsistency
     */
    checkInconsistency() {

        let diff = this.rows[0].getY();
        let index = 0;
        for (let i = 0; i < this.rows.length; i++)
            var row = this.rows[i];
        {
            if ((row.getY() == diff && index > 0)) {
                //row.setY(row.getY())
            }
            diff = row.getY();
            index++;
        }

    }

    /**
     * @method positionExists
     * @param {flexiciousNmsp.RowPositionInfo} seed
     * @return {Boolean}
     */
    positionExists(seed) {
        for (const row of this.rows) {
            if (row.rowPositionInfo == seed) return true;
        }

        return false;
    }

    /**
     * @method keyDownHandler
     * @param event
     */
    keyDownHandler(event) {



    }

    /**
     * 
     */
    validateDisplayList() {

        //        var vs=this.getVerticalScrollPosition();
        //        var mVs=this.getMaxVerticalScrollPosition();
        //        var hs=this.getHorizontalScrollPosition();
        //        var mHs=this.getMaxHorizontalScrollPosition();
        //        flexiciousNmsp.FlexDataGridContainerBase.prototype.validateDisplayList.apply(this,[]);
        //        var newVs=this.getVerticalScrollPosition();
        //        var newMvs=this.getMaxVerticalScrollPosition();
        //        var newHs=this.getHorizontalScrollPosition();
        //        var newMhs=this.getMaxHorizontalScrollPosition();
        //
        //        if((vs>newVs)&&(mVs>newMvs)){
        //            //this means we expanded and we were at the bottom before...
        //            this.recycle(this.grid.getColumnLevel(),false);
        //
        //        }
        //        if((hs>newHs)&&(mHs>newMhs)){
        //            //this means we expanded and we were at the bottom before...
        //            this.grid.recycleH(false);
        //        }

        this.grid.placeBottomBar(); //after we are done setting up scrollbars, need to show or hide the graphics box to prevent whitespace from showing through.

    }

    /**
     * @method rowExists
     * @param {flexiciousNmsp.RowPositionInfo} rowPos
     * @return {Boolean}
     */
    rowExists(rowPos) {
        for (const row of this.rows) {
            if (row.rowPositionInfo == rowPos)
                return true;
        }

        return false;
    }
    render() {
        return super.render();
    }

    /**
     * @method removeMatchSticks
     */
    removeMatchSticks() {
        while (this.matchSticks.length > 0) {
            //if(this.matchSticks[0].domElement.parentNode)
            const mt = this.matchSticks[this.matchSticks.length - 1];
            if(mt.parent){
                mt.parent.vMatch = null;
                mt.parent.hMatch = null;
                mt.parent.removeChild(mt);    
            }
            this.matchSticks.pop();
        }
    }

    /**
     * @method removeAllComponents
     * @param {Boolean} recycle
     */
    removeAllComponents(recycle) {
        //        this.removeMatchSticks();

        this.pauseState();
        if (recycle) {
            if (this.floatingHeaderRowInfo) {
                this.hideFloatingRow();
            }
        }
        else {
            if (this.floatingHeaderRowInfo) {
                this.hideFloatingRow();
                while (this.floatingHeaderRowInfo.cells.length > 0) {
                    this.removeComponent(this.floatingHeaderRowInfo.cells[0]);
                }
            }
            this.floatingHeaderRowInfo = null;
        }


        if (this._inEdit) this.endEdit(this.getEditor());
        if (!recycle) {
            flexiciousNmsp.FlexDataGridContainerBase.prototype.removeAllComponents.apply(this, [recycle]);
            if (this.grid.enableFillerRows) {
                this.grid.invalidateFiller();
            }
            while (this.fillerRows.length > 0) {
                const fillerRow = this.fillerRows.pop();
                this.removeComponents(fillerRow);
            }
            this._rowCache = {};
        }
        else {
            for (const row of this.rows) {
                this.saveRowInCache(row);
            }

            this.rows = [];
        }

        this.resumeState();

    }
    pauseState(){
        super.pauseState();
        this.grid.getLeftLockedContent().pauseState();
        this.grid.getRightLockedContent().pauseState();
    }
    resumeState(){
        super.resumeState();
        this.grid.getLeftLockedContent().resumeState();
        this.grid.getRightLockedContent().resumeState();
    }
    /**
     * @method saveRowInCache
     * @param {flexiciousNmsp.RowInfo} row
     */
    saveRowInCache(row) {
        row.showHide(false);
        const key = `${row.rowPositionInfo.getLevelNestDepth()}${row.rowPositionInfo.getRowType()}`;
        if (!this._rowCache[key]) {
            this._rowCache[key] = [];
        }
        //trace(key + "" + _rowCache[key].length)
        if (!this._rowCache[key].includes(row))
            this._rowCache[key].push(row);

        for (const comp of row.cells) {
            const headerCell = comp.component.implementsOrExtends("FlexDataGridHeaderCell") ? comp.component : null;
            if (headerCell) {
                headerCell.destroySortIcon();
            }
        }
    }

    /**
     * Inserts the column specified column before the specified column
     * 
     * @method shiftColumns
     * @param {flexiciousNmsp.FlexDataGridColumn} columnToInsert
     * @param {flexiciousNmsp.FlexDataGridColumn} insertBefore
     * 
     */
    shiftColumns(columnToInsert, insertBefore) {
        const allRows = this.getAllRows();

        for (const row of allRows) {
            const cells = row.cells.slice();
            const arrayCollection = (cells);
            const cellToInsert = row.getCellForColumn(columnToInsert);
            const cellInsertBefore = row.getCellForColumn(insertBefore);
            if (cellToInsert && cellInsertBefore) {
                if (arrayCollection.includes(cellToInsert))
                    arrayCollection.splice(arrayCollection.indexOf(cellToInsert), 1);
                if (arrayCollection.includes(cellInsertBefore))
                    arrayCollection.splice(arrayCollection.indexOf(cellInsertBefore), 0, cellToInsert);
            }
            row.cells = arrayCollection;
        }
    }

    /**
     * @method drawRows
     * @param {Boolean} forceFiller 
    */
    drawRows(forceFiller) {
        if (typeof forceFiller == "undefined") forceFiller = false;

        this.addMatchSticks();
        this.setVisibleRange();
        const start = this._visibleRange[0];
        const end = this._visibleRange[1];

        if (this._visibleRange.length == 0 || this._visibleRange[0] == null)
            this.grid.currentPoint.contentY = 0;
        else
            this.grid.currentPoint.contentY = this._visibleRange[0].getVerticalPosition();
        this.grid.currentPoint.contentX = this.grid.currentPoint.leftLockedContentX = this.grid.currentPoint.rightLockedContentX = 0;
        if (start != null && end != null) {
            this.drawRowsUsingCache(start, end, true, false);
        }
        if ((this._calculatedTotalHeight < this.getHeight()) || forceFiller) {
            this.drawFiller(end);
        }
        return true;


    }

    /**
     * @method drawFiller
     * @param {flexiciousNmsp.RowPositionInfo} end
     */
    drawFiller(end) {
        //this means we need to fill...
        if (this.grid.enableFillerRows) {
            this.grid.invalidateFiller();
            return;
        }

        const filler = this.fillerRows.length > 0 ? this.fillerRows[0].rowPositionInfo :
            new flexiciousNmsp.RowPositionInfo(new Object(), end ? end.getRowIndex() + 1 : 1, 0, this.getHeight(),
                this.grid.getColumnLevel(), flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILL);
        if (this.fillerRows.length == 0)
            this.fillerRows.push(null);
        this.fillerRows[0] = this.processItem(filler, false, this.fillerRows[0]);
        const fillerRow = this.fillerRows[0];
        const items = fillerRow.cells;

        for (const item of items) {
            if (item.component)
                item.component.setStyleAttribute("zIndex", -1);
        }
    }

    /**
     * @method adjustFiller
     * @param {Number} offset
     */
    adjustFiller(offset) {

        if (this.grid.enableFillerRows) {
            this.grid.invalidateFiller();
            return;
        }
        if (this.fillerRows.length > 0) {
            const fillerRow = this.fillerRows[0];
            const items = fillerRow.cells;

            for (const item of items) {
                if (item.component)
                    item.component.setHeight(item.component.getHeight() + offset);
            }
        }

    }

    /**
     * @method setBackgroudFillerSize
     */
    setBackgroudFillerSize() {

        let ht = this.getHeight();
        ht -= this.grid.isVScrollBarVisible;
        ht -= this._calculatedTotalHeight;
        if (ht < 0) ht = 1;
        this.backgroundForFillerRows.setHeight(ht - this.grid.isHScrollBarVisible);
        this.backgroundForFillerRows.setWidth(this.getWidth() - (this.grid.isVScrollBarVisible));

        if (ht > 0) {
            const w = this.grid.getColumnLevel().getSumOfColumnWidths([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_NONE]);
            this.backgroundForFillerRows.move(Math.min(this.getHorizontalScrollPosition(), w - this.getWidth()), this.grid.getBodyContainer().getCalculatedTotalHeight());

        }
        if (ht < 2) {
            this.backgroundForFillerRows.setStyleAttribute("display", "none");
        } else {
            this.backgroundForFillerRows.setStyleAttribute("display", "");
        }
    }

    /**
     * @method onWidthChanged
     * @param event
     */
    onWidthChanged(event) {

        if ((this._calculatedTotalHeight < this.getHeight())) {
            if (this.grid.enableFillerRows) {
                this.grid.invalidateFiller();
            }
        }
    }

    /**
     * @method onHeightChanged
     * @param event
     */
    onHeightChanged(event) {

        if (this.grid.enableFillerRows) {
            this.grid.invalidateFiller();
            return;
        }
        if ((this._calculatedTotalHeight < this.getHeight())) {
            if (this.fillerRows.length == 0)
                this.drawFiller(this._visibleRange.length ? this._visibleRange[1] : null);
        }
        this.resizeFillerCells();

    }

    /**
     * @method resizeFillerCells
     */
    resizeFillerCells() {

        if (this.grid.enableFillerRows) {
            this.grid.invalidateFiller();
            return;
        }
        if (this.fillerRows.length > 0) {
            const fillerRow = this.fillerRows[0];
            const items = fillerRow.cells;

            for (const item of items) {
                if (item.component) {
                    item.component.setHeight(this.getHeight() - this.grid.isHScrollBarVisible);
                }
            }
        }

    }

    /**
     * @method  processRowPositionInfo
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     * @param {Boolean} scrollDown
     * @param {flexiciousNmsp.RowInfo} existingRow
     * @param {Array} existingComponents
     * @return {flexiciousNmsp.RowInfo}
     * 
     */
    processRowPositionInfo(rowPositionInfo, scrollDown, existingRow, existingComponents) {
        if (typeof scrollDown == "undefined") scrollDown = true;
        if (typeof existingRow == "undefined") existingRow = null;
        if (typeof existingComponents == "undefined") existingComponents = null;

        if (rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA) {
            return this.processItem(rowPositionInfo, scrollDown, existingRow, existingComponents);
        }
        else {
            return flexiciousNmsp.FlexDataGridContainerBase.prototype.processRowPositionInfo.apply(this, [rowPositionInfo, scrollDown, existingRow, existingComponents]);
        }

    }

    /**
     * @method addEventListeners
     * @param {flexiciousNmsp.ComponentInfo} comp
     */
    addEventListeners(comp) {

        if (!comp || comp.row.getIsFillRow()) return;

        flexiciousNmsp.FlexDataGridContainerBase.prototype.addEventListeners.apply(this, [comp]);
        const dataCell = comp.component;
        if (dataCell) {

            if (this.grid.enableDrag) {
                dataCell.addEventListener(this, Constants.EVENT_DRAG_COMPLETE, this.onCellDragComplete);
                dataCell.addEventListener(this, Constants.EVENT_MOUSE_MOVE, this.onCellDragMouseMove);
                dataCell.addEventListener(this, Constants.EVENT_MOUSE_DOWN, this.onCellDragMouseDown);
                dataCell.addEventListener(this, Constants.EVENT_MOUSE_UP, this.onCellDragMouseUp);
            }
            if (this.grid.enableDrop) {
                dataCell.addEventListener(this, Constants.EVENT_DRAG_ENTER, this.onDragEnter);
                dataCell.addEventListener(this, Constants.EVENT_MOUSE_UP, this.onDragDrop);
                if (!this.grid.enableDrag)
                    dataCell.addEventListener(this, Constants.EVENT_MOUSE_MOVE, this.onCellDragMouseMove);
            }
        }
        if (dataCell.getIsExpandCollapseCell()) {
            dataCell.addEventListener(this, flexiciousNmsp.FlexDataGridExpandCollapseCell.EVENT_EXPAND_COLLAPSE, this.onExpandCollapse);
        }

    }

    /**
     * @method onCellDragComplete
     * @param event
     */
    onCellDragComplete(event) {

        this.grid.dragComplete(event.target)

    }

    /**
     * @method onCellDragMouseMove
     * @param event
     */
    onCellDragMouseMove(event) {
        if (!this.grid.allowInteractivity) return;
        if (this.grid.getHeaderContainer().columnDraggingDragCell || this.grid.getHeaderContainer().columnResizingCell) return;

        if (this.grid.enableDrop && flexiciousNmsp.DisplayList.isMouseDown) {
            this.grid.dragAcceptReject(event.currentTarget);
        }

        if (this.onDisabledCell) return;
        if (!this.isMouseDown) return;
        if (!flexiciousNmsp.DisplayList.isMouseDown) return;
        const cell = event.currentTarget;
        if (cell && cell.implementsOrExtends('IFlexDataGridDataCell')) {
            const that = this;
            if (UIUtils.isWebKit() && !flexiciousNmsp.FlexDataGrid.currentDragCell) {
                setTimeout(() => {
                    if (that.isMouseDown) {
                        if (!that.grid._isScrolling) {
                            that.grid.dragBegin(event)

                        }
                    }
                }, 300);
            }
            else {
                if (!that.grid._isScrolling) {
                    that.grid.dragBegin(event);
                }
            }
        }
    }

    /**
     * @method onCellDragMouseUp
     * @param event
     */
    onCellDragMouseUp(event) {
        this.isMouseDown = 0;
    }

    /**
     * @method onCellDragMouseDown
     * @param event
     */
    onCellDragMouseDown(event) {

        if (!this.grid.allowInteractivity) return;
        if (this.grid.getHeaderContainer().columnDraggingDragCell || this.grid.getHeaderContainer().columnResizingCell) return;
        const cell = event.currentTarget;
        this.onDisabledCell = cell && !cell.getEnabled();
        this.isMouseDown = 1;
    }

    /**
     * @method onDragEnter
     * @param event
     */
    onDragEnter(event) {

        this.grid.dragEnterInternal(event)

    }

    /**
     * @method onDragDrop
     * @param event
     */
    onDragDrop(event) {
        if (flexiciousNmsp.FlexDataGrid.currentDragCell && this.grid.enableDrop) {
            flexiciousNmsp.FlexDataGrid.currentDropCell = event.currentTarget;
            if (this.grid.dropIndicator.getVisible()) {
                this.grid.dragDropInternal(flexiciousNmsp.FlexDataGrid.currentDropCell);
            }
        }
    }

    /**
     * @method onCellDropMouseMove
     * @param event
     */
    onCellDropMouseMove(event) {

        if (!this.grid.allowInteractivity) return;
        if (this.grid.enableDrop && flexiciousNmsp.FlexDataGrid.currentDragCell) {
            const cell = flexiciousNmsp.FlexDataGrid.currentDragCell;
            if (this.isOutOfVisibleArea(cell.rowInfo)) {
                const pt = this.grid.globalToLocal(cell.localToGlobal(new flexiciousNmsp.Point(0, 0)));
                if (pt.getY() > (this.grid.getHeight() / 2))
                    this.grid.scrollToExistingRow(this.getVerticalScrollPosition() + cell.getHeight(), true);
                else
                    this.grid.scrollToExistingRow(this.getVerticalScrollPosition() - cell.getHeight(), false);

            }
        }

    }

    /**
     * @method removeEventListeners
     * @param {flexiciousNmsp.ComponentInfo} comp
     */
    removeEventListeners(comp) {

        if (!comp || comp.row.getIsFillRow()) return;
        flexiciousNmsp.FlexDataGridContainerBase.prototype.removeEventListeners.apply(this, [comp]);
        if ((comp.component).getIsExpandCollapseCell()) {
            comp.component.removeEventListener(flexiciousNmsp.FlexDataGridExpandCollapseCell.EVENT_EXPAND_COLLAPSE, this.onExpandCollapse);
        }
        else if (comp.component.implementsOrExtends('IFlexDataGridDataCell')) {
            const dataCell = comp.component;

            if (this.grid.enableDrag) {
                dataCell.removeEventListener(Constants.EVENT_DRAG_COMPLETE, this.onCellDragComplete);
                dataCell.removeEventListener(Constants.EVENT_MOUSE_MOVE, this.onCellDragMouseMove);
                dataCell.removeEventListener(Constants.EVENT_MOUSE_DOWN, this.onCellDragMouseDown);
                dataCell.removeEventListener(Constants.EVENT_MOUSE_UP, this.onCellDragMouseUp);
            }
            if (this.grid.enableDrop) {
                dataCell.removeEventListener(Constants.EVENT_DRAG_ENTER, this.onDragEnter);
                dataCell.removeEventListener(Constants.EVENT_MOUSE_UP, this.onDragDrop);
                if (!this.grid.enableDrag)
                    dataCell.removeEventListener(Constants.EVENT_MOUSE_MOVE, this.onCellDragMouseMove);
            }
        }


    }

    /**
     * @method handleSpaceBar
     * @param {flexiciousNmsp.FlexDataGridCell} cell
     * @param {*} triggerEvent
     */
    handleSpaceBar(cell, triggerEvent) {

        const keyboardEvent = triggerEvent.type.toLowerCase().includes('key') ? triggerEvent : null;

        if (keyboardEvent && keyboardEvent.ctrlKey) {
            const rowPosInfo = this.getRowPositionInfoFromRows(cell.rowInfo.getData());

            const expCollCell = cell.rowInfo.getExpandCollapseCell();

            if (expCollCell && expCollCell.hasChildren) {
                this.handleExpandCollapse(expCollCell);
                expCollCell.refreshCell()
            }
        } else {
            flexiciousNmsp.FlexDataGridContainerBase.prototype.handleSpaceBar.apply(this, [cell, triggerEvent]);
        }

    }

    /**
     * @method handleArrowKey
     * @param {flexiciousNmsp.FlexDataGridCell} cell
     * @param {Number} keyCode
     * @param {*} triggerEvent
     */
    handleArrowKey(cell, keyCode, triggerEvent) {

        if (flexiciousNmsp.FlexDataGridContainerBase.prototype.handleArrowKey.apply(this, [cell, keyCode, triggerEvent])) {
            if (this.setCurrentRowAtScrollPosition(this.getVerticalScrollPosition())) {
                this.recycle(this.grid.getColumnLevel(), (keyCode == Constants.KEYBOARD_DOWN));//only recycle if our seed actually changed..
                return true;
            }
            return false;
        }

    }

    /**
     * @method processItem
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     * @param {Boolean} scrollDown
     * @param {flexiciousNmsp.RowInfo} existingRow
     * @param {Array} existingComponents
     * @return {flexiciousNmsp.RowInfo}
     */
    processItem(rowPositionInfo, scrollDown, existingRow, existingComponents) {
        if (typeof scrollDown == "undefined") scrollDown = true;

        const level = rowPositionInfo.getLevel(this.grid);
        const paddingY = level.getStyleValue("horizontalGridLines") ? 1 : 0;
        const item = rowPositionInfo.rowData;
        let row;
        const rowHeight = rowPositionInfo.getRowHeight();
        if (!existingRow) {
            row = this.addRow(rowHeight, scrollDown, rowPositionInfo);
        }
        else {
            this.grid.currentPoint.contentX = this.grid.currentPoint.leftLockedContentX = this.grid.currentPoint.rightLockedContentX = 0;
            row = existingRow;
        }


        if (this.grid.enableDefaultDisclosureIcon) {
            if ((this.grid.lockDisclosureCell || this.isInVisibleHorizontalRange(this.grid.currentPoint.contentX, level.nestIndent * (rowPositionInfo.getRowNestlevel() - 1)))
                && (!existingRow || !row.paddingExists()))
                this.addPadding(rowPositionInfo.getRowNestlevel(), row, rowHeight, level);
            else if (!this.grid.lockDisclosureCell)
                this.grid.currentPoint.contentX += level.nestIndent * (rowPositionInfo.getRowNestlevel() - 1);
            if ((level.nextLevel || level.nextLevelRenderer)) {
                if ((this.grid.lockDisclosureCell || this.isInVisibleHorizontalRange(this.grid.currentPoint.contentX, level.nestIndent))
                    && (!existingRow || !row.disclosureExists())) {
                    const disclosureCell = this.grid.rendererCache.popInstance(level.expandCollapseCellRenderer);
                    disclosureCell.level = level;
                    disclosureCell.setActualSize(level.getMaxDisclosureCellWidth() - this.getBorderWidth(disclosureCell), rowHeight - this.getBorderHeight(disclosureCell));
                    disclosureCell.rowInfo = row;
                    this.addEventListeners(this.addCell(disclosureCell, row).componentInfo);
                    disclosureCell.refreshCell();

                }
                else if (!this.grid.lockDisclosureCell)
                    this.grid.currentPoint.contentX += level.getMaxDisclosureCellWidth();
            }
        }
        let cols = level.getVisibleColumns([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_NONE, flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_LEFT]);
        for (var colIndex = 0; colIndex < cols.length; colIndex++) {
            var col = cols[colIndex];
            if ((this.isInVisibleHorizontalRange(this.grid.currentPoint.contentX, col.getWidth())
                || col.getIsLocked())
                && (!existingRow || !row.columnCellExists(col))) {
                this.pushCell(col, row, existingComponents);
            }
            else if (!col.getIsLocked())
                this.grid.currentPoint.contentX += col.getWidth();
        }
        //now that we're done with the left locked and unlocked cols, do the right locked ones..
        cols = level.getVisibleColumns([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_RIGHT]);
        let wSoFar = 0;
        for (colIndex = 0; colIndex < cols.length; colIndex++) {
            col = cols[colIndex];
            this.grid.currentPoint.contentX = this.getWidth() - wSoFar - col.getWidth() + 1;
            if (!existingRow || !row.columnCellExists(col))
                this.pushCell(col, row, existingComponents);
            wSoFar += col.getWidth();
        }
        //do we have something right locked? if not, add something so we dont show a hole
        //if(!row.rightLockedExists())
        //this.addPadding(rowPositionInfo.getRowNestlevel(),row,rowHeight,level,true);
        this.grid.placeComponents(row);
        return row;

    }

    /**
     * @method pushCell
     * @param {flexiciousNmsp.FlexDataGridColumn} column
     * @param {flexiciousNmsp.RowInfo} row
     * @param {Array} existingComponents
     * @return {flexiciousNmsp.ComponentAdditionResult}
     */
    pushCell(column, row, existingComponents) {

        if (!column.getWidth())
            column.setWidth(75);
        const rendererFactory = column.deriveRenderer(row.rowPositionInfo.getRowType());
        let isNewCell = false;
        let dataCell;
        let found = false;
        if (existingComponents) {
            var existingComponent = this.getExistingCell(existingComponents, rendererFactory, column);
            if (existingComponent) {
                found = true;
                isNewCell = false;
                dataCell = existingComponent.component;
            }
        }
        if (!found) {
            dataCell = this.grid.rendererCache.popInstance(column.getDataCellRenderer(), rendererFactory, this.grid.useRendererCache?row:null);
            isNewCell = true;
        }
        dataCell.setRendererFactory(rendererFactory);
        dataCell.level = row.rowPositionInfo.getLevel(this.grid);
        dataCell.rowInfo = row;
        dataCell.setColumn(column);
        dataCell.setActualSize(column.getWidth() - this.getBorderWidth(dataCell), row.getHeight() - this.getBorderHeight(dataCell));
        dataCell.setWordWrap(column.wordWrap);

        const result = this.addCell(dataCell, row, existingComponent);
        if (row.getIsDataRow() && isNewCell)
            this.addEventListeners(result.componentInfo);
        dataCell.refreshCell();
        return result;

    }

    /**
     * @method gridMouseOut
     */
    gridMouseOut() {
        if (!this.grid)
            return;
        //return;
        if (this.grid.getIsRowSelectionMode() && this.grid.currentCell) {
            this.grid.highlightRow(null, this.grid.currentCell.rowInfo, false);
        }
        if (this.grid.currentCell) {
            this.grid.highlightRow(this.grid.currentCell, this.grid.currentCell.rowInfo, false);
            this.grid.currentCell.rowInfo.invalidateCells();
        }
        this.grid.currentCell = null;

    }

    /**
     * @method onExpandCollapse
     * @param event
     */
    onExpandCollapse(event) {
        //in client mode, do the expand collapse
        const disclosureCell = event.target;
        this.handleExpandCollapse(disclosureCell.getIExpandCollapseComponent());

    }

    /**
     * @method handleExpandCollapse
     * @param {flexiciousNmsp.ExpandCollapseComponent} disclosureCell
     */
    handleExpandCollapse(disclosureCell) {

        const expandingDataItem = disclosureCell.getRowInfo().getData();
        const rowInfoPsoInfo = disclosureCell.getRowInfo().rowPositionInfo;
        const level = disclosureCell.getLevel();
        if (!disclosureCell.hasChildren || !level) return;
        let evt;
        let wasOpen = false;
        if (!level.isItemOpen(expandingDataItem)) {
            wasOpen = false;
            evt = new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridEvent.ITEM_OPENING, level.grid, level, null, disclosureCell.getCell(), expandingDataItem);
            level.dispatchEvent(evt);
        } else {
            wasOpen = true;
            evt = new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridEvent.ITEM_CLOSING, level.grid, level, null, disclosureCell.getCell(), expandingDataItem);
            level.dispatchEvent(evt);
        }
        if (evt.isDefaultPrevented()) {
            return;
        }


        if (!level.getIsClientItemLoadMode()) {
            //in server mode, wait for data to come down from the server to load details
            if (!this.isLoading(expandingDataItem, disclosureCell.getLevel()) && !this.grid.enableVirtualScroll) {
                const nested =
                    new flexiciousNmsp.ExtendedFilterPageSortChangeEvent(flexiciousNmsp.ExtendedFilterPageSortChangeEvent.ITEM_LOAD,
                        this.createFilter(level.nextLevel, expandingDataItem));
                disclosureCell.getLevel().dispatchEvent(nested);
                const itemLoadInfo = new flexiciousNmsp.ItemLoadInfo(expandingDataItem, level, undefined);
                itemLoadInfo.pageIndex = 0;
                this._loadedItems.push(itemLoadInfo);
            } else {
                this.expandCollapse(expandingDataItem, rowInfoPsoInfo);
            }
        }
        else {
            this.expandCollapse(expandingDataItem, rowInfoPsoInfo);
        }

        if (wasOpen) {
            level.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridEvent.ITEM_CLOSE, level.grid, level, null, disclosureCell.getCell(), expandingDataItem));
        } else {
            level.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridEvent.ITEM_OPEN, level.grid, level, null, disclosureCell.getCell(), expandingDataItem));
        }
        this.grid.invalidateCells();

        this.resizeFillerCells();


    }

    /**
     * Given a row index (less than the total number of rows, goes to the row in question).
     * Row index becomes the first visible row. Please note this is a 1 based index.
     * 
     * @method gotoRow
     * @param {Number} rowIndex
     * @return {Boolean}
     * 
     */
    gotoRow(rowIndex) {
        if (rowIndex >= 0 && rowIndex < this.itemVerticalPositions.length) {
            rowIndex = rowIndex > 0 ? rowIndex - 1 : 0;
            const itemPosition = this.itemVerticalPositions[rowIndex];
            this.gotoVerticalPosition(itemPosition.getVerticalPosition());
            return true;
        }
        return false;

    }

    /**
     * Selects the provided text in the currently visible cells
     * that have the default item renderer, or if the item renderer
     * has the selection property
     * 
     * @method selectText
     * @param {String} txt
     * 
     */
    selectText(txt) {
        if (!txt) return;
        txt = txt.toLowerCase();
        const items = this.rows;

        for (const row of items) {
            for (const cell of row.cells) {
                if (row.getIsDataRow()) {
                    const fCell = cell.component;
                    if (fCell) {
                        const txtFld = fCell.getRenderer();
                        if (txtFld) {
                            if (!txtFld.text) continue;
                            const txtFldtext = txtFld.text.toLowerCase();
                            const idx = txtFldtext.indexOf(txt);
                            if (idx >= 0) {
                                fCell.currentBackgroundColors = fCell.getStyleValue("selectionColor");
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Iterates through the data provider to get a list of objects
     * that match the text provided.
     * 
     * @method quickFind
     * @param {String} whatToFind
     * @param {Object} flat
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Array} result
     * 
     */
    quickFind(whatToFind, flat, level, result) {
        if (!whatToFind) return [];
        if (!flat) flat = this.getRootFlat();
        if (!level) level = this.grid.getColumnLevel();
        if (!result) result = [];
        const itemIndex = 0;

        for (const item of flat) {
            for (const col of level.getVisibleColumns()) {
                const txt = col.itemToLabel(item);
                if (txt && txt.toLowerCase().includes(whatToFind.toLowerCase())) {
                    result.push(item);
                    break;
                }
            }

            if (level.nextLevel) {
                const nextFlat = this.filterPageSort(level.getChildren(item), level, item, true, false, true);
                this.quickFind(whatToFind, nextFlat, level.nextLevel, result);
            }
        }

        return result;
    }

    /**
     * Given a data item, scrolls to that item in the datagrid
     * If the use item Key For compare parameter is set to true, uses
     * the selectedKeyField value of the item to identify the item.
     *
     * Flexicious by default will only build RowPositionInfo objects
     * for open items that the user can visually scroll to. So items that
     * are closed, or are on a different page, will not be built, and cannot be scrolled to.
     * If you set rebuildRowPositions to true, the function will inspect the data
     * provider to find the item in question, and build the RowPositionInfo for it
     * if its not found in the currently built RowPositionInfo objects.
     * 
     * @method gotoItem
     * @param {Object} item
     * @param {Boolean} highlight
     * @param {Boolean} useItemKeyForCompare
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Boolean} rebuild
     * @return {Boolean}
     * 
     */
    gotoItem(item, highlight, useItemKeyForCompare, level, rebuild) {
        if (typeof level == "undefined") level = null;
        if (typeof rebuild == "undefined") rebuild = true;

        if (this.grid.enableVirtualScroll) throw new Error("This method is not supported when enableVirtualScroll=true");

        if (!level) level = this.grid.getColumnLevel();
        const items = this.itemVerticalPositions;
        for (var i = 0; i < items.length; i++) {
            const rowPositionInfo = items[i];
            const rowLevel = rowPositionInfo.getLevel(this.grid);
            if (rowPositionInfo.getIsDataRow()
                &&
                ((rowLevel == level && rowLevel.isItemEqual(item, rowPositionInfo.rowData, useItemKeyForCompare)) ||
                    (!useItemKeyForCompare && (item == rowPositionInfo.rowData)))
            ) {
                this.gotoRow(rowPositionInfo.getRowIndex() + 1);
                if (highlight) {
                    this.validateNow();

                    for (const row of this.rows) {
                        if (row.rowPositionInfo == rowPositionInfo) {
                            for (const cell of row.cells) {
                                const fdgc = (cell.component);
                                if (fdgc) {
                                    this.grid.currentCell = fdgc;
                                    this.grid.highlightRow(fdgc, row, true);
                                    return true;
                                }
                            }
                        }
                    }
                }
                return true;
            }
        }
        if (!rebuild)
            return false;
        //this means we have not build the RowPositionInfo for this yet
        const stack = [];
        this.getObjectStack(item, stack, useItemKeyForCompare);

        for (i = 0; i < stack.length; i++) {
            const itemPositionInfo = stack[i];
            if (itemPositionInfo.level.nextLevel && !itemPositionInfo.level.isItemOpen(itemPositionInfo.item)) {
                this.addOpenItem(itemPositionInfo.item);
            }
            let pageIndex = itemPositionInfo.pageIndex;
            pageIndex = Math.floor(pageIndex);
            if (pageIndex != -1) {
                const filterItem = itemPositionInfo.level.getParentLevel() == null ? null : (i < stack.length - 1) ? stack[i + 1].item : itemPositionInfo.item;

                if (!this.grid.itemFilters.getValue(filterItem == null ? Constants.TOP_LEVEL_FILTER : filterItem)) {
                    this.grid.itemFilters.addItem(filterItem == null ? Constants.TOP_LEVEL_FILTER : filterItem, new flexiciousNmsp.AdvancedFilter());
                }
                this.grid.itemFilters.getValue(filterItem == null ? Constants.TOP_LEVEL_FILTER : filterItem).pageIndex = pageIndex;

                if (itemPositionInfo.level.getParentLevel() == null) {
                    //were at root..
                    this.grid.getPagerControl().setPageIndex(pageIndex);
                    this.grid.validateNow();//make sure we have rows...
                    this.grid.rebuildBody();
                    //  this.gotoItem(item,highlight,useItemKeyForCompare,stack[0].level,false);//hopefully now we should have our row position info built.
                    this.gotoItem(item, highlight, useItemKeyForCompare, level, false);//hopefully now we should have our row position info built.
                }
            }
        }
        return true;

    }
    /**
     * @method addOpenItem
     * @param {Object} item
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     * @param {Boolean} check
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} lvl
     */
    addOpenItem(item, rowPositionInfo, check, lvl) {
        if (typeof rowPositionInfo == "undefined") rowPositionInfo = null;
        if (typeof check == "undefined") check = true;

        lvl = lvl || (rowPositionInfo ? rowPositionInfo.getLevel(this.grid) : {});
        if (lvl.selectedKeyField) {
            const items = this.openItems;
            const isSimple = lvl.selectedKeyField.indexOf(".") == -1;
            for (const openItem of items) {
                if (isSimple) {
                    if (item[lvl.selectedKeyField] == openItem[lvl.selectedKeyField]) {
                        return;
                    }
                } else {
                    if (UIUtils.resolveExpression(item, lvl.selectedKeyField) == UIUtils.resolveExpression(openItem, lvl.selectedKeyField)) {
                        return; //we already have an open item with this id.
                    }
                }
            }

            if (!(this.openItems.indexOf(item)) >= 0)
                this.openItems.push(item);
        }
        else if (!check || !this.openItems.indexOf(item) >= 0)
            this.openItems.push(item);

    }
    /**
     * Given an object at any level, returns an array containing information about that object,
     * followed by its parent, until the top of the tree is reached, in reverse order.
     * The array contains a list of ItemPositionInfo objects, which contains that object
     * itself, its index, as well as the page number it appears on, if enablePaging is
     * set to true else -1.
     * 
     * 
     * @method getObjectStack
     * @param {Object} itemToFind
     * @param {Array} stackSoFar
     * @param {Boolean} useItemKeyForCompare
     * @param {Object} flat
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * 
     * @return {Boolean}
     * 
     */
    getObjectStack(itemToFind, stackSoFar, useItemKeyForCompare, flat, level) {
        if (!flat) flat = this.getRootFlat();
        if (!level) level = this.grid.getColumnLevel();
        let itemIndex = 0;

        for (const item of flat) {
            if (level.isItemEqual(itemToFind, item, useItemKeyForCompare)) {
                stackSoFar.push(new flexiciousNmsp.ItemPositionInfo(item, level, level.enablePaging ? itemIndex / level.pageSize : -1, itemIndex));
                return true;
            }
            else if (level.nextLevel) {
                const nextFlat = this.filterPageSort(level.getChildren(item), level, item, true, false, true);
                if (this.getObjectStack(itemToFind, stackSoFar, useItemKeyForCompare, nextFlat, level.nextLevel)) {
                    stackSoFar.push(new flexiciousNmsp.ItemPositionInfo(item, level, level.enablePaging ? itemIndex / level.pageSize : -1, itemIndex));
                    return true;
                }
            }
            itemIndex++;
        }

        return false;
    }

    /**
     * Scrolls to a row that is already drawn.
     * @method scrollToExistingRow
     * @param {Number} vsp
     * @param {Bolean} scrollDown
     */
    scrollToExistingRow(vsp, scrollDown) {

        if ((this.getVerticalScrollPosition() != vsp) && (vsp <= (this.getMaxVerticalScrollPosition() + this.grid.getColumnLevel().getRowHeight()))) {
            this.setVerticalScrollPosition(vsp);
            this.grid.traceValue = vsp;
            this.recycle(this.grid.getColumnLevel(), scrollDown, 0, false);
            this.grid.synchronizeLockedVerticalScroll();
        }

    }

    /**
     * Returnes true if the row's vertical position is less than the grids vertical scroll bar position
     * or if the rows vertical position plus rows height is greater than the grids vertical position plus grid height.
     * 
     * @method isOutOfVisibleArea
     * @param {flexiciousNmsp.RowInfo} row
     * 
     */
    isOutOfVisibleArea(row) {

        return this.isYOutOfVisibleArea(row.getY(), row.getHeight())

    }

    /**
     * Returnes true if the row's vertical position is less than the grids vertical scroll bar position
     * or if the rows vertical position plus rows height is greater than the grids vertical position plus grid height.
     * 
     * @method isYOutOfVisibleArea
     * @param {Number} numY
     * @param {Number} numH
     * 
     * @return {Number} 
     * 
     */
    isYOutOfVisibleArea(numY, numH) {

        return (numY < this.getVerticalScrollPosition()) || (numY + numH > this.getVerticalScrollPosition() + this.getHeight() - (this.getHorizontalScrollBar() ? Constants.HORIZONTAL_SCROLLBAR_HEIGHT : 0));

    }

    /**
     * Goes to the vertical position specified by the parameter.
     * Recycles the renderers
     * 
     * @method gotoVerticalPosition
     * @param {Number} vsp vertical Scroll Position
     * 
     */
    gotoVerticalPosition(vsp) {

        if (this.grid.getDataProvider() == null) {
            this._verticalScrollPending = vsp;
            this.doInvalidate();
            return;
        }
        if (vsp < 0) vsp = 0;
        if ((vsp + this.getHeight()) > this.calculateTotalHeight() && !(this.implementsOrExtends("PrintFlexDataGridBodyContainer"))) {
            vsp = Math.max(0, this.calculateTotalHeight() - this.getHeight() + this.grid.getRowHeight());
        }
        if (this.getVerticalScrollPosition() != vsp) {
            this.setVerticalScrollPosition(vsp);
            this._drawDirty = true;
            this._recreateRows = false;
            this.doInvalidate();
        }
        if (vsp > 0) {
            this.checkStickyHeader();
        }
    }

    /**
     * @method isLoading
     * 
     * @param {object} item
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Boolean} useSelectedKeyField
     * 
     * @return {Boolean}
     */
    isLoading(item, level, useSelectedKeyField) {
        if (typeof useSelectedKeyField == "undefined") useSelectedKeyField = false;

        return (this.findLoadingInfo(item, level, useSelectedKeyField) != null)

    }

    /**
     * 
     * @method setChildData
     * 
     * @param {Object} item
     * @param {Object} schildren
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Number} totalRecords
     * @param {Boolean} useSelectedKeyField
     * 
     * @return {Boolean}
     * 
     */
    setChildData(item, schildren, level, totalRecords, useSelectedKeyField) {
        if (typeof totalRecords == "undefined") totalRecords = -1;

        this.grid.clearFlattenedCache();
        const items = this._loadedItems;

        for (const loadingItem of items) {
            if (loadingItem.isEqual(item, level, useSelectedKeyField)) {
                //found our guy...
                let addToSelection = false;
                if (!this.grid.enableSelectionExclusion && this.grid.enableSelectionCascade) {
                    if (loadingItem.level.isItemSelected(loadingItem.item)) {
                        addToSelection = true;
                    }
                }
                loadingItem.totalRecords = (totalRecords == -1) ? this.grid.getLength(schildren) : totalRecords;
                if (loadingItem.level.childrenCountField) {
                    if (loadingItem.item.hasOwnProperty(loadingItem.level.childrenCountField)) {
                        loadingItem.item[loadingItem.level.childrenCountField] = loadingItem.totalRecords;
                        const iexpCell = this.getCellForRowColumn(loadingItem.item, null, true);
                        if (iexpCell) {
                            iexpCell.refreshCell();
                        }
                    }
                }
                let destColl = loadingItem.level.getChildren(loadingItem.item);
                destColl = [];

                for (const child of schildren) {
                    (destColl).push(child);
                    if (addToSelection && loadingItem.level.nextLevel) {
                        loadingItem.level.nextLevel.addSelectedItem(child);
                    }
                }

                if (destColl instanceof Array)
                    loadingItem.item[loadingItem.level.childrenField] = destColl;
                if (!loadingItem.hasLoaded) {
                    loadingItem.hasLoaded = true;
                    if (loadingItem.totalRecords > 0)
                        this.expandCollapse(item);
                }
                else {
                    this.expandCollapse(item);
                    this.expandCollapse(item);
                }
                break;
            }
        }

        if (this.grid.enableDynamicLevels) {
            this.grid.ensureLevelsCreated();
        }
    }

    /**
     * @method expandCollapse
     * 
     * @param {Object} expandingDataItem
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     * @param {Boolean} forceScrollBar
     * 
     */
    expandCollapse(expandingDataItem, rowPositionInfo, forceScrollBar) {
        if (typeof rowPositionInfo == "undefined") rowPositionInfo = null;
        if (typeof forceScrollBar == "undefined") forceScrollBar = true;

        let pushUp = false;
        let expanding;
        let i;
        if (rowPositionInfo == null) {
            var items = this.itemVerticalPositions;
            for (i = 0; i < items.length; i++) {
                const vrowPositionInfo = items[i];
                if (vrowPositionInfo.rowData == expandingDataItem &&
                    (vrowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA)) {
                    rowPositionInfo = vrowPositionInfo;
                    break;
                }
            }
        }
        const rowPosLevel = rowPositionInfo.getLevel(this.grid);

        if (rowPosLevel.isItemOpen(expandingDataItem)) {
            this.removeOpenItem(expandingDataItem, rowPositionInfo);
            if (this.grid.enableVirtualScroll) return;

            expanding = rowPositionInfo;
            const dataItemsRemoved = [];
            dataItemsRemoved.push(expandingDataItem);
            let howManyToRemove = 0;
            var heightRemoved = 0;
            //we've hit... so start removing from here
            let nextRow = this.itemVerticalPositions.length - 1 > rowPositionInfo.getRowIndex() ? this.itemVerticalPositions[rowPositionInfo.getRowIndex() + 1] : null;
            const rowPositionsToRemove = [];
            while (nextRow && (nextRow.getRowNestlevel() > rowPositionInfo.getRowNestlevel())) {
                //remove
                howManyToRemove++;
                if (!rowPositionsToRemove.includes(nextRow)) {
                    rowPositionsToRemove.push(nextRow);
                    dataItemsRemoved.push(nextRow.rowData);

                    //if(openItems.indexOf(nextRow.rowData)>=0){
                    if (nextRow.getIsDataRow() && nextRow.getLevel(this.grid).isItemOpen(nextRow.rowData)) {
                        this.removeOpenItem(nextRow.rowData, nextRow);
                    }
                }
                nextRow = this.itemVerticalPositions.length - 1 > rowPositionInfo.getRowIndex() ? this.itemVerticalPositions[rowPositionInfo.getRowIndex() + howManyToRemove] : null;
            }

            howManyToRemove--;
            let row;
            if (howManyToRemove > 0) {
                const removed = this.itemVerticalPositions.splice(rowPositionInfo.getRowIndex() + 1, howManyToRemove);
                nextRow = this.itemVerticalPositions.length - 1 > rowPositionInfo.getRowIndex() ? this.itemVerticalPositions[rowPositionInfo.getRowIndex() + 1] : null;

                const rowsToRemove = [];
                for (i = 0; i < this.rows.length; i++) {
                    row = this.rows[i];
                    if (row.getIsChromeRow() && (dataItemsRemoved.includes(row.getData()))) {
                        rowsToRemove.push(row);
                    }
                    else if (row.getIsRendererRow() && (dataItemsRemoved.includes(row.getData()))) {
                        rowsToRemove.push(row);
                    }
                    else {
                        for (const toRemove of rowPositionsToRemove) {
                            if (row.rowPositionInfo == toRemove) {
                                rowsToRemove.push(row);
                                break;
                            }
                        }
                    }


                }
                if (nextRow)
                    heightRemoved = nextRow.getVerticalPosition() - rowPositionInfo.getVerticalPosition() - rowPositionInfo.getRowHeight();
                else if (removed.length > 0)
                    heightRemoved = ((removed[removed.length - 1].getVerticalPosition() + removed[removed.length - 1].getRowHeight()) - (expanding.getVerticalPosition() + expanding.getRowHeight()));
                for (i = 0; i < rowsToRemove.length; i++) {
                    row = rowsToRemove[i];
                    this.rows.splice(this.rows.indexOf(row), 1);
                    this.saveRowInCache(row);
                }
                this.adjustRowPositions(rowPositionInfo, 0 - howManyToRemove, 0 - heightRemoved);
                //if we have removed enough rows that there.implementsOrExtends('nothing below us, need to draw more rows.')){
                this.setVisibleRange();
                const visibleRangeEnd = this._visibleRange[1];
                this._visibleRange[1] = this.getItemAtPosition(visibleRangeEnd.getVerticalPosition() + heightRemoved);
                if (!this._visibleRange[1]) this._visibleRange[1] = visibleRangeEnd;
                if (visibleRangeEnd != this._visibleRange[1]) {
                    this.grid.currentPoint.contentY = this.rows[this.rows.length - 1].getY() + this.rows[this.rows.length - 1].getHeight();
                }
                else {
                    pushUp = true;
                }
            }
        }
        else {
            this.addOpenItem(expandingDataItem, rowPositionInfo);
            if (this.grid.enableVirtualScroll) return;
            expanding = rowPositionInfo;
            const howManyToAdd = 0;
            const heightAdded = 0;
            const addedRows = [];

            const flat = rowPosLevel.nextLevelRenderer ? [rowPositionInfo.rowData] : rowPosLevel.getChildren(rowPositionInfo.rowData,
                rowPosLevel.nextLevel ? rowPosLevel.nextLevel.getEnableFilters() : false);
            const htAdded = this.calculateTotalHeight(flat, rowPosLevel.nextLevelRenderer ? rowPosLevel : rowPosLevel.nextLevel,
                rowPositionInfo.getVerticalPosition() + expanding.getRowHeight(),
                rowPositionInfo.getRowNestlevel() + 1, rowPositionInfo,
                addedRows, expandingDataItem);
            this.adjustRowPositions(rowPositionInfo, addedRows.length, htAdded);
            var items = addedRows.reverse();

            for (const newPos of items) {
                this.itemVerticalPositions.splice(rowPositionInfo.getRowIndex() + 1, 0, newPos);
            }

            this.grid.currentPoint.contentY = expanding.getVerticalPosition() + expanding.getRowHeight();
        }


        if (this.grid.showSpinnerOnFilterPageSort) {
            this.grid.hideSpinner();
        }

        const oldVsp = this.getVerticalScrollPosition();
        const oldHsp = this.getHorizontalScrollPosition();
        this.removeAllComponents(true);

        //this.grid.synchronizeLockedVerticalScroll();
        this.drawRows();
        if (forceScrollBar) {
            if (this.getVerticalScrollBar()) {

                if (oldVsp >= this.getMaxVerticalScrollPosition()) {
                    if (!isNaN(heightRemoved)) {
                        this.setVerticalScrollPosition(Math.max(0, oldVsp - heightRemoved));
                    }
                } else if (oldVsp < 0) {
                    this.setVerticalScrollPosition(0);
                }
                if (oldVsp <= this.getMaxVerticalScrollPosition()) {
                    this.setVerticalScrollPosition(oldVsp + 1);
                    this.setVerticalScrollPosition(oldVsp - 1);
                }
            }

        }
        this.grid.currentCell = this.getCellForRowColumn(expanding.rowData, null, true);
        if (this.grid.currentCell && this.grid.currentCell.getEnabled()) {
            this.grid.highlightRow(this.grid.currentCell, this.grid.currentCell.rowInfo, true);
        }
        if (!this.grid.getForceColumnsToFitVisibleArea() && oldHsp > 0) {
            this.setHorizontalScrollPosition(oldHsp);
            this.recycleH();
        }
    }

    /**
     * @method adjustRowPositions
     * 
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     * @param {number} howManyToRemove
     * @param {number} heightRemoved
     * 
     */
    adjustRowPositions(rowPositionInfo, howManyToRemove, heightRemoved) {
        let nextRow;
        for (let pointer = rowPositionInfo.getRowIndex() + 1; pointer < this.itemVerticalPositions.length; pointer++) {
            nextRow = this.itemVerticalPositions[pointer];
            nextRow.setRowIndex(nextRow.getRowIndex() + howManyToRemove);
            nextRow.setVerticalPosition(nextRow.getVerticalPosition() + heightRemoved);
        }
        const prevCalc = this._calculatedTotalHeight;
        this._calculatedTotalHeight += heightRemoved;
        this.checkScrollChange(prevCalc);

        this.checkAutoAdjust();

        for (const row of this.rows) {
            row.setY(row.rowPositionInfo.getVerticalPosition());
        }
        //invalidateDisplayList();
    }

    /**
     * @method invalidateCalculatedHeight
     */
    invalidateCalculatedHeight() {

        this._heightCalculated = false;

    }

    /**
     * @method setCurrentRowAtScrollPosition
     * @param {number} vsp
     * @return {boolean}
     */
    setCurrentRowAtScrollPosition(vsp) {

        const item = this.currentRowPointer ? this.currentRowPointer.rowData : null;
        const currentRowType = this.currentRowPointer ? this.currentRowPointer.getRowType() : null;
        this.currentRowPointer = this.getItemAtPosition(vsp);
        return (this.currentRowPointer != null && ((item != this.currentRowPointer.rowData) ||
            (currentRowType != this.currentRowPointer.getRowType()) || this.grid.recalculateSeedOnEachScroll));//only recycle if our seed actually changed..

    }

    /**
     * @method getFilteredPagedSortedData
     * 
     * @param {Dictionary} dictionary
     * @param {Boolean} applyFilter
     * @param {Boolean} applyPaging
     * @param {Boolean} applySort
     * @param {Array} pages
     * @param {Boolean} flatten
     * 
     * @return {Object}
     */
    getFilteredPagedSortedData(dictionary, applyFilter, applyPaging, applySort, pages, flatten) {
        if (typeof applyFilter == "undefined") applyFilter = true;
        if (typeof applyPaging == "undefined") applyPaging = true;
        if (typeof applySort == "undefined") applySort = true;
        if (typeof pages == "undefined") pages = null;
        if (typeof flatten == "undefined") flatten = false;

        const result = [];
        this.gatherFilteredPagedSortedData(dictionary, result, this.getRootFlat(),
            this.grid.getColumnLevel(), null, applyFilter, applyPaging, applySort, pages, flatten);
        return result;

    }

    /**
     * @method gatherFilteredPagedSortedData
     * 
     * @param {Dictionary} dictionary
     * @param {ArrayCollection} result
     * @param {Object} flat
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Object} parentObject
     * @param {Boolean} applyFilter
     * @param {Boolean} applyPaging
     * @param {Boolean} applySort
     * @param {Array} pages
     * @param {Boolean} flatten
     * 
     */
    gatherFilteredPagedSortedData(
        dictionary,
        result,
        flat,
        level,
        parentObject,
        applyFilter,
        applyPaging,
        applySort,
        pages,
        flatten) {
        flat = this.filterPageSort(flat, level, parentObject, applyFilter, applyPaging, applySort, pages);

        for (const item of flat) {
            dictionary[item] = level;
            result.push(item);
            if (level.nextLevel && flatten)
                this.gatherFilteredPagedSortedData(dictionary, result, level.getChildren(item), level.nextLevel, item, applyFilter, applyPaging, applySort, pages, flatten);
        }
    }

    /**
     * @method getRootFlat
     * @return {Object}
     */
    getRootFlat() {

        return this.grid.getDataProvider();

    }

    /**
     * Dual purpose function, one to calculate height, second to figure out that at
     * vertical position Y, the seed record to be shown is?.
     * 
     * @method calculateTotalHeight
     * 
     * @param {Object} flat
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {number} heightSoFar
     * @param {number} nestLevel
     * @param {flexiciousNmsp.RowPositionInfo} expanding
     * @param {Array} addedRows
     * @param {Object} parentObject
     * @param {boolean} isRecursive
     * 
     * @return {number} total Height
     */
    calculateTotalHeight(
        flat,
        level,
        heightSoFar,
        nestLevel,
        expanding,
        addedRows,
        parentObject,
        isRecursive) {
        if (typeof flat == "undefined") flat = null;
        if (typeof level == "undefined") level = null;

        if (this._heightCalculated && (expanding == null)) //if we are not calculating the seed.
            return this._calculatedTotalHeight;

        if (this.grid.variableRowHeight && flexiciousNmsp.FlexDataGrid.measurerText.parent == null)
            this.grid.addChild(flexiciousNmsp.FlexDataGrid.measurerText);
        if (this.grid.variableRowHeightUseRendererForCalculation && this.variableRowHeightRenderers == null && !isRecursive)//this.implementsOrExtends(not a recursive call to ourselves.
            this.variableRowHeightRenderers = new flexiciousNmsp.KeyValuePairCollection();

        let prevCalc = 0;
        if (!isRecursive) {
            prevCalc = this._calculatedTotalHeight;
        }

        let totalHeight = 0;
        if (!flat) {
            if (!expanding) {
                this.itemVerticalPositions = [];
            }
            flat = this.getRootFlat();
        }
        if (!level) {
            level = this.grid.getColumnLevel();
        }
        if (level.getIsClientFilterPageSortMode()) {
            flat = this.filterPageSort(flat, level, parentObject, true, true, true, null, (expanding == null && level.getNestDepth() == 1))

        }
        let heightAdded = 0;
        let section;
        let i;
        const expandingLevel = expanding ? expanding.getLevel(this.grid) : null;
        if (expanding && !expandingLevel.nextLevelRenderer) {
            var sections = expandingLevel.nextLevel.displayOrder.split(",");
            for (i = 0; i < sections.length; i++) {
                section = sections[i];
                var htAdded = this.processSection(addedRows, expanding.getRowIndex(), parentObject, heightSoFar, level, parentObject, this.getChromeType(section), expanding, expandingLevel);
                totalHeight += htAdded;
                heightSoFar += htAdded;
                if (section == "body")
                    break;
            }
        }
        let itemIndex = 0;
        const maxRows = this.grid.getLength(flat);
        for (let rowPointer = 0; rowPointer < maxRows; rowPointer++) {
            const item = flat[rowPointer];
            this.parentMap[level.getItemKey(item)] = parentObject;
            if (item) {

                itemIndex++;
                if (!expanding) {
                    heightAdded = this.addToVerticalPositions(item, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
                }
                else if (!expandingLevel.nextLevelRenderer) {
                    heightAdded = this.addToExpandingPositions(addedRows, expanding.getRowIndex(), item, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
                }

                totalHeight += heightAdded;
                heightSoFar += heightAdded;
                const open = (level.nextLevel || (level.nextLevelRenderer && (level.grid.hasChildrenFunction == null || level.grid.hasChildrenFunction(item) == true))) && level.isItemOpen(item);
                if (!open && level.nextLevel == null) {
                    if (this.grid.enableDynamicLevels && this.grid.dynamicLevelHasChildrenFunction(item)) {
                        level.nextLevel = this.grid.getColumnLevel().clone(false);
                        level.nextLevel.reusePreviousLevelColumns = true;
                        this.grid.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridEvent.DYNAMIC_LEVEL_CREATED, this.grid,
                            level.nextLevel));
                        this.grid.getColumnLevel().initializeLevel(this.grid);
                        this.grid.reDraw();
                    }
                }

                if (open) {
                    if (level.nextLevelRenderer) {
                        if ((level.nextLevelRenderer && (level.grid.hasChildrenFunction == null || level.grid.hasChildrenFunction(item) == true))) {
                            if (!level.levelRendererHeight) {
                                const renderer = level.nextLevelRenderer.newInstance();
                                if(renderer.hasOwnProperty('getHeight'))
                                    level.levelRendererHeight = renderer.getHeight();
                            }
                            if (!expanding) {
                                heightAdded = this.addToVerticalPositions(item, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_RENDERER);
                            }
                            else {
                                heightAdded = this.addToExpandingPositions(addedRows, expanding.getRowIndex(), item, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_RENDERER);
                            }


                            totalHeight += heightAdded;
                            heightSoFar += heightAdded;

                        }
                    }
                    else {
                        nestLevel++;
                        let nextLevel = 0;
                        const result = level.getChildren(item);
                        if (this.grid.getLength(result) > 0) {

                            sections = level.nextLevel.displayOrder.split(",");
                            for (i = 0; i < sections.length; i++) {
                                section = sections[i];
                                htAdded = this.processSection(addedRows, expanding ? expanding.getRowIndex() - 1 : 0, item, heightSoFar, level.nextLevel, item
                                    , this.getChromeType(section), expanding, expandingLevel);
                                totalHeight += htAdded;
                                heightSoFar += htAdded;
                                if (section == "body")
                                    break;
                            }
                            nextLevel = this.calculateTotalHeight(result, level.nextLevel,
                                heightSoFar, nestLevel, expanding, addedRows, item, true);

                            totalHeight += nextLevel;
                            heightSoFar += nextLevel;

                            let hit1 = false;
                            sections = level.nextLevel.displayOrder.split(",");
                            for (i = 0; i < sections.length; i++) {
                                section = sections[i];
                                if (hit1) {
                                    htAdded = this.processSection(addedRows, expanding ? expanding.getRowIndex() : -1, item, heightSoFar, level.nextLevel, item,
                                        this.getChromeType(section), expanding, expandingLevel);
                                    totalHeight += htAdded;
                                    heightSoFar += htAdded;
                                }
                                if (section == "body")
                                    hit1 = true;
                            }
                        }
                        nestLevel--;
                    }
                }
            }
        }
        if (expanding && !expandingLevel.nextLevelRenderer) {
            let hit = false;
            sections = expandingLevel.nextLevel.displayOrder.split(",");
            for (i = 0; i < sections.length; i++) {
                section = sections[i];
                if (hit) {
                    htAdded = this.processSection(addedRows, expanding.getRowIndex(), parentObject, heightSoFar, level, parentObject,
                        this.getChromeType(section), expanding, expandingLevel);
                    totalHeight += htAdded;
                    heightSoFar += htAdded;
                }
                if (section == "body")
                    hit = true;
            }
        }
        if (!expanding) {
            if (nestLevel == 0) {
                this._heightCalculated = true;
                this._calculatedTotalHeight = totalHeight;
                this.verticalSpill = (this._calculatedTotalHeight > this.getHeight());
                this.checkAutoAdjust();
            }
        }
        if (this.grid.variableRowHeight && flexiciousNmsp.FlexDataGrid.measurerText.parent != null)
            this.grid.removeChild(flexiciousNmsp.FlexDataGrid.measurerText);
        if (this.grid.variableRowHeightUseRendererForCalculation && this.variableRowHeightRenderers != null && !isRecursive) {
            this.clearVariableRowHeightRenderes();
        }

        if (!isRecursive) {
            this.checkScrollChange(prevCalc);


        }
        return totalHeight;
    }

    /**
     * 
     * @method checkScrollChange
     * @param {number} prevCalc previous calculation
     * 
     */
    checkScrollChange(prevCalc) {
        //if(this.grid.getForceColumnsToFitVisibleArea())
        {
            let myHt = this.getHeight();
            if (!prevCalc) prevCalc = 0;
            if (this.grid.enableHeightAutoAdjust) {
                //prevCalc = this._calculatedTotalHeight;
                myHt = this.grid.maxAutoAdjustHeight;
            }
            if (prevCalc < myHt && this._calculatedTotalHeight > myHt) {
                //this means we had no scrollbar before, but now we do.
                this.grid.showHideVScroll(true, Constants.VERTICAL_SCROLLBAR_WIDTH);
                this.setBackgroudFillerSize();
            } else if (prevCalc > myHt && this._calculatedTotalHeight < myHt) {
                //this means we had a scrollbar before, but now its gone
                this.grid.showHideVScroll(false, Constants.VERTICAL_SCROLLBAR_WIDTH);
                this.setBackgroudFillerSize();
            }
            if (this._calculatedTotalHeight < myHt) {
                this.setStyleAttribute("overflowY", "hidden");
            } else {
                this.setStyleAttribute("overflowY", "auto");
            }

            if (!this.grid.getForceColumnsToFitVisibleArea()) {
                const cols = this.grid.getColumnLevel().getVisibleColumns([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_NONE]);
                if (cols.length > 0) {
                    const lstCol = cols[cols.length - 1];
                    if (this.grid.getHorizontalScrollPolicy() == "on") {
                        this.setStyleAttribute("overflowX", "scroll");
                    }
                    else if (lstCol.getWidth() > lstCol._naturalLastColWidth) {
                        this.setStyleAttribute("overflowX", "hidden");
                    } else {
                        this.setStyleAttribute("overflowX", "auto");
                    }
                }
            }
            if (this.grid.implementsOrExtends('PrintFlexDataGrid')) {
                this.setStyleAttribute("overflow", "hidden");
            }
        }
    }

    /**
     * Clears out the variable row height renderers
     * @method clearVariableRowHeightRenderes
     */
    clearVariableRowHeightRenderes() {
        //this.implementsOrExtends('not a recursive call to ourselves, and we are don')){
        const items = this.variableRowHeightRenderers.keys;

        for (const col of items) {
            this.grid.removeChild(this.variableRowHeightRenderers.getValue(col));
        }

        this.variableRowHeightRenderers.clear();
        this.variableRowHeightRenderers = null;
    }

    /**
     * @method processSection
     * 
     * @param {Array} addedRows
     * @param {number} rowIndex
     * @param {Object} parentObject
     * @param {number} heightSoFar
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Object} parentObject5
     * @param {number} chromeType
     * @param {flexiciousNmsp.RowPositionInfo} expanding
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} expandingLevel
     * 
     * @return {number} 
     */
    processSection(
        addedRows,
        rowIndex,
        parentObject,
        heightSoFar,
        level,
        parentObject5,
        chromeType,
        expanding,
        expandingLevel) {

        const totalHeight = 0;
        let heightAdded = 0;
        if (chromeType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER) {
            if (level.enablePaging || level.forcePagerRow) {
                if (!expanding) {
                    heightAdded = this.addToVerticalPositions(parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER);
                }
                else
                    heightAdded = this.addToExpandingPositions(addedRows, expanding.getRowIndex(), parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER);
            }
        }
        if (!level.reusePreviousLevelColumns) {
            if (chromeType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILTER) {
                if (level.getEnableFilters()) {
                    if (!expanding) {
                        heightAdded = this.addToVerticalPositions(parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILTER);
                    }
                    else
                        heightAdded = this.addToExpandingPositions(addedRows, expanding.getRowIndex(), parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILTER);
                }
            }
            if (chromeType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER) {
                if (!expanding) {
                    heightAdded = this.addToVerticalPositions(parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER);
                }
                else
                    heightAdded = this.addToExpandingPositions(addedRows, expanding.getRowIndex(), parentObject, heightSoFar, level, parentObject,
                        flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER);
            }
        }
        if (chromeType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER) {
            if (level.getEnableFooters()) {
                if (!expanding) {
                    heightAdded = this.addToVerticalPositions(parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER);
                }
                else
                    heightAdded = this.addToExpandingPositions(addedRows, expanding.getRowIndex(), parentObject, heightSoFar, level, parentObject, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER);
            }
        }
        return heightAdded;

    }

    /**
     * @method getChromeType
     * @param {String} name name of chrome type
     * @return {number}
     */
    getChromeType(name) {

        if (name == "header") {
            return flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER
        }
        else if (name == "footer") {
            return flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER
        }
        else if (name == "pager") {
            return flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER
        }
        else if (name == "filter") {
            return flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILTER
        }
        else if (name == "body") {
            return flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA
        }
        else
            throw new Error(`Invalid section specified for the displayOrder property ${name}`);


    }

    /**
     * @method checkAutoAdjust
     */
    checkAutoAdjust() {

        this.ensureAutoAjustHeight();


    }

    /**
     * @method ensureAutoAjustHeight
     */
    ensureAutoAjustHeight() {

        if (this.grid.enableHeightAutoAdjust) {
            const borderThickness = this.grid.getStyle("borderThickness");
            const paddingBottom = this.grid.hasBorderSide("bottom") ? borderThickness : 0;
            let requiredHt = Math.min(this._calculatedTotalHeight + this.grid.getHeaderSectionHeight() + (this._calculatedTotalHeight == 0 ? this.grid.getRowHeight() : 0) +
                this.grid.getFooterRowHeight() + paddingBottom + (this.grid.getHorizontalScrollBar() ? Constants.HORIZONTAL_SCROLLBAR_HEIGHT : 0)
                + ((this.grid.noDataMessage && (this._calculatedTotalHeight == 0)) ? (this.grid.spinner && this.grid.spinner.getHeight() ? this.grid.spinner.getHeight() : 2 * this.grid.getRowHeight()) : 0),
                this.grid.maxAutoAdjustHeight);
            requiredHt += 2;



            if (!isNaN(requiredHt) && requiredHt != this.grid.getHeight()) {
                this.grid.setHeight(requiredHt);
                this.grid.invalidateDisplayList();
            }
        }

    }

    /**
     * @method addToVerticalPositions
     * 
     * @param {Object} item
     * @param {number} heightSoFar
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Object} parent
     * @param {number} rowType
     * 
     * @return {number}
     */
    addToVerticalPositions(item, heightSoFar, level, parent, rowType) {

        const heightAdded = this.grid.calculateRowHeight(item, level, rowType);
        let depth = 1;
        if (rowType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER) {
            depth = level.getMaxColumnGroupDepth();
            //heightAdded/=depth;
            for (let i = 1; i < depth; i++) {
                this.itemVerticalPositions.push(new flexiciousNmsp.RowPositionInfo(item, this.itemVerticalPositions.length,
                    heightSoFar, heightAdded, level, flexiciousNmsp.RowPositionInfo.ROW_TYPE_COLUMN_GROUP)
                );
                heightSoFar += heightAdded;
            }
        }

        this.itemVerticalPositions.push(new flexiciousNmsp.RowPositionInfo(item, this.itemVerticalPositions.length,
            heightSoFar, heightAdded, level, rowType)
        );
        return heightAdded * depth;

    }

    /**
     * @method addToExpandingPositions
     * 
     * @param {Array} array
     * @param {number} parentRowIndex
     * @param {object} item
     * @param {number} heightSoFar
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {Object} parent
     * @param {number} rowType
     * 
     * @return {number}
     */
    addToExpandingPositions(array, parentRowIndex, item, heightSoFar, level, parent, rowType) {


        const heightAdded = this.grid.calculateRowHeight(item, level, rowType);
        let depth = 1;
        if (rowType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER) {
            depth = level.getMaxColumnGroupDepth();
            //heightAdded/=depth;
            for (let i = 1; i < depth; i++) {
                array.push(new flexiciousNmsp.RowPositionInfo(item, parentRowIndex + array.length + 1,
                    heightSoFar, heightAdded, level, flexiciousNmsp.RowPositionInfo.ROW_TYPE_COLUMN_GROUP)
                );
                heightSoFar += heightAdded;
            }
        }
        array.push(new flexiciousNmsp.RowPositionInfo(item, parentRowIndex + array.length + 1,
            heightSoFar, heightAdded, level, rowType));
        return heightAdded * depth;

    }

    /**
     * @method addCell
     * 
     * @param {UIComponent} component
     * @param {flexiciousNmsp.RowInfo} row
     * @param {flexiciousNmsp.ComponentInfo} existingComponent
     * 
     * @return {flexiciousNmsp.ComponentAdditionResult}
     * 
     */
    addCell(component, row, existingComponent) {
        if (typeof existingComponent == "undefined") existingComponent = null;

        if (!component.getVisible())
            component.setVisible(true);
        const result = flexiciousNmsp.FlexDataGridContainerBase.prototype.addCell.apply(this, [component, row, existingComponent]);
        if (this.currentRowPointer) {
            result.verticalSpill = (this.grid.currentPoint.contentY > (this.currentRowPointer.getVerticalPosition() + this.getHeight() + 1));
        }

        result.horizontalSpill = this.grid.currentPoint.contentX > this._visibleRangeH[1];
        if (row.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILL) {
            component.parent.setChildIndex(component, 0);
            component.setHeight(this.getHeight() - this.grid.isHScrollBarVisible);
        }
        return result;

    }

    /**
     * @method onHeaderCellClicked
     * 
     * @param {flexiciousNmsp.FlexDataGridHeaderCell} headerCell
     * @param {*} triggerEvent
     * @param {boolean} isMsc
     * 
     */
    onHeaderCellClicked(headerCell, triggerEvent, isMsc) {
        if (typeof isMsc == "undefined") isMsc = false;

        flexiciousNmsp.FlexDataGridContainerBase.prototype.onHeaderCellClicked.apply(this, [headerCell, triggerEvent, isMsc]);

        if (headerCell.level != this.grid.getColumnLevel()) {
            this.onChildHeaderClicked(headerCell);
        }

    }

    /**
     * @method onChildHeaderClicked
     * @param {flexiciousNmsp.FlexDataGridHeaderCell} headerCell
     */
    onChildHeaderClicked(headerCell) {

        if (headerCell.getColumn() && headerCell.getColumn().sortable) {
            this.grid.rebuildBody();
        }

    }

    /**
     * @method onSelectAllChanged
     * @param {*} event
     */
    onSelectAllChanged(event) {

        flexiciousNmsp.FlexDataGridContainerBase.prototype.onSelectAllChanged.apply(this, [event]);
        this.grid.invalidateSelection();

    }

    /**
     * @method addRow
     * 
     * @param {number} ht
     * @param {boolean} scrollDown
     * @param {RowPositionInfo} rowPositionInfo
     * 
     */
    addRow(ht, scrollDown, rowPositionInfo) {

        const result = flexiciousNmsp.FlexDataGridContainerBase.prototype.addRow.apply(this, [ht, scrollDown, rowPositionInfo]);
        this.verticalSpill = this.grid.currentPoint.contentY > this.getHeight();
        return result;

    }

    /**
     * @method getAllRows
     * @return {ArrayCollection} new rows
     */
    getAllRows() {
        const newRows = (this.rows.slice());
        let i;
        let j;
        const items = this.fillerRows;
        for (i = 0; i < items.length; i++) {
            const fillerRow = items[i];
            newRows.push(fillerRow);
        }
        for (const key in this._rowCache) {
            for (j = 0; j < this._rowCache[key].length; j++) {
                const row = this._rowCache[key][j];
                newRows.push(row);
            }
        }
        return newRows;
    }

    /**
     * @method nextPage
     * Goes to the next page of data.
     */
    nextPage() {

        this.gotoVerticalPosition(this.getVerticalScrollPosition() + this.getHeight());

    }

    /**
     * @method getColumnGroupDepth
     * @param {flexiciousNmsp.RowInfo} row
     * @return {number} depth
     *  - not supported for now. Column groups are only supported at the top level.
     */
    getColumnGroupDepth(row) {

        const index = this.itemVerticalPositions.indexOf(row.rowPositionInfo);
        const MAX_COL_GROUP_DEPTH = 5;
        let depth = 1;
        for (let i = index; i < MAX_COL_GROUP_DEPTH; i++) {
            if (i + index > this.itemVerticalPositions.length) {
                return depth;
            }
            if (this.itemVerticalPositions[i + index].rowInfo.getIsHeaderRow()) {
                return depth;
            }
            depth++;
        }
        return depth;

    }

    /**
     * @method clearAllCollections
     */
    clearAllCollections() {

        if (this.grid.clearOpenItemsOnDataProviderChange)
            this.clearOpenItems();

    }

    /**
     * @method clearOpenItems
     */
    clearOpenItems() {

        this.openItems = [];
        this._loadedItems = [];

        this.parentMap = {};
    }

    /**
     * @method removeOpenItem
     * 
     * @param {Object} item
     * @param {RowPositionInfo} rowPositionInfo
     * 
     */
    removeOpenItem(item, rowPositionInfo) {
        if (typeof rowPositionInfo == "undefined") rowPositionInfo = null;

        if (rowPositionInfo && rowPositionInfo.getLevel(this.grid).selectedKeyField) {
            const lvl = rowPositionInfo.getLevel(this.grid);
            const items = this.grid.getOpenItems();

            for (const openItem of items) {
                if (UIUtils.resolveExpression(item, lvl.selectedKeyField) == UIUtils.resolveExpression(openItem, lvl.selectedKeyField)) {
                    this.openItems.splice(this.openItems.indexOf(openItem), 1);
                    return;
                }
            }
        }
        else if (this.openItems.includes(item))
            this.openItems.splice(this.openItems.indexOf(item), 1);

    }

    /**
     * In lazy loaded grid levels, (filterPageSortMode=server), when
     * the user expands a level for the first time, since the data
     * is not loaded, the level dispatches the filterPageSortChange, or itemLoad.
     *
     * This event is then handled by client code, and when the data
     * is retreieved from the server, the setChildData method is called. This method will
     * store the results in this array collection.
     *
     * @method getLoadedItems
     * 
     */
    getLoadedItems() {
        return this._loadedItems;
    }

    /**
     * @method setVerticalScrollPosition
     * @param {number} val
     *
     */
    setVerticalScrollPosition(val) {
        if (val < 0) {
            val = 0;
        }
        flexiciousNmsp.FlexDataGridContainerBase.prototype.setVerticalScrollPosition.apply(this, [val]);
    }

    /**
     * Returns true if there is another page of data.
     * 
     * @method getValidNextPage
     * @return {boolean} 
     */
    getValidNextPage() {
        return this.getHeight() > 0 && (this.calculateTotalHeight() > this.getHeight()) && (this.getVerticalScrollPosition() < (this.calculateTotalHeight() - this.getHeight()));
    }

    /**
     * Returns the number of rows currently visible.
     * 
     * @method getOnScreenRows
     * @return {Array} on screen rows
     */
    getOnScreenRows() {
        if (this._drawDirty) {
            this.validateNow();
        }
        const osRows = [];

        for (const row of this.rows) {
            if (row.getY() >= this.getVerticalScrollPosition() && (row.getY() < (this.getVerticalScrollPosition() + this.getHeight())))
                osRows.push(row);
        }

        return osRows;
    }

    /**
     * @method getCalculatedTotalHeight
     * @return {number} total calculated height.
     */
    getCalculatedTotalHeight() {
        return this._calculatedTotalHeight;
    }

    /**
     * @method getNumTotalRows
     * @return {number} total number of rows.
     */
    getNumTotalRows() {
        return this.itemVerticalPositions.length;
    }

    /**
     * Gets the vertical scroll bar props
     * 
     * @method getVerticalScrollBar
     * @return {flexiciousNmsp.Rectangle} vertical scrollbar
     */
    getVerticalScrollBar() {
        return new flexiciousNmsp.Rectangle(0, 0, this.domElement ? this.domElement.offsetWidth - this.domElement.clientWidth : 0, this.getHeight());
    }

    /**
     * Gets the horizontal scroll bar props
     * 
     * @method getVerticalScrollBar
     * @return {flexiciousNmsp.Rectangle} horizotal scrollbar
     */
    getHorizontalScrollBar() {
        return new flexiciousNmsp.Rectangle(0, 0, this.getWidth(), this.domElement ? this.domElement.offsetHeight - this.domElement.clientHeight : 0);
    }


    /**
     * Gets the scrollLeft
     */
    // setHorizontalScrollPosition(val) {
    //     this.setAttribute("scrollLeft", val);
    // }

    /**
     * Gets the scrollLeft
     * 
     * @method getMaxHorizontalScrollPosition
     * @return {number} maximum horizontal scroll position
     * 
     */
    getMaxHorizontalScrollPosition() {
        if (this.domElement) {
            return this.domElement.scrollWidth - this.domElement.clientWidth;
        } else {
            const scrollableWidth = this.grid.getColumnLevel().getSumOfColumnWidths([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_NONE]);
            return scrollableWidth - this.getWidth();
        }
    }

    /**
     * Gets the scrollLeft
     * 
     * @method getMaxVerticalScrollPosition
     * @return {number} maximum vertical scroll position
     * 
     */
    getMaxVerticalScrollPosition() {
        return this._calculatedTotalHeight - this.getHeight();
    }

    /**
     * @method addMatchSticks
     */
    addMatchSticks(p) {
        //if(this.matchSticks.length==0)
        {
            this.addMatchStick(this);
            this.addMatchStick(this.grid.getLeftLockedContent());
            this.addMatchStick(this.grid.getRightLockedContent());
            this.addHMatchStick(this);
            this.addHMatchStick(this.grid.getFilterContainer());
            this.addHMatchStick(this.grid.getFooterContainer());
            this.addHMatchStick(this.grid.getHeaderContainer());
        }
    }

    /**
     * @method addMatchStick
     * @param p
     */
    addMatchStick(p) {
        if (!p.vMatch) {
            p.vMatch = new flexiciousNmsp.UIComponent("div");
            p.vMatch.setAttribute("className", "matchStick");
            p.addChild(p.vMatch);
            this.matchSticks.push(p.vMatch);
        }
        p.vMatch.setHeight(this._calculatedTotalHeight);
        if (p != this) {
            //left locked and right locked content needs a longer matchstick
            p.vMatch.setHeight(this._calculatedTotalHeight + Constants.HORIZONTAL_SCROLLBAR_HEIGHT);
        }

        this.maxVerticalScrollPosition = this._calculatedTotalHeight - this.height;
    }

    /**
     * @method addHMatchStick
     * @param p
     */
    addHMatchStick(p) {
        if (!p.hMatch) {
            if (!this.grid.getForceColumnsToFitVisibleArea()) {
                //insert vmatches.
                p.hMatch = new flexiciousNmsp.UIComponent("div");
                p.hMatch.setAttribute("className", "matchStick");
                p.addChild(p.hMatch);
                this.matchSticks.push(p.hMatch);
            }
        }
        if (p.hMatch) {
            var w = this.grid.getColumnLevel().getSumOfColumnWidths([flexiciousNmsp.FlexDataGridColumn.LOCK_MODE_NONE]) - 1;
            this.maxHorizontalScrollPosition = w - this.width;
            p.hMatch.setWidth(w);
            if (p != this) {
                //left locked and right locked content needs a longer matchstick
                p.hMatch.setWidth(w + Constants.VERTICAL_SCROLLBAR_WIDTH)
            }
        }
    }

    /**
     * @method kill
     */
    kill() {
        flexiciousNmsp.FlexDataGridContainerBase.prototype.kill.apply(this);
        this.openItems = [];

        for (const dataitem of this.itemVerticalPositions) {
            dataitem.kill();
        }

        this.itemVerticalPositions = [];
        this._rowCache = {};
        this.parentMap = {};
        this.fillerRow = null;
        while (this.matchSticks.length > 0) {
            const mt = this.matchSticks[this.matchSticks.length - 1];
            mt.kill();
            this.matchSticks.pop();
        }
        this.matchSticks = [];
        this.currentRowPointer = null;
        this._visibleRange = [];
        this._visibleRangeH = [];
    }

    kill() {
        flexiciousNmsp.UIComponent.prototype.kill.apply(this);
        const rows = this.getAllRows();

        for (const row of rows) {
            for (const cell of row.cells) {
                const comp = cell.component;
                comp.kill();
            }

            row.kill();
        }

        this.grid = null;
        this.rows = [];
    }

    /**
     * @private
     * @method getRowForRowPositionInfo
     * @param {flexiciousNmsp.RowPositionInfo} rowPos
     * 
     * @return {flexiciousNmsp.RowInfo} row
     */
    getRowForRowPositionInfo(rowPos) {
        for (const row of this.rows) {
            if (row.rowPositionInfo == rowPos)
                return row;
        }

        return null;
    }

    /**
     * @method checkStickyHeader
     */
    checkStickyHeader() {

        if (this.getMaxVerticalScrollPosition() < this.getHeight()) {
            //no vertical scroll, everything is visible.
            return;
        }

        //step 1 go through visible rows,
        let nestedSubRow;
        for (var i = 0; i < this.rows.length; i++) {
            const row = this.rows[i];
            if (row.getIsDataRow() && row.rowPositionInfo.getLevelNestDepth() > 1 && row.y > this.getVerticalScrollPosition()) {
                const clevel1 = (row.rowPositionInfo.getLevel(this.grid));
                if (clevel1 && clevel1.enableStickyHeaders) {
                    //this means we have atleast one data row which is at a inner level (sub row)
                    nestedSubRow = row;
                    break;
                }

            }
        }
        let nestedHeaderRow;
        if (this.floatingHeaderRowInfo) {
            this.hideFloatingRow();
        }
        if (nestedSubRow) {
            const clevel = (nestedSubRow.rowPositionInfo.getLevel(this.grid));

            if (clevel && clevel.enableStickyHeaders) {
                for (i = nestedSubRow.rowPositionInfo.getRowIndex(); i > 0; i--) {
                    const prevRow = this.itemVerticalPositions[i];
                    if (prevRow.getIsHeaderRow()) {
                        // we found the header rowInfo for this row;
                        nestedHeaderRow = prevRow;
                        break;
                    }
                }
                const existingNestedHeaderRow = this.getRowForRowPositionInfo(nestedHeaderRow);
                if (nestedHeaderRow && (existingNestedHeaderRow == null || existingNestedHeaderRow.y < this.getVerticalScrollPosition())) {
                    if (this.floatingHeaderRowInfo) {
                        //this.rows.removeItem(floatingHeaderRowInfo);
                        //this.saveRowInCache(floatingHeaderRowInfo);
                    }
                    //we have found the nested header row, but it is not drawn. So lets draw it!
                    const floatingHeaderRowPositionInfo = new flexiciousNmsp.RowPositionInfo(nestedHeaderRow.rowData,
                        nestedHeaderRow.getRowIndex(), this.getVerticalScrollPosition(), nestedHeaderRow.getRowHeight(),
                        nestedHeaderRow.getLevel(this.grid), nestedHeaderRow.getRowType());
                    if (this.floatingHeaderRowInfo) {
                        this.floatingHeaderRowInfo.setRowPositionInfo(floatingHeaderRowPositionInfo);
                    }
                    this.floatingHeaderRowInfo = this.processRowPositionInfo(floatingHeaderRowPositionInfo, true, this.floatingHeaderRowInfo, null);
                    this.floatingHeaderRowInfo.showHide(true);
                    if (this.rows.includes(this.floatingHeaderRowInfo)) {
                        UIUtils.removeFromArray(this.rows, this.floatingHeaderRowInfo);
                        //this.rows.removeItemAt(this.rows.getItemIndex(floatingHeaderRowInfo));
                    }

                    for (i = 0; i < this.floatingHeaderRowInfo.cells.length; i++) {
                        const comp = this.floatingHeaderRowInfo.cells[i];
                        //comp.component.parent.addChildAt(comp.component,comp.component.parent.numChildren);
                        comp.component.setStyleAttribute("zIndex", "989");
                        comp.component.setStyleAttribute("opacity", clevel.stickyHeadersAlpha);
                    }
                }
            }
        }

    }

    /**
     * @method hideFloatingRow
     */
    hideFloatingRow() {
        this.floatingHeaderRowInfo.showHide(false);

        for (const comp of this.floatingHeaderRowInfo.cells) {
            const headerCell = comp.component;
            if (headerCell && headerCell.typeName == 'FlexDataGridHeaderCell') {
                headerCell.destroySortIcon();
            }
        }
    }

    /**
     * @private
     * @method snapToColumnWidths
     */
    snapToColumnWidths() {

        flexiciousNmsp.FlexDataGridContainerBase.prototype.snapToColumnWidths.apply(this, []);
        if (this.floatingHeaderRowInfo && this.floatingHeaderRowInfo.cells.length > 0 && this.floatingHeaderRowInfo.visible) {
            this.snapRowToColumnWidth(this.floatingHeaderRowInfo);
        }

    }

    componentDidMountCustom() {
        if(this.grid) {
            if(this.grid.domElement && this.grid.enableHideBuiltInContextMenuItems) {
                this.grid.domElement.oncontextmenu = (e) => {return false}; 
            }
            this.grid.dispatchEvent(new FlexDataGridEvent(FlexDataGridEvent.BODY_CONTAINER_MOUNTED));
            if(this.grid.enableHorizontalScrollOptimizationsBETA){
                this.addEventListener(this.grid, Constants.EVENT_SCROLL, this.grid.onContainerScroll);
            }
    
        }
    }
}

flexiciousNmsp.FlexDataGridBodyContainer = FlexDataGridBodyContainer; //add to name space
FlexDataGridBodyContainer.prototype.typeName = FlexDataGridBodyContainer.typeName = 'FlexDataGridBodyContainer';//for quick inspection

/**
 *
 */