
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import FlexDataGridColumn from './FlexDataGridColumn'
import TriStateCheckBox from '../../controls/TriStateCheckBox'

/**
 * A column to add checkbox selection to the grid.
 * @constructor
 * @class FlexDataGridCheckBoxColumn
 * @namespace flexiciousNmsp
 */
export default class FlexDataGridCheckBoxColumn extends FlexDataGridColumn {
    constructor() {
        super();
        /**
         * In single select datagrids, set this to true for the icon to appear as if it is a radio button
         * 
         * @type {boolean}
         * @property radioButtonMode
         * @default false
         */
        this.radioButtonMode = false;
        /**
         * Flag to enable/disable the top level select checkbox. You would use this if you need the checkbox selection behavior, but do not wish for the user to be able to select all.
         * @type {boolean}
         * @property allowSelectAll
         * @default true
         */
        this.allowSelectAll = true;

        /**
         * Flag to show checkbox as well as a display label. In header cells, shows checkbox and headerText. When you set this to true, the column width  mode switches from fixed to none, and the width switches from 25 to 100 (default column width)
         * @type {boolean}
         * @property enableLabelAndCheckBox
         * @default false
         */
        this.enableLabelAndCheckBox = false;
        //this.setStyle("paddingLeft",6);
        //this.setStyle("headerPaddingLeft",6);
        this.setColumnWidthMode(flexiciousNmsp.FlexDataGridColumn.COLUMN_WIDTH_MODE_FIXED);
        this.setDataCellRenderer(flexiciousNmsp.FlexDataGridCheckBoxColumn.static_FlexDataGridDataCellUIComponent);
        
        /**
         * @type {null}
         * @property itemRenderer
         * @default FlexDataGridCheckBoxColumn.static_TriStateCheckBox
         */
        this.itemRenderer = FlexDataGridCheckBoxColumn.static_TriStateCheckBox;
        
        /**
         * @type {null}
         * @property headerRenderer
         * @default FlexDataGridCheckBoxColumn.static_TriStateCheckBox
         */
        this.headerRenderer = FlexDataGridCheckBoxColumn.static_TriStateCheckBox;

        this.setWidth(45);

        /**
         * @type {boolean}
         * @property sortable
         * @default false
         */
        this.sortable = false;
        
        /**
         * @type {boolean}
         * @property resizable
         * @default false
         */
        this.resizable = false;
        
        /**
         * @type {boolean}
         * @property excludeFromExport
         * @default true
         */
        this.excludeFromExport = true;
        
        /**
         * @type {boolean}
         * @property excludeFromPrint
         * @default true
         */
        this.excludeFromPrint = true;
        
        /**
         * @type {boolean}
         * @property excludeFromSettings
         * @default true
         */
        this.excludeFromSettings = true;

        this.setEditable(false);

        /**
         * @type {boolean}
         * @property enableDataCellOptmization
         * @default false
         */
        this.enableDataCellOptmization = false;

        /**
         * @type {number}
         * @property paddingLeft
         * @default 1
         */
        this.paddingLeft = 1;

        /**
         * @type {number}
         * @property paddingRight
         * @default 1
         */
        this.paddingRight = 1;

        /**
         * @type {number}
         * @property headerPaddingLeft
         * @default 1
         */
        this.headerPaddingLeft = 1;

        /**
         * @type {number}
         * @property headerPaddingRight
         * @default 1
         */
        this.headerPaddingRight = 1;

        this.setHeaderText("");

    }

    getClassNames() {
        return ["FlexDataGridCheckBoxColumn", "FlexDataGridColumn", "CSSStyleDeclaration"
            , "IFactory", "IFixedWidth", "IDataGridFilterColumn"];
    }

    /**
     * For printing, we need to be a  IFactory
     * @method newInstance
     * @return {flexiciousNmsp.FlexDataGridCheckBoxColumn}
     *
     */
    newInstance() {
        return new flexiciousNmsp.FlexDataGridCheckBoxColumn();
    }

    /**
     * Flag to enable/disable the top level select checkbox. You would use this if you need the checkbox selection behavior, but do not wish for the user to be able to select all.
     * @method getEnableSelectAll
     * @return {boolean}
     */
    getEnableSelectAll() {
        return this.allowSelectAll;
    }

    /**
     * @method setEnableSelectAll
     * @param {boolean} value
     */
    setEnableSelectAll(value) {
        this.allowSelectAll = value;
    }

    /**
     * In single select datagrids, set this to true for the icon to appear as if it is a radio button
     * @method getEnableRadioButtonMode
     * @return {boolean} 
     */
    getEnableRadioButtonMode() {
        return this.radioButtonMode;
    }

    /**
     * In single select datagrids, set this to true for the icon to appear as if it is a radio button
     * @method setEnableRadioButtonMode
     * @param {boolean} value
     */
    setEnableRadioButtonMode(value) {
        this.radioButtonMode = value;
    }
}

flexiciousNmsp.FlexDataGridCheckBoxColumn = FlexDataGridCheckBoxColumn; //add to name space 
FlexDataGridCheckBoxColumn.prototype.typeName = FlexDataGridCheckBoxColumn.typeName = 'FlexDataGridCheckBoxColumn';//for quick inspection
FlexDataGridCheckBoxColumn.static_TriStateCheckBox = new flexiciousNmsp.ClassFactory(flexiciousNmsp.TriStateCheckBox, undefined);
FlexDataGridCheckBoxColumn.static_FlexDataGridDataCellUIComponent = new flexiciousNmsp.ClassFactory(flexiciousNmsp.FlexDataGridDataCell, undefined);