/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import FlexDataGridContainerBase from './FlexDataGridContainerBase'

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 

/**
 * The container for header, footer, filter and pager sections of the top level. These
 * rows are locked so they stay in place, i.e, they do not scroll when the user scrolls vertically.
 *
 * When the user scrolls horizontally, these rows will scroll.
 *
 * @constructor
 * @class FlexDataGridHeaderContainer
 * @namespace flexiciousNmsp
 */
export default class FlexDataGridHeaderContainer extends FlexDataGridContainerBase {
    constructor(grid) {
        super(grid);
        /**
         * The type of this container.
         * Can be one of:
         * <ul>
         * <li>ROW_TYPE_HEADER</li>
         * <li>ROW_TYPE_FOOTER</li>
         * <li>ROW_TYPE_PAGER</li>
         * <li>ROW_TYPE_FILTER</li>
         * <li>ROW_TYPE_DATA</li>
         * <li>ROW_TYPE_FILL</li>
         * <li>ROW_TYPE_RENDERER</li>
         * <li>ROW_TYPE_COLUMN_GROUP</li>
         * </ul>
         */
        this.chromeType = flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER;

        /**
         * @type {String}
         * @property verticalScrollPolicy
         * @default "off""
         */
        this.verticalScrollPolicy = "off";

        /**
         * @type {String}
         * @property horizontalScrollPolicy
         * @default "off"
         */
        this.horizontalScrollPolicy = "off";
        this.addEventListener("widthChanged", this.onWidthChanged);
    }

    getClassNames() {
        return ["FlexDataGridHeaderContainer", "FlexDataGridContainerBase", "Container", "UIComponent"];
    }

    /**
     * @method setWidth
     * @param {number} value
     */
    setWidth(value) {
        flexiciousNmsp.FlexDataGridContainerBase.prototype.setWidth.apply(this, [value]);
        this.dispatchEvent(new flexiciousNmsp.BaseEvent("widthChanged"));
    }

    /**
     * @method onWidthChanged
     * @param event
     */
    onWidthChanged(event) {

        if (this.chromeType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER) {
            const cellComp = this.getPagerCell();
            if (cellComp) {
                const cell = cellComp.component;
                if (cell) {
                    cell.setActualSize(this.getPagerWidth(), cell.getHeight());
                    cell.invalidateDisplayList();
                }
            }
        }

    }

    /**
     * @method onCellDropMouseMove
     * @param event
     */
    onCellDropMouseMove(event) {
        if (this.grid.getHeaderContainer().columnDraggingDragCell || this.grid.getHeaderContainer().columnResizingCell) return;

        if (this.grid.enableDrop && (flexiciousNmsp.DisplayList.isMouseDown > 0)) {
            const cell = event.currentTarget;
            if (this.getY() > this.grid.getBodyContainer().getY())//i am below the container
                this.grid.scrollToExistingRow(this.grid.getBodyContainer().getVerticalScrollPosition() + cell.getHeight(), true);
            else
                this.grid.scrollToExistingRow(this.grid.getBodyContainer().getVerticalScrollPosition() - cell.getHeight(), false);

        }

    }

    /**
     * @method getScrollableRect
     * @return {flexiciousNmsp.Rectangle}
     */
    getScrollableRect() {

        const bounds = flexiciousNmsp.FlexDataGridContainerBase.getScrollableRect.apply(this);
        if (this.grid.getBodyContainer().getVerticalScrollBar())
            bounds.setWidth(bounds.getWidth() + this.grid.getBodyContainer().getVerticalScrollBar().getWidth());
        return bounds;

    }

    /**
     * @method createComponents
     * 
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {number} currentScroll
     * @param {object} flat
     */
    createComponents(level, currentScroll, flat) {
        if (typeof currentScroll == "undefined") currentScroll = 0;

        flexiciousNmsp.FlexDataGridContainerBase.prototype.createComponents.apply(this, [level]);
        this.pauseState();
        //        var flat = this.grid.getDataProvider();
        //        if(level.getIsClientFilterPageSortMode())
        //            flat=this.filterPageSort(flat,this.grid.getColumnLevel(),null,true,false,true);
        if (!flat) {
            flat = this.grid.getDataProvider();
            if (level.getIsClientFilterPageSortMode())
                flat = this.filterPageSort(flat, this.grid.getColumnLevel(), null, true, false, true);
        }
        let heightAdded = this.getHeight();
        let heightSoFar = 0;
        const arr = [];
        if (this.chromeType == flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER) {
            var depth = level.getMaxColumnGroupDepth();
            heightAdded /= depth;
            for (let i = 1; i < depth; i++) {
                const pos1 = new flexiciousNmsp.RowPositionInfo(flat, i - 1,
                    heightSoFar, heightAdded, this.grid.getColumnLevel(), flexiciousNmsp.RowPositionInfo.ROW_TYPE_COLUMN_GROUP);
                heightSoFar += heightAdded;

                this.processHeaderLevel(level, pos1, false, flat, flexiciousNmsp.RowPositionInfo.ROW_TYPE_COLUMN_GROUP);

            }
        }
        this.grid.currentPoint.leftLockedHeaderX = this.grid.currentPoint.rightLockedHeaderX = 0;
        const pos = new flexiciousNmsp.RowPositionInfo(flat, depth, heightSoFar, heightAdded,
            this.grid.getColumnLevel(), this.chromeType);
        this.processHeaderLevel(level, pos, false, flat, this.chromeType);
        this.resumeState();
    }

    /*grid.currentPoint.contentY=0;
     for each(var cgPos:RowPositionInfo in arr){
     processHeaderLevel(level,cgPos,false,flat,RowPositionInfo.ROW_TYPE_COLUMN_GROUP);
    }*/
    
    /**
     * @method rootPageChange
     * @param event 
     */
    rootPageChange(event) {

        const evt = new flexiciousNmsp.ExtendedFilterPageSortChangeEvent(flexiciousNmsp.ExtendedFilterPageSortChangeEvent.PAGE_CHANGE);
        evt.triggerEvent = event;
        this.dispatchEvent(evt);

    }

    /**
     * @method onFilterChange
     * @param event 
     */
    onFilterChange(event) {

        flexiciousNmsp.FlexDataGridContainerBase.prototype.onFilterChange.apply(this, [event]);
        const evt = new flexiciousNmsp.ExtendedFilterPageSortChangeEvent(flexiciousNmsp.ExtendedFilterPageSortChangeEvent.FILTER_CHANGE);
        evt.triggerEvent = event;
        this.dispatchEvent(evt);

    }

    /**
     * @method getRowsForRecycling
     * @return {Array}
     */
    getRowsForRecycling() {

        return (this.rows.slice().reverse());

    }

    /**
     * @method getPagerCell
     * @return {flexiciousNmsp.FlexDataGridPagerCell}
     */
    getPagerCell() {
        for (const row of this.rows) {
            for (const cell of row.cells) {
                if (cell.component.implementsOrExtends('FlexDataGridPagerCell')) {
                    return cell;
                }
            }
        }

        return null;
    }

    /**
     * @method getPager
     * @return {flexiciousNmsp.FlexDataGridPagerCell}
     */
    getPager() {

        const cell = this.getPagerCell();
        if (cell && cell.component && (cell.component.implementsOrExtends('FlexDataGridPagerCell')))
            return (cell.component).getRenderer();
        return null;

    }

    /**
     * @method addCell
     * 
     * @param component
     * @param {flexiciousNmsp.RowInfo} row
     * @param {flexiciousNmsp.ComponentInfo} existingComponent
     * 
     * @return 
     */
    addCell(component, row, existingComponent) {
        if (typeof existingComponent == "undefined") existingComponent = null;

        const result = flexiciousNmsp.FlexDataGridContainerBase.prototype.addCell.apply(this, [component, row, existingComponent]);
        result.horizontalSpill = false;
        return result;

    }

    /**
     * Sets the filter value at the top level for the specified field
     * @method setFilterValue
     * @param col
     * @param val
     */
    setFilterValue(col, val) {
        for (const row of this.rows) {
            for (const cell of row.cells) {
                const fdg = cell.component;
                if (fdg && fdg.getColumn() && fdg.getColumn().getSearchField() == col) {
                    const filterControl = fdg.getRenderer();
                    if (filterControl.implementsOrExtends('ISelectFilterControl')) {
                        filterControl.validateNow();
                    }
                    filterControl.setValue(val);
                    filterControl.validateNow();
                    break;
                }
            }
        }
    }

    /**
     *Returns the filter at the top level.
     * @method getFilterArguments
     * @return {Array}
     */
    getFilterArguments() {

        for (let i = 0; i < this.rows.length; i++) {
            const row = this.rows[i];
            if (i == 0)
                return row.getFilterArguments();
        }
        return [];

    }

    /**
     * Sets the filter at the top level.
     *
     * @param {String} fld	 The string to match the searchField property of the filter control with.
     * @return {boolean} True if focus was set, false if otherwise.<br/>
     * Focus may not be set if the given search field does not exist, <br/>
     * or if the filter control for the given search field does not  <br/>
     * implement IFocusManagerComponent.
     */
    setFilterFocus(fld) {

        for (let i = 0; i < this.rows.length; i++) {
            const row = this.rows[i];
            if (i == 0)
                return row.setFilterFocus(fld);
        }
        return false;

    }

    /**
     * @method getMaxHorizontalScrollPosition
     * @return {number}
     */
    getMaxHorizontalScrollPosition() {
        return this.grid.getBodyContainer().getMaxHorizontalScrollPosition();
    }

    /**
     * @method setWidth
     * @param {number} w width
     */
    setWidth(w) {
        flexiciousNmsp.UIComponent.prototype.setWidth.apply(this, [w]);
        if (this.grid && this.grid.getPager() && this == this.grid.getPagerContainer()) {
            this.grid.getPager().parent.setWidth(this.getPagerWidth());
        }
    }
}


flexiciousNmsp.FlexDataGridHeaderContainer = FlexDataGridHeaderContainer; //add to name space
FlexDataGridHeaderContainer.prototype.typeName = FlexDataGridHeaderContainer.typeName = 'FlexDataGridHeaderContainer';//for quick inspection