/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import FlexDataGridBodyContainer from './FlexDataGridBodyContainer'
import Rectangle from '../utils/Rectangle'
/**
 * The container for the body that supports virtual scrolling
 * @constructor
 * @class FlexDataGridVirtualBodyContainer
 */
export default class FlexDataGridVirtualBodyContainer extends FlexDataGridBodyContainer {
    constructor(grid) {
        super(grid);

        /**
         * @private  
         */
        this._openItemVerticalPositions=[];

        /**
         * @private 
         */
        this._pendingActiveCell = null;
        /**
         * The timer that launches the actual scroll so we do not fire server calls as the user is scrolling.
         * @property virtualScrollTimer
         * @default null
         */
        this.virtualScrollTimer = null;

        /**
         * Delay in milliseconds before "virtualScroll" event is dispatched.
         * @type {Number}
         * @property virtualScrollDelay
         * @default 500
         */
        this.virtualScrollDelay = 500;

        /**
         * The current viewport rectange
         * @type {flexiciousNmsp.Rectangle}
         * @property loadedViewPort
         * @default flexiciousNmsp.Rectangle
         */
        this.loadedViewPort = new flexiciousNmsp.Rectangle();

    }
    /**
     * @method getClassNames
     * @return Array
     */ 
    getClassNames() {
        return ["FlexDataGridVirtualBodyContainer","FlexDataGridBodyContainer","FlexDataGridContainerBase","Container","UIComponent"];
    }

    /**
     * @method calculateTotalHeight
     * 
     * @param {object} flat
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {number} heightSoFar
     * @param {number} nestLevel
     * @param {flexiciousNmsp.RowPositionInfo} expanding
     * @param {Array} addedRows
     * @param {object} parentObject
     * @param {boolean} isRecusirve
     * 
     * @return {number} total height
     */
    calculateTotalHeight(
        flat,
        level,
        heightSoFar,
        nestLevel,
        expanding,
        addedRows,
        parentObject,
        isRecusirve) {
        if(typeof flat=="undefined")flat=null;
        if(typeof  level=="undefined") level=null;
        if(typeof  heightSoFar=="undefined") heightSoFar=0;
        if(typeof  nestLevel=="undefined") nestLevel=0;
        if(typeof  expanding=="undefined") expanding=null;
        if(typeof  addedRows=="undefined") addedRows=null;
        if(typeof  parentObject=="undefined") parentObject=null;
        if(typeof  isRecusirve=="undefined") isRecusirve=false;

        if(!this.grid.enableVirtualScroll || expanding)
            return flexiciousNmsp.FlexDataGridBodyContainer.prototype.calculateTotalHeight.apply(this,[flat, level, heightSoFar, nestLevel, expanding, addedRows, parentObject,isRecusirve]);


        if(this._heightCalculated) //if we are not calculating the seed.
            return this._calculatedTotalHeight;

        if(flat==null)//this is the first time we call calculate height. super will take care of populating the visible records.
        {
            flexiciousNmsp.FlexDataGridBodyContainer.prototype.calculateTotalHeight.apply(this,[flat, level, heightSoFar, nestLevel, expanding, addedRows, parentObject]);
            this._calculatedTotalHeight = this.grid.getTotalRecords()*(this.grid.getRowHeight());

        }
        return this._calculatedTotalHeight;

    }

    /**
     * @method handleArrowKey
     * 
     * @param {flexiciousNmsp.FlexDataGridCell} cell
     * @param {number} keyCode
     * @param triggerEvent
     * 
     * @return {boolean}
     */
    handleArrowKey(cell, keyCode, triggerEvent) {

        if(this.grid.enableVirtualScroll){
            if(keyCode==Constants.KEYBOARD_END){
                this.gotoVerticalPosition(this.getMaxVerticalScrollPosition());
            }
            else if(keyCode==Constants.KEYBOARD_HOME){
                this.gotoVerticalPosition(0);
            }
            else if(keyCode==Constants.KEYBOARD_PAGE_DOWN){
                this.gotoVerticalPosition(Math.min(this.getVerticalScrollPosition()+this.getHeight(),this.getMaxVerticalScrollPosition()));
            }
            else if(keyCode==Constants.KEYBOARD_PAGE_UP){
                this.gotoVerticalPosition(Math.max(this.getVerticalScrollPosition()-this.getHeight(),0));
            }
            else{
                if(keyCode==Constants.KEYBOARD_UP||keyCode==Constants.KEYBOARD_DOWN){
                    this._keyboardhandled=true;
                    if(this.isYOutOfVisibleArea(cell.rowInfo.getY()-cell.getHeight()-cell.getHeight(),cell.getHeight()) && keyCode==Constants.KEYBOARD_UP){
                        this._pendingActiveCell = new CellInfo(0,cell.getColumn());
                        this.gotoVerticalPosition(Math.max(this.getVerticalScrollPosition()-this.grid.getColumnLevel().getRowHeight(),0));
                    }else if(this.isYOutOfVisibleArea(cell.rowInfo.getY()+cell.getHeight(),cell.getHeight()) && keyCode==Constants.KEYBOARD_DOWN){
                        this._pendingActiveCell = new CellInfo(this.rows.length,cell.getColumn());
                        this.gotoVerticalPosition(Math.min(this.getVerticalScrollPosition()+this.grid.getColumnLevel().getRowHeight(),this.getMaxVerticalScrollPosition()));
                    }else
                        flexiciousNmsp.FlexDataGridBodyContainer.prototype.handleArrowKey.apply(this,[cell,keyCode,triggerEvent]);
                    return false;
                }
                else
                    return flexiciousNmsp.FlexDataGridBodyContainer.prototype.handleArrowKey.apply(this,[cell,keyCode,triggerEvent]);
            }

            return false;
        }else{
            return flexiciousNmsp.FlexDataGridBodyContainer.prototype.handleArrowKey.apply(this,[cell,keyCode,triggerEvent]);
        }

    }

    /**
     * @method gotoVerticalPosition
     * @param {number} vsp
     */
    gotoVerticalPosition(vsp) {
        if(this.getHeight()==0)return;
        if(this.grid.enableVirtualScroll){
            this.setVerticalScrollPosition(vsp);
            this.grid.synchronizeLockedVerticalScroll();
            this.dispatchScroll(vsp,undefined);
        }else{
            flexiciousNmsp.FlexDataGridBodyContainer.prototype.gotoVerticalPosition.apply(this,[vsp]);
        }

    }

    /**
     * The current viewport rectange
     * @method setCurrentRowAtScrollPosition
     * @param {number} vsp
     * 
     * @return {boolean} 
     */
    setCurrentRowAtScrollPosition(vsp) {

        if(!this.grid.enableVirtualScroll){
            return flexiciousNmsp.FlexDataGridBodyContainer.prototype.setCurrentRowAtScrollPosition.apply(this,[vsp]);
        }
        const requestedViewport = new flexiciousNmsp.Rectangle();
        requestedViewport.setY(Math.max(0,vsp-this.grid.getColumnLevel().getRowHeight()));
        requestedViewport.setHeight(this.getHeight());
        requestedViewport.setWidth(this.getWidth()-1);
        requestedViewport.setX(0);


        if(this.grid.showSpinnerOnFilterPageSort){
            this.grid.showSpinner();
        }

        if(this.grid.enableDrawAsYouScroll||!this.grid._isScrolling){
            if(this.virtualScrollDelay>0 && !this.grid.enableDrawAsYouScroll){
                if (this.virtualScrollTimer == null)
                {
                    this.virtualScrollTimer = new flexiciousNmsp.Timer(this.virtualScrollDelay, 1);
                    this.virtualScrollTimer.addEventListener(this,Constants.EVENT_TIMER_COMPLETE, this.onTimerComplete,false,0,true);
                }
                this.virtualScrollTimer.reset(); // reset if already set...
                this.virtualScrollTimer.repeatCount = 1;
                // starts the timer ticking
                this.virtualScrollTimer.start();
            }
            else{
                this.dispatchScroll(vsp,undefined);
            }
        }

        return false;//we will recycle when we get data from server.

    }

    /*if(loadedViewPort.containsRect(requestedViewport))return false;*/

    /**
     * @method onTimerComplete
     * @param event
     */
    onTimerComplete(event) {
        if(this.virtualScrollTimer){
            this.virtualScrollTimer.stop();
            this.virtualScrollTimer.removeEventListener(Constants.EVENT_TIMER_COMPLETE, this.onTimerComplete,false);
        }
        this.dispatchScroll(this.getVerticalScrollPosition(),undefined);
        this.virtualScrollTimer=null;

    }

    /**
     * @method getOpenVirtualScrollInfo
     * 
     * @param {number} vsp
     * @param {number} levelNestDepth
     * 
     * @return {object}
     */
    getOpenVirtualScrollInfo(vsp, levelNestDepth) {
        if(typeof levelNestDepth=="undefined")levelNestDepth=-1;

        const arr = this._openItemVerticalPositions.slice();
        arr.sort((a, b) => a.getLevelNestDepth()-b.getLevelNestDepth());
        arr.reverse();

        const withinItemArea=false;

        for (const vsli of arr) {
            if(vsli.getVerticalScrollPosition()<=vsp &&
                vsp<(vsli.verticalScrollPosition+vsli.getItemArea()-1)){
                //this means we're in the open area of vsli.
                if(levelNestDepth==-1 || levelNestDepth==vsli.levelNestDepth)
                    return vsli
            }
        }

        return null;
    }

    /**
     * @method doScroll
     */
    doScroll() {

        this.dispatchScroll(this.getVerticalScrollPosition(),undefined);

    }

    /**
     * @method dispatchScroll
     * 
     * @param {number} vsp
     * @param {array} recordsOnPage
     * 
     */
    dispatchScroll(vsp, recordsOnPage) {
        if(typeof recordsOnPage=="undefined")recordsOnPage=null;

        let pointer=Math.max(0,vsp-this.grid.getColumnLevel().getRowHeight());
        this.loadedViewPort.setY(pointer);
        recordsOnPage=[];
        const end=Math.min(this.getMaxVerticalScrollPosition()+this.getHeight(),
            (vsp+(this.getHeight())+(3*this.grid.getColumnLevel().getRowHeight())));
        while(pointer<end)
        {
            if(pointer>=this.getCalculatedTotalHeight()){
                break;
            }
            pointer+=this.addRecord(recordsOnPage,pointer);

        }

        this.loadedViewPort.setHeight(end);
        this.loadedViewPort.setWidth(this.getWidth());
        this.loadedViewPort.setX(0);

        const vsce = new flexiciousNmsp.FlexDataGridVirtualScrollEvent(flexiciousNmsp.FlexDataGridVirtualScrollEvent.VIRTUAL_SCROLL,this.grid,recordsOnPage);
        this.grid.dispatchEvent(vsce);

    }

    /**
     * @method getVpFromRecordIndex
     * 
     * @param {number} vsp
     * @param {number} recordIndex
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} expandingLevel
     * 
     * @return {boolean}
     */
    getVpFromRecordIndex(vsp, recordIndex, expandingLevel) {

        let openArea = expandingLevel.getNestDepth()==1?null:this.getOpenVirtualScrollInfo(vsp,expandingLevel.getNestDepth()-1);
        let atTop=false;
        if(!openArea){
            openArea = new flexiciousNmsp.VirtualScrollLoadInfo(this.grid.getColumnLevel(),0,0,this._calculatedTotalHeight,null,null,flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
            atTop=true;
        }
        const anchorArea=this.getAnchorArea(openArea,vsp,expandingLevel);
        if(!anchorArea){
            //this means there are no open areas, or open areas are below us, just use recordIndex.
            return openArea.verticalScrollPosition+(expandingLevel.reusePreviousLevelColumns||expandingLevel.getNestDepth()==1?0:expandingLevel.getHeaderHeight())+(recordIndex*expandingLevel.getRowHeight())+(atTop?0:openArea.level.getRowHeight());
        }else{
            return anchorArea.verticalScrollPosition+anchorArea.getItemArea()+((recordIndex-anchorArea.recordIndex)*expandingLevel.getRowHeight())-expandingLevel.getRowHeight();
        }

    }

    /**
     * @method getTopLevelRecordIndex
     * @param {number} vsp
     * @return {number} top level record index
     * 
     */
    getTopLevelRecordIndex(vsp) {

        let openArea = this.getOpenVirtualScrollInfo(vsp,1);//get open area at root level
        if(!openArea){
            openArea = new flexiciousNmsp.VirtualScrollLoadInfo(this.grid.getColumnLevel(),0,0,this._calculatedTotalHeight,null,null,flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
            const anchorArea=this.getAnchorArea(openArea,vsp,this.grid.getColumnLevel());
            if(!anchorArea){
                return Math.floor(vsp/this.grid.getColumnLevel().getRowHeight());//above all open areas
            }else{
                return Math.floor( (anchorArea.recordIndex)+((vsp-anchorArea.getVerticalScrollPosition()-anchorArea.getItemArea())/this.grid.getColumnLevel().getRowHeight()));
            }
        }else{
            return openArea.recordIndex;
        }

    }

    /**
     * @method addRecord
     * 
     * @param {Array} recordsOnPage
     * @param {number} vsp
     * 
     * @return {number} 
     */
    addRecord(recordsOnPage, vsp) {


        let openArea = this.getOpenVirtualScrollInfo(vsp);
        let levelBeingAdded;
        let atTop=false;
        if(!openArea){
            openArea = new flexiciousNmsp.VirtualScrollLoadInfo(this.grid.getColumnLevel(),0,0,this._calculatedTotalHeight,null,null,flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
            levelBeingAdded = this.grid.getColumnLevel();
            atTop=true
        }else{
            levelBeingAdded=openArea.level.nextLevel?openArea.level.nextLevel:openArea.level;
        }
        const anchorArea=this.getAnchorArea(openArea,vsp,levelBeingAdded);
        let myRecordIndex=0;
        let myVp=0;
        let parent=null;
        if(!anchorArea){
            if(vsp<(openArea.verticalScrollPosition+openArea.level.getRowHeight())){
                //this means we are in the parent record.
                myRecordIndex=openArea.recordIndex;
                levelBeingAdded=openArea.level;
                myVp=openArea.verticalScrollPosition;
                parent=levelBeingAdded.getNestDepth()==1?null:this.getParentOpenItem(openArea.verticalScrollPosition,openArea.levelNestDepth).item;
            }else if(levelBeingAdded&&
                (levelBeingAdded.getNestDepth()>1) &&
                (!levelBeingAdded.reusePreviousLevelColumns) &&
                (vsp<(openArea.verticalScrollPosition+openArea.level.getRowHeight()+levelBeingAdded.getHeaderHeight()))){
                //this means we are in the header section of the next elvel.
                //just add the header height, and return;
                parent=levelBeingAdded.getNestDepth()==1?null:openArea.item;
                recordsOnPage.push(new flexiciousNmsp.VirtualScrollLoadInfo(levelBeingAdded,
                    -1
                    ,vsp
                    ,levelBeingAdded.getHeaderHeight()
                    ,parent
                    ,parent
                    ,flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER));
                return levelBeingAdded.getHeaderHeight();
            }
            else{
                //this means we're above any open areas within this open area, or there are no child open areas.
                myRecordIndex=(vsp-openArea.verticalScrollPosition-(atTop?0:openArea.level.getRowHeight()))/levelBeingAdded.getRowHeight();
                myRecordIndex=Math.floor(myRecordIndex);
                let headerHeight=0;
                if(levelBeingAdded.getNestDepth()>1 && !levelBeingAdded.reusePreviousLevelColumns){
                    myRecordIndex--;
                    headerHeight=levelBeingAdded.getHeaderHeight();
                }
                myVp = openArea.verticalScrollPosition + (myRecordIndex*levelBeingAdded.getRowHeight()) + (atTop?0:openArea.level.getRowHeight()) + headerHeight;
                parent=levelBeingAdded.getNestDepth()==1?null:openArea.item;
            }
        }else{
            //this means we are below a referencable open area, and we can calculate our row index.
            myRecordIndex=(anchorArea.recordIndex)+((vsp-anchorArea.verticalScrollPosition-anchorArea.itemHeight)/levelBeingAdded.getRowHeight());
            myRecordIndex=Math.floor(myRecordIndex);
            myVp = anchorArea.verticalScrollPosition +anchorArea.getItemArea() + ((myRecordIndex-anchorArea.recordIndex-1)*levelBeingAdded.getRowHeight());
            parent=levelBeingAdded.getNestDepth()==1?null:openArea.item;
        }
        const heightAdded=levelBeingAdded.getRowHeight();
        if(openArea.level.enableFooters){

            if(vsp>=(openArea.verticalScrollPosition+openArea.getItemArea()-openArea.level.getFooterRowHeight())){
                //we are adding the last record of a child level.
                recordsOnPage.push(new flexiciousNmsp.VirtualScrollLoadInfo(levelBeingAdded,
                    -1
                    ,vsp
                    ,levelBeingAdded.getFooterRowHeight()
                    ,parent
                    ,parent
                    ,flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER));
                return levelBeingAdded.getFooterRowHeight();
            }
        }
        recordsOnPage.push(new flexiciousNmsp.VirtualScrollLoadInfo(levelBeingAdded,
            myRecordIndex
            ,myVp
            ,levelBeingAdded.getRowHeight()
            ,null
            ,parent
            ,flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA));
        return heightAdded;

    }

    /**
     * @method getAnchorArea
     * 
     * @param {flexiciousNmsp.VirtualScrollLoadInfo} openArea
     * @param {number} vsp
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * 
     * @return {flexiciousNmsp.VirtualScrollLoadInfo} 
     */
    getAnchorArea(openArea, vsp, level) {
        //get the child open area immediately preceding the VSP in the given open area.
        const nextLevelChildren=[];
        const arr = this._openItemVerticalPositions.slice();
        arr.sort((a, b) => a.verticalScrollPosition-b.verticalScrollPosition);
        arr.reverse();

        for (const vsliLvl of arr) {
            if((vsliLvl.level==level)
                && (vsliLvl.verticalScrollPosition>=openArea.verticalScrollPosition)
                && ((vsliLvl.verticalScrollPosition+vsliLvl.getItemArea())<=(openArea.verticalScrollPosition+openArea.getItemArea()))
                && ((vsliLvl.verticalScrollPosition+vsliLvl.getItemArea())<=vsp)
                )
                return (vsliLvl);
        }

        return null;
    }

    /**
     * @method clearOpenItems
     */
    clearOpenItems() {

        this._openItemVerticalPositions=[];
        flexiciousNmsp.FlexDataGridBodyContainer.prototype.clearOpenItems.apply(this,[]);

    }

    /**
     * @method findRowPositionInfo
     * @param {Object} item
     * @return {flexiciousNmsp.RowPositionInfo} 
     */
    findRowPositionInfo(item) {

        //need to keep track of vertical positions of open items.
        let rowPositionInfo=null;
        const expCell = this.getCellForRowColumn(item,null,true);
        if(expCell){
            rowPositionInfo=expCell.rowInfo.rowPositionInfo;
        }
        if(rowPositionInfo==null){
            //this means we are expanding something that has not been drawn. With virtual scrolling this.implementsOrExtends('not possible, because we need to track the vertical positions of all open item')){
            throw new Error("Cannot expand an item that has not been drawn when enableVirtualScroll='true'.")
        }
        return rowPositionInfo;

    }

    /**
     * @method getChromeHeight
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} nextLvl
     * @return {number}
     */
    getChromeHeight(nextLvl) {

        return ((nextLvl.reusePreviousLevelColumns?0:nextLvl.getHeaderHeight()) +(nextLvl.enableFooters?nextLvl.getFooterRowHeight():0)+(nextLvl.forcePagerRow?nextLvl.getPagerRowHeight():0));

    }

    /**
     * @method getParentOpenItemFromObject
     * @param {object} item
     * @return {object}
     */
    getParentOpenItemFromObject(item) {
        for (const row of this.itemVerticalPositions) {
            if(row.virtualScrollLoadInfo && (row.virtualScrollLoadInfo.level.areItemsEqual(row.virtualScrollLoadInfo.item,item))){
                return row.virtualScrollLoadInfo.parent;
            }
        }

        return null;
    }

    /**
     * @method getParentOpenItem
     * 
     * @param {number} verticalPosition
     * @param {number} myNestDepth
     * 
     * @return {flexiciousNmsp.VirtualScrollLoadInfo}
     */
    getParentOpenItem(verticalPosition, myNestDepth) {
        const arr = this._openItemVerticalPositions.slice();
        arr.sort((a, b) => a.getVerticalPosition()-b.getVerticalPosition());
        arr.reverse();

        for (const item of arr) {
            if(item.verticalScrollPosition<verticalPosition
                && item.levelNestDepth<myNestDepth){
                return item;
            }
        }

        return null;
    }

    /**
     * @method onChildHeaderClicked
     * @param {flexiciousNmsp.FlexDataGridHeaderCell} headerCell
     */
    onChildHeaderClicked(headerCell) {

        if(this.grid.enableVirtualScroll){
            //simply dispatchScroll, which should handle filter/sort
            this.dispatchScroll(this.getVerticalScrollPosition());
        }
        else
            flexiciousNmsp.FlexDataGridBodyContainer.prototype.onChildHeaderClicked.apply(this,[headerCell]);

    }

    /**
     * @method clearAllCollections
     */
    clearAllCollections() {

        flexiciousNmsp.FlexDataGridBodyContainer.prototype.clearAllCollections.apply(this,[]);
        this._openItemVerticalPositions=[];

    }

    /**
     * @method addOpenItem
     * 
     * @param {Object} item
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     * @param {boolean} check
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} lvl
     */
    addOpenItem(item, rowPositionInfo, check, lvl) {
        if(typeof rowPositionInfo=="undefined")rowPositionInfo=null;
        if(typeof check=="undefined")check=true;


        flexiciousNmsp.FlexDataGridBodyContainer.prototype.addOpenItem.apply(this,[item,rowPositionInfo,check,lvl]);
        if(this.grid.enableVirtualScroll){
            let itemPosition=rowPositionInfo.getVerticalPosition();
            if(rowPositionInfo==null){
                rowPositionInfo=this.findRowPositionInfo(item);

            }
            const cell=this.getCellForRowColumn(rowPositionInfo.rowData,null,true);
            var lvl=rowPositionInfo.getLevel(this.grid);
            const nextLvl=lvl.nextLevel;
            const numChildren=UIUtils.resolveExpression(item,lvl.childrenCountField);
            let vsli;

            if(rowPositionInfo.virtualScrollLoadInfo){
                vsli = new flexiciousNmsp.VirtualScrollLoadInfo(lvl,rowPositionInfo.virtualScrollLoadInfo.recordIndex,
                    this.getVpFromRecordIndex(rowPositionInfo.getVerticalPosition(),rowPositionInfo.virtualScrollLoadInfo.recordIndex,lvl)
                    ,lvl.getRowHeight(),rowPositionInfo.rowData,null,flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
            }else{
                itemPosition =  cell.setY();//-(cell.getY()%lvl.rowHeight);
                const itemRecords=[];
                this.addRecord(itemRecords,itemPosition);
                if(itemRecords.length==0){
                    //this will only be zero if we are in the header on basis of vsp.
                    itemPosition+=lvl.getRowHeight();
                    this.addRecord(itemRecords,itemPosition);
                }
                vsli=itemRecords[0];
                vsli.item=rowPositionInfo.rowData;
                vsli.verticalScrollPosition=itemPosition;

            }
            vsli.itemHeight = (numChildren*nextLvl.getRowHeight())+this.getChromeHeight(nextLvl);


            for (const openItem of this._openItemVerticalPositions) {
                if(openItem.verticalScrollPosition>=vsli.verticalScrollPosition
                ){
                    openItem.verticalScrollPosition+=(vsli.itemHeight);
                    //move all other open areas down.
                }
            }

            let parent=this.getParentOpenItem(vsli.verticalScrollPosition,vsli.levelNestDepth);
            while(parent){
                parent.itemHeight+=vsli.itemHeight;
                parent=this.getParentOpenItem(parent.verticalScrollPosition,parent.levelNestDepth);
            }
            this._openItemVerticalPositions.push(vsli);
            this._calculatedTotalHeight+=(vsli.itemHeight);

            this.dispatchScroll(this.getVerticalScrollPosition());
            this.setVirtualScrollProperties();
        }

    }

    /**
     * @method removeOpenItem
     * @param {Object} item
     * @param {flexiciousNmsp.RowPositionInfo} rowPositionInfo
     */
    removeOpenItem(item, rowPositionInfo) {
        if(typeof rowPositionInfo=="undefined")rowPositionInfo=null;

        if(this.grid.enableVirtualScroll){
            if(rowPositionInfo==null){
                rowPositionInfo=this.findRowPositionInfo(item);
            }
            const itemLevel=rowPositionInfo.getLevel(this.grid);
            for(let i=0;i<this._openItemVerticalPositions.length;i++){
                var vscli=this._openItemVerticalPositions[i];
                const lvl= vscli.level;
                if((itemLevel==lvl) &&
                    lvl.areItemsEqual(vscli.item,item)){
                    break;
                }
            }
            this.removeOpenItemVerticalPosition(vscli,rowPositionInfo.getVerticalPosition());
            if(this._openItemVerticalPositions.length!=this.openItems.length){
                throw new Error("Invalid state - cannot synchronize open items and openItemVerticalPositions. This may happen if you directly modify the openItems collection when enableVirtualScroll=true. Please use the addOpenItem and removeOpenItem methods instead.")


            }
            this.dispatchScroll(this.getVerticalScrollPosition());
            this.setVirtualScrollProperties();
        }else{
            flexiciousNmsp.FlexDataGridBodyContainer.prototype.removeOpenItem.apply(this,[item,rowPositionInfo]);

        }


    }

    /**
     * @method removeOpenItemVerticalPosition
     * @param {flexiciousNmsp.VirtualScrollLoadInfo} toRemove
     * @param {number} vPos
     */
    removeOpenItemVerticalPosition(toRemove, vPos) {
        let openItem;
        let i;
        const childrenToRemove=[];
        for(i=0;i<this._openItemVerticalPositions.length;i++){
            openItem=this._openItemVerticalPositions[i];
            if(toRemove.isParentOf(openItem)){
                childrenToRemove.push(openItem);

            }
        }
        for(i=0;i<childrenToRemove.length;i++){
            const child=childrenToRemove[i];
            this.removeOpenItemVerticalPosition(child,child.verticalScrollPosition)
        }

        this.openItems.splice(this.openItems.indexOf(toRemove.item),1);
        const removed=(this._openItemVerticalPositions.splice(this._openItemVerticalPositions.indexOf(toRemove),1)[0]);

        this._calculatedTotalHeight-=(removed.itemHeight);
        for(i=0;i<this._openItemVerticalPositions.length;i++){
            openItem=this._openItemVerticalPositions[i];

            if(openItem.verticalScrollPosition<=vPos &&
                vPos<(openItem.verticalScrollPosition+openItem.itemHeight)){
                //this means we are within another open area, hwhich.implementsOrExtends('our parent, so we need to remove from i')){
                openItem.itemHeight-=removed.itemHeight;
            }
            if(openItem.verticalScrollPosition>vPos){
                openItem.verticalScrollPosition-=(removed.itemHeight);
            }
        }
    }

    /**
     * @method setVirtualScrollProperties
     */
    setVirtualScrollProperties() {

//        var newViewableHeight = this.getHeight();
//        newViewableHeight = Math.floor(newViewableHeight);
//        if (this.getHorizontalScrollBar() && this.horizontalScrollPolicy != ScrollPolicy.ON)
//            newViewableHeight -= this.getHorizontalScrollBar().minHeight;
//        var vm = this.viewMetrics;
//        newViewableHeight -= (vm.top + vm.bottom);
//        if(this.getVerticalScrollBar())
//            this.getVerticalScrollBar().setScrollProperties(
//                newViewableHeight, 0,
//                _calculatedTotalHeight-newViewableHeight, verticalPageScrollSize);
        //todo

    }

    /**
     * @method setVirtualScrollData
     * @param {Array} arr
     */
    setVirtualScrollData(arr) {
        if(this.grid.showSpinnerOnFilterPageSort){
            this.grid.hideSpinner();
        }
        const _pendingHeaders=[];
        this.itemVerticalPositions=[];
        let soFar=-1;
        let i;
        for(i=0;i<arr.length;i++){
            const vsli=arr[i];
            if(soFar==-1)
                soFar=vsli.verticalScrollPosition;
            if(vsli.getRowType()==flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA){
                soFar+=this.addToVerticalPositions(vsli.item,soFar,vsli.level,null,flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA);
                this.itemVerticalPositions[this.itemVerticalPositions.length-1].virtualScrollLoadInfo=vsli;
            }else{
                soFar+=this.processSection([],0
                    ,vsli.parent,
                    soFar,vsli.level,vsli.parent,
                    vsli.getRowType(),null
                    ,null);
            }
            this.itemVerticalPositions[this.itemVerticalPositions.length-1].rowData=vsli.item;
        }
        this.grid.currentPoint.reset();
        this.removeAllComponents(true);
        this.addMatchSticks(null);
        for(i=0;i<this.itemVerticalPositions.length;i++){
            const ipos=this.itemVerticalPositions[i];
            this.processItemPositionInfoUsingCache(ipos,this.rows.length,true);
        }

        if(this._pendingActiveCell && this.rows.length>0) {

            const rowInfo=this._pendingActiveCell.rowData.toString()=="0"?this.getFirstVisibleRow():this.getLastVisibleRow();
            if(rowInfo){
                let cell=rowInfo.getCellForColumn(this._pendingActiveCell.getColumn()).component;
                if(!cell && rowInfo.cells.length>0){
                    cell=rowInfo.cells[0]
                }
                if(cell){
                    this.grid.currentCell=cell;
                    this.grid.highlightRow(cell,cell.rowInfo,true);
                    this.grid.traceValue=cell.getText();
                }
                this._pendingActiveCell=null;
            }
        }
    }

    /**
     * @method getFirstVisibleRow
     * @return {flexiciousNmsp.RowInfo}
     */
    getFirstVisibleRow() {
        const rowsCopy=this.rows.slice();
        UIUtils.sortOn(rowsCopy,"y");

        for (const row of rowsCopy) {
            if(row.getY()>this.getVerticalScrollPosition()){
                return row;
            }
        }

        if(rowsCopy.length>0)
            return rowsCopy[0];
        return null;
    }

    /**
     * @method getLastVisibleRow
     * @return {flexiciousNmsp.RowInfo}
     */
    getLastVisibleRow() {
        const rowsCopy=this.rows.slice();
        UIUtils.sortOn(rowsCopy,"y");

        for (const row of rowsCopy) {
            if((row.getY()+(2*row.rowPositionInfo.getLevel(this.grid).getRowHeight()))>(this.getVerticalScrollPosition()+this.getHeight())){
                return row;
            }
        }

        if(rowsCopy.length>0)
            return rowsCopy[rowsCopy.length-1];
        return null;
    }

    /**
     * @method recycle
     * 
     * @param {flexiciousNmsp.FlexDataGridColumnLevel} level
     * @param {boolean} scrollDown
     * @param {number} scrollDelta
     * @param {boolean} scrolling
     */
    recycle(level, scrollDown, scrollDelta, scrolling) {
        if(typeof  scrollDelta=="undefined") scrollDelta=0;
        if(typeof scrolling=="undefined")scrolling=true;

        if(!this.grid.enableVirtualScroll){
            flexiciousNmsp.FlexDataGridBodyContainer.prototype.recycle.apply(this,[level,scrollDown,scrollDelta,scrolling]);
        }

    }
}

flexiciousNmsp.FlexDataGridVirtualBodyContainer = FlexDataGridVirtualBodyContainer; //add to name space 
FlexDataGridVirtualBodyContainer.prototype.typeName = FlexDataGridVirtualBodyContainer.typeName = 'FlexDataGridVirtualBodyContainer';//for quick inspection