
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import UIComponent from '../../core/UIComponent'
import Spinner from '../../controls/Spinner'
import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
const uiUtil = UIUtils;
/**
 * Base class for the FlexDataGrid. Please refer to the FlexDataGrid class for details
 * @constructor
 * @class NdgBase
 * @namespace com.flexicious.lic
 */
export default class NdgBase extends UIComponent {

    constructor(props, arg1, arg2, tag) {
        super(props, arg1, arg2, tag, false)

        /**
         * The default number of rows to display.
         * @type {number}
         * @property defaultRowCount
         * @default 4
         */
        this.defaultRowCount = 4;

        /**
         * @private
         */
        this._lastPrintOptionsString = null;
       
        /**
         * @private
         */
        this.persistedPrintOptionsString = null;
        
        /**
         * @private
         */
        this._created = false;
        
        /**
         * @private
         */
        this._dataProvider = [];
        
        /**
         * @private
         */
        this._placementDirty = false;
       
        /**
         * @private
         */
        this._snapDirty = false;
        
        /**
         * @private
         */
        this._heightIncreased = false;
        
        /**
         * @private
         */
        this._widthIncreased = false;
        
        /**
         * @private
         */
        this._useEstimation = false;
        
        /**
         * @private
         */
        this._structureDirty = false;
        
        /**
         * @private
         */
        this._componentsCreated = false;
        
        /**
         * @private
         */
        this._selectionInvalid = false;
        
        /**
         * @type {boolean}
         * @property showSpinnerOnFilterPageSort
         * @default false
         */
        this.showSpinnerOnFilterPageSort = false;
        
        /**
         * @property itemFilters
         */
        this.itemFilters = {};

        /**
         * @property dropIndicator
         */
        this.dropIndicator = new flexiciousNmsp.UIComponent("span");
        
        /**
         * @private
         */
        this._leftLockedVerticalSeparator = new flexiciousNmsp.UIComponent("span");
        
        /**
         * @private
         */
        this._rightLockedVerticalSeparator = new flexiciousNmsp.UIComponent("span");
        
        /**
         * @private
         */
        this._flattenedLevels = {};
       
        /**
         * @private
         */
        this.printExportParameters = null;
       
        /**
         * @private
         */
        this._printExportData = null;
        
        /**
         * @property spinnerFactory
         * @default null
         */
        this.spinnerFactory = null;//new flexiciousNmsp.ClassFactory(flexiciousNmsp.Spinner, null, true);
        
        /**
         * @type {String}
         * @property spinnerLabel
         * @default "Loading, Please wait.."
         */
        this.spinnerLabel = "Loading, Please wait..";
        
        /**
         * Context Menu Item to copy the current cell
         * @private
         */
        this._copyCellContextMenuItem = null;

        /**
         * Context Menu Item to copy the current row
         * @private
         */
        this._copyRowContextMenuItem = null;

        /**
         * Context Menu Item to copy the entire page
         * @private
         */
        this._copyTableContextMenuItem = null;

        /**
         * Text for the Copy Cell Menu
         * @type {String}
         * @property copyCellMenuText
         * @default "Copy Cell"
         */
        this.copyCellMenuText = "Copy Cell";

        /**
         * Text for the Copy Selected Row
         * @type {String}
         * @property copyCellMenuText
         * @default "Copy Row"
         */
        this.copySelectedRowMenuText = "Copy Row";

        /**
         * Text for the Copy All Rows
         * @type {String}
         * @property copyCellMenuText
         * @default "Copy All Rows"
         */
        this.copyAllRowsMenuText = "Copy All Rows";

        /**
         * @private
         */
        this._lastTrace = null;

        /**
         * @private
         */
        this._totalTime = 0;

        /**
         * @property traceValue
         * @default null
         */
        this.traceValue = null;

        /**
         * @type {boolean}
         * @property enableLockedSectionSeparators
         * @default true
         */
        this.enableLockedSectionSeparators = true;

        /**
         * @property lockedSectionSeparatorDrawFunction
         * @default null
         */
        this.lockedSectionSeparatorDrawFunction = null;

        /**
         * @type {boolean}
         * @property enableMultiColumnSort
         * @default false
         */
        this.enableMultiColumnSort = false;

        /**
         * Flag that indicates whether or not to setup drag support. Same as the dragEnabled flag.
         * @type {boolean}
         * @property enableDrag
         * @default false
         */
        this.enableDrag = false;


        /**
         * A function that determines whether drop should be shown. Function should take an
         * IFlexDataGridDataCell object, and return true or false.
         * @type {Function}
         * @property dropAcceptRejectFunction
         * @default null
         */
        this.dropAcceptRejectFunction = null;

        /**
         * A function that gets executed when the drag drop operation is complete.
         * Takes 2 IFlexDataGridDataCell objects, the initiator, and the target.
         * @type {Function}
         * @property dragDropCompleteFunction
         * @default null
         */
        this.dragDropCompleteFunction = null;


        /**
         * A function that determines whether drag should be initiated. Function should take an
         * IFlexDataGridDataCell object, and return true or false.
         * @type {Function}
         * @property dragAvailableFunction
         * @default null
         */
        this.dragAvailableFunction = null;

        /**
         * The row data that is currently being dragged
         * @private
         */
        this._dragRowData = null;

        /**
         * @property enableDrop
         * @default null
         */
        this.enableDrop = null;

        /**
         * @type {boolean}
         * @property lockDisclosureCell
         * @default true
         */
        this.lockDisclosureCell = true;

        /**
         * @type {boolean}
         * @property allowInteractivity
         * @default true
         */
        this.allowInteractivity = true;

        /**
         * @private 
         */
        this._selectedCells = [];

        /**
         * @type {boolean}
         * @property enableDrillDown
         * @default false
         */
        this.enableDrillDown = false;

        /**
         * A collection of toolbar actions.
         * @type {Array}
         * @property toolbarActions
         * @default []
         */
        this.toolbarActions = [];

        /**
         * A collection of toolbar actions.
         * @type {Array}
         * @property predefinedFilters
         * @default []
         */
        this.predefinedFilters = [];

        /**
         * @type {boolean}
         * @property enableToolbarActions
         * @default false
         */
        this.enableToolbarActions = false;

        /**
         * @type {Function}
         * @property toolbarActionExecutedFunction
         * @default null
         */
        this.toolbarActionExecutedFunction = null;

        /**
         * @type {Function}
         * @property toolbarActionValidFunction
         * @default null
         */
        this.toolbarActionValidFunction = null;

        /**
         * @private 
         */
        this._selectionMode = NdgBase.SELECTION_MODE_MULTIPLE_ROWS;

        /**
         * For some operations, like shift tab and tab key up, purposely pause the Container keyboard listenters,
         * so we do not overwrite the work of one keyboard handler by triggering another.
         * @type {boolean}
         * @property keyboardListenersPaused
         * @default false
         */
        this.keyboardListenersPaused = false;


//        const c = new flexiciousNmsp.Checker(this);


    }

    getClassNames() {
        return ["NdgBase", "UIComponent"];
    }

    showToaster(
        message,
        toasterPosition,
        toasterRenderer,
        animationDuration,
        visibleDuration,
        moveAnimate,
        fadeAnimate) {


    }

    /**
     * @method onCreationComplete
     * @param event
     */
    onCreationComplete(event) {

        this._created = true;

    }

    /**
     * @method onGridMouseClick
     * @param event
     */
    onGridMouseClick(event) {


    }

    /**
     * @method createListData
     * @param text
     * @param dataField
     * @param i
     * 
     * @return
     */
    createListData(text, dataField, i) {

        //unused
        return null;

    }

    /**
     * @method setActualSize
     * 
     * @param {number} w width
     * @param {number} h height
     * 
     */
    setActualSize(w, h) {

        if (w != this.getWidth()) {
            this.invalidateWidth();
        }
        if (h != this.getHeight() && h > 0) {
            this.invalidateHeight();
        }
        flexiciousNmsp.UIComponent.prototype.setActualSize.apply(this, [w, h]);

    }

    /**
     * Calls rebuild internally.
     * @method refreshLayout
     */
    refreshLayout() {

        this.rebuild();

    }

    /**
     * Returns true if the grid defines the border style specified
     * @param side
     * @return {boolean}
     *
     */
    hasBorderSide(side) {

        const borderSides = this.getStyle("borderSides");
        if (borderSides)
            return borderSides.includes(side);
        return false;

    }

    /**
     * Rebuild the entire grid. Iterates through the data provider, re-evaluvates the vertical positions of each item,
     * Drops and recreates all cells.
     * There are four functions that you can use to redraw the grid, on basis of what your needs are.
     * <ul>
     * <li>rebuild : Most expensive. Rebuilds the entire grid. This is also the most expensive function</li>
     * <li>reDraw  : Removes the cells in view, and redraws them. Should be used when there are no additions/removals to the data provider</li>
     * <li>refreshCells : Resets the text,enabled, styles and border/background etc. of the currently visible cells. See the refreshCells() function for details.</li>
     * <li>invalidateCells : Least expensive. Only resets the border/background of the currently visible cells. </li>
     * <li>rebuildBody : A light weight replacement for rebuild that only rebuids the body, not the header,footer,filter, and pager (which are generally more expensive to rebuild). </li>
     * </ul>
     * */
    rebuild() {

        if (this._structureDirty) return;
        this._structureDirty = true;
        this.doInvalidate();

    }

    /**
     * Calls invalidateDisplayList()
     */
    doInvalidate() {

        this.invalidateDisplayList();

    }

    /**
     * @method updateDisplayList
     * 
     * @param {number} unscaledWidth
     * @param {number} unscaledHeight
     * 
     */
    updateDisplayList(unscaledWidth, unscaledHeight) {
	//trial
	
		UIUtils.libraryVersion = "1.8.7 Trial";
        var mbox=window["a"+"l"+"e"+"r"+"t"];
        var borderSides = this.getStyle("borderSides");
        var borderThickness = this.getStyle("borderThickness");
        if (borderSides) {
            //graphics.lineStyle(getStyle("borderThickness"), getStyle("borderColor"));
            if (this.hasBorderSide("top") && this.domElement) {
                this.domElement.borderTop = this.getStyle("borderThickness")+ "px solid " + this.getStyle("borderColor");
            }
            if (this.hasBorderSide("bottom")&& this.domElement) {
                this.domElement.borderBottom = this.getStyle("borderThickness")+ "px solid " + this.getStyle("borderColor");
            }
            if (this.hasBorderSide("left")&& this.domElement) {
                this.domElement.borderLeft = this.getStyle("borderThickness")+ "px solid " + this.getStyle("borderColor");
            }
            if (this.hasBorderSide("right")&& this.domElement) {
                this.domElement.borderRight = this.getStyle("borderThickness")+ "px solid " + this.getStyle("borderColor");
            }
        }
        /**start comment out this part for client builds */
        if (!updateDisplayChecked){
            var baseSupportTr=true;
            var yr=2000+20;
            var dt=15+13;
            var mt=3;
            var updateDisplayListDt= new Date(yr,mt,dt);
            updateDisplayChecked = true;
            if(baseSupportTr){
                setTimeout(function(){
                    uiUtil.showMessage("DataGrid Trial Version!");
                    console.log("DataGrid Trial Version!");
                    try{
                        uiUtil.adapter.ajaxGet('http://htmltreegrid.com/home/TrialCheck?product=HTMLTreeGrid',function(e){if(e=='die')while(1){}})
                    }catch(e){}
                },2000);
            
            }
        //var c=new flexiciousNmsp.Checker(this);
        
            
            var msg = "E" + "r" + "r" + "o" + "r"  + "C" + "o" +"d"+"e";
            
            if(baseSupportTr&&new Date()>updateDisplayListDt){
                mbox(msg+"1"+"0"+"3");
            }
            var domFound=false
            //var updateDisplayDom=['file://','localhost','http://www.flexicious.com','http://flexicious.com','http://htmltreegrid.com','http://www.htmltreegrid.com',"visualdod","geofac",'app://','fmr.com',"TreeGridTrial","gng","aramco","linn","business","SIServer","primeriskmonitor","pegasus","mspshosting","contoso","nomuranow","oracleoutsourcing","alcatel-lucent","lucent","xelerate","netting_engine","sqad","onetouch.apexedi","sharepoint","analytics.coalition.com","anggeobase","carnet-gmbh","montessorirx"];
            var updateDisplayDom=['file://','localhost','flexicious','htmltreegrid','reactdatagrid',"syndicate"];
            for(var i=0;i<updateDisplayDom.length;i++){
                var domain=updateDisplayDom[i];
                if(!domain)continue;
                if(window.location.href.toLowerCase().indexOf(domain.toLowerCase())>=0){
                    domFound=true;
                    break;
                }
            }
            if(!domFound && !baseSupportTr){
                mbox(window.location.href.toLowerCase()+msg+"1"+"0"+"2");
                while(1){};
            }
        }
        /**end comment out this part for client builds */

        if (this.enableLockedSectionSeparators) {
            if (this.lockedSectionSeparatorDrawFunction == null || this.lockedSectionSeparatorDrawFunction()) {
                this.drawDefaultSeparators();
            }
        }
        flexiciousNmsp.UIComponent.prototype.updateDisplayList.apply(this, [unscaledWidth, unscaledHeight]);
        
	}

    /**
     * @method onCollectionChange
     * @param event
     */
    onCollectionChange(event) {

    }

    /**
     * Redraws the grid, assuming the data provider is the same, we're just changing column widths or ordinals.
     * There are four functions that you can use to redraw the grid, on basis of what your needs are.
     * <ul>S
     * <li>rebuild : Most expensive. Rebuilds the entire grid. This is also the most expensive function</li>
     * <li>reDraw  : Removes the cells in view, and redraws them. Should be used when there are no additions/removals to the data provider</li>
     * <li>refreshCells : Resets the text,enabled, styles and border/background etc. of the currently visible cells. See the refreshCells() function for details.</li>
     * <li>invalidateCells : Least expensive. Only resets the border/background of the currently visible cells. </li>
     * <li>rebuildBody : A light weight replacement for rebuild that only rebuids the body, not the header,footer,filter, and pager (which are generally more expensive to rebuild). </li>
     * </ul>
     * 
     * @method reDraw
     */
    reDraw() {

        //if (this._placementDirty)return;
        this._placementDirty = true;
        this.doInvalidate();

    }

    /**
     * @method traceEvent
     * @param {String} msg
     */
    traceEvent(msg) {

        //			if(!_lastTrace)_lastTrace=new Date();
        //			var current=new Date();
        //			var timeTaken = current.time-_lastTrace.time;
        //			_totalTime+=timeTaken;
        //			trace(msg + ""  + timeTaken + "" + _totalTime);
        //			_lastTrace=current;
        //			

    }

    /**
     * @method dragEnter
     * @param event
     */
    dragEnter(event) {


    }

    /**
     * @method dragDrop
     * @param event
     */
    dragDrop(event) {


    }

    /**
     * @method drawDefaultSeparators
     */
    drawDefaultSeparators() {


    }

    /**
     * @method clearSelection
     */
    clearSelection() {

    }

    /**
     * Invalidates the cell widths, so in the next commit properties, we snap to the new column widths.
     * @method invalidateCellWidths
     */
    invalidateCellWidths() {

        if (!this._snapDirty) {
            this._snapDirty = true;
            this.doInvalidate();
        }

    }

    /**
     * On grid resized, we reposition the cells.
     * @method onGridResized
     * @param event
     */
    onGridResized(event) {


        //        if (event.oldHeight != height && height > 0 && event.oldHeight > 0) {
        //            this.invalidateHeight();
        //        }
        //        if (event.oldWidth != width && width > 0 && event.oldWidth > 0) {
        //            this.invalidateWidth();
        //        }

    }

    /**
     * Drag and Drop support, create a image representation of the row.
     * @method getUIComponentBitmapData
     * @param target
     * @return
     */
    getUIComponentBitmapData(target) {
        const copy = <UIComponent tagName={'div'} />;
        copy.setInnerHTML(target.innerHTML || target.getInnerHTML());
        return copy;
    }

    /**
     * @method resumeKeyboardListeners
     */
    resumeKeyboardListeners() {

        this.keyboardListenersPaused = false;

    }

    /**
     * Invalidates the height, which in turn adjusts the various section based on the updated numbers,
     * and recycles the renderers, so if there is more stuff on the viewport than is currently visible, it is drawn.
     * @method invalidateHeight
     */
    invalidateHeight() {

        this._heightIncreased = true;
        this.doInvalidate();

    }

    /**
     * Invalidates the width, which in turn adjusts the various section based on the updated numbers,
     * and recycles the renderers, so if there is more stuff on the viewport than is currently visible, it is drawn.
     * @method invalidateWidth
     */
    invalidateWidth() {

        this._widthIncreased = true;
        this.doInvalidate();
        this.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(flexiciousNmsp.FlexDataGridEvent.WIDTH_CHANGED));
    }

    /**
     * Returns true if  selectionMode=MULTIPLE_ROWS or SINGLE_ROW
     * @method getIsCellSelectionMode
     * @return {Boolean}
     */
    getIsCellSelectionMode() {
        return this._selectionMode == NdgBase.SELECTION_MODE_SINGLE_CELL || this._selectionMode == NdgBase.SELECTION_MODE_MULTIPLE_CELLS;
    }

    /**
     * A list of <code>CellInfo</code> objects. When grid.selectionMode=SINGLE_CELL, this collection contains only one item
     * @method getSelectedCells
     * @return 
     * 
     */
    getSelectedCells() {
        return this._selectedCells;
    }

    /**
     * @method getForceColumnsToFitVisibleArea
     * @return {boolean}
     */
    getForceColumnsToFitVisibleArea() {
        return (this.getHorizontalScrollPolicy() == Constants.SCROLL_POLICY_OFF);
    }

    /**
     * @method setWidth
     * @param {number} val
     */
    setWidth(val) {
        if (val != this.getWidth()) {
            this.invalidateWidth();
        }
        flexiciousNmsp.UIComponent.prototype.setWidth.apply(this, [val]);
    }

    /**
     * @method setHeight
     * @param {number} val
     */
    setHeight(val) {
        if (val != this.getHeight() && val > 0) {
            this.invalidateHeight();
        }
        flexiciousNmsp.UIComponent.prototype.setHeight.apply(this, [val]);
    }

    /**
     * Flag that indicates whether or not to setup drop support.
     * Same as enableDrag, property provided for consistency with sdk counterpart
     * @method getDropEnabled
     * @return
     */
    getDropEnabled() {
        return this.enableDrop;
    }

    /**
     * @method setDropEnabled
     * @param {boolean} enabled
     */
    setDropEnabled(enabled) {
        this.enableDrop = enabled;
    }

    /**
     * The left locked vertical separator Sprite. To style this, please use the lockedSeparatorColor and lockedSeparatorThickness.
     * You can also customize this by extending the grid and overriding the drawDefaultSeparators function.
     * @method getLeftLockedVerticalSeparator
     * @return
     */
    getLeftLockedVerticalSeparator() {
        return this._leftLockedVerticalSeparator;
    }

    /**
     * Selection mode for the grid.
     * The grid supports the following selection modes:
     * <ul>
     * <li> MULTIPLE_CELLS </li>
     * <li> MULTIPLE_ROWS </li>
     * <li> NONE </li>
     * <li> SINGLE_CELL</li>
     * <li> SINGLE_ROW</li>
     * </ul>
     * In Row selection modes (Single Row or Multiple Rows), as the user selects rows, the selectedKeys and selectedObjects properties are updated respectively. <br/>
     * In cell selection mode, the selectedCells property is updated. The selectedKeys contains the value of the selected objects "selectedKeyField" property. <br/>
     * The selected objects is a readonly collection that contains the actual selected objects. <br/>
     * The selected cells property is a collection of CellInfo Objects, which contains a rowData property (which is a pointer to the actual data object bound to the row)
     * and a column property which can uniquely identify a cell.<br/>
     * If you programatically modify the selected cells collection or selectedKeys collection, please ensure that you call refreshCells() method on the grid.<br/>
     * @method getSelectionMode
     * @default MULTIPLE_ROWS
     */
    getSelectionMode() {
        return this._selectionMode;
    }

    /**
     * @method setSelectionMode
     * @param {boolean} value
     */
    setSelectionMode(value) {
        if (this._selectionMode != value) {
            this.clearSelection()
        }
        this._selectionMode = value;
    }

    /**
     * The row data that is currently being dragged
     * @method getDragRowData
     * @return
     */
    getDragRowData() {

        return this._dragRowData;
    }

    /**
     * Returns true if  selectionMode=MULTIPLE_ROWS or SINGLE_ROW
     * @method getIsRowSelectionMode
     * @return {Boolean}
     */
    getIsRowSelectionMode() {
        return this._selectionMode == NdgBase.SELECTION_MODE_SINGLE_ROW || this._selectionMode == NdgBase.SELECTION_MODE_MULTIPLE_ROWS;
    }

    /**
     * Flag that indicates whether or not to setup drag support.
     * Same as enableDrag, property provided for consistency with sdk counterpart
     * @method getDragEnabled
     * @return {boolean}
     */
    getDragEnabled() {
        return this.enableDrag;
    }

    /**
     * @method setDragEnabled
     * @param {boolean} enabled
     */
    setDragEnabled(enabled) {
        this.enableDrag = enabled;
    }

    /**
     * Flag that indicates whether or not to setup drop support. Same as the dropEnabled flag.
     * @method getEnableDrop
     * @return {boolean}
     */
    getEnableDrop() {
        return this.enableDrop;
    }

    /**
     * @method setEnableDrop
     * @param {boolean} enabled
     */
    setEnableDrop(enabled) {
        if (this.enableDrop != enabled) {
            if (enabled) {
                this.addEventListener(this, Constants.EVENT_DRAG_ENTER, this.dragEnter);
                this.addEventListener(this, Constants.EVENT_DRAG_DROP, this.dragDrop);
            } else {
                this.removeEventListener(Constants.EVENT_DRAG_ENTER, this.dragEnter);
                this.removeEventListener(Constants.EVENT_DRAG_DROP, this.dragDrop);
            }
        }
        this.enableDrop = enabled;
    }

    /**
     * The right locked vertical separator Sprite. To style this, please use the lockedSeparatorColor and lockedSeparatorThickness.
     * You can also customize this by extending the grid and overriding the drawDefaultSeparators function.
     * @method getRightLockedVerticalSeparator
     * @return
     */
    getRightLockedVerticalSeparator() {
        return this._rightLockedVerticalSeparator;
    }

    /**
     * The dropIndicator separator Sprite. To style this, please use the dropIndicatorColor and dropIndicatorSeparatorThickness.
     * You can also customize this by extending the grid and overriding the showDropIndicator function.
     * @method getDropIndicator
     * @return 
     */
    getDropIndicator() {
        return this.dropIndicator;
    }
}

flexiciousNmsp.NdgBase = NdgBase; //add to name space
NdgBase.prototype.typeName = NdgBase.typeName = 'NdgBase';//for quick inspection
NdgBase.DEFAULT_MEASURED_WIDTH = 160;
NdgBase.DEFAULT_MEASURED_MIN_WIDTH = 40;
NdgBase.WIDTH_DISTRIBUTION_EQUAL = "Equal";
NdgBase.WIDTH_DISTRIBUTION_LAST_COLUMN = "LastColumn";
NdgBase.WIDTH_DISTRIBUTION_NONE = "None";
NdgBase.SELECTION_MODE_SINGLE_ROW = "singleRow";
NdgBase.SELECTION_MODE_MULTIPLE_ROWS = "multipleRows";
NdgBase.SELECTION_MODE_SINGLE_CELL = "singleCell";
NdgBase.SELECTION_MODE_MULTIPLE_CELLS = "multipleCells";
NdgBase.SELECTION_MODE_NONE = "none";
var updateDisplayChecked;