import TypedObject from '../../../core/TypedObject'
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

/**
 * An equivalent of a Java HashMap, that allws you to have object keys and values.
 * @constructor
 * @class KeyValuePairCollection
 * @namespace flexiciousNmsp
 */
export default class KeyValuePairCollection extends TypedObject {
    constructor() {
        super();
        this.keys = [];
        this.values = [];
        this.clear();
    }

    getClassNames() {
        return "KeyValuePairCollection";
    }

    /**
     * Add a key value pair
     * @param {Object} key 
     * @param {Object} value 
     */
    addItem(key, value) {
        const keyIdx = this.keys.indexOf(key);
        if (keyIdx == -1) {
            this.keys.push(key);
            this.values.push(value);
        }
        else {
            this.values[keyIdx] = value;
        }

    }

    /**
     * Remove item with associate key
     * @param {Object} key 
     */
    removeItem(key) {
        const keyIdx = this.keys.indexOf(key);
        if (keyIdx != -1) {
            this.keys.splice(keyIdx, 1);
            this.values.splice(keyIdx, 1);
        }

    }

    /**
     * Get value for the key from key Value pair
     * @param {Object} key 
     */
    getValue(key) {
        const keyIdx = this.keys.indexOf(key);
        if (keyIdx == -1)
            return null;
        return this.values[keyIdx];


    }

    /**
     * Check whether key exists or not
     * @param {Object} key 
     */
    hasKey(key) {
        return this.keys.includes(key);

    }

    /**
     * Remove all key value pairs
     */
    clear() {
        this.keys = [];
        this.values = [];

    }

    kill() {
        for (const item of this.keys) {
            if (typeof item["kill"] == "function")
                item.kill();
        }

        for (const item1 of this.values) {
            if (typeof item1["kill"] == "function")
                item1.kill();
        }

        this.clear();
    }
}

const uiUtil = flexiciousNmsp.UIUtils;
const flxConstants = flexiciousNmsp.Constants;
KeyValuePairCollection.prototype.push = KeyValuePairCollection.prototype.addItem;
KeyValuePairCollection.prototype.setValue = KeyValuePairCollection.prototype.addItem;
flexiciousNmsp.KeyValuePairCollection = KeyValuePairCollection;