
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
 
import KeyValuePairCollection from './KeyValuePairCollection'
import TypedObject from '../../../core/TypedObject'
import FactoryKey from './FactoryKey'
/**
 * Class to keep renderers around...
 *
 * Each Cell in the grid, is a container UI component, and the actual renderer UI component
 * The _dict object here holds a bucket for each container type/renderer type combination
 * The _rendererFactories holds a map of each cell and the corresponding bucket key (FactoryKey) that
 * was used to generate it, so we are able to put it back into the bucket that it came from.
 *
 * The idea is, there should always be only just as many things in memory as are currently visibule
 * plus a little outside so we scroll smoothly...
 * @constructor
 * @class RendererCache
 * @namespace flexiciousNmsp
 */
export default class RendererCache extends TypedObject {
    constructor(grid) {
        super();
        this.grid=grid;
        this.factoryKeys=[];
        this.dict= new KeyValuePairCollection();
        this.rendererFactories= new KeyValuePairCollection();
        this.factoryKeys= [];
    }

    getClassNames() {
        return "RendererCache";
    }

    /**
     * Returns an instance of the specified factory/subfactory combination from the cache, if once exists.
     * Else, instantiates a new instance, and stores the factory key that was used to create it.
     * @param factory
     * @param subFactory
     * @return
     */
    popInstance(factory, subFactory, dataRow) {
        if (typeof subFactory == "undefined") subFactory = null;

        if (!subFactory) subFactory = factory;
        var key = this.getFactoryKey(factory, subFactory);
        if (!this.dict.hasKey(key))
            this.dict.addItem(key, new Array());

        var retVal;
        var cellsForRenderer = this.dict.getValue(key);
        if (dataRow) {
            for (var i = 0; i < cellsForRenderer.length; i++) {
                var cell = cellsForRenderer[i];
                if (dataRow == cell.componentInfo.row) {
                    cellsForRenderer.splice(i, 1);
                    retVal = cell;
                    break;
                }
            }
        } else {
            retVal = cellsForRenderer.shift();
        }
        if (!retVal) {
            retVal = factory.newInstance();
            if (this.grid.dispatchCellCreated)
                this.grid.dispatchEvent(new flexiciousNmsp.FlexDataGridEvent(
                    flexiciousNmsp.FlexDataGridEvent.CELL_CREATED, this.grid, null, null, retVal
                ));
            this.rendererFactories.push(retVal, key);
        }
        return retVal;

    }

    /**
     * Returns a previously created renderer into the bucket for reuse later.
     */
    pushInstance(instance) {
       this.dict.getValue(this.rendererFactories.getValue(instance)).push(instance);
    }

    /**
     * 
     * @param {flexiciousNmsp.ClassFactory} factory 
     * @param {flexiciousNmsp.ClassFactory} subFactory 
     * 
     * @return {flexiciousNmsp.FactoryKey}
     */
    getFactoryKey(factory, subFactory) {
        for (const fKey of this.factoryKeys) {
            if(fKey.factory==factory && fKey.subFactory==subFactory){
                return fKey;
            }
        }
        const newKey=new flexiciousNmsp.FactoryKey(factory,subFactory);
        this.factoryKeys.push(newKey);
        return newKey;
    }

    kill() {
        this.dict.kill();
        this.rendererFactories.kill();
        this.factoryKeys=[];
        this.grid=null;
    }
}
flexiciousNmsp.RendererCache = RendererCache;
