
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */



import Constants from '../../../utils/Constants'
import UIUtils from '../../../utils/UIUtils' 
import TypedObject from '../../../core/TypedObject'
/**
 * A Class that encapsulates functionality that containers of filter controls should have.
 * @class FilterContainerImpl
 * @constructor
 * @param iEventDispatcher
 *
 */
export default class FilterContainerImpl extends TypedObject {
    constructor(iEventDispatcher) {
        super()
        this.iEventDispatcher = iEventDispatcher;
        this.filterControls = [];
    }

    getClassNames() {
        return "FilterContainerImpl";
    }

    /**
     * Return filter arguments
     * @return {Array}
     */
    getFilterArguments() {
        const argumentsCollection = [];

        for (const iFilterControl of this.filterControls) {
            const filterExpression = this.getFilterExpression(iFilterControl);
            if (filterExpression != null) {
                argumentsCollection.push(filterExpression);
            }
        }

        return argumentsCollection;
    }

    /**
     * Remove all filter controls
     */
    resetFilterControls() {
        for (const iFilterControl of this.filterControls) {
            iFilterControl.dispatchEvent(new flexiciousNmsp.ExtendedFilterPageSortChangeEvent(flexiciousNmsp.ExtendedFilterPageSortChangeEvent.DESTROY));
        }

        flexiciousNmsp.UIUtils.removeAll(this.filterControls);
    }

    /**
     * Unresgister a filter control component
     * @param {*} iFilterControl 
     */
    unRegisterIFilterControl(iFilterControl) {
        if (iFilterControl.filterTriggerEvent.toLowerCase() == "none") {
            //user specified trigger event, so dont refresh on control change
        }
        else if (iFilterControl.filterTriggerEvent.toLowerCase() == "enterkeyup") {
            iFilterControl.removeEventListener(Constants.EVENT_KEY_UP, this.onKeyUp);
        }
        else if (iFilterControl.filterTriggerEvent.toLowerCase() == "enterkeyuporfocusout") {
            iFilterControl.removeEventListener(this, Constants.EVENT_KEY_UP, this.onKeyUp);
            iFilterControl.removeEventListener(this, Constants.EVENT_FOCUS_OUT, this.onFocusOut);
        }
        else {
            //user specified trigger event, so dont refresh on control change
            if (iFilterControl.implementsOrExtends('IDelayedChange')) {
                iFilterControl.removeEventListener("delayedChange", this.onChangeHandler);
            }
            else {
                iFilterControl.removeEventListener("change", this.onChangeHandler);
            }

        }
        this.filterControls.splice(this.filterControls.indexOf(iFilterControl), 1);


    }

    /**
     * Check whether filter control component already resgister or not
     * @param {*} iFilterControl 
     */
    isIFilterControlRegistered(iFilterControl) {
        return this.filterControls.includes(iFilterControl);

    }

    /**
     * Resgister a filter control component
     * @param {*} iFilterControl 
     */
    registerIFilterControl(iFilterControl) {
        this.filterControls.push(iFilterControl);
        if (!iFilterControl.filterTriggerEvent) {

        }
        else if (iFilterControl.filterTriggerEvent.toLowerCase() == "none") {
            //user specified trigger event, so dont refresh on control change
        }
        else if (iFilterControl.filterTriggerEvent.toLowerCase() == "enterkeyup") {
            iFilterControl.addEventListener(this, Constants.EVENT_KEY_UP, this.onKeyUp);
        }
        else if (iFilterControl.filterTriggerEvent.toLowerCase() == "enterkeyuporfocusout") {
            iFilterControl.addEventListener(this, Constants.EVENT_KEY_UP, this.onKeyUp);
            iFilterControl.addEventListener(this, Constants.EVENT_FOCUS_OUT, this.onFocusOut);
        }
        else {
            //user specified trigger event, so dont refresh on control change
            if (iFilterControl.implementsOrExtends('IDelayedChange')) {
                iFilterControl.addEventListener(this, "delayedChange", this.onChangeHandler);
            }
            else {
                iFilterControl.addEventListener(this, "change", this.onChangeHandler);
            }

        }


    }

    /**
     * @param fld	The string to match the searchField property of the filter control with.
     * @return 		True if focus was set, false if otherwise.
     * Focus may not be set if the given search field does not exist, <br/>
     * or if the filter control for the given search field does not  <br/>
     * implement IFocusManagerComponent.
     */
    setFilterFocus(fld) {
        for (const child of this.filterControls) {
            if (child.searchField == fld) {
                if (child.implementsOrExtends('IFocusManagerComponent')) {
                    this.setFocusOnChild(child);
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * set focus on its child component
     * @param {*} child 
     */
    setFocusOnChild(child) {

        if (typeof child["focus"] == "function") {
            child.focus();
        }
        if (typeof child["setSelection"] == "function") {
            child.setSelection();
        }

    }

    /**
     * 
     * @param event 
     */
    getNextFilter(event) {
        //TODO IMPLEMENT THIS;

        let myIndex;

        for (let i = 0; i < this.filterControls.length; i++) {
            const child = this.filterControls[i];
            if (event.currentTarget == child) {
                myIndex = i;
                break;
            }
        }
        const backward = (event.keyCode == Constants.KEYBOARD_LEFT) || ((event.keyCode == Constants.KEYBOARD_TAB) && event.shiftKey);
        const forward = (event.keyCode == Constants.KEYBOARD_RIGHT) || (event.keyCode == Constants.KEYBOARD_TAB) && !backward;

        if (!forward && !backward) return null;
        return this.getNextFocusableFilter(myIndex, forward)

    }

    /**
     * Returns next focusable filter 
     * @param {Number} myIndex current focusable filter index
     * @param {Boolean} forward forward or backward mode
     * @param {Boolean} inclusive exclusive or inclusive mode
     */
    getNextFocusableFilter(myIndex, forward, inclusive) {
        //TODO IMPLEMENT THIS;
        if (typeof inclusive == "undefined") inclusive = false;


        if (this.filterControls.length == 0) return null;
        if (myIndex >= 0 && myIndex <= this.filterControls.length && inclusive && (this.filterControls[myIndex].implementsOrExtends('IFocusManagerComponent'))) {
            return this.filterControls[myIndex];
        }

        let myFilter = this.filterControls[myIndex];
        if (myFilter.implementsOrExtends('ITriStateCheckBoxFilterControl')) {
            myFilter = myFilter;
        }
        let nextFilter;
        const dg = this.iEventDispatcher["dataGrid"];

        const myX = dg.getFilterX(myFilter);
        for (let i = 0; i < this.filterControls.length; i++)
            var iFilterControl = this.filterControls[i];
        {
            if (iFilterControl.implementsOrExtends('IFocusManagerComponent')) {
                const otherX = dg.getFilterX(iFilterControl);
                if (forward) {
                    if (nextFilter) {
                        if ((otherX < (nextFilter).getX())
                            && (otherX > myX)
                            && (iFilterControl.implementsOrExtends('IFocusManagerComponent'))
                            && (iFilterControl != myFilter)) {
                            nextFilter = iFilterControl;
                        }
                    }
                    else if (otherX > myX
                        && (iFilterControl != myFilter)) {
                        nextFilter = iFilterControl;
                    }
                }
                else {//backward
                    if (nextFilter) {
                        if ((otherX > (nextFilter).getX())
                            && (otherX < myX)
                            && (iFilterControl.implementsOrExtends('IFocusManagerComponent'))
                            && (iFilterControl != myFilter)) {
                            nextFilter = iFilterControl;
                        }
                    }
                    else if (otherX < myX
                        && (iFilterControl != myFilter)) {
                        nextFilter = iFilterControl;
                    }
                }
            }
        }
        return nextFilter;

    }

    /**
     * Set filter value to column
     * @param {flexiciousNmsp.FlexDataGridColumn} column 
     * @param {Object} value 
     */
    setFilterValue(column, value) {
        //TODO IMPLEMENT THIS;

        for (let i = 0; i < this.filterControls.length; i++)
            var iFilterControl = this.filterControls[i];
        {
            if (iFilterControl.searchField == column) {
                iFilterControl.setValue(value);
            }
        }

    }

    /**
     * Get filter value from column
     * @param {flexiciousNmsp.FlexDataGridColumn} column 
     * @return {Object} value  
     */
    getFilterValue(column) {
        //TODO IMPLEMENT THIS;

        for (let i = 0; i < this.filterControls.length; i++)
            var iFilterControl = this.filterControls[i];
        {
            if (iFilterControl.searchField == column) {
                return iFilterControl.getValue();
            }
        }
        return null;

    }

    /**
     * Check whether filter control search field belongs to this column or not
     * @param {flexiciousNmsp.FlexDataGridColumn} column 
     * @return {Boolean}
     */
    hasField(column) {
        //TODO IMPLEMENT THIS;

        for (let i = 0; i < this.filterControls.length; i++)
            var iFilterControl = this.filterControls[i];
        {
            if (iFilterControl.searchField == column) {
                return true;
            }
        }
        return false;

    }

    /**
     * Process filter
     */
    processFilter() {
        this.onChangeHandler(null);

    }

    /**
     * Clear all filter controls
     */
    clearFilter() {
        for (const iFilterControl of this.filterControls) {
            iFilterControl.clear();
        }
        //this.onChangeHandler(null);
    }

    // clearFilter() {
    //     for (const iFilterControl of this.filterControls) {
    //         iFilterControl.clear();
    //     }
    //     //this.onChangeHandler(null);
    // }

    onKeyUp(event) {
        if (event.keyCode == Constants.KEYBOARD_ENTER) {
            this.onChangeHandler(event);
        }

    }

    onFocusOut(event) {
        if( !event.target.owns(document.activeElement) ) {
            this.onChangeHandler(event);
        }
    }

    onChangeHandler(event) {
        const target = event ? event.currentTarget || event.srcElement : this;

        const filter = new flexiciousNmsp.Filter();
        filter.setFilterExpressions(this.getFilterArguments());
        const filterPageSortChangeEvent = new flexiciousNmsp.ExtendedFilterPageSortChangeEvent(flexiciousNmsp.ExtendedFilterPageSortChangeEvent.FILTER_CHANGE, filter);
        filterPageSortChangeEvent.triggerEvent = event;
        this.iEventDispatcher.dispatchEvent(filterPageSortChangeEvent);

    }

    /**
     * Get filter expression from filter control
     * @param {*} iFilterControl 
     * @return {flexiciousNmsp.FilterExpression}
     */
    getFilterExpression(iFilterControl) {
        if (!iFilterControl.getEnabled() || !iFilterControl.implementsOrExtends("IFilterControl")) return null;
        let expression;
        let filterExpression = new flexiciousNmsp.FilterExpression();
        filterExpression.columnName = iFilterControl.searchField;
        filterExpression.filterOperation = iFilterControl.filterOperation;
        filterExpression.wasContains = (filterExpression.filterOperation == flexiciousNmsp.FilterExpression.FILTER_OPERATION_TYPE_CONTAINS);
        filterExpression.filterComparisionType = iFilterControl.filterComparisionType;
        filterExpression.filterControl = iFilterControl;
        filterExpression.filterControlValue = iFilterControl.getValue();
        if (iFilterControl.implementsOrExtends('ICustomMatchFilterControl')) {
            filterExpression.columnName = iFilterControl.searchField;
            if(filterExpression.filterControlValue === Constants.EMPTY_FILTER){
                return null;
            }
            filterExpression.filterControl = iFilterControl;
            filterExpression.expression = filterExpression.filterControlValue;//new Object();//this.implementsOrExtends('ignored...
            return filterExpression;
        }
        else if (iFilterControl.implementsOrExtends('IDynamicFilterControl')) {
            const iDynamicFilterControl = (iFilterControl);
            filterExpression = iDynamicFilterControl.getFilterExpression();
            if (filterExpression == null) return null;
            filterExpression.columnName = iFilterControl.searchField;
            filterExpression.filterComparisionType = iFilterControl.filterComparisionType;
            filterExpression.filterControl = iFilterControl;
            return filterExpression;
        }
        else if (iFilterControl.implementsOrExtends('IRangeFilterControl')) {
            filterExpression.filterOperation = flexiciousNmsp.FilterExpression.FILTER_OPERATION_TYPE_BETWEEN;
            const iRangeFilterControl = iFilterControl;
            if (iRangeFilterControl.getSearchRangeStart() == null || iRangeFilterControl.getSearchRangeEnd() == null)
                return null;
            expression = [];
            expression.push(iRangeFilterControl.getSearchRangeStart());
            expression.push(iRangeFilterControl.getSearchRangeEnd());
            //expression=expression;
        }
        else if (iFilterControl.implementsOrExtends('ISingleSelectFilterControl')) {
            const iSingleSelectFilterControl = iFilterControl;
            const si = iSingleSelectFilterControl.getSelectedValue();

            if (si == null || si.toString() == iSingleSelectFilterControl.addAllItemText)
                return null;

            expression = si;
        }
        else if (iFilterControl.implementsOrExtends('IMultiSelectFilterControl')) {
            const iMultiSelectFilterControl = iFilterControl;
            expression = [];
            filterExpression.filterOperation = flexiciousNmsp.FilterExpression.FILTER_OPERATION_TYPE_IN_LIST;
            const items = iMultiSelectFilterControl.getValue();

            for (const obj of items) {
                if (obj != null) {
                    if (obj == iMultiSelectFilterControl.addAllItemText)
                        return null;
                    expression.push(obj)
                }
            }

            if (expression.length == 0) return null;
        }

        else if (iFilterControl.implementsOrExtends('ITextFilterControl')) {
            if (filterExpression.filterOperation == null)
                filterExpression.filterOperation = flexiciousNmsp.FilterExpression.FILTER_OPERATION_TYPE_BEGINS_WITH;
            const iTextFilterControl = iFilterControl;
            if((iTextFilterControl.getValue() || "").length==0)
                return null;
            expression = iTextFilterControl.getValue();
        }
        else if (iFilterControl.implementsOrExtends('ITriStateCheckBoxFilterControl')) {
            const iTriStateCheckBoxFilterControl = iFilterControl;
            if (iTriStateCheckBoxFilterControl.getSelectedState() == "middle")
                return null;
            expression = (iTriStateCheckBoxFilterControl.getSelectedState() == "checked");
        }
        else if (iFilterControl.implementsOrExtends('ISelectedBitFilterControl')) {
            expression = (iFilterControl).getSelected();
        }
        else
            return null;
        if (filterExpression.filterOperation == null)
            filterExpression.filterOperation = flexiciousNmsp.FilterExpression.FILTER_OPERATION_TYPE_EQUALS;

        filterExpression.expression = expression;
        filterExpression.filterControlValue = iFilterControl.getValue();
        if (iFilterControl.gridColumn && (iFilterControl.gridColumn).hasOwnProperty("enableRecursiveSearch"))
            filterExpression.recurse = (iFilterControl.gridColumn)["enableRecursiveSearch"];
        return filterExpression;

    }

    kill() {
        flexiciousNmsp.EventDispatcher.prototype.kill.apply(this);
        this.filterControls = [];
        this.iEventDispatcher = [];
    }
}
flexiciousNmsp.FilterContainerImpl = FilterContainerImpl;