/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import TypedObject from '../../core/TypedObject'

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 

/**
 * Class added in 2.9 to support multiple preferences. Each IPersistable has
 * a gridpreferences object, that contains an array of PreferenceInfo objects
 * @constructor
 * @class GridPreferencesInfo
 * @namespace flexiciousNmsp
 * 
 */
export default class GridPreferencesInfo extends TypedObject {
    constructor() {
        super();
        /**
         * Flag that tells the grid to load the default preferences on creation complete
         * @default true
         * @type {Boolean}
         * @property loadDefaultPreferenceOnCreationComplete
         */
        this.loadDefaultPreferenceOnCreationComplete = true;
        /**
         * Name of the default preference
         * @default Default
         * @type {String}
         * @property defaultPreferenceName
         */
        this.defaultPreferenceName = "Default";
        /**
         * List of saved preferences.
         * @property savedPreferences
         * @default []
         * @type {Array}
         */
        this.savedPreferences = [];
    }

    /**
     *
      * @return {Array}
     */
    getClassNames() {
        return ["GridPreferencesInfo", "TypedObject"];
    }

    /**
     * A function that converts the current preference object into a string.
     * @param uso
     * @return {String}
     */
    toPreferenceString(uso) {
        let str = "";
        str += (this.loadDefaultPreferenceOnCreationComplete ? 'y' : 'n') + uso.multiPrefGridPrefPropDelimiter;
        str += this.defaultPreferenceName + uso.multiPrefGridPrefPropDelimiter;

        for (const pref of this.savedPreferences) {
            str += pref.name + uso.multiPrefPrefPropDelimiter;
            str += pref.preferences + uso.multiPrefPrefPropDelimiter;
            str += uso.multiPrefDelimiter;
        }

        return str;
    }

    /**
     *  Converts a string resut of toPreferenceString to a GridPreferencesInfo object
     * @param uso
     * @param str
     * @return {*}
     */
    fromPreferenceString(uso, str) {

        const gpi = this;
        const thisProps = str.split(uso.multiPrefGridPrefPropDelimiter);
        if (thisProps && thisProps.length == 1) {
            //this means we are multi pref grid, but we have a single pref stored.
            gpi.loadDefaultPreferenceOnCreationComplete = true;
            const po1 = new flexiciousNmsp.PreferenceInfo();
            po1.name = gpi.defaultPreferenceName;
            po1.preferences = thisProps[0];
            gpi.savedPreferences.push(po1);
            return gpi;
        }
        gpi.loadDefaultPreferenceOnCreationComplete = (thisProps[0] == 'y');
        gpi.defaultPreferenceName = thisProps[1];
        const prefArr = thisProps[2].split(uso.multiPrefDelimiter);
        for (let i = 0; i < prefArr.length; i++) {
            str = prefArr[i];
            if (!str) continue;
            const po = new flexiciousNmsp.PreferenceInfo();
            const prefPropArr = str.split(uso.multiPrefPrefPropDelimiter);
            po.name = prefPropArr[0];
            po.preferences = prefPropArr[1];
            gpi.savedPreferences.push(po);
        }
        return gpi;

    }
}

flexiciousNmsp.GridPreferencesInfo = GridPreferencesInfo; //add to name space 
GridPreferencesInfo.prototype.typeName = GridPreferencesInfo.typeName = 'GridPreferencesInfo';//for quick inspection