/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import UIComponent from '../../core/UIComponent'

/**
 * A OpenSettingsPopup that which can be used within the filtering/binding infrastructure.
 * @constructor
 * @class OpenSettingsPopup
 * @namespace flexiciousNmsp
 * 
 */
export default class OpenSettingsPopup extends UIComponent {
    constructor() {
        super({}, "div")
        this.attachClass("flexiciousGrid");
        this.setWidth(600);
    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["OpenSettingsPopup", "UIComponent"];
    }

    setGrid(val) {
        this.grid = val;
        this.gpi = this.grid.getGridPreferencesInfo();
        this.currentPref = this.grid.getCurrentPreferenceInfo();

    }

    /**
     * Initializes the auto complete and watermark plugins
     */
    onClearSettings() {

        this.grid.clearPreferences();
        UIUtils.showMessage("Preferences Cleared!");
        UIUtils.removePopUp(this);

    }

    onSaveSettings() {

        this.grid.persistPreferences(this.currentPref.name, this.currentPref.name == this.gpi.defaultPreferenceName);
        if (this.grid.preferencePersistenceMode != "server")
            UIUtils.showMessage("Preferences Saved!");
        UIUtils.removePopUp(this);

    }

    onCancel(evt) {
        UIUtils.removePopUp(this);
    }

    openSettings_cbDefaultChangeHandler() {
        //TODO
        /* if(event.currentTarget.selected)
        gpi.defaultPreferenceName=(rptPrefs.dataProvider.getItemAt(event.currentTarget.repeaterIndex) as PreferenceInfo).name*/
    }

    onDeleteLinkBtnlick(evt) {
        this.onDeletePref(rptPrefs.dataProvider.getItemAt(event.currentTarget.repeaterIndex).implementsOrExtends("PreferenceInfo"));
    }

    onDeletePref(pref) {

        if (pref.isSystemPref) {
            window.alert('This is a system preference, it cannot be deleted');
            return;
        }
        if (window.confirm("Are you sure you wish to delete this preference?")) {
            this.gpi.savedPreferences.splice(this.gpi.savedPreferences.indexOf(pref), 1);
        }
    }

    onDeleteLinkBtnlick(i) {
        this.onDeletePref(this.gpi.savedPreferences[i]);
    }

    onApplyPref(pref) {
        const prefs = pref.preferences;
        this.grid.setCurrentPreferenceInfo(pref, true)
        this.currentPref = pref;
        UIUtils.removePopUp(this);

    }

    /**
     * Initializes the auto complete and watermark plugins
     */
    initialize() {
        flexiciousNmsp.UIComponent.prototype.initialize.apply(this);
        let html = "<div>The preferences you specify below will be retained when this grid is loaded in the future:</div>";
        if (this.grid.enableMultiplePreferences) {
            html += `<div ></span><span id="${this.grid.id}-link-button-${i}" class="openSettingslinkbtn"  tabindex="0"><label>  ${Constants.OPEN_SETTINGS_DEFAULT}</label><label>  ${Constants.OPEN_SETTINGS_PREFERENCE_NAME}</label><label>  ${Constants.OPEN_SETTINGS_DELETE}</label><label>  ${Constants.OPEN_SETTINGS_APPLY}</label></div><div style='...'/>`;
        }

        html += "<table border='0' style='...'>" +
            "<tr>" +
            "<td>Name</td>" +
            "<td>Is Default?</td>" +
            "<td>Apply</td>" +
            "<td>Delete</td>" +
            "</tr>";

        const items = this.grid.getGridPreferencesInfo().savedPreferences;
        for (var i = 0; i < items.length; i++) {
            const pi = items[i];
            html += "<tr>" +
                "<td> + item.name + </td>" +
                "<td> + item.defaultPreferenceName + </td>" +
                "</tr>";
        }


        html += "<table border='0' style='...'> <tr><td>" +
            "   <div style='...'>" +
            "<input class='cbDefault' onchange='openSettings_cbDefaultChangeHandler'>" + "</input>" +
            "<label class='txtName'></label>" +
            "<a href=''>[Delete]</a>" +
            "<a href=''>[Apply]</a>" +
            "</td></tr></table>" +
            "</div>";

        html += `<div class='bottomButtonBar'> <a  class='BTN_REMOVE_PREFS button' alt='${Constants.OPEN_SETTINGS_REMOVE_ALL_SAVED_PREFERENCES}'> ${Constants.OPEN_SETTINGS_REMOVE_ALL_SAVED_PREFERENCES}</a> <a  class='BTN_SAVE_CHANGES button' alt='${Constants.OPEN_SETTINGS_SAVE_CHANGES}'> ${Constants.OPEN_SETTINGS_SAVE_CHANGES}</a> <a class='BTN_CLOSE button' alt='${Constants.OPEN_SETTINGS_CLOSE}'> ${Constants.OPEN_SETTINGS_CLOSE}</a></div>`;

        this.setInnerHTML(html);


        const deleteLinkBtn = UIUtils.findElementWithClassName(this.domElement, 'DELETE_LINK_BTN');
        const applyLinkBtn = UIUtils.findElementWithClassName(this.domElement, 'APPLY_LINK_BTN');

        const removeBtn = UIUtils.findElementWithClassName(this.domElement, 'BTN_REMOVE_PREFS');
        const saveBtn = UIUtils.findElementWithClassName(this.domElement, 'BTN_SAVE_CHANGES');
        const closeBtn = UIUtils.findElementWithClassName(this.domElement, 'BTN_CLOSE');


        UIUtils.addDomEventListener(this, cbDefaultBtn, "click", evt => {
            UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'flexiciousSortPopup').component.onDefaultRadioButtonClick();
        });
        UIUtils.addDomEventListener(this, deleteLinkBtn, "click", evt => {
            UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'flexiciousSortPopup').component.onDeleteLinkBtnlick();
        });
        UIUtils.addDomEventListener(this, applyLinkBtn, "click", evt => {
            UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'flexiciousSortPopup').component.onApplyLinkBtnClick();
        });

        UIUtils.addDomEventListener(this, removeBtn, "click", evt => {
            UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'flexiciousGrid').component.onClearSettings();
        });
        UIUtils.addDomEventListener(this, saveBtn, "click", evt => {
            UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'flexiciousGrid').component.onSaveSettings();
        });
        UIUtils.addDomEventListener(this, closeBtn, "click", evt => {
            UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'flexiciousGrid').component.onCancel();
        });
    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.OpenSettingsPopup = OpenSettingsPopup; //add to name space
OpenSettingsPopup.prototype.typeName = OpenSettingsPopup.typeName = 'OpenSettingsPopup';//for quick inspection