/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'
import PreferencePersistenceEvent from '../events/PreferencePersistenceEvent'
/**
 * Class responsible for preference persistence.
 * @constructor
 * @class UserSettingsController
 * @namespace flexiciousNmsp
 * 
 */
export default class UserSettingsController extends TypedObject {


    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["UserSettingsController", "TypedObject"];
    }

    /**
     *
     * @return {*}
     */
    static instance() {
        if (!UserSettingsController._instance) {
            UserSettingsController._instance = new UserSettingsController();
        }
        return UserSettingsController._instance;

    }

    /**
     * Clears preferences
     *  @param userSettingsOptions
     */
    clearPreferences(userSettingsOptions) {

        const grid = userSettingsOptions.persistable;
        if (grid.preferencePersistenceMode == "client") {
            if (UIUtils.hasLocalStorage()) {
                window.localStorage[grid.getPreferencePersistenceKey()] = null;
            } else {
                //use cookies??
                UIUtils.showError('Local Storage is not supported for your browser. Please implement server persistence or disable persistence');
            }

        }
        else {
            const preferencePersistenceEvent = new flexiciousNmsp.PreferencePersistenceEvent(
                flexiciousNmsp.PreferencePersistenceEvent.CLEAR_PREFERENCES, grid.getPreferencePersistenceKey()
                , null);
            grid.dispatchEvent(preferencePersistenceEvent);
        }
        if (grid.enableMultiplePreferences) {
            grid.getGridPreferencesInfo().savedPreferences = [];//if we remove, remove from everywhere.
        }


    }

    /**
     *  Loads preferences
     * @param userSettingsOptions
     */
    loadPreferences(userSettingsOptions) {

        const grid = userSettingsOptions.persistable;
        const key = grid.getPreferencePersistenceKey();

        const preferencePersistenceEvent = new flexiciousNmsp.PreferencePersistenceEvent(flexiciousNmsp.PreferencePersistenceEvent.PREFERENCES_LOADING, key, null);
        grid.dispatchEvent(preferencePersistenceEvent);
        if (grid.preferencePersistenceMode == "client") {
            let prefs;
            if (UIUtils.hasLocalStorage()) {
                try {
                    prefs = localStorage[grid.getPreferencePersistenceKey()];
                } catch (e) {
                    prefs = null;
                }
            } else {
                //use cookies??
                UIUtils.showError('Local Storage is not supported for your browser. Please implement server persistence or disable persistence');
            }
            var customData = "";
            if (grid.enableMultiplePreferences && prefs != null) {
                var gpo = new flexiciousNmsp.GridPreferencesInfo().fromPreferenceString(userSettingsOptions, prefs);
                grid.setGridPreferencesInfo(gpo, false);

                for (const pi of gpo.savedPreferences) {
                    if (pi.name == gpo.defaultPreferenceName) {
                        prefs = pi.preferences;
                        grid.setCurrentPreferenceInfo(pi, false);
                    }
                }
            }

            if (prefs != null && prefs != "null") {
                if (prefs.includes(userSettingsOptions.prefCustomDataDelimiter)) {
                    const prefParts = prefs.split(userSettingsOptions.prefCustomDataDelimiter);
                    //prefs=prefParts[0];
                    customData = prefParts[1];
                }
                if (!grid.enableMultiplePreferences || gpo.loadDefaultPreferenceOnCreationComplete)
                    grid.setPreferences(prefs);
            }
        }
        else {

            const preferencePersistenceEvent1 = new flexiciousNmsp.PreferencePersistenceEvent(
                flexiciousNmsp.PreferencePersistenceEvent.LOAD_PREFERENCES, key, null);
            preferencePersistenceEvent1.customData = customData;
            grid.dispatchEvent(preferencePersistenceEvent1);
        }

    }

    /**
     * Saves the preferences
     * @param userSettingsOptions
     * @param name
     * @param isDefault
     */
    persistPreferences(userSettingsOptions, name, isDefault) {
        if (typeof name == "undefined") name = 'Default';
        if (typeof isDefault == "undefined") isDefault = false;


        const grid = userSettingsOptions.persistable;
        let prefs = this.getPreferencesString(userSettingsOptions);
        const preferencePersistenceEvent = new flexiciousNmsp.PreferencePersistenceEvent(flexiciousNmsp.PreferencePersistenceEvent.PERSIST_PREFERENCES, grid.getPreferencePersistenceKey(), prefs);
        preferencePersistenceEvent.preferenceName = name;
        preferencePersistenceEvent.isDefault = isDefault;
        grid.dispatchEvent(preferencePersistenceEvent);

        grid.preferencesSet = true;
        if (preferencePersistenceEvent.customData) {
            prefs += userSettingsOptions.prefCustomDataDelimiter + preferencePersistenceEvent.customData;
        }
        if (grid.enableMultiplePreferences) {
            let preferenceInfo = null;

            for (const po of grid.getGridPreferencesInfo().savedPreferences) {
                if (po.name == name) {
                    preferenceInfo = po;
                    break;
                }
            }

            if (!preferenceInfo) {
                preferenceInfo = new flexiciousNmsp.PreferenceInfo();
                preferenceInfo.name = name;
                grid.getGridPreferencesInfo().savedPreferences.push(preferenceInfo);
            }
            preferenceInfo.preferences = prefs;
            if (isDefault) {
                grid.getGridPreferencesInfo().defaultPreferenceName = name;
            }
            if (grid.preferencePersistenceMode == "client") {
                if (UIUtils.hasLocalStorage()) {
                    localStorage[grid.getPreferencePersistenceKey()] = grid.getGridPreferencesInfo().toPreferenceString(userSettingsOptions);
                } else {
                    //use cookies??
                    UIUtils.showError('Local Storage is not supported for your browser. Please implement server persistence or disable persistence');
                }

            }
            grid.setCurrentPreferenceInfo(preferenceInfo, false);
        } else {

            if (grid.preferencePersistenceMode == "client") {

                if (UIUtils.hasLocalStorage()) {
                    localStorage[grid.getPreferencePersistenceKey()] = prefs;
                } else {
                    //use cookies??
                    UIUtils.showError('Local Storage is not supported for your browser. Please implement server persistence or disable persistence');
                }

            }
        }


    }

    /**
     * Gets the preferences string for the supplied grid
     * @param userSettingsOptions
     * @return {*}
     */
    getPreferencesString(userSettingsOptions) {

        const grid = userSettingsOptions.persistable;
        return this.getCompactPreferencesString(userSettingsOptions);
    }

    /**
     *
     * @param userSettingsOptions
     * @param val
     * @return {*}
     */
    parsePreferences(userSettingsOptions, val) {

        if (!val || val.length == 0) return [];
        const grid = userSettingsOptions.persistable;
        try {
            return this.parseCompactPreferencesString(userSettingsOptions, val);
        } catch (e) {
            if (userSettingsOptions.silentFailure) return [];
            if (userSettingsOptions.allowClearOnCorruption) {
                if (window.confirm(userSettingsOptions.showErrorMessageWhenCorrupt.replace("__ERRORMESSAGE__", e.message)))
                    grid.clearPreferences();
            } else {
                throw e;
            }
        }
        return [];

    }

    /**
     *
     * @param userSettingsOptions
     * @param val
     * @return {Array}
     */
    parseCompactPreferencesString(userSettingsOptions, val) {
        const result = [];
        const prefs = val.split(userSettingsOptions.prefDelimiter);
        const colInfos = prefs[0].split(userSettingsOptions.prefColDelimiter);
        const noVal = "o";
        const colOrders = [];
        const colVisibilities = [];
        const colWidths = [];
        const colFilters = [];
        const colSorts = [];
        const grid = userSettingsOptions.persistable;

        for (const colInfoData of colInfos) {
            const colInfo = colInfoData.split(userSettingsOptions.prefColPrefDelimiter);
            const colHeaderText = colInfo[0];
            colOrders.push(colHeaderText);
            if (colInfo[1] != noVal) {
                colVisibilities.push({ key: colHeaderText, property: colInfo[1] == "y" })
            }
            if (colInfo[2] != noVal) {
                colWidths.push({ key: colHeaderText, property: colInfo[2] })
            }
            if (colInfo[3] != noVal) {
                if (colInfo[3] != null)
                    colFilters.push({ key: colHeaderText, property: UIUtils.fromPersistenceString(colInfo[3]) });
            }
            if (colInfo[4] != noVal) {
                const fs = new flexiciousNmsp.FilterSort();
                const items = grid.getColumns();

                for (const col of items) {
                    if (col.getUniqueIdentifier() == colHeaderText) {
                        fs.sortColumn = col.getSortFieldName();
                        fs.isAscending = colInfo[4] == "y";
                        colSorts.push(fs);
                        break;
                    }
                }
            }
        }

        result.push({ key: Constants.PERSIST_COLUMN_ORDER, data: colOrders });
        if (colVisibilities.length > 0)
            result.push({ key: Constants.PERSIST_COLUMN_VISIBILITY, data: colVisibilities });
        if (colWidths.length > 0)
            result.push({ key: Constants.PERSIST_COLUMN_WIDTH, data: colWidths });
        if (colSorts.length > 0)
            result.push({ key: Constants.PERSIST_SORT, data: colSorts });
        if (colFilters.length > 0)
            result.push({ key: Constants.PERSIST_FILTER, data: colFilters });

        if (prefs[1] != noVal) {
            result.push({ key: Constants.PERSIST_VERTICAL_SCROLL, data: (prefs[1].toString()) });
        }
        if (prefs[2] != noVal) {
            result.push({ key: Constants.PERSIST_HORIZONTAL_SCROLL, data: (prefs[2].toString()) });
        }
        if (prefs[3] != noVal) {
            result.push({ key: Constants.PERSIST_FOOTER_FILTER_VISIBILITY, data: ([prefs[3].toString().charAt(0) == "y", prefs[3].toString().charAt(1) == "y"]) });
        }
        if (prefs[4] != noVal) {
            result.push({ key: Constants.PERSIST_PAGE_SIZE, data: (prefs[4].toString()) });
        }
        if (prefs[5] != noVal) {
            result.push({ key: Constants.PERSIST_PRINT_SETTINGS, data: (prefs[5].toString()) });
        }

        return result;
    }

    /**
     *
     * @param userSettingsOptions
     * @return {String}
     */
    getCompactPreferencesString(userSettingsOptions) {
        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;
        const prefBuffer = "";
        const preferenceKeys = grid.preferencesToPersist.split(",");
        const prefStrings = [];

        const cols = grid.getColumns();
        let colSettings = "";
        let colIndex = 0;
        const delim = userSettingsOptions.prefColPrefDelimiter;
        const sorts = grid.createFilter().sorts;

        for (const col of cols) {
            //var colIdentifier=col.hasOwnProperty("uniqueIdentifier")?col.uniqueIdentifier:col.getHeaderText();
            const colIdentifier = col.getUniqueIdentifier();
            let filterValue = null;
            if (col.getSearchField())
                filterValue = grid.getFilterValue(col.getSearchField());
            let sort = null;

            for (const srt of sorts) {
                if (srt.sortColumn == col.getSortFieldName()) {
                    sort = srt;
                    break;
                }
            }

            colSettings += colIdentifier + delim;
            colSettings += (preferenceKeys.indexOf(Constants.PERSIST_COLUMN_VISIBILITY) ? col.getVisible() ? "y" : "n" : "o") + delim;
            colSettings += (preferenceKeys.indexOf(Constants.PERSIST_COLUMN_WIDTH) ? parseInt(col.getWidth()).toString() : "o") + delim;
            colSettings += ((preferenceKeys.indexOf(Constants.PERSIST_FILTER) && (filterValue != null) && filterValue != "") ? UIUtils.toPersistenceString(filterValue) : "o") + delim;
            colSettings += ((preferenceKeys.indexOf(Constants.PERSIST_SORT) && sort) ? (sort.isAscending ? "y" : "n") : "o") + delim;

            colSettings += userSettingsOptions.prefColDelimiter;
            colIndex++;
        }

        colSettings += userSettingsOptions.prefDelimiter;
        colSettings += (preferenceKeys.indexOf(Constants.PERSIST_VERTICAL_SCROLL) ? grid.getVerticalScrollPosition() : "o") + userSettingsOptions.prefDelimiter;
        colSettings += (preferenceKeys.indexOf(Constants.PERSIST_HORIZONTAL_SCROLL) ? grid.getHorizontalScrollPosition() : "o") + userSettingsOptions.prefDelimiter;
        colSettings += (preferenceKeys.indexOf(Constants.PERSIST_FOOTER_FILTER_VISIBILITY) ? (grid.footerVisible ? "y" : "n") + (grid.filterVisible ? "y" : "n") : "o") + userSettingsOptions.prefDelimiter;
        colSettings += (preferenceKeys.indexOf(Constants.PERSIST_PAGE_SIZE) ? grid.getPageSize() : "o") + userSettingsOptions.prefDelimiter;
        colSettings += (preferenceKeys.indexOf(Constants.PERSIST_PRINT_SETTINGS) ? grid.lastPrintOptionsString ? grid.lastPrintOptionsString : grid.persistedPrintOptionsString : "o") + userSettingsOptions.prefDelimiter;

        return colSettings;
    }

    /**
     *
     * @param userSettingsOptions
     * @return {Array}
     */
    getLegacyPreferences(userSettingsOptions) {

        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;

        const preferenceKeys = grid.preferencesToPersist.split(",");
        const preferences = [];
        for (let i = 0; i < preferenceKeys.length; i++)
            var preferenceKey = preferenceKeys[i];
        {
            if (preferenceKey == Constants.PERSIST_COLUMN_ORDER) {
                preferences.push({ key: preferenceKey, data: this.getColumnOrder(userSettingsOptions) });
            }
            else if (preferenceKey == Constants.PERSIST_COLUMN_VISIBILITY) {
                preferences.push({ key: preferenceKey, data: this.getColumnVisibility(userSettingsOptions) });
            }
            else if (preferenceKey == Constants.PERSIST_COLUMN_WIDTH) {
                preferences.push({ key: preferenceKey, data: this.getColumnWidths(userSettingsOptions) });
            }
            else if (preferenceKey == Constants.PERSIST_FILTER) {
                const filterPrefs = [];
                const items = grid.getColumns();
                let filterValue = null;
                for (const column of items) {
                    if (column.getSearchField())
                        filterValue = grid.getFilterValue(column.getSearchField());
                    if (filterValue) {
                        filterPrefs.push({ key: column.getUniqueIdentifier(), property: filterValue });
                    }
                }

                preferences.push({ key: preferenceKey, data: filterPrefs });
            }
            else if (preferenceKey == Constants.PERSIST_SORT) {
                preferences.push({ key: preferenceKey, data: grid.createFilter().sorts });
            }
            else if (preferenceKey == Constants.PERSIST_VERTICAL_SCROLL) {
                preferences.push({ key: preferenceKey, data: grid.getVerticalScrollPosition() });
            }
            else if (preferenceKey == Constants.PERSIST_HORIZONTAL_SCROLL) {
                preferences.push({ key: preferenceKey, data: grid.getHorizontalScrollPosition() });
            }
            else if (preferenceKey == Constants.PERSIST_FOOTER_FILTER_VISIBILITY) {
                preferences.push({ key: preferenceKey, data: ([grid.footerVisible, grid.filterVisible]) });
            }
            else if (preferenceKey == Constants.PERSIST_PAGE_SIZE) {
                preferences.push({ key: preferenceKey, data: grid.getPageSize() });
            }
            else if (preferenceKey == Constants.PERSIST_PRINT_SETTINGS) {
                preferences.push({ key: preferenceKey, data: grid.lastPrintOptionsString ? grid.lastPrintOptionsString : grid.persistedPrintOptionsString });
            }
        }
        return preferences;

    }

    /**
     * Gets the order of columns
     * @param userSettingsOptions
     * @return {Array}
     */
    getColumnOrder(userSettingsOptions) {
        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;
        const cols = [];
        const items = grid.getColumns();

        for (const column of items) {
            cols.push(column.getUniqueIdentifier());
        }

        return cols;
    }

    /**
     *
     * @param userSettingsOptions
     * @param property
     * @return {Array}
     */
    getPropertyValues(userSettingsOptions, property) {
        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;
        const prefs = [];
        const items = grid.getColumns();

        for (const column of items) {
            if (column.hasOwnProperty(property))
                prefs.push({ key: column.getUniqueIdentifier(), property: column[property] });
        }

        return prefs;
    }

    /**
     *
     * @param userSettingsOptions
     */
    getColumnVisibility(userSettingsOptions) {

        return this.getPropertyValues(userSettingsOptions, "visible");

    }

    /**
     * Gets the widths of the columns
     * @param [userSettingsOptions]
     * @return
     */
    getColumnWidths(userSettingsOptions) {
        return this.getPropertyValues(userSettingsOptions, "width");
    }

    /**
     * Applies previously set preferences..
     * @param [userSettingsOptions]
     * @param [arrayCollection]
     *
     */
    setPreferences(userSettingsOptions, arrayCollection) {

        try {
            var grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;

            for (const item of arrayCollection) {
                if (item.key == Constants.PERSIST_COLUMN_ORDER) {
                    this.setColumnOrder(userSettingsOptions, item.data);
                }
                else if (item.key == Constants.PERSIST_COLUMN_VISIBILITY) {
                    this.setColumnVisibility(userSettingsOptions, item.data);
                }
                else if (item.key == Constants.PERSIST_COLUMN_WIDTH) {
                    this.setColumnWidths(userSettingsOptions, item.data);
                }
                else if (item.key == Constants.PERSIST_SORT) {
                    grid.processSort(item.data);
                }
                else if (item.key == Constants.PERSIST_FILTER) {
                    grid.clearFilter();
                    //grid.validateNow();
                    const items = grid.getColumns();

                    for (const column of items) {
                        for (const pref of item.data) {
                            if (column && pref && (column.getUniqueIdentifier() == pref.key)) {
                                if (pref.property != null) {
                                    grid.setFilterValue(column.getSearchField(), pref.property.hasOwnProperty("item") ? pref.property.item : pref.property, false);
                                }
                            }
                        }
                    }
                    //grid.validateNow();
                }

                else if (item.key == Constants.PERSIST_VERTICAL_SCROLL) {
                    grid.setVerticalScrollPosition(item.data);
                }
                else if (item.key == Constants.PERSIST_HORIZONTAL_SCROLL) {
                    grid.horizontalScrollPosition = item.data;
                }
                else if (item.key == Constants.PERSIST_FOOTER_FILTER_VISIBILITY) {
                    grid.footerVisible = item.data[0];
                    grid.filterVisible = item.data[1];
                }
                else if (item.key == Constants.PERSIST_PAGE_SIZE) {
                    grid.setPageSize(parseInt(item.data));
                }
                else if (item.key == Constants.PERSIST_PRINT_SETTINGS) {
                    grid.persistedPrintOptionsString = item.data ? item.data.toString() : "";
                }
            }
        }
        catch (e) {
            if (userSettingsOptions.silentFailure)
                return;
            if (userSettingsOptions.allowClearOnCorruption) {
                if (window.confirm(userSettingsOptions.showErrorMessageWhenCorrupt.replace("__ERRORMESSAGE__", e.message)))
                    grid.clearPreferences();
            } else {
                throw e;
            }
        }


    }

    /**
     * Sets the order of columns
     * @param [userSettingsOptions]
     * @param [val]
     *
     */

    setColumnOrder(userSettingsOptions, val) {

        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;
        const persistedColumns = [];
        //grid.columns=persistedColumns.source;
        //create a new array collection of the size of the existing columns
        let persistedCols = [];
        if (grid.getHasGroupedColumns()) {
            //if has column groups{
            const groupedCols = grid.getGroupedColumns();
            persistedCols = (new Array(Math.max(groupedCols.length, val.length)));

            //for(var i=0;i<.length;i++)
            //	var s=[i];

            for (const col of groupedCols) {
                var colIndex = 0;
                if (col.implementsOrExtends("FlexDataGridColumnGroup")) {//if it is a column group
                    //get the index of its first persisted column.
                    //add the column group at that index
                    const firstCol = this.getFirstColumn(col);
                    if (firstCol) {
                        colIndex = val.indexOf(firstCol.getUniqueIdentifier());
                    }
                    //if this is a "column only" columnGroup, re arrange its columns.
                    this.rearrangeColumns(col, val, userSettingsOptions);
                } else {
                    //get its index in the preferences string, and add it to the array collection at its persisted index
                    colIndex = val.indexOf(col.getUniqueIdentifier());
                }
                //if it does not exist in the preferences string, add it to the array collection at its current index
                if (colIndex == -1) {
                    colIndex = grid.getColumns().indexOf(col);
                }
                if (colIndex == -1) {
                    colIndex = groupedCols.indexOf(col);
                }
                while (persistedCols[colIndex] && (colIndex < persistedCols.length)) {
                    colIndex++
                }
                if (persistedCols[colIndex]) {
                    persistedCols.push(col)
                }
                else
                    persistedCols[colIndex] = col;
            }

            grid.setGroupedColumns(this.removeUndefined(persistedCols));
        }
        else {
            const persistedColsArray = (new Array(grid.getColumns().length));
            const items = grid.getColumns();

            for (const col2 of items) {
                //get its index in the preferences string, and add it to the array collection at its persisted index
                colIndex = val.indexOf(col2.getUniqueIdentifier());
                //if it does not exist in the preferences string, add it to the array collection at its current index
                if (colIndex == -1) {
                    colIndex = grid.getColumns().indexOf(col2);
                }
                while (persistedColsArray[colIndex] && (colIndex < persistedColsArray.length)) {
                    colIndex++
                }
                if (persistedColsArray[colIndex]) {
                    persistedColsArray.push(col2)
                }
                else
                    persistedColsArray[colIndex] = col2;
                //var val2 = UIUtils.extractPropertyValues(persistedCols,"headerText").source.join("|");
            }

            grid.setColumns(this.removeUndefined(persistedColsArray));
            //var val1 = UIUtils.extractPropertyValues(new flexiciousNmsp.ArrayCollection(grid.columns),"headerText").source.join("|");
        }
        //set the new grouped columns
    }

    /**
     *
     * @param col
     * @param val
     * @param userSettingsOptions
     */
    rearrangeColumns(col, val, userSettingsOptions) {

        //if this is a "column only" columnGroup, re arrange its columns on basis of preferences.
        const isColumnOnly = col.implementsOrExtends("FlexDataGridColumnGroup") ? col.getIsColumnOnly() : this.getIsColumnOnly(col);
        if (isColumnOnly) {
            const columns = col.getColumns();
            const resultCols = [];

            for (const uq of val) {
                for (const childCol of columns) {
                    if (childCol.getUniqueIdentifier() == uq) {
                        resultCols.push(childCol);
                    }
                }
            }

            for (const childCol1 of columns) {
                if (!resultCols.includes(childCol1)) {
                    resultCols.push(childCol1);
                }
            }

            if (col.implementsOrExtends("FlexDataGridColumnGroup") && (userSettingsOptions.persistable))
                col.setColumns(resultCols);
            else {
                col.setColumns(resultCols);
                //this is ultimate
            }
        } else {
            const cgs = col.hasOwnProperty("columnGroups") ? col["columnGroups"] : col.children;

            for (const childGrp of cgs) {
                if (childGrp.implementsOrExtends("FlexDataGridColumnGroup")) {
                    this.rearrangeColumns(childGrp, val, userSettingsOptions);
                }
            }
        }

    }

    /**
     *
     * @param col
     * @return {Boolean}
     */
    getIsColumnOnly(col) {
        const children = col.getAllColumns();

        for (const child of children) {
            if (child.implementsOrExtends("FlexDataGridColumnGroup")) {
                return false;
            }
        }

        return true;
    }

    /**
     *
     * @param source
     * @return {Array}
     */
    removeUndefined(source) {
        const result = [];

        for (const item of source) {
            if (item)
                result.push(item);
        }

        return result;
    }

    /**
     *
     * @param col
     * @return {*}
     */
    getFirstColumn(col) {

        const children = col.getAllColumns();
        for (let i = 0; i < children; i++) {
            const child = children[i];
            if (!child.implementsOrExtends("FlexDataGridColumnGroup")) {
                //this means this is a column
                return child;
            } else {
                //this means this is a column group
                const result = this.getFirstColumn(child);
                if (result)
                    return result;
            }
        }
        return null;

    }

    /**
     * @param [userSettingsOptions]
     * @param [val]
     * @param [property]
     *
     */
    setPropertyValues(userSettingsOptions, val, property) {
        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;
        const items = grid.getColumns();

        for (const column of items) {
            for (const pref of val) {
                if (column.getUniqueIdentifier() == pref.key) {
                    if (userSettingsOptions.userWidthsOverrideFitToContent && property == "width" && column.hasOwnProperty("columnWidthMode")
                        && column.getColumnWidthMode() == "fitToContent") {
                        column.setColumnWidthMode("fixed");
                    }
                    if (property == "width" && pref.property) {
                        pref.property = parseInt(pref.property);
                    }
                    UIUtils.checkSetterAndApply(column, property, pref.property);
                }
            }
        }
    }

    /**
     * Sets the visibility of the columns
     * @param userSettingsOptions
     * @param val
     */
    setColumnVisibility(userSettingsOptions, val) {
        this.setPropertyValues(userSettingsOptions, val, "visible");
    }

    /**
     * Sets the widths of the columns
     * @param [userSettingsOptions]
     * @param [val]
     *
     */
    setColumnWidths(userSettingsOptions, val) {

        const grid = userSettingsOptions.persistable.implementsOrExtends("IExtendedDataGrid") ? userSettingsOptions.persistable : null;
        this.setPropertyValues(userSettingsOptions, val, "width");
    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.UserSettingsController = UserSettingsController; //add to name space
UserSettingsController.prototype.typeName = UserSettingsController.typeName = 'UserSettingsController';//for quick inspection