/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'

/**
 * Class responsible for storing information about the user preference persistence settings
 * @constructor
 * @class UserSettingsOptions
 * @namespace flexiciousNmsp
 * 
 */
export default class UserSettingsOptions extends TypedObject {
    constructor() {
        super();
        /**
         * If there is an error during parse preferences, dont apply preferences, and fail silently.
         * @type {Boolean}
         * @property silentFailure
         * @default false
         */
        this.silentFailure = false;
        /**
         * If for any reason, the preferences become corrupt, prompt the user to clear them as
         * opposed to throwing an error?
         * @type {Boolean}
         * @property allowClearOnCorruption
         * @default true
         */
        this.allowClearOnCorruption = true;
        /**
         * If for any reason, the preferences become corrupt, prompt the user to clear them as
         * opposed to throwing an error, and in the prompt, the error message to show?
         * @type {String}
         * @property showErrorMessageWhenCorrupt
         * @default Error occurred while applying preferences: __ERRORMESSAGE__. Do you wish to clear preferences?
         */
        this.showErrorMessageWhenCorrupt = "Error occurred while applying preferences: __ERRORMESSAGE__. Do you wish to clear preferences?";
        /**
         ** If set to true, uses the newer preference persistence scheme introduced in
         * Flexicious 2.0, which stores the preferences as custom string as opposed to XML,
         * making it a lot less verbose. For Ultimate, this is always true
         * @type {Boolean}
         * @property useCompactPreferences
         * @default true
         */
        this.useCompactPreferences = true;
        /**
         * The delimiter that separates the string for two preferences, defaults to ~|
         * @type {String}
         * @property prefDelimiter
         * @default ~|
         */

        this.prefDelimiter = "~|";
        /**
         * The delimiter that separates the string for two columns within a single preference
         * @type {String}
         * @property prefColDelimiter
         */
        this.prefColDelimiter = "@|";
        /**
         * The delimiter that separates the string for two preferences within columns within a single preference
         * @type {String}
         * @property prefColPrefDelimiter
         * @default &|
         */
        this.prefColPrefDelimiter = "&|";
        /**
         * The delimiter that separates the IPersistables data from custom data
         * @type {String}
         * @property prefCustomDataDelimiter
         * @default #|
         */
        this.prefCustomDataDelimiter = "#|";
        /**
         * The delimiter that separates two preferences from each other
         * @type {String}
         * @property multiPrefDelimiter
         * @default $|
         */
        this.multiPrefDelimiter = "$|";
        /**
         * The delimiter that separates the properties of the GridPreferencesInfo object
         * @type {String}
         * @property multiPrefGridPrefPropDelimiter
         * @default *|
         */
        this.multiPrefGridPrefPropDelimiter = "*|";
        /**
         * The delimiter that separates the properties of the PreferenceInfo object
         * @type {String}
         * @property multiPrefPrefPropDelimiter
         * @default %|
         */
        this.multiPrefPrefPropDelimiter = "%|";
        /**
         * The IPersistable being persisted
         * @type {null}
         * @property persistable
         * @default null
         */
        this.persistable = null;
        /**
         * Class responsible for showing the UI that prompts the user to save or clear preferences
         * @type {flexiciousNmsp.ClassFactory}
         * @property saveSettingsPopupRenderer
         */
        this.saveSettingsPopupRenderer = new flexiciousNmsp.ClassFactory(flexiciousNmsp.SaveSettingsPopup, null);
        /**
         * Class responsible for showing the UI that prompts the user to save or clear preferences
         * @type {flexiciousNmsp.ClassFactory}
         * @property openSettingsPopupRenderer
         */
        this.openSettingsPopupRenderer = new flexiciousNmsp.ClassFactory(flexiciousNmsp.OpenSettingsPopup, null);
        /**
         *Class responsible for showing the UI that prompts the user to display the preferences
         * @type {flexiciousNmsp.ClassFactory}
         * @property settingsPopupRenderer
         */
        this.settingsPopupRenderer = new flexiciousNmsp.ClassFactory(flexiciousNmsp.SettingsPopup, null);
        /**
         * If set to true, will mark the columns that have column width mode = fitToContent as fixed when
         * preferences are applied. This will cause them to no longer size according to their contents, but
         * respect the width that the user set.
         * @type {Boolean}
         * @property userWidthsOverrideFitToContent
         * @default false
         */
        this.userWidthsOverrideFitToContent = false;


    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["UserSettingsOptions", "TypedObject"];
    }

    /**
     * Returns an options object for this grid.
     * @param grid
     */
    static create(grid) {
        const options = new UserSettingsOptions();
        options.persistable = grid;
        options.useCompactPreferences = true;
        options.grid = grid;
        return options;
    }

    /**
     * Quick accessor for persistable grid
     * @return {null}
     */

    getGrid() {
        return this.persistable;
    }
}

flexiciousNmsp.UserSettingsOptions = UserSettingsOptions; //add to name space
UserSettingsOptions.prototype.typeName = UserSettingsOptions.typeName = 'UserSettingsOptions';//for quick inspection