/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

(function(window)
{
    "use strict";
    var PrintController, uiUtil = flexiciousNmsp.UIUtils, flxConstants = flexiciousNmsp.Constants;
    /**
     *  This class is responsible for actual print and
     * print preview functionality
     * @constructor
     * @namespace com.flexicious.print
     * 
     */
    PrintController=function(){

        this._extendedDataGrid = null;

        this._printWindow = null;

        this._printOptions = null;

        this.totalDirty = true;

        this.printJob = null;

        this.serverDataRetrieved = false;

        this.worker = null;

        this._pageShown = false;

        this._instance = null;


        flexiciousNmsp.TypedObject.apply(this);

    };
    flexiciousNmsp.PrintController = PrintController; //add to name space
    PrintController.prototype = new flexiciousNmsp.TypedObject(); //setup hierarchy
    PrintController.prototype.typeName = PrintController.typeName = 'PrintController';//for quick inspection
    PrintController.prototype.getClassNames=function(){
        return ["PrintController","TypedObject"];
    };
    PrintController._instance = new PrintController();
    PrintController.instance=function(){
        return PrintController._instance;
    };
    /**
     * Called by the grid to print its contents to the printer on basis
     * of the passed in print options. If the preview property of the
     * printOptions parameter is true, it will popup a preview window.
     * @param grid			IExtendedDataGrid to print
     * @param [printOptions]	PrintOptions to use for printing
     */
    PrintController.prototype.print=function(printable, printOptions){
        if(typeof  printOptions=="undefined") printOptions=null;

        if(printOptions.asynch && ! printOptions.preview){
            printOptions.preview=true;
        }
        flexiciousNmsp.UIUtils.removeAll(printOptions.availableColumns);
        flexiciousNmsp.UIUtils.removeAll(printOptions.columnsToPrint);
        flexiciousNmsp.UIUtils.removeAll(printOptions.columnWidths);

        var grid=printable;
        //If no print options, create default.
        if (printOptions == null)
            printOptions=new flexiciousNmsp.PrintOptions();
        printOptions.printable=grid;
        if(grid.preferencesToPersist.indexOf(Constants.PERSIST_PRINT_SETTINGS))
        {
            var preferences=grid.lastPrintOptionsString?grid.lastPrintOptionsString:
                grid.persistedPrintOptionsString;
            if(preferences)
                printOptions.loadFromPersistedString(grid,preferences);
        }
        var colIndex=0;
        if(printOptions.hideHiddenColumns){
            flexiciousNmsp.UIUtils.removeAll(printOptions.columnsToPrint);
        }
        var items=grid.getPrintableColumns(printOptions);
        for(var i=0;i<items.length;i++) {
            var col=items[i];

            var colName=col.getUniqueIdentifier();
            printOptions.availableColumns.push({'name':colName,'headerText':col.getConcatenatedHeaderText()});
            if(printOptions.hideHiddenColumns){
                if(col.getVisible()){
                    printOptions.columnsToPrint.push({'name':colName});
                }
            }
            colIndex++;
        }

        this._extendedDataGrid=grid;
        this._printOptions=printOptions;
        this._printOptions.printExportParameters=this._extendedDataGrid.printExportParameters;

        var printView=printOptions.printOptionsViewrenderer.newInstance();
        //this is the options popup
        printView.setGrid(this._extendedDataGrid);
        printView.printOptions=(printOptions);
        printOptions.printOptionsView=printView;
        printView.addEventListener(this,Constants.EVENT_CLOSE, this.onPrintOptionsClose,false,0,true);
        if(this._printOptions.showColumnPicker){
            //if user clicks ok, print/preview
            var popupParent=printOptions.popupParent;
            if(!popupParent)popupParent=UIUtils.getTopLevelApplication();
            UIUtils.addPopUp(printView , popupParent ,printOptions.modalWindows,null,printOptions.printOptionsWindowTitle, {context : this, method : "onPrintOptionsCancel"});
        }else{
            var evt = {target:printView,detail:Constants.ALERT_OK};
            this.onPrintOptionsClose(evt);
        }
    };

    /**
     * @param e
     * callback for popup window getting close by the X button.
     */
    PrintController.prototype.onPrintOptionsCancel = function(e){
        this._extendedDataGrid.dispatchEvent(new flexiciousNmsp.PrintPreviewEvent(flexiciousNmsp.PrintPreviewEvent.PRINT_CANCELED));
    };

    PrintController.prototype.onPrintOptionsClose=function (event){

        (event.target).removeEventListener(Constants.EVENT_CLOSE, this.onPrintOptionsClose);
        if (event.detail == Constants.ALERT_OK)
        {
            if(this._extendedDataGrid.getFilterPageSortMode()=="server" &&
                (this.getPrintOptions().printExportOption==flexiciousNmsp.PrintExportOptions.PRINT_EXPORT_ALL_PAGES ||
                    this.getPrintOptions().printExportOption==flexiciousNmsp.PrintExportOptions.PRINT_EXPORT_SELECTED_PAGES) &&
                !this.serverDataRetrieved){
                //here, we're in server mode, user has asked us to print data that we dont have.
                //so go back to server, get the data, and then continue.
                var printExportFilter=new PrintExportFilter();
                printExportFilter.copyFrom(this._extendedDataGrid.createFilter());
                printExportFilter.printExportOptions=this.getPrintOptions();
                this._extendedDataGrid.addEventListener(this,flexiciousNmsp.PrintExportDataRequestEvent.PRINT_EXPORT_DATA_RECD, this.onPrintRequestDataRecieved,false,0,true);
                var printExportDataRequestEvent=new flexiciousNmsp.PrintExportDataRequestEvent(flexiciousNmsp.PrintExportDataRequestEvent.PRINT_EXPORT_DATA_REQUEST,printExportFilter);
                this._extendedDataGrid.dispatchEvent(printExportDataRequestEvent);
            }
            else//we have all the data...
                this.printWithOptions(this._extendedDataGrid, event.target.printOptions);
        } else {
            this._extendedDataGrid.dispatchEvent(new flexiciousNmsp.PrintPreviewEvent(flexiciousNmsp.PrintPreviewEvent.PRINT_CANCELED));
        }

    };
    /**
     *   If the user chose to specify which columns to print, this function
     * is used to identify which columns to print on basis of user selection.
     */
    PrintController.prototype.checkColumn=function (col,  printOptions){

        if (!printOptions.columnsToPrint || printOptions.columnsToPrint.length == 0)
        {
            return true;
        }
        else
        {
            for(var i=0;i<printOptions.columnsToPrint.length;i++)
            {
                var colName=printOptions.columnsToPrint[i];
                if ((colName.name == col.getDataField()) || (colName.name == col.getHeaderText()))
                {
                    return true;
                }
            }
        }
        return false;

    };
    /**
     *  Called by both the print and the preview, to populate the grid on basis of the provided options.
     */
    PrintController.prototype.setupPrintWindow=function (printable, printOptions, printWindow){
        //this is done in ExtendedPrintController.
    };
    PrintController.prototype.cloneColumn=function ( cols, col){

        //we're cloning the column here, because we need to give it to the PrintDataGrid
        //to layout itself. If we just used the DataGrid columns they would become resize
        //with the print grid.
        var newCol;
        if(col.implementsOrExtends('IFactory')){//scenario when the concrete class is a derivative of EDGC or EADGC and potentially in a different swc.
            newCol = (col).newInstance();
        }
        else{
            var clazz=new flexiciousNmsp.FlexDataGridColumn();
        }
        var colCopy=col;

        for(var colProp in colCopy) //we're cloning columns here so as to not upset the orignial grid
        {
            if (col[colProp] && colProp != "width" && colProp != "owner" && colProp != "children"
                &&newCol[colProp])
                newCol[colProp]=col[colProp];//we want to clone all properties except the width and owner
            //do not want sort in preview
            newCol.sortable=false;
        }
        //flexiciousNmsp.FlexVersionSpecific.transferColumnStyles(col,newCol);
        cols.push(newCol);
        if(col.implementsOrExtends("FlexDataGridColumnGroup"))
        {
            for(var i=0;i<col.children.length;i++){
                var child=col.children[i];
                this.cloneColumn(newCol.children,child);
            }
        }

    };

    PrintController.prototype.calculateTotalPages = function (iCollectionView, hasColumnGroups) {

    };
    PrintController.prototype.calculatePreAndPostPages = function (printWindow, totalPages) {

        if (printOptions.preReportPagesFactory && printOptions.preReportPages == 0) {
            var comp = printOptions.preReportPagesFactory.newInstance();
            comp.initializeFromPrintable(printOptions.printable);
            printOptions.preReportPages = comp.calculateTotalPages();
        }
        if (printOptions.postReportPagesFactory && printOptions.postReportPages == 0) {
            var comp1 = printOptions.postReportPagesFactory.newInstance();
            comp1.initializeFromPrintable(printOptions.printable);
            printOptions.postReportPages = comp1.calculateTotalPages();
        }
        printWindow.totalPages = printOptions.preReportPages + totalPages + printOptions.postReportPages;//just so we know in advance what the total number of pages is.

    };
    PrintController.prototype.calculateSpillover = function (printWindow, printOptions, iCollectionView, whichPage) {

        return false;

    };


    /**
     * Called just once to setup the print popup
     * @param grid
     * @param [printOptions]
     * @return
     *
     */
    PrintController.prototype.createPrintWindow=function (printable, printOptions){

        this.totalDirty=true;
        var printWindow=this.instantiatePrintWindow();
        printWindow.printable=printable;
        printWindow.setPrintOptions(printOptions);
        printOptions.printComponentRenderer=printable.createPrintComponentFactory();
        //sets the properties of the print page
        for(var key in printOptions.windowStyleProperties)
        {
            printWindow.setStyle(this.key, printOptions.windowStyleProperties[this.key]);
        }
        //client code should not need to override this
        if (printOptions.printComponentRenderer)
            printWindow.setPrintComponentRenderer(printOptions.printComponentRenderer);
        printOptions.printComponent = printWindow.printComponent;

        //The regular header and footer renderers.
        printWindow.reportHeaderRenderer=printOptions.reportHeaderRenderer;
        printWindow.reportFooterRenderer=printOptions.reportFooterRenderer;
        printWindow.pageHeaderRenderer=printOptions.pageHeaderRenderer;
        printWindow.pageFooterRenderer=printOptions.pageFooterRenderer;

        this._printWindow=printWindow;
        return printWindow;

    };
    /*if (grid is AdvancedDataGrid && !printOptions.printComponentRenderer)
     {
     //Print Data Grid for ExtendedAdvancedDataGrid
     printOptions.printComponentRenderer=new ClassFactory(ExtendedAdvancedPrintDataGrid);
     }*/
    PrintController.prototype.instantiatePrintWindow=function (){

        return new flexiciousNmsp.PrintWindow();

    };
    PrintController.prototype.onPrintRequestDataRecieved=function (event){

        //now we have the data we need, set the flag that we dont need
        //to get data, remove the listener, and continue
        this._extendedDataGrid.removeEventListener(PrintExportDataRequestEvent.PRINT_EXPORT_DATA_RECD, this.onPrintRequestDataRecieved);
        this.serverDataRetrieved=true;
        this.printWithOptions(this._extendedDataGrid,this._printOptions)

    };
    /**
     * The main function to call while printing the grid.
     * @param grid			The IExtendedDataGrid to print
     * @param [printOptions]	The print options
     *
     */
    PrintController.prototype.printWithOptions=function(printable, printOptions){

        this.serverDataRetrieved=false;
        this._printOptions=printOptions;
        flexiciousNmsp.UIUtils.removeAll(printOptions.printedPages);




        if (printOptions.preview && !printOptions.printToPdf)
        {
            this.previewWithOptions(printable, printOptions);
            return ;
        }

        //if (!this.printJob &&!printOptions.printToPdf){
        //    this.printJob=new flexiciousNmsp.FlexPrintJob();
        //    this.printJob.printAsBitmap = printOptions.printAsBitmap;
        //}

        if(1)//if (printOptions.printToPdf || this.printJob.start() )
        {
            if(printOptions.printToPdf){
                var pdfPrinter = printable.pdfPrinter ? printable.pdfPrinter : flexiciousNmsp.JsPdfPrinter ? new flexiciousNmsp.JsPdfPrinter() : null;
                if(!pdfPrinter){
                    alert("Cannot print the grid. PdfPrinter is require.");
                    return;
                }
                printOptions.printable.pdfPrinter.beginDocument(printOptions);
            }
            if(printOptions.asynch && printOptions.previewWindow){
                printOptions.previewWindow.content.removeAllChildren();
                printOptions.printContainer=printOptions.previewWindow.content;
            }


            var printWindow=this.createPrintWindow(printable, printOptions);
            UIUtils.addChild(
                printOptions.printContainer!=null?printOptions.printContainer
                    :document.body,printWindow);
            printWindow.setWidth(printOptions.pageSize.getWidth());
            printWindow.setHeight(printOptions.pageSize.getHeight());
            this.setupPrintWindow(printable, printOptions, printWindow);
            printWindow.setCurrentPage(1);
            if (printWindow.getTotalPages() > 1)
            {
                printWindow.showFirstPage(true);
            }
            else
            {
                printWindow.showSinglePage(true);
            }
            printWindow.validateNow();
            this.gotoCurrentPage();

            if(printOptions.asynch && printOptions.previewWindow){
                if (this.worker)
                {
                    if(printOptions.asynchDelayCapture){
                        this.worker.removeEventListener(Constants.EVENT_TIMER,this.onPrintWorkerTimerDelayedAsych);
                    }else{
                        this.worker.removeEventListener(Constants.EVENT_TIMER,this.onPrintWorkerTimer);
                    }
                    this.worker.stop();
                    this.worker = null;
                }
                this.worker = new flexiciousNmsp.Timer(printOptions.asynchTimeInterval);
                if(printOptions.asynchDelayCapture){
                    this.worker.addEventListener(this,Constants.EVENT_TIMER,this.onPrintWorkerTimerDelayedAsych);
                }else{
                    this.worker.addEventListener(this,Constants.EVENT_TIMER,this.onPrintWorkerTimer);
                }
                this.incapture=false;
                this.capturedPages=[];
                this.getPrintOptions().generatedHTML="";

               /* if(this.getPrintOptions().printToPdf)
                {
                    this.getPrintOptions().printable.pdfPrinter = pdfPrinter;
                    pdfPrinter.beginDocument(this.getPrintOptions());
                    *//*this.jsPdfDoc=new jsPDF(this.getPrintOptions().pageSize.isLandscape?'l':'p');
                    this.jsPdfPrinter = new flexiciousNmsp.jsPdfPrinter();
                    this.jsPdfPrinter.pdfObject = this.jsPdfDoc;*//*
                }*/

                this.worker.start();
            }else{
                this.onPrintWorkerTimer(null);
            }

            UIUtils.disableElement(printOptions.previewWindow.topBar.domElement);
            UIUtils.disableElement(printOptions.previewWindow.bottomBar.domElement);
        }
        else{
            this.printJob=null;
            if(printOptions.previewWindow){
                var pw = printOptions.previewWindow;
                if(pw && pw["isPrinting"]){
                    pw.isPrinting=false;
                }
            }
        }

    };


    PrintController.prototype.printProcessComplete = function(){
        if(this.getPrintOptions().asynch && this.worker){
            this.worker.removeEventListener(Constants.EVENT_TIMER,this.onPrintWorkerTimer);
            this.worker.stop();
            this.worker = null;
        }
        this.onPrintComplete();
    };

    PrintController.prototype.onPrintWorkerTimer=function (event){

        var done=false;
        var printWindow = this._printWindow;

   /*     if(!this.getPrintOptions().pdfUseScreenShot){
            this.addCurrentPage();
            this.printProcessComplete();
            return;
        }

        if(this.incapture)
        {
            return;
        }
*/

        if(printWindow.getCurrentPage()<=printWindow.getTotalPages())
        {

            /*if(this.capturedPages.indexOf(printWindow.getCurrentPage())==-1 && !this.getPrintOptions().printToPdf) {
                this.incapture=true;
                this.addCurrentPage();
                return;
            }
*/
            this.printCurrentPage(true);

            if(printWindow.getCurrentPage()==printWindow.getTotalPages()){
                done=true;
            }
            else {
                printWindow.setCurrentPage(printWindow.getCurrentPage() + 1);
            }
            if(this.getPrintOptions().asynch && this.getPrintOptions().previewWindow){
                //do nothing, we will come back to this method once the UI is updated.
                this.getPrintOptions().previewWindow.progress=printWindow.getCurrentPage()+"|"+printWindow.getTotalPages();
            }else{
                if(!done)
                    this.onPrintWorkerTimer(null);
            }
        }
        else{
            done=true;
        }
        if(done){
            this.printProcessComplete();
        }

    };

    PrintController.prototype.printCurrentPage=function (add){

        var printWindow = this._printWindow;
        printWindow.setCurrentPage(printWindow.getCurrentPage());
        if (printWindow.getTotalPages() > 1)
        {
            if (printWindow.getCurrentPage() == 1)
                printWindow.showFirstPage(true);
            else if (printWindow.getCurrentPage() == printWindow.getTotalPages())
                printWindow.showLastPage(true);
            else
                printWindow.showMiddlePage(true);
            if(add){
                if(printWindow.getCurrentPage() != 1) { // look like we are already in the first page on the preview
                    this.gotoCurrentPage();
                    printWindow.validateNow();
                }
                this.addCurrentPage();
            }
        }
        else
        {
            printWindow.showSinglePage(true);
            if(add){
                this.gotoCurrentPage();
                printWindow.validateNow();
                this.addCurrentPage();
            }
        }

    };

    PrintController.prototype.onPrintWorkerTimerDelayedAsych=function (event){

        var done=false;
        var printWindow = this._printWindow;
        if(this._pageShown){
            this.addCurrentPage();
            printWindow.setCurrentPage(printWindow.getCurrentPage()+1);
        }
        if((printWindow.getCurrentPage()>printWindow.getTotalPages())&&this._pageShown){
            done=true;
            this.onPrintComplete();
            this.worker.removeEventListener(Constants.EVENT_TIMER,this.onPrintWorkerTimerDelayedAsych);
            this.worker.stop();
            this.worker = null;
            return;
        }
        if(printWindow.getCurrentPage()<=printWindow.getTotalPages())
        {
            this.printCurrentPage(false);
            this._pageShown=true;
            this.gotoCurrentPage();
            printWindow.validateNow();
            //do nothing, we will come back to this method once the UI is updated.
            this.getPrintOptions().previewWindow.progress=printWindow.getCurrentPage()+"|"+printWindow.getTotalPages();
        }

    };

    PrintController.prototype.onPrintComplete=function (){

        // Send the job to the printer.
        if(!this.getPrintOptions().printToPdf)
        {
            var cssStr="";
            //this.printJob.send();
            //this.printJob=null;
            for( var i in document.styleSheets ){
                if( document.styleSheets[i].href) {
                    if(document.styleSheets[i].href.indexOf('flexicious.css')>0)
                    cssStr+='<link rel="stylesheet" href="'+document.styleSheets[i].href+'" type="text/css"/>                    '
                }
            }



            var finalHtmlStr=cssStr+"<style>* {float:none !important;}.css-element {overflow: visible !important;float:none;}</style>"+this.getPrintOptions().generatedHTML;
            var win= UIUtils.openBrowserPopup('about:blank');
            if(!win)
            {
                alert(Constants.DISABLE_POPUP_BLOCKERS);
                return;
            }
            win.document.title=this.getPrintOptions().exportFileName;
            var ht =(this.getPrintOptions().pageSize.getHeight()*this._printWindow.getTotalPages());
            setTimeout(function(){
                UIUtils.setHtml(win.document.body,finalHtmlStr);
                win.document.body.style.overflow="auto";
                win.document.body.style.height = ht +"px";
                setTimeout(function(){
                    win.print();
                    win.close();

                },1000);
            },1000);

        }
        else
        {
           // this.jsPdfDoc.save(this.getPrintOptions().exportFileName);
           // this.jsPdfDoc=null;
            if(this.getPrintOptions().printable.pdfPrinter)
                this.getPrintOptions().printable.pdfPrinter.endDocument(this.getPrintOptions());
        }
        if(this.getPrintOptions().printOptionsView)
            UIUtils.removePopUp(this.getPrintOptions().printOptionsView);

        if(this._printWindow.parent)
            UIUtils.removeChild(this._printWindow.parent,this._printWindow);

        var pe = new flexiciousNmsp.PrintPreviewEvent(flexiciousNmsp.PrintPreviewEvent.AFTER_PRINT);
        pe.printOptions=this.getPrintOptions();
        this._printWindow.getPrintable().dispatchEvent(pe);
        if(this.getPrintOptions().previewWindow){
            UIUtils.removePopUp(this.getPrintOptions().previewWindow);
            this.getPrintOptions().previewWindow=null;
        }
        //if(this.getPrintOptions().printToPdf && this._printWindow.getPrintable().pdfPrinter){
        //    this._printWindow.getPrintable().pdfPrinter.endDocument(this.getPrintOptions());
        //}

    };
    PrintController.prototype.addCurrentPage=function (){


        //if(this.getPrintOptions().printToPdf){
        //    var img=this.convertToImage(this._printWindow, this._printOptions);
        //    if(this.getPrintOptions().printable.pdfPrinter){
        //        this.getPrintOptions().previewWindow.content.addChild(img);
        //        this.getPrintOptions().previewWindow.validateNow();
        //        this.getPrintOptions().printable.pdfPrinter.addPage(img,this.getPrintOptions());
        //        this.getPrintOptions().previewWindow.content.removeChild(img);
        //    }else{
        //        this.getPrintOptions().printedPages.push(img);
        //    }
        //}
        //else
        {
            var that=this;
            this.incapture=true;

            if(this.getPrintOptions().printToPdf)
            {
                if(!this.getPrintOptions().pdfUseScreenShot && this.getPrintOptions().printable.pdfPrinter)
                {
                    this.getPrintOptions().previewWindow.validateNow();
                    this._printWindow.printComponent.validateNow();
                    this.getPrintOptions().printable.pdfPrinter.addPage(this._printWindow,this.getPrintOptions());
                }
                else
                {
                    this.getPrintOptions().previewWindow.content.domElement.style.overflow="visible";
                    this.getPrintOptions().previewWindow.content.domElement.style.height = this.getPrintOptions().previewWindow.content.domElement.scrollHeight +"px";
                    html2canvas(this.getPrintOptions().previewWindow.content.domElement,{async:false}).then(function(canvas) {
                        var img=canvas.toDataURL("image/png");
                      /*  that.jsPdfDoc.addImage(img,'png',20,20);
                        if(that._printWindow.getCurrentPage()!=that._printWindow.getTotalPages()){
                            that.jsPdfDoc.addPage();
                        }*/
                        that.getPrintOptions().printable.pdfPrinter.addPage(img,that.getPrintOptions());
                        //document.getElementById('themeContainer').appendChild(canvas);
                        that.incapture=false;
                        that.capturedPages.push(that._printWindow.getCurrentPage());
                        that.printOptions.previewWindow.content.domElement.style.overflow="auto";
                    });
                }
            }
            else{
                that.incapture=false;
                that.capturedPages.push(that._printWindow.getCurrentPage());
            }


            //this.printJob.addObject(this._printWindow,flexiciousNmsp.FlexPrintJobScaleType.FILL_PAGE);
            if(!this.getPrintOptions().generatedHTML)
            {
                this.getPrintOptions().generatedHTML="";
            }
            //this.getPrintOptions().previewWindow.content.domElement.style.pageBreakInside="avoid";
            //this.getPrintOptions().previewWindow.content.domElement.style.pageBreakAfter="always";
            //this.getPrintOptions().previewWindow.content.domElement.style.webkitRegionBreakInside="avoid";
            this.getPrintOptions().generatedHTML += this.getPrintOptions().previewWindow.content.domElement.innerHTML;
            this.getPrintOptions().generatedHTML += "<div style='page-break-after: always;'></div>"
        }

    };
    /**
     * Creates a preview of the grid and displays it.
     * @param printable			The IExtendedDataGrid to preview
     * @param [printOptions]	The Print Options (See the print options class)
     *
     */
    PrintController.prototype.previewWithOptions=function(printable, printOptions){

        this._extendedDataGrid=printable.implementsOrExtends("IExtendedDataGrid")?printable:null;
        this._printOptions=printOptions;
        var printWindow=this.createPrintWindow(printable, printOptions);
        if (!printOptions.printPreviewViewrenderer)
        {
            //user may override this if needed.
            printOptions.printPreviewViewrenderer=new flexiciousNmsp.ClassFactory(flexiciousNmsp.PrintPreview,undefined);
        }
        var printPreview=printOptions.printPreviewViewrenderer.newInstance();
        printOptions.previewWindow = printPreview;
        printPreview.setHeight(640);
        printPreview.setWidth(this.getPrintOptions().pageSize.getWidth()+50);

        printPreview.addEventListener(this,flexiciousNmsp.PrintPreviewEvent.PAGE_OPTIONS_CHANGED, this.onPreviewPageOptionsChanged,false,0,true);
        printPreview.addEventListener(this,flexiciousNmsp.PrintPreviewEvent.PAGE_INDEX_CHANGED, this.onPreviewPageIndexChanged,false,0,true);
        printPreview.addEventListener(this,flexiciousNmsp.PrintPreviewEvent.COLUMNS_CHANGED, this.onPreviewColumnsChanged,false,0,true);
        printPreview.addEventListener(this,flexiciousNmsp.PrintPreviewEvent.DATAGRID_RECREATE_REQUIRED, this.onDataGridRecreateRequest,false,0,true);
        printPreview.addEventListener(this,flexiciousNmsp.PrintPreviewEvent.COLUMNS_RESIZED, this.onDataGridColumnsResized,false,0,true);
        printPreview.addEventListener(this,flexiciousNmsp.PrintPreviewEvent.PRINT_REQUESTED, this.onPreviewPrintRequested,false,0,true);
        printPreview.addEventListener(this,"popupRemoved", this.onPreviewClose,false,0,true);
        var popupParent=printOptions.popupParent;
        if(!popupParent)popupParent=UIUtils.getTopLevelApplication();
        UIUtils.addPopUp(printPreview, popupParent,printOptions.modalWindows,null,printOptions.printPreviewWindowTitle,{context : this, method : "onPrintOptionsCancel"});

        printPreview.addEventListener(this, flexiciousNmsp.PrintPreviewEvent.PRINT_CANCELED, this.onPrintOptionsCancel);

        this.generatePreview(printPreview, printWindow);
        printPreview.setPrintOptions(printOptions);
        //flexiciousNmsp.PopUpManager.centerPopUp(printPreview);

    };
    PrintController.prototype.onPreviewClose=function (event){

    };
    /**
     * Fired when the preview button on the print preview screen is clicked.
     * @param event
     *
     */
    PrintController.prototype.onPreviewPrintRequested=function (event){

        this._printOptions.preview=false;
        this.printWithOptions(this._extendedDataGrid?this._extendedDataGrid:this._printOptions.printable, this._printOptions);

    };
    /**
     * fires when the user changes any preview settings that warrant
     * regeneration of the preview image
     * @param event
     *
     */
    PrintController.prototype.onPreviewPageOptionsChanged=function (event){

        var printPreview=(event.target );
        printPreview.setCurrentPage(1);
        this.generatePreview(printPreview, this.createPrintWindow(this._extendedDataGrid?this._extendedDataGrid:this._printOptions.printable, this.getPrintOptions()));
        //flexiciousNmsp.PopUpManager.centerPopUp(printPreview);


    };
    /**
     * Just prints the next page, without recreating the print window.
     * @param event
     *
     */
    PrintController.prototype.onPreviewPageIndexChanged=function (event){

        var printPreview=(event.target );
        this.generatePreview(printPreview, this._printWindow);

    };
    /**
     * Just prints the next page, without recreating the print window.
     * @param event
     *
     */
    PrintController.prototype.onPreviewColumnsChanged=function (event){

        flexiciousNmsp.UIUtils.removeAll(this._printOptions.columnWidths);
        this.onDataGridRecreateRequest(event);

    };
    PrintController.prototype.onDataGridColumnsResized=function (event){

        this._extendedDataGrid.lastPrintOptionsString = this._printOptions.toPersistenceString(this._extendedDataGrid);

    };
    PrintController.prototype.onDataGridRecreateRequest=function (event){

        var printPreview=(event.target);
        printPreview.setCurrentPage(1);
        this.generatePreview(printPreview, this.createPrintWindow(this._extendedDataGrid?this._extendedDataGrid:this._printOptions.printable, this.getPrintOptions()));

    };

    PrintController.prototype.generatePreview=function(printPreview, printWindow){

        var pageSize=this._printOptions.pageSize;
        if(!printPreview.initialized)
        printPreview.initialize();
        printPreview.domElement.parentNode.style.width=(this.getPrintOptions().pageSize.getWidth()+50)+"px";

        printPreview.content.removeAllChildren();
        printPreview.content.addChild(printWindow);

        printWindow.setWidth(this.getPrintOptions().pageSize.getWidth());
        printWindow.setHeight(this.getPrintOptions().pageSize.getHeight());

        printPreview.content.setWidth(this.getPrintOptions().pageSize.getWidth()+25);
        printPreview.content.setHeight(screen.height - 400);


        if(!printWindow.initialized){
            printWindow.initialize();
        }
        printWindow.getPrintDataGrid().setWidth(this.getPrintOptions().pageSize.getWidth());

        if(!printWindow.getPrintComponent().implementsOrExtends("PrintFlexDataGrid"))
        {
            printWindow.showSinglePage(true);
        }
        this.setupPrintWindow(printWindow.printable, this.getPrintOptions(), printWindow);

        if (printWindow.getTotalPages() > 1)
        {
            if (printPreview.getCurrentPage() == 1)
                printWindow.showFirstPage(false);
            else if (printPreview.getCurrentPage() == printPreview.getTotalPages())
                printWindow.showLastPage(false);
            else
                printWindow.showMiddlePage(false);
        }
        else
        {
            printWindow.showSinglePage(false);
        }
        printPreview.setTotalPages(printWindow.getTotalPages());

        printWindow.setCurrentPage(printPreview.getCurrentPage());
        //var items = printWindow.getPrintDataGrid().printListItems.length==0;
        //var needToReEnter=printWindow.getPrintDataGrid()&&items;
        //if(needToReEnter)
        //{
        //    printWindow.getPrintDataGrid().gotoRow(0);
        //    printWindow.validateNow();
        //    if(printWindow.getPrintDataGrid().getValidNextPage())
        //        printWindow.getPrintDataGrid().nextPage();
        //    printWindow.validateNow();
        //    printWindow.getPrintDataGrid().gotoRow(printWindow.getPageStart());
        //
        //}
        //else
        {
            this.gotoCurrentPage();
        }
        printWindow.callValidate();


        //UIUtils.removeChild(_extendedDataGrid.parentDocument,printWindow);
        /*if(this.getPrintOptions().printable){
            var pe = new flexiciousNmsp.PrintPreviewEvent(flexiciousNmsp.PrintPreviewEvent.AFTER_PRINT);
            pe.printOptions=this.getPrintOptions();
            pe.printOptions.printable.dispatchEvent(pe);
        }*/

    };
    PrintController.prototype.gotoCurrentPage=function (){



    };
    PrintController.prototype.gotoCurrentPage=function (){

        if(this.getPrintOptions().preReportPages==0 && this.getPrintOptions().postReportPages==0 ){
            this._printWindow.getPrintDataGrid().gotoRow(this._printWindow.getPageStart());
        }else{
            this._printWindow.showPrintComponentBasedOnPageIndex(true);
        }

    };

    PrintController.prototype.getPrintOptions = function() {
        return this._printOptions;
    };
}(window));
