
/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

(function(window)
{
    "use strict";
    var PrintFlexDataGridBodyContainer, uiUtil = flexiciousNmsp.UIUtils, flxConstants = flexiciousNmsp.Constants;
    /**
     * Specialized extension of the FlexDataGridBodyContainer that is used for printing. Hides the
     * vertical scrollbar, and shows only what can be displayed on the page given the
     * height constraints of the page being printed to.
     * @constructor
     * @namespace flexiciousNmsp.print
     */
    PrintFlexDataGridBodyContainer=function(grid){

        this._currentPositions = [];



        flexiciousNmsp.FlexDataGridVirtualBodyContainer.apply(this,[grid]);
        this.horizontalScrollPolicy="off";
        this.verticalScrollPolicy="off";
        this._verticalMask=0;
        this._horizontalMask=0;
        this.domElement.className=" flexDataGridVirtualBodyContainer";

        UIUtils.addDomEventListener(this, this.domElement, "mousewheel", function (evt){
            //UIUtils.findAncestorByClassName(evt.target || evt.srcElement, 'printPreview').component.onPageFooterCbClick(evt.target || evt.srcElement);
            evt.preventDefault();
            evt.stopPropagation();
        });

    };
    flexiciousNmsp.PrintFlexDataGridBodyContainer = PrintFlexDataGridBodyContainer; //add to name space
        PrintFlexDataGridBodyContainer.prototype = new flexiciousNmsp.FlexDataGridVirtualBodyContainer(); //setup hierarchy*/
    PrintFlexDataGridBodyContainer.prototype.typeName = PrintFlexDataGridBodyContainer.typeName = 'PrintFlexDataGridBodyContainer';//for quick inspection
    PrintFlexDataGridBodyContainer.prototype.getClassNames=function(){
        return ["PrintFlexDataGridBodyContainer","FlexDataGridVirtualBodyContainer",
            "FlexDataGridBodyContainer","FlexDataGridContainerBase","UIComponent"];
    };

    PrintFlexDataGridBodyContainer.prototype.getScrollableRect=function (){

        var bounds = flexiciousNmsp.getScrollableRect();
        var grid = this.grid.implementsOrExtends("PrintFlexDataGrid")?this.grid:null;
        if(grid.totalPagesHeight>0){
            bounds.height=grid.totalPagesHeight;
        }
        return bounds;

    };
    PrintFlexDataGridBodyContainer.prototype.recycle=function(level, scrollDown, scrollDelta,scrolling){

    }
    /**
     * @private
     * @return
     */
    PrintFlexDataGridBodyContainer.prototype.showPositions=function(positions){

        if(!positions){
            this._currentPositions=null;
        }else{
            this._currentPositions=positions;
        }
        this._drawDirty=true;
        if(this._currentPositions)
        {
            var start = this._currentPositions[0];
            var end = this._currentPositions[1];
            this._visbleRange=[start,end];
            this.doInvalidate();
            this.validateNow();
        }
        else
            this.removeAllComponents(true);

    };

    PrintFlexDataGridBodyContainer.prototype.drawRows=function (forceFiller){
         if(typeof forceFiller=="undefined")forceFiller=false;

        if(this._currentPositions==null || this._currentPositions.length!=3){
            flexiciousNmsp.FlexDataGridVirtualBodyContainer.prototype.drawRows.apply(this,[forceFiller]);
            this.removeAllComponents(true);
            return true;
        }
        this.removeAllComponents(true);
        var start = this._currentPositions[0];
        var end = this._currentPositions[1];

        //this.setVerticalScrollPosition(start.getVerticalPosition());
        var lastPage=((this.getCalculatedTotalHeight()-this.getVerticalScrollPosition())<this.getHeight()) &&(this.getCalculatedTotalHeight()>this.getHeight());
        this.grid.currentPoint.contentY = 0;//start.getVerticalPosition();
        this.grid.currentPoint.contentX=this.grid.currentPoint.leftLockedContentX=this.grid.currentPoint.rightLockedContentX=0;

        if(start!=null && end !=null){
            //trace("drawing " + start.getRowIndex() + " to " + end.getRowIndex())
            var vPos=start.getVerticalPosition();
            for ( var rowIndex = start.getRowIndex(); rowIndex <= end.getRowIndex(); rowIndex++ ) {
                var seed = this.itemVerticalPositions[rowIndex];
                if(!seed.printReduced){
                    seed.printReduced=true;
                    seed.setVerticalPosition(seed.getVerticalPosition()-vPos);
                }
            }
            this.drawRowsUsingCache(start,end,true,true);
        }


        if(this.fillerRows.length==0){
            var filler=new flexiciousNmsp.RowPositionInfo({},end?end.getRowIndex()+1:1,0,this.getHeight(),
                this.grid.getColumnLevel(),flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILL);
            this.fillerRows.push(this.processItem(filler,false));
        }

        if(this.fillerRows.length>0){
            var items=this.fillerRows[0].cells;
            for(var i=0;i<items.length;i++){
                var item=items[i];
                if(item.component){
                    item.component.setY(0);
                    item.component.setHeight(this.getHeight());
                    item.component.domElement.style.zIndex=-1;
                }
            }
        }


        this.grid._heightIncreased=false;
        //var that=this;
        //setTimeout(function(){
        //    that.setVerticalScrollPosition(start.getVerticalPosition())
        //},1);
        return true;

    };

    PrintFlexDataGridBodyContainer.prototype.getCalculatedTotalHeight = function() {
        return this._calculatedTotalHeight;
    };

    /**
     * Gets the vertical scroll bar props
     */
    PrintFlexDataGridBodyContainer.prototype.getVerticalScrollBar=function(){
        return new flexiciousNmsp.Rectangle(0,0,1,this.getHeight());
    };
    /**
     * Gets the horizontal scroll bar props
     */
    PrintFlexDataGridBodyContainer.prototype.getHorizontalScrollBar=function(){
        return new flexiciousNmsp.Rectangle(0,0,this.getWidth(),1);
    };
    PrintFlexDataGridBodyContainer.prototype.addPadding=function (nestLevel,row,paddingHeight,
                                                             level,forceRightLock,scrollPad,width){
        if(scrollPad)
        {

        }
        else{
            flexiciousNmsp.FlexDataGridVirtualBodyContainer.prototype.addPadding.apply(this,[nestLevel,row,paddingHeight,
                level,forceRightLock,scrollPad,width]);

        }
    };
    PrintFlexDataGridBodyContainer.prototype.addMatchStick=function(p){
        if(!p.vMatch){
            p.vMatch=new flexiciousNmsp.UIComponent("div");
            p.vMatch.domElement.className="matchStick";
            UIUtils.addChild(p,p.vMatch);
            this.matchSticks.push(p.vMatch);
        }
        p.vMatch.setHeight(this.grid.totalPagesHeight);
        if(p!=this){
            //left locked and right locked content needs a longer matchstick
            p.vMatch.setHeight(this.grid.totalPagesHeight+Constants.HORIZONTAL_SCROLLBAR_HEIGHT);
        }

        this.maxVerticalScrollPosition=this._calculatedTotalHeight-this.height;
    };

}(window));
