/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
const uiUtil = flexiciousNmsp.UIUtils;
const flxConstants = flexiciousNmsp.Constants;
import PrintExportOptions from '../utils/PrintExportOptions'
import PageSize from './PageSize'
/**
 * Class to print the datagrid, either one page at a time <br/>
 * or specific pages, or all pages.
 *
 *
 *
 * @constructor
 * @namespace com.flexicious.print
 */
export default class PrintOptions extends PrintExportOptions {
 constructor() {
     super();
     /**
      * Set this to true if you want the print controller <br/>
      * to perform a preview before printing. Default is true
      */
     this.preview=true;
     /**
      * This is the actual instance of the preview window being used to render
      * the preview.
      */
     this.previewWindow=null;
     /**
      * Added for integration with Alive PDF. <br/>
      * Defaults to false.  When set to true,
      * creates an image of each page and stores <br/>
      * it in the printedPages array instead of sending
      * them to the printer
      */
     this.printToPdf=false;
     /**
      * An image of each page and stores <br/>
      * it in the printedPages array instead of sending
      * them to the printer, when printToPdf is set to true
      */
     this.printedPages=[];
     /**
      * Size of the paper in the printer. Valid values are
      * PAGE_SIZE_A3
      * PAGE_SIZE_A4
      * PAGE_SIZE_A5
      * PAGE_SIZE_LETTER
      * PAGE_SIZE_LEGAL
      *
      * Default is PAGE_SIZE_LETTER
      *
      */
     this.pageSize=PageSize.PAGE_SIZE_LETTER.clone();
     /**
      * The renderer component for the overall print header. Prints only on the <br/>
      * first page of the printout. Has to extend from com.flexicious.print.PrintArea <br/>
      * or implement com.flexicious.print.IPrintArea. Default implementation is provided <br/>
      * that simply displays today's date. See com.flexicious.print.PrintHeader
      *
      * PLEASE NOTE: If you specify a custom report or page header or footer renderer, please specify a fixed height
      * to it. Currently, variable row height headers and footers are not supported in print preview.
      *
      */
     this.reportHeaderRenderer=new flexiciousNmsp.ClassFactory(flexiciousNmsp.ReportHeader);
     /**
      * The renderer component for the overall print footer. Prints only on the last <br/>
      * page of the printout. Has to extend from com.flexicious.print.PrintArea <br/>
      * or implement com.flexicious.print.IPrintArea. Default implementation is provided <br/>
      * that simply displays today's date. See com.flexicious.print.PrintFooter <br/>
      */
     this.reportFooterRenderer=new flexiciousNmsp.ClassFactory(flexiciousNmsp.ReportFooter);
     /**
      * The renderer component for the page header. Prints on each page of the print out
      * Has to extend from com.flexicious.print.PrintArea
      * or implement com.flexicious.print.IPrintArea. Default implementation is provided
      * that simply displays the page number. See com.flexicious.print.PageHeader
      */
     this.pageHeaderRenderer= new flexiciousNmsp.ClassFactory(flexiciousNmsp.PageHeader);
     /**
      * The renderer component for the page footer. Prints on each page of the print out.
      * Has to extend from com.flexicious.print.PrintArea
      * or implement com.flexicious.print.IPrintArea. Default implementation is provided
      * that simply displays the page number. See com.flexicious.print.PageFooter
      */
     this.pageFooterRenderer= new flexiciousNmsp.ClassFactory(flexiciousNmsp.PageHeader);
     /**
      * Please note - this property has been deprecated.
      * To change the default renderer for the print component, please
      * use the printComponentFactory on the IPrintable (ExtendedDataGrid,
      * ExtendedAdvancedDataGrid,PrintDataGrid or DashboardContainer)
      *  object instead
      *
      *
      * The component that does the actual print. Following classes do this:
      * For ExtendedDataGrid -> ExtendedPrintDataGrid
      * For ExtendedAdvancedDataGrid -> ExtendedAdvancedPrintDataGrid
      * For FlexDataGrid (Ultimate) -> PrintFlexDataGrid
      * For DashboardContainer -> PrintDashboardContainer
      */
     this.printComponentRenderer=null;
     /**
      * The view renderer for the print options. The default view
      * shows page options, as well as an option for the user to preview
      * Default implementation is com.flexicious.print.PrintOptionsView.
      * You can provide your own implementation, see example #4.
      */
     this.printOptionsViewrenderer=new flexiciousNmsp.ClassFactory(flexiciousNmsp.PrintOptionsView);
     /**
      * The view renderer for the preview.
      * Default implementation is com.flexicious.print.PrintPreview.
      * You can provide your own implementation, see example #4.
      */
     this.printPreviewViewrenderer= new flexiciousNmsp.ClassFactory(flexiciousNmsp.PrintPreview);
     /**
      *
      */
     this.availableColumns=[];
     /**
      *
      */
     this.columnsToPrint=[];
     /**
      *
      */
     this.columnWidths=[];
     /**
      * Whether or not to include the page header
      * Default is true
      */
     this.includePageHeader=true;
     /**
      * Whether or not to include the page footer
      * Default is true
      */
     this.includePageFooter=true;
     /**
      * Whether or not to include the report header
      * Default is true
      */
     this.includePrintHeader=true;
     /**
      * Whether or not to include the report footer
      * Default is true
      */
     this.includePrintFooter=true;
     /**
      * Any custom data that is passed into the print/export mechanism.<br/>
      * This will then be available on Print/Export Options when the Print or Export is run.<br/>
      * Please note, this is not persisted in preferences.
      */
     this.printExportParameters=null;
     /**
      * Flag, which defaults to false, launches the preview and options screen in modal mode.
      */
     this.modalWindows=true;
     /**
      * Sets the printAsBitmap property of the print job <br/>
      * Defaults to true, similar to the default for the FlexPrintJob class
      *
      */
     this.printAsBitmap=false;
     /**
      * Before attempting to print, we add the print window to a display object container.
      * This is required for the print datagrid to render. If set to null, we default to the
      * parent of the grid being printed.
      */
     this.printContainer=null;
     /**
      * The printable being printed. This is the grid that you pass in to the print controller
      */
     this.printable=null;
     /**
      * The print representation of the printable being printed.
      */
     this.printComponent=null;
     /**
      * A dictionary of style properties and their values
      * to be applied to the print canvas.
      * By default the following properties are added:
      * backgroundColor: #FFFFFF
      * paddingTop: 20
      * paddingBottom: 20
      * paddingLeft: 20
      * paddingRight: 20
      */
     this.windowStyleProperties={};
     /**
      * The properties to transfer from the grid being printed to the print version.
      * Not yet supported. For future release
      */
     this.propertiesToTransfer=[];
     /**
      * The styles to transfer from the grid being printed to the print version.
      * Not yet supported. For future release
      */
     this.stylesToTransfer=[];
     /**
      * If you set this to true, columns that are set to visible=false on the parent grid will
      * be printed. (Applicable to Classic only);
      * @deprecated - use hideHiddenColumns instead
      */
     this.includeInvisibleColumns=true;

     this.asynch = false;
     /**
      * The time interval, in milliseconds, used by the timer of the <code>execute()</code> method
      * to iteratively process pages.
      * You can set it to a higher value if you want to wait for longer before the next page is processed. This is used in cases
      * where you are loading external images in your item renderers. Please make sure you also set asynchDelayCapture to true
      * in this scenario.
      * The default value is 1, which allows enough time for the UI to update itself to show the end user a progress bar
      * and also prevents timeouts while printing or exporting to pdf a large number of pages.
      */
     this.asynchTimeInterval= 100;
     /**
      * Flag to wait for the grid to completely render prior to capturing a snapshot for printing.
      * The way the print works, is that it renders each page of the grid takes a snapshot and
      * adds it to the print object to be spooled to the printer. Same concept applies to pdf
      * where instead of adding to the print object the snapshot is converted to an image and
      * added to the pdf byte stream. This does not work well in scenarios where you have external
      * images that are loaded by url as opposed to using embedded assets. In scenarios like these
      * you may need to add a delay to the printing mechanism to ensure that enough time is available
      * for the external images to load. This flag, in combination with the asynchTimeInterval lets you
      * control the amount of time that the print mechanism will wait after the grid is rendered to capture the snapshot.
      */
     this.asynchDelayCapture=false;


     //defaults.
     this.windowStyleProperties["backgroundColor"]="#FFFFFF";
     this.windowStyleProperties["paddingTop"]="20";
     this.windowStyleProperties["paddingBottom"]="20";
     this.windowStyleProperties["paddingLeft"]="20";
     this.windowStyleProperties["paddingRight"]="20";


     /**
      * A class factory that is responsible for instantiating a ICustomPrintComponent
      * that will be printed before the report.
      */
     this.preReportPagesFactory=null;
     /**
      * A class factory that is responsible for instantiating a ICustomPrintComponent
      * that will be printed after the report.
      */
     this.postReportPagesFactory=null;
     /**
      * If you define a preReportPagesFactory, you may optionally provide a number
      * for this. If you do, we will not attempt to calculate how many pages by calling
      * calculateTotalPages on your ICustomPrintComponent.
      */
     this.preReportPages=0;
     /**
      * If you define a postReportPagesFactory, you may optionally provide a number
      * for this. If you do, we will not attempt to calculate how many pages by calling
      * calculateTotalPages on your ICustomPrintComponent
      */
     this.postReportPages=0;

     /**
      * File name for the download. Defaults to download.pdf where pdf is the extension.
      * @property exportFileName
      * @type String
      * @default download
      */
     this.exportFileName = "download.pdf";

     /**
      * Title for the print options dialog
      * @property printOptionsWindowTitle
      * @type String
      * @default Print Options
      */
     this.printOptionsWindowTitle = Constants.PRINT_OPTIONS_TITLE;

     /**
      * Title for the print preview dialog
      * @property printPreviewWindowTitle
      * @type String
      * @default Print Options
      */
     this.printPreviewWindowTitle = Constants.PRINT_PREVIEW_TITLE;


     /**
      * Flag added in 3.3 to enable passing an image to the pdf printer as opposed to the
      * actual display object. Defaults to true, because currently we just add a snapshot of the
      * current view to the PDF page. If this flag is set to false, we will pass the actual display object
      * (and not the image snapshot) to the pdf printer. The pdf printer then has to be smart enough to
      * parse through the Display Object, and call the appropriate APIs on the pdf library to convert
      * the Display Object to PDF version of it.
      * */
     this.pdfUseScreenShot=false;

     flexiciousNmsp.PrintExportOptions.apply(this);

 }

 getClassNames() {
     return ["PrintOptions","PrintExportOptions","TypedObject"];
 }

 /**
  * Hydrates previously persisted property values for the user interactive
  * persistable properties of the print options from the string value created
  * earlier.
  * @param grid
  * @param val	The string value
  */
 loadFromPersistedString(grid, val) {
  let col;
  let i;
  if(grid.useCompactPreferences){
      const parts=val.split("~/");
      if(parts.length==8){
          let cols= UIUtils.fromPersistenceString(parts[0]);
          for( i=0;i<cols.length;i++){
              col=cols[i];
              this.columnsToPrint.push({name:col});
          }
          cols= UIUtils.fromPersistenceString(parts[1]);
          for( i=0;i<cols.length;i++){
              col=cols[i];
              this.columnWidths.push(parseInt(col));
          }
          this.includePageFooter=parts[2]=='y';
          this.includePageHeader=parts[3]=='y';
          this.includePrintFooter=parts[4]=='y';
          this.includePrintFooter=parts[5]=='y';
          this.pageSize = flexiciousNmsp.PageSize.getByName(parts[6]);
          this.pageSize.isLandscape=parts[7]=='y';
      }
  }
 }

 /**
  * Converts the user interactive persistable properties of the print options to a
  * string that can be used for persistence. Non persitable properties
  * (like the renderers) are ignored.
  * @return
  */
 toPersistenceString(grid) {

     if(grid.useCompactPreferences){
      let buffer="";
      const colsToPrint = [];

      for (const item of this.columnsToPrint) {
       colsToPrint.push(item.name)
      }

      const delim="~/";
      buffer+= UIUtils.toPersistenceString(colsToPrint)+delim;
      buffer+= UIUtils.toPersistenceString(this.columnWidths)+delim;
      buffer+= (this.includePageFooter?'y':'n')+delim;
      buffer+= (this.includePageHeader?'y':'n')+delim;
      buffer+= (this.includePrintFooter?'y':'n')+delim;
      buffer+= (this.includePrintFooter?'y':'n')+delim;
      buffer+= this.pageSize.name +delim;
      buffer+= (this.pageSize.isLandscape?'y':'n');
      return buffer;
     }
 }

 /**
  * Convenience method to create an exporter of a specified type
  */
 static create(toPdf) {
     if(typeof toPdf=="undefined")toPdf=false;

     const printOptions = new PrintOptions();
     printOptions.printToPdf=toPdf;
     return printOptions;

 }

 /**
  * For backwards compatibility, this is same as printComponentRenderer
  */
 getPrintDataGridRenderer() {
     return this.printComponentRenderer;
 }

 setPrintDataGridRenderer(val) {
     this.printComponentRenderer=val;
 }

 /**
  * The grid being printed. This is the grid that you pass in to the print controller
  */
 getExtendedDataGrid() {
     return this.printable;
 }

 setExtendedDataGrid(val) {
     this.printable = val;
 }

 /**
  * The print representation of the grid being printed. If the extendedDataGrid is a ExtendedDataGrid, this is a ExtendedPrintDataGrid.
  * IF the extendedDataGrid is an ExtendedAdvancedDataGrid, this is an printAdvancedDataGrid. If extendedDataGrid is FlexDataGrid(Ultimate)
  * then this is a PrintFlexDataGrid.
  */
 getPrintDataGrid() {
     return this.printComponent;
 }

 setPrintDataGrid(val) {
     this.printComponent = val;
 }
}

flexiciousNmsp.PrintOptions = PrintOptions; //add to name space 
PrintOptions.prototype.typeName = PrintOptions.typeName = 'PrintOptions';//for quick inspection
