/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

(function (window) {
    "use strict";
    var JsPdfPrinter, uiUtil = flexiciousNmsp.UIUtils, flxConstants = flexiciousNmsp.Constants;
    /**
     * A class that takes the PDF Pages as they are generated. This functionality
     * was introduced in 2.9 to provide a responsive UI during the generation of the PDF.
     * Previously, the print controller created image snapshots in memory, and handed them all to the pdf engine, which processed
     * them individually by adding them to the display tree in one go.
     * Instead, now, the step of the snapshot itself adds the image to the pdf doc and this whole thing
     * runs inside a timer so the UI has an opportunity to update itself.
     *
     * First, you specify a class that implements IPDFPrinter as the pdfPrinter property.
     * The jsPdfPrinter is a sample implementation of IPDFPrinter. This class should expose the
     * methods defined in the IPDFPrinter which are used as such:
     * The beginDocument method is called when the Print controller is ready to send out the pages.
     * For each page generated by the Print Controller, the addPage method is called.
     * Once all pages are added to the doc, the endDocument method is called.
     *
     * Update - 3.3 - This class was extensivly enhanced to support the notion of "pure native pdf".
     * @constructor
     * @namespace
     * 
     */
    JsPdfPrinter = function () {

        this.pdfObject = null;

        this.pdfFontSize = 12;

        this.multiplier = 22.5/72;

        /**
         * @type {boolean}
         *
         */
        this.pdfWrapText = true;

        /**
         *
         * @type {boolean}
         * in wrap text mode, text are wrap based on words
         */
        this.wordWrap = true;

        /**
         * @type {string}
         * pdf font
         */
        this.pdfFontName = "Arial";

        /**
         * used to while genearate cells, need to told cells in which row kind of.
         * @type {number}
         */
        this.linePointer = 0;

        /**
         * @type {{left: number, top: number, bottom: number, right: number}}
         * margin the for the pdf pages
         */
        this.margin = {left : 10, top : 10, bottom : 10, right : 10};

        /**
         * @type {boolean}
         * used to figured out is writting the fist page of the process.
         */
        this.isFirstPage = true;
    };
    flexiciousNmsp.JsPdfPrinter = JsPdfPrinter; //add to name space
    JsPdfPrinter.prototype = new flexiciousNmsp.TypedObject(); //setup hierarchy
    JsPdfPrinter.prototype.typeName = JsPdfPrinter.typeName = 'jsPdfPrinter';//for quick inspection
    JsPdfPrinter.prototype.getClassNames = function () {
        return ["jsPdfPrinter", "TypedObject"];
    };

    JsPdfPrinter.FONT_STYLE_NORMAL = "normal";
    JsPdfPrinter.FONT_STYLE_BOLD = "bold";
    JsPdfPrinter.FONT_STYLE_ITALIC = "italic";
    JsPdfPrinter.CELL_ALIGN_CENTER = "center";
    JsPdfPrinter.CELL_ALIGN_LEFT = "left";
    JsPdfPrinter.CELL_ALIGN_RIGHT = "right";
    JsPdfPrinter.CELL_ALIGN_JUSTIFIED = "justified";

    // margin and other house keepings before burst in the LIB

    JsPdfPrinter.prototype.text = function(text, x, y, flags, angle, align){
        if(x < this.margin.left)
            x = this.margin.left;
        this.pdfObject.text(text, x, y, flags, angle, align);
    };

    JsPdfPrinter.prototype.lastCellPos = { x: undefined, y: undefined, w: undefined, h: undefined, ln: undefined };

    JsPdfPrinter.prototype.getLastCellPosition = function(){
        return this.lastCellPos;
    };

    JsPdfPrinter.prototype.setLastCellPosition = function(x, y, w, h, ln){
        this.lastCellPos = { 'x': x, 'y': y, 'w': w, 'h': h, 'ln': ln };
    };

    JsPdfPrinter.prototype.x = function(uiComponent){
        if(!uiComponent)
            return 0;
        return uiComponent.hasOwnProperty("x") ? uiComponent.getX() : uiComponent.domElement.offsetLeft;
    };

    JsPdfPrinter.prototype.y = function(uiComponent){
        if(!uiComponent)
            return 0;
        return uiComponent.hasOwnProperty("y") ? uiComponent.getY() : uiComponent.domElement.offsetTop;
    };

    JsPdfPrinter.prototype.width = function(uiComponent){
        if(!uiComponent)
            return 0;
        return uiComponent.getWidth() ? uiComponent.getWidth() : uiComponent.domElement.offsetWidth;
    };

    JsPdfPrinter.prototype.height = function(uiComponent){
        if(!uiComponent)
            return 0;
        return uiComponent.getHeight() ? uiComponent.getHeight() : uiComponent.domElement.offsetHeight;
    };

    JsPdfPrinter.prototype.cell = function(x, y, w, h, txt, ln, align){
        var padding = 0;
        var curCell = this.getLastCellPosition();
        if (curCell.ln !== undefined) {
            if (curCell.ln === ln) {
                x = curCell.x + curCell.w;
                y = curCell.y;
            } else {
                y = (this.getLastCellPosition().y + this.getLastCellPosition().h);
            }
        }

        this.rect(x, y, w, h, "F");
        if (txt[0] !== undefined) {
            if (align === 'right') {
                if (!(txt instanceof Array)) {
                    txt = [txt];
                }
                for (var i = 0; i < txt.length; i++) {
                    var currentLine = txt[i];
                    var textSize = this.pdfObject.getStringUnitWidth(currentLine) * this.pdfObject.internal.getFontSize();
                    this.text(currentLine, this.margin.left + x + w - textSize - padding, this.margin.top + y + this.pdfObject.internal.getLineHeight()*(i+1));
                }
            } else {
                this.text(txt, this.margin.left + x + padding, this.margin.top + y + (this.pdfObject.internal.getLineHeight() * this.multiplier));
            }
        }
        this.setLastCellPosition(x, y ? y : 0, w, h, ln);
    };

    JsPdfPrinter.prototype.line = function(x1, y1, x2, y2){
            x1 += this.margin.left;
            x2 += this.margin.left;
            y1 += this.margin.top;
            y2 += this.margin.top;
        this.lines([[x2 - x1, y2 - y1]], x1, y1);
    };

    JsPdfPrinter.prototype.lines = function(lines, x, y, scale, style, closed){
        return this.pdfObject.lines(lines, x, y, scale, style, closed);
    };

    JsPdfPrinter.prototype.rect = function(x, y, w, h, style){
        x += this.margin.left;
        y += this.margin.top;
        this.pdfObject.rect(x, y, w, h, style);
    };

    JsPdfPrinter.prototype.addImage = function(imageData, format, x, y, w, h, alias, compression, rotation){
        if(x < this.margin.left)
            x = this.margin.left;
        this.pdfObject.addImage(imageData, format, x, y, w, h, alias,compression,rotation);
    };

    /**
     * @param cell
     * @return {null}
     *
     * for now its used to only for the expand collapsecell.
     * look like the  img sit in the cell dom. not in the render dom.
     * so it is only search in the cell dom. other wise we need to drill down search for the img.
     * it is return first founded img.
     */
    JsPdfPrinter.prototype.getImageDOM = function(cell){
        if(!cell || !cell.domElement || !cell.domElement.children.length) // error and violation cases
            return null;
        var img = cell.domElement.getElementsByTagName("img");
        if(!img || !img.length)
            return null;
        return img[0];
    };

    /**
     * @param cell
     * @return {*}
     */
    JsPdfPrinter.prototype.getWrapedCellText = function(cell){
        if(!cell)
            return "";
        var multiplier = this.multiplier,
            numberOfPossibleLines = Math.floor(cell.getHeight() / this.pdfFontSize),
            numberOfWrittenLines = 0,
            dump = "",
            formatted = "",
            text = cell.getText(),
            pointer = 0;
        if(text.indexOf("\n") != -1)// assumes already formatted..
            return;

        for (pointer = 0; pointer < text.length; pointer++) {
            dump += text.charAt(pointer);
            if (this.getStringWidth(dump) > ((this.width(cell)/* - paddingLeft - paddingLeft*/) * multiplier)) {
                if(numberOfPossibleLines-1 == numberOfWrittenLines)
                    break;
                if (this.wordWrap) {
                    var spilted = dump.split(" ");
                    var last = spilted.pop();
                    if (!spilted.length) {
                        formatted += dump + "\n";
                        dump = "";
                    } else {
                        formatted += spilted.join(" ") + "\n";
                        dump = last;
                    }
                } else {
                    formatted += dump + "\n";
                    dump = "";
                }
                numberOfWrittenLines++;
            }
        }
        formatted += dump;
        spilted = formatted.split("\n");
        var poped = spilted.pop();
        if (!poped) {
            formatted = spilted.join("\n");
        }
        return formatted
    };

    JsPdfPrinter.prototype.addPage = function (flexPage, printOptions) {
        if(this.isFirstPage) {
            this.pdfObject.deletePage(1); // look like the  pdf library automatically add a page on startup
            this.isFirstPage = false;
        }

        var multiplier = this.multiplier;
        var isLandscapse = printOptions.pageSize.isLandscape;
        var pdfObject = this.pdfObject;
        var page=pdfObject.addPage(printOptions.pageSize.name.toLowerCase(), "p");
        this.setLastCellPosition(0, 0, undefined, undefined);

        if(printOptions.pdfUseScreenShot) {
            pdfObject.addImage(flexPage,"png",0,0); //account for pdf page padding of 20 px

        } else {
        //===========================Changes in 3.3 to support pure pdf printing===============================================//
            pdfObject.setFont(this.pdfFontName, flexiciousNmsp.JsPdfPrinter.FONT_STYLE_NORMAL);
            var pw = flexPage;
            this.isLastPage = pw.getCurrentPage() == pw.getTotalPages();
            var n = pw.numChildren();
            var lineHeight = this.pdfFontSize * multiplier;
            var offsetY = 0;
            var offsetX = 0;
            for (var i = 0; i < n; i++) {

                var child = pw.getChildAt(i);
                if ((child).visible && (child).includeInLayout) {
                    //keep in mind that the code below spits out the pdf equivalent of the built in report & page header and footer renderers
                    //if you have customized them, you'll need to make the modifications to the following code - to basically render that in 
                    //pdf format.

                    //we are no longer taking screenshots, instead replicating the renderer in PDF here. If you have customized renderers you may need to get familiar
                    //with the alive pdf API.
                    var txt, align;
                    if (child.implementsOrExtends('ReportHeader')) {
                        pdfObject.setDrawColor(255, 255, 255);
                        pdfObject.setFillColor(255, 255, 255);
                        txt = "Report Generated On " + UIUtils.formatDate(new Date());
                        align = child.getStyle("textAlign") || "right";
                        this.cell(this.x(child) * multiplier, (this.y(child)) * multiplier,
                            this.width(child) * multiplier, this.height(child) * multiplier, txt, -2, "left");
                            //(align == "right" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_RIGHT : align == "center" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_CENTER : flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT));
                    }
                    else if (child.implementsOrExtends('PageHeader')) {
                        pdfObject.setDrawColor(255, 255, 255);
                        pdfObject.setFillColor(255, 255, 255);
                       // txt = "Page " + (child).currentPage + " of " + (child).totalPages;
                        txt = "";
                        align = child.getStyle("textAlign") || "right";
                        this.cell(this.x(child) * multiplier, (this.y(child)) * multiplier,
                            this.width(child) * multiplier, this.height(child) * multiplier, txt, -1,
                            (align == "right" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_RIGHT : align == "center" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_CENTER : flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT));
                    }
                    else if (child.implementsOrExtends('IExtendedDataGrid')) {
                        //Now the grid.
                        var grid = this.grid = child;
                        var cell;
                        var headerX = this.x(grid) + this.x(grid.getHeaderContainer());
                        var headerY = this.y(grid) + this.y(grid.getHeaderContainer());

                        var footerX = this.x(grid) + this.x(grid.getFooterContainer());
                        var footerY = this.y(grid) + this.y(grid.getFooterContainer());

                        var bodyX = this.x(grid) + this.x(grid.getBodyContainer());
                        var bodyY = this.y(grid) + this.y(grid.getBodyContainer()) - grid.getVerticalScrollPosition();
                        var j = 0;

                        this.linePointer = 0;

                       /// var borderColorRGB = flexiciousNmsp.JsPdfPrinter.extractRGB(grid.getStyle("borderColor"));
                        //pdfObject.setDrawColor(borderColorRGB[0], borderColorRGB[1], borderColorRGB[2]);
                       // pdfObject.rect(child.getX() * multiplier, child.getY() * multiplier, child.getWidth() * multiplier, child.getHeight() * multiplier);
                       // pdfObject.setDrawColor(255,225,255);
                       // this.rect(child.getX() * multiplier, child.getY() * multiplier, child.getWidth() * multiplier, child.getHeight() * multiplier, "F");


                        //left locked headers (only used for hierarchical grids)
                        for (j = 0; j < grid.getLeftLockedHeader().numChildren(); j++) {
                            var leftLockedHeaderCell = grid.getLeftLockedHeader().getChildAt(j);
                            if (!leftLockedHeaderCell)continue;
                            this.drawCell(leftLockedHeaderCell, pdfObject,this.x(grid) + this.x(grid.getLeftLockedHeader()), headerY, true, leftLockedHeaderCell.rowInfo && leftLockedHeaderCell.rowInfo.cells[0] &&  leftLockedHeaderCell.rowInfo.cells[0].component == leftLockedHeaderCell);
                        }


                        //write out the header cells
                        for (j = 0; j < grid.getHeaderContainer().numChildren(); j++) {
                            cell = grid.getHeaderContainer().getChildAt(j);// as FlexDataGridCell;
                            if (cell.implementsOrExtends('FlexDataGridPaddingCell'))continue;
                            if (!cell)continue;
                            this.drawCell(cell, pdfObject, headerX, headerY, true, cell.rowInfo && cell.rowInfo.cells[0].component == cell);
                        }

                        this.linePointer = 1;
                        var lastY = null;

                        //write out the body cells.
                        var rows = grid.getBodyContainer().rows.slice();
                        for (j = 0; j < rows.length; j++) {
                            var cells  = rows[j].cells;
                            for(var k = 0; k < cells.length; k++){
                                 cell = cells[k].component;
                                if (!cell)continue;
                                if(cell.implementsOrExtends("FlexDataGridExpandCollapseCell") || cell.implementsOrExtends("FlexDataGridPaddingCell"))
                                   continue;
                                if ((bodyY + this.y(cell)) > (this.y(grid) + this.height(grid)))continue;
                                if ((bodyY + this.y(cell)) < (this.y(grid)))continue;
                                if(cell.rowInfo && cell.rowInfo.rowPositionInfo.getRowType() == 6)
                                    continue;
                                this.drawCell(cell, pdfObject, bodyX, bodyY, false, cell.rowInfo.cells[0].component == cell);
                            }
                            this.linePointer += 1;
                        }

                        this.linePointer = 1;
                        lastY = null;

                        //write out footer cells
                        for (j = 0; j < grid.getFooterContainer().numChildren(); j++) {
                            cell = grid.getFooterContainer().getChildAt(j);// as FlexDataGridCell;
                            if (cell.implementsOrExtends('FlexDataGridPaddingCell'))continue;
                            if (!cell)continue;
                            if(lastY == null){
                                lastY = this.y(cell);
                            } else if(lastY != this.y(cell)) {
                                lastY = this.y(cell);
                                this.linePointer++;
                            }
                            this.drawCell(cell, pdfObject, footerX, footerY);
                        }

                        this.linePointer = 1;
                        lastY = null;

                        for (j = 0; j < grid.getLeftLockedContent().numChildren(); j++) {
                            var leftLockedCell = grid.getLeftLockedContent().getChildAt(j);
                            if (!leftLockedCell)continue;
                            if(!leftLockedCell.implementsOrExtends("IFlexDataGridCell"))continue;
                            if ((bodyY + this.y(leftLockedCell)) > (this.y(grid) + this.height(grid)))continue;
                            if ((bodyY + this.y(leftLockedCell)) < (this.y(grid)))continue;
                            if(lastY == null){
                                lastY = this.y(leftLockedCell);
                            } else if(lastY != this.y(leftLockedCell)) {
                                lastY = this.y(leftLockedCell);
                                this.linePointer++;
                            }
                            if(leftLockedCell.rowInfo && (leftLockedCell.rowInfo.rowPositionInfo.getRowType() == 6 || !leftLockedCell.rowInfo.visible))
                                continue;
                            var drawLeft = leftLockedCell.rowInfo && leftLockedCell.rowInfo.cells[0] && leftLockedCell.rowInfo.cells[0].component === leftLockedCell, dontRight = false;
                            if(leftLockedCell.implementsOrExtends("FlexDataGridPaddingCell") && leftLockedCell.rowInfo){
                                var nxtIdx = UIUtils.extractPropertyValues(leftLockedCell.rowInfo.cells, "component").indexOf(leftLockedCell);
                                nxtIdx = nxtIdx != -1 ? nxtIdx+1 : -1;
                                if(nxtIdx != -1) {
                                    var nextCell = leftLockedCell.rowInfo.cells[nxtIdx].component;
                                    dontRight = nextCell.implementsOrExtends("FlexDataGridExpandCollapseCell");
                                }
                            }
                            this.drawCell(leftLockedCell, pdfObject, this.x(grid) + this.x(grid.getLeftLockedContent()), bodyY, false, drawLeft, dontRight);
                        }

                        this.linePointer = 1;
                        lastY = null;


                        //write out left locked footer cells.
                        for (j = 0; j < grid.getLeftLockedFooter().numChildren(); j++) {
                            var leftLockedFooterCell = grid.getLeftLockedFooter().getChildAt(j);
                            if (!leftLockedFooterCell)continue;
                            if(lastY == null){
                                lastY = this.y(leftLockedFooterCell);
                            } else if(lastY != this.y(leftLockedFooterCell)) {
                                lastY = this.y(leftLockedFooterCell);
                                this.linePointer++;
                            }
                            if(leftLockedFooterCell.rowInfo && leftLockedFooterCell.rowInfo.rowPositionInfo.getRowType() == 6)
                                continue;
                            this.drawCell(leftLockedFooterCell, pdfObject, this.x(grid) + this.x(grid.getLeftLockedFooter()), footerY, false , leftLockedFooterCell.rowInfo && leftLockedFooterCell.rowInfo.cells[0] &&  leftLockedFooterCell.rowInfo.cells[0].component == leftLockedFooterCell);
                        }
                    }
                    else if (child.implementsOrExtends('PageFooter')) {
                        this.text("Page " + (child).currentPage + " of " + (child).totalPages, (this.width(child) * multiplier) - 10, (this.y(child)) * multiplier + (lineHeight * 2));
                    }
                    else if (child.implementsOrExtends('ReportFooter')) {
                        this.text("Report Generated On " + UIUtils.formatDate(new Date()), this.x(child) * multiplier, (this.y(child)) * multiplier + (lineHeight * 2));
                    }
                }
            }

        }
    };

    JsPdfPrinter.prototype.checkCellHorizontalBound = function(cell, containerX){
        if(!cell)
            return;
    };

    JsPdfPrinter.prototype.checkCellWidth = function(cell, containerX){
        if(!cell)
            return;
        var len = containerX + this.x(cell) + this.width(cell), grid = this.grid;
        var bound = this.pdfObject.internal.pageSize.width - (this.margin.left + this.margin.right);
        if(bound > len * this.multiplier)
            return;
        var excess = (this.x(cell) + this.width(cell)) - grid.getWidth();
        cell.setWidth(cell.getWidth()- excess);
    };

    JsPdfPrinter.prototype.drawCell = function (cell, pdfObject, containerX, containerY, drawTop, drawLeft, dontDrawRight) {
        var multiplier = this.multiplier;

        if(!cell || !cell.implementsOrExtends || !cell.implementsOrExtends("IFlexDataGridCell"))
            return;

        //draw the background
        var color = cell.getBackgroundColors();
        var textColor = cell.getTextColors();
        var bgColor = color instanceof Array ? color[0] : color;
        var rgbColor = flexiciousNmsp.JsPdfPrinter.extractRGB(bgColor);
       // pdfObject.setDrawColor(rgbColor[0], rgbColor[1], rgbColor[2]);
        pdfObject.setFillColor(rgbColor[0], rgbColor[1], rgbColor[2]);
        rgbColor = flexiciousNmsp.JsPdfPrinter.extractRGB(textColor);
        pdfObject.setDrawColor(rgbColor[0], rgbColor[1], rgbColor[2]);

        this.pdfFontSize = parseInt(cell.getStyleValue("fontSize") || this.pdfFontSize+"");
        var lineHeight = this.pdfFontSize * multiplier;

        pdfObject.setFont(this.pdfFontName, cell.getStyleValue("fontStyle") == "bold" ? flexiciousNmsp.JsPdfPrinter.FONT_STYLE_BOLD : flexiciousNmsp.JsPdfPrinter.FONT_STYLE_NORMAL);
        pdfObject.setFontSize(this.pdfFontSize);

        var paddingLeft = UIUtils.nanToZero(cell.getStyleValue(cell.getPrefix() + this.capitalizeFirstLetterIfPrefix(cell.getPrefix(), "paddingLeft")));
        var paddingTop = UIUtils.nanToZero(cell.getStyleValue(cell.getPrefix() + this.capitalizeFirstLetterIfPrefix(cell.getPrefix(), "paddingTop")));

        var align = cell.getStyleValue("textAlign");
        if(cell.implementsOrExtends("FlexDataGridExpandCollapseCell")){
            this.cell((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell)) * multiplier, // + (this.height(cell) / 2) - (this.pdfFontSize / 2)) * multiplier,
                this.width(cell) * multiplier, (this.height(cell)) * multiplier, "", this.linePointer,
                (align == "left" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT : align == "right" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_RIGHT : align == "center" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_CENTER : flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT/*JUSTIFIED*/));
            var img = this.getImageDOM(cell);
            var style = img ? window.getComputedStyle(img) : null;
            if(img && img.src && style.visibility.toLowerCase() == "visible" && style.display.toLowerCase() != "none"){
                img.crossOrigin = "anonymous";
                var x, y;
                x = this.margin.left + (containerX+this.x(cell))*multiplier;
                x+=(img.offsetLeft + 2)*multiplier;
                y = (containerY+this.y(cell)+ cell.getHeight() +3)*multiplier;
                y+=img.offsetTop*multiplier;
                this.addImage(img, img.src.split(".")[img.src.split(".").length-1],x,y,
                    img.offsetWidth * multiplier, img.offsetHeight * multiplier, cell.open ? "FlexDataGridExpandIcon" : "FlexDataGridCollapseIcon");
            }

        } else if (this.getStringWidth(cell.getText()+"") < ((this.width(cell) - paddingLeft - paddingLeft) * multiplier)) {
            this.cell((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell)) * multiplier, // + (this.height(cell) / 2) - (this.pdfFontSize / 2)) * multiplier,
                            this.width(cell) * multiplier, (this.height(cell)) * multiplier, cell.getText()+"", this.linePointer,
                            (align == "left" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT : align == "right" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_RIGHT : align == "center" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_CENTER : flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT/*JUSTIFIED*/));
        } else {
            var content = cell.getText()+""; // to get rid of integer
            var currentLine = "";
            var pointer = 0;
            if (this.pdfWrapText && cell.getColumn().wordWrap) {
                this.cell((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell)) * multiplier, // + (this.height(cell) / 2) - (this.pdfFontSize / 2)) * multiplier,
                    this.width(cell) * multiplier, (this.height(cell)) * multiplier, this.getWrapedCellText(cell), this.linePointer,
                    (align == "left" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT : align == "right" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_RIGHT : align == "center" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_CENTER : flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT/*JUSTIFIED*/));
            } else {
                //just print whatever can fit in one line
                //if i can fit in one line, add me at the middle
                for (pointer = 0; pointer < content.length; pointer++) {
                    currentLine += content.charAt(pointer);
                    if (this.getStringWidth(currentLine) > ((this.width(cell) - paddingLeft - paddingLeft) * multiplier)) {
                        currentLine = currentLine.substr(0, currentLine.length - 1);
                        break;
                    }
                }
                this.cell((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell)) * multiplier, // + (this.height(cell) / 2) - (this.pdfFontSize / 2)) * multiplier,
                    this.width(cell) * multiplier, (this.height(cell)) * multiplier, currentLine, this.linePointer,
                    (align == "left" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT : align == "right" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_RIGHT : align == "center" ? flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_CENTER : flexiciousNmsp.JsPdfPrinter.CELL_ALIGN_LEFT/*JUSTIFIED*/));
                    //this.text(currentLine, ((containerX + this.x(cell) + 2) * multiplier), ((containerY + this.y(cell) + (this.height(cell) / 2) - (this.pdfFontSize / 2) - 2) * multiplier));

            }
        }

        //draw the border

        if (cell.hasHorizontalGridLines()) {
            rgbColor = flexiciousNmsp.JsPdfPrinter.extractRGB(cell.getHorizontalGridLineColor());
            pdfObject.setDrawColor(rgbColor[0], rgbColor[1], rgbColor[2]);
            pdfObject.setLineWidth(cell.getHorizontalGridLineThickness() * multiplier);
            this.line((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell) + this.height(cell) - 1) * multiplier, (containerX + this.x(cell) + this.width(cell)) * multiplier, (containerY + this.y(cell) + this.height(cell) - 1) * multiplier);
        }
        if (cell.getDrawTopBorder() || drawTop) {
            rgbColor = flexiciousNmsp.JsPdfPrinter.extractRGB(cell.getHorizontalGridLineColor());
            pdfObject.setDrawColor(rgbColor[0], rgbColor[1], rgbColor[2]);
            pdfObject.setLineWidth(cell.getHorizontalGridLineThickness() * multiplier);
            this.line((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell)) * multiplier, (containerX + this.x(cell) + this.width(cell)) * multiplier, (containerY + this.y(cell)) * multiplier);
        }

        if (cell.column) {
            if (cell.column.isLastUnLocked || cell.column.isLastrightLocked) {
                return;
            }
        }
        //now draw the right border
        var paddingX = cell.hasVerticalGridLines() ? cell.getVerticalGridLineThickness() : 0;
        var paddingY = cell.hasHorizontalGridLines() ? cell.getHorizontalGridLineThickness() : 0;

        if(drawLeft){
            this.line((containerX + this.x(cell) /*- 1*/) * multiplier, (containerY + this.y(cell)) * multiplier, (containerX + this.x(cell) /*- 1*/) * multiplier,
                (containerY + this.y(cell) + this.height(cell) - paddingY) * multiplier);
        }

        var cgc = (cell.implementsOrExtends('FlexDataGridColumnGroupCell') ? cell : null);
        var sbp = (cell.implementsOrExtends('FlexDataGridPaddingCell') ? cell : null);
        if (cgc && cgc.background)return;
        if (cell.hasVerticalGridLines()) {
            rgbColor = flexiciousNmsp.JsPdfPrinter.extractRGB(cell.getVerticalGridLineColor());
            pdfObject.setDrawColor(rgbColor[0], rgbColor[1], rgbColor[2]);
            pdfObject.setLineWidth(cell.getVerticalGridLineThickness() * multiplier);
            if (sbp) {
                if (sbp.scrollBarPad) {
                    this.line((containerX + this.x(cell)) * multiplier, (containerY + this.y(cell)) * multiplier, (containerX + this.x(cell)) * multiplier,
                                    (containerY + this.y(cell) + this.height(cell) - paddingY) * multiplier);
                }
            }
            else if (cgc) {
                var colBeforeMe;
                var drawLeftBorder = true;
                var startCol = cgc.columnGroup ? cgc.columnGroup.startColumn : null;
                if (drawLeftBorder) {
                    if (startCol && startCol.colIndex > 0) {
                        colBeforeMe = startCol.level.getVisibleColumns()[startCol.colIndex - 1];
                        if (colBeforeMe && colBeforeMe.columnGroup && colBeforeMe.columnGroup.endColumn == colBeforeMe) {
                            drawLeftBorder = false;
                        }
                    }
                    this.line((containerX + this.x(cell) - 1) * multiplier, (containerY + this.y(cell)) * multiplier, (containerX + this.x(cell) - 1) * multiplier,
                        (containerY + this.y(cell) + this.height(cell) - paddingY) * multiplier);
                }
                if (cgc.columnGroup.endColumn.isLastUnLocked || cgc.columnGroup.endColumn.isLastrightLocked) {
                    return;
                }
            }
            if(!dontDrawRight)
            this.line((containerX + this.x(cell) + this.width(cell) - paddingX) * multiplier, (containerY + this.y(cell)) * multiplier,
                (containerX + this.x(cell) + this.width(cell) - paddingX) * multiplier,
                (containerY + this.y(cell) + this.height(cell) - paddingY) * multiplier);
        }
    };
    JsPdfPrinter.prototype.capitalizeFirstLetterIfPrefix = function (prefix, val) {
        return prefix.length > 0 ? this.doCap(val) : val;
    };
    JsPdfPrinter.prototype.doCap = function (val) {
        return val.substr(0, 1).toUpperCase() + val.substr(1);
    };

    JsPdfPrinter.prototype.measurer = null;

    JsPdfPrinter.prototype.getStringWidth = function (content) {
        var  w = 0;
        if(!this.measurer)
            this.measurer = new flexiciousNmsp.UIComponent();
        w = this.measurer.measureText(content ? content : "");
        return w.width /** (this.pdfFontSize*/ * this.multiplier/*) * .001*/;
    };

    JsPdfPrinter.prototype.beginDocument = function (printOptions) {
        this.pdfObject = new jsPDF({
            orientation : printOptions.pageSize.isLandscape ? "l" : "p",
            unit : "mm",
            format : printOptions.pageSize.name.toLowerCase(),
            compress : 1,
            fontSize : this.pdfFontSize,
            lineHeight : 1///*this.pdfFontSize * */  /*(25.4/72)*/ (36/72)
        });
        this.linePointer = 0;
        this.isFirstPage = true;
    };
    JsPdfPrinter.prototype.endDocument = function (printOptions) {
        this.saveDoc(printOptions);
    };
    JsPdfPrinter.prototype.saveDoc = function (printOptions, confirmed) {
        if(confirm("Pdf Generated successfully. do you want to save it. ?"))
            this.pdfObject.output("save");
    };

    /**
     *
     * @param src originl color. either uint or #------ format
     */
    JsPdfPrinter.extractRGB = function(src){
        if(!src)
            return [0,0,0];

        if(typeof src == "string"){
            src = parseInt(src.replace("#", ""), 16);
        }

        var rgb = [];
        rgb[0] = src >> 16 & 0xFF;
        rgb[1] = src >> 8 & 0xFF;
        rgb[2] = src & 0xFF;
        return rgb;
    }

}(window));