/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import React from 'react'
import UIUtils from '../../utils/UIUtils'
import Constants from '../../utils/Constants'
import FilterSort from '../grids/filters/FilterSort'
import UIComponent from '../../core/UIComponent'
import ReactDataGrid from '../../table/ReactDataGrid'
import ReactDataGridColumn from '../../table/ReactDataGridColumn'
import ToolbarAction from '../../flexgrid/valueobjects/ToolbarAction'
// import { RadioButton, RadioButtonGroup } from 'material-ui/RadioButton'

/**
 * A MultiColumnSortPopup that which can be used within the sorting/binding infrastructure.
 * @constructor
 * @class MultiColumnSortPopup
 * @namespace flexiciousNmsp
 */
export default class MultiColumnSortPopup extends UIComponent {
    constructor() {
        super({}, "div")
        this.attachClass("flexiciousGrid");
        this.setWidth(600);
        this.setHeight(145);

        this.setGrid = (val) => {
            this.grid = val;
            this.allColumns = val.getColumns();

        }

        this.loadColumns = (lvl, dp, loadSorts) => {
            if (typeof loadSorts == "undefined") loadSorts = false;

            let item = new Object();
            const sortableCols = lvl.getSortableColumns();
            for (var i = 0; i < sortableCols.length; i++) {
                var col = sortableCols[i];
                item = new Object();
                item.label = col.getHeaderText();
                item.column = col;
                item.data = col.getUniqueIdentifier();
                dp.push(item);

            }
            if (loadSorts) {
                for (var i = 0; i < lvl.currentSorts.length; i++) {
                    const sort = lvl.currentSorts[i];
                    const sortables = lvl.getSortableColumns();
                    for (let j = 0; j < sortables.length; j++) {
                        var col = sortables[j];
                        if (sort.sortColumn == col.getSortFieldName()) {
                            sort.column = col
                            this.currentSorts.push(sort)
                        }
                    }
                }
            }
            if (lvl.nextLevel && !lvl.nextLevel.reusePreviousLevelColumns) {
                dp.push({ label: "", type: "separator" });
                item = new Object();
                item.label = "...";
                item.data = lvl;
                item.children = [];
                dp.push(item);
                this.loadColumns(lvl.nextLevel, item.children, loadSorts)
            }
        }
    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["MultiColumnSortPopup", "UIComponent"];
    }

    showDialog() {
        this.currentSorts = [];
        this.cols = [];
        this.loadColumns(this.grid.getColumnLevel(), this.cols, false);
        this.loadColumns(this.grid.getColumnLevel(), [], true);
        const actions = [
            ToolbarAction.create(Constants.MCS_BTN_CLEAR_ALL_LABEL, this.onClear.bind(this), true),
            ToolbarAction.create(Constants.MCS_BTN_APPLY_LABEL, this.onApply.bind(this), true),
            ToolbarAction.create(Constants.MCS_BTN_CANCEL_LABEL, this.onCancel.bind(this), true)
        ];
        this.popup = UIUtils.addPopUp(this.render(), this.grid, false, null, Constants.MCS_LBL_HEADER_TEXT, actions);
        this.grid.addPopup(this.popup);
    }

    onClear(evt) {
        this.grid.removeAllSorts();
        this.grid.removePopup(this.popup);
    }

    onCancel(evt) {
        this.grid.removePopup(this.popup);
    }

    onApply(evt) {
        const grid = this.grid;
        grid.removeAllSorts();
        for (let i = 0; i < grid.multiColumnSortNumberFields; i++) {
            const hiddenInput = document.getElementById(`${grid.id}-sort-value-${i}`);
            if (hiddenInput.value) {
                const colUnique = hiddenInput.value;
                const col = grid.getColumnLevel().getColumnByDataField(colUnique, "uniqueIdentifier", true);
                const ascendingRadioGroup = this['rbg' + i];
                const srt = new flexiciousNmsp.FilterSort();
                srt.column = col;
                srt.sortColumn = col.getSortFieldName();
                srt.isAscending = ascendingRadioGroup.getSelectedValue() !== "desc";
                srt.column.getLevel().addSort(srt);
            }
        }
        grid.rebuildBody();
        grid.rebuildHeader();
        grid.removePopup(this.popup);
    }

    onSelectionHandler(rowIndex, evt) {
        let col = this.allColumns[evt.target.selectedIndex];
        const hiddenInput = document.getElementById(`${this.grid.id}-sort-value-${rowIndex}`);
        hiddenInput.value = col.getUniqueIdentifier();
    }

    /**
     * Initializes the auto complete and watermark plugins
     */
    render() {
        let html = [];

        for (let i = 0; i < this.grid.multiColumnSortNumberFields; i++) {
            const srt = this.currentSorts.length > i ? this.currentSorts[i] : null;
            html.push(<div class={"sortPopupRow"}>
                <span class={"sortPopupLabel"}>{i == 0 ? Constants.MCS_LBL_SORT_BY_TEXT : Constants.MCS_LBL_THEN_BY_TEXT}</span>
                <input type="hidden" id={this.grid.id + '-sort-value-' + i} value={srt == null ? '' : srt.column.getUniqueIdentifier()} />
                
                <div>
                    <label className={"sortPopupDropdown ui-widget ui-corner-all ui-state-default"}>
                        {srt == null ? Constants.MCS_LBL_CHOOSE_COLS : srt.column.getHeaderText()}
                    </label>
                    <select onChange={this.onSelectionHandler.bind(this, [i])} defaultValue={srt == null ? Constants.MCS_LBL_CHOOSE_COLS : srt.column.getHeaderText()}>
                        {
                            this.allColumns.map((col, index) => {
                                return <option key={"option" + index} value={col.getLabel()}>{col.getLabel()}</option>
                            })
                        }
                    </select>
                </div>                
            </div>);
                            /*<RadioButtonGroup ref={(rbg) => { this['rbg' + i] = rbg; }} name={'radioButtonGroup'} defaultSelected={srt && srt.isAscending ? "asc" : "desc"}>
                    <RadioButton disabled={srt == null ? "disabled" : ''} name={srt + '' + i} label={Constants.MCS_RBN_ASCENDING_LABEL} className={"sortPopupLabel"} value={"asc"} />
                    <RadioButton disabled={srt == null ? "disabled" : ''} name={srt + '' + i} label={Constants.MCS_RBN_DESCENDING_LABEL} className={"sortPopupLabel"} value={"desc"} />
                </RadioButtonGroup>*/

        }

        return (
            <div className={'flexiciousSortPopup'}>
                {
                    html.map((data) => {
                        return data;
                    })
                }
            </div>
        );

    }
}



/**
 *
 * @type {Function}
 * */
flexiciousNmsp.MultiColumnSortPopup = MultiColumnSortPopup; //add to name space
MultiColumnSortPopup.prototype.typeName = MultiColumnSortPopup.typeName = 'MultiColumnSortPopup';//for quick inspection
