/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'

/**
 * Stores user selection for records to print or to export. Only applicable
 * when the filterPageSortMode of the grid is set to server,
 * and the user requests to print all pages or specific pages
 * of data that are not currently loaded.
 * @return
 * @constructor
 * @class PrintExportOptions
 * @namespace flexiciousNmsp
 * 
 */
export default class PrintExportOptions extends TypedObject {
    constructor() {
        super();
        /**
         * Specifies what to export, valid options are:
         * EXPORT_CURRENT_PAGE
         * EXPORT_ALL_PAGES
         * EXPORT_SPECIFIED_PAGES
         * EXPORT_SELECTED_RECORDS
         *
         * The All pages and specified pages are only available
         * if paging is enabled and the grid is in client mode.
         * @property printExportOption
         */
        this.printExportOption = PrintExportOptions.PRINT_EXPORT_CURRENT_PAGE;
        /**
         * In conjunction with printOption/exportOption = PRINT_EXORT_SPECIFIED_PAGES
         * determines which pages of a grid with paging enabled
         * to print.
         * @property pageFrom
         */
        this.pageFrom = -1;
        /**
         * In conjunction with printOption/exportOption = PRINT_EXORT_SPECIFIED_PAGES
         * determines which pages of a grid with paging enabled
         * to print.
         * @property pageTo
         */
        this.pageTo = -1;
        /**
         */
        this.popupParent = null;
        /**
         * Flag to control whether or not to show the column picker. Defaults to true.
         * @property showColumnPicker
         */
        this.showColumnPicker = true;
        /**
         */
        this.showWarningMessage = true;
        /**
         * A flag that will hide any columns that are not visible from being printed or exported by default. User can still go into
         * column picker and include these columns. To disable the user from printing or exporting invisible columns altogether,
         * please use the excludeHiddenColumns instead. Please note, when you set this flag, the columnsToPrint will get overwritten, so
         * any changes you make to that array will be ignored.
         * @default false
         * @property hideHiddenColumns
         */
        this.hideHiddenColumns = false;
        /**
         * A flag that will To disable the user from printing or exporting invisible columns altogether . User cannot go into
         * column picker and include these columns. To hide any columns that are not visible from being printed or exported by default,
         * please use the hideHiddenColumns instead
         * @default false
         * @property excludeHiddenColumns
         */
        this.excludeHiddenColumns = false;
        /**
         * Not used.
         */
        this.saveFileMessage = "File generated. Download?";

    }

    getClassNames() {
        return ["PrintExportOptions", "TypedObject"];
    }
}

flexiciousNmsp.PrintExportOptions = PrintExportOptions; //add to name space
PrintExportOptions.prototype.typeName = PrintExportOptions.typeName = 'PrintExportOptions';//for quick inspection
PrintExportOptions.PRINT_EXPORT_CURRENT_PAGE = "Current Page";
PrintExportOptions.PRINT_EXPORT_ALL_PAGES = "All Pages";
PrintExportOptions.PRINT_EXPORT_SELECTED_PAGES = "Selected Pages";
PrintExportOptions.PRINT_EXPORT_SELECTED_RECORDS = "Selected Records";