/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'
/**
 * A Class that holds row and column information
 * @constructor
 * @class ChangeInfo
 * @namespace flexiciousNmsp
 * 
 */
export default class ChangeInfo extends TypedObject {
    constructor(
        changedItem,
        changeLevelNestDepth,
        changedProperty,
        previousValue,
        newValue,
        changeType) {
        super();
        /**
         * The level nest depth at which the change happened.
         * @property changeLevelNestDepth
         * @type {int}
         * @default changeLevelNestDepth
         */
        this.changeLevelNestDepth = changeLevelNestDepth;
        /**
         * The item that was changed
         * @property changedItem
         * @type {Object}
         * @default changedItem
         */
        this.changedItem = changedItem;
        /**
         * The property on the item that was changed
         * @property changedProperty
         * @type {String}
         * @default changedProperty
         */
        this.changedProperty = changedProperty;
        /**
         * The value before the change
         * @property previousValue
         * @type {*}
         * @default previousValue
         */
        this.origValue = previousValue;
        /**
         * The value before the change
         * @property previousValue
         * @type {*}
         * @default previousValue
         */
        this.previousValue = previousValue;
        /**
         * The value after the change
         * @property newValue
         * @type {*}
         * @default newValue
         */
        this.newValue = newValue;
        /**
         * One of the three change types : CHANGE_TYPE_INSERT, CHANGE_TYPE_UPDATE or CHANGE_TYPE_DELETE
         * @property changeType
         * @type {String}
         * @default changeType
         */
        this.changeType = changeType;

    }

    /**
     *
     * @return {Array}
     */
    getClassNames() { //for support of "is" keyword
        return ["TypedObject", this.typeName];
    }

    /**
     * Creates a human readable string out of the given ChangeInfo object.
     * @return {String}
     */

    toString() {
        return ` NestDepth:${this.changeLevelNestDepth} previousValue:${this.previousValue} newValue::${this.newValue} changeType::${this.changeType} changedProperty::${this.changedProperty}\n`;
    }
}

/**
 *
 * @type {Function}
 */
flexiciousNmsp.ChangeInfo = ChangeInfo; //add to name space
/**
 *
 * @type {String}
 */
/**
 *
 * @type {String}
 */
ChangeInfo.prototype.typeName = ChangeInfo.typeName = "ChangeInfo";//for quick inspection
/**
 *
 * @type {String}
 */
ChangeInfo.CHANGE_TYPE_INSERT = "insert";

/**
 *
 * @type {String}
 */
ChangeInfo.CHANGE_TYPE_UPDATE = "update";
/**
 *
 * @type {String}
 */
ChangeInfo.CHANGE_TYPE_DELETE = "delete";
/**
 *
 * @type {String}
 */
ChangeInfo.CHANGE_TYPE_RESET = "reset";
