/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'
import RowInfo from './RowInfo'
/**
 * Creates an instance of ComponentInfo.
 * @param {FlexDataGridCell} component
 * @param {Number} x
 * @param {RowInfo} row
 * @constructor
 * @class ComponentInfo
 */
export default class ComponentInfo extends TypedObject {

    constructor(component, x, row) {

        super();
        this.component = null;
        this.x = null;
        /**
         * @property row
         * @type {RowInfo}
         * @default null
         */
        this.row = null;
        /**
         *
         * @type {Boolean}
         * @property inCornerAreas
         * @default false
         */
        this.inCornerAreas = false;

        /**
         *
         * @type {FlexDataGridCell}
         * @property component
         * @default component
         */
        this.component = component;
        /**
         *
         * @type {Number}
         * @property x
         * @default x
         */
        this.x = x;
        /**
         *
         * @type {RowInfo}
         * @property {RowInfo}
         * @default row
         */
        this.row = row;


    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["ComponentInfo", "TypedObject"];
    }

    /**
     *
     * @return {int}
     */
    getColIndex() {
        const cell = this.component;
        if (!cell) return -1;
        const pad = cell.implementsOrExtends("FlexDataGridPaddingCell") ? cell : null;
        if (pad) {
            if (pad.scrollBarPad)
                return 99999;
            else if (pad.forceRightLock)
                return 99998;
            else
                return -2;
        }
        return cell.getColumn() ? cell.getColumn().getColIndex() : -1;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLocked() {
        const cell = this.component;
        return cell && cell.getIsLocked();
    }

    /**
     *If left locked, returns X, if right locked returns leftLockedWidth+unlockedWidth+x
     * if unlocked returns leftLockedX + x
     @return {Boolean}
     */

    getPerceivedX() {
        const cell = this.component;
        if (!cell) return this.x;
        return cell.getPerceivedX();
    }

    /**
     *
     * @return {Boolean}
     */
    getIsRightLocked() {
        const cell = this.component;
        return cell && cell.getIsRightLocked();
    }

    /**
     *
     * @return {Boolean}
     */
    getIsLeftLocked() {
        const cell = this.component;
        return cell && cell.getIsLeftLocked();
    }

    /**
     * Returns true if we are a data cell or if we area a chrome cell
     * at a nest depth of greater than 1. These cells are all drawn in the
     * content area, hence have to scroll
     */
    getIsContentArea() {
        const cell = this.component;
        return cell && (cell.getIsContentArea());
    }

    /**
     *
     * @return {Number}
     */
    getX() {
        return this.x;
    }

    /**
     *
     * @param val
     */

    setX(val) {
        this.x = val;
    }

    /**
     *
     * @return {Number}
     */
    getY() {
        return this.y;
    }

    /**
     *
     * @param val
     */
    setY(val) {
        this.y = val;
    }
}

/**
 *
 * @type {Function}
 */

flexiciousNmsp.ComponentInfo = ComponentInfo; //add to name space

/**
 *
 * @type {String}
 */
ComponentInfo.prototype.typeName = ComponentInfo.typeName = 'ComponentInfo';//for quick inspection