/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */



import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import EventDispatcher from '../../core/EventDispatcher'
import FilterContainerImpl from '../grids/filters/FilterContainerImpl'
import ComponentInfo from './ComponentInfo'
/**
 * @constructor
 * @class RowInfo
 * @namespace flexiciousNmsp
 */
export default class RowInfo extends EventDispatcher {
    constructor(height, y, grid) {
        super();
        this.height = 0;
        this.y = 0;
        this.grid = null;
        this.cells = [];
        this.rowPositionInfo = null;

        /**
         *
         * @type {Boolean}
         * @property  hasScrollbarPad
         * @default false
         */
        this.hasScrollbarPad = false;
        /**
         *
         * @type {Boolean}
         * @property  hasNestPad
         * @default false
         */
        this.hasNestPad = false;
        this.paddingCell = null;
        /**
         *
         * @type {Boolean}
         * @property  hasRightLockPad
         * @default false
         */
        this.hasRightLockPad = false;
        /**
         *
         * @type {Boolean}
         * @property  hasDisclosure
         * @default false
         */
        this.hasDisclosure = false;

        /**
         *
         * @type {FilterContainerImpl}
         * @property filterContainerInterface
         * @default null
         */

        this.filterContainerInterface = null;
        /**
           *
           * @type {int}
           * @property height
           * @default height
           */
        this.height = height;
        /**
         *
         * @type {int}
         * @property y
         * @default y
         */
        this.y = y;
        /**
         *
         * @type {IExtendedDataGrid}
         * @property grid
         * @default grid
         */
        this.grid = grid;
        /**
         *
         * @type {Array}
         * @property cells
         * @default []
         */
        this.cells = [];

        this.visible = true;
        //TODO - conditional instantiation
        this.filterContainerInterface = new FilterContainerImpl(this);
    }

    /**
     *
     * @return {Array}
     */
    getClassNames() {
        return ["RowInfo", "EventDispatcher"];
    }

    /**
     *
     * @return {Object}
     */
    getData() {
        return this.rowPositionInfo ? this.rowPositionInfo.rowData : null;
    }

    /**
     *
     * @return {Number}
     */
    getY() {
        return this.y;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsHeaderRow() {
        return this.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsColumnGroupRow() {
        return this.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_COLUMN_GROUP;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsFooterRow() {
        return this.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsPagerRow() {
        return this.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsFilterRow() {
        return this.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILTER;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsRendererRow() {
        return this.rowPositionInfo ? this.rowPositionInfo.getIsRendererRow() : false;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsFillRow() {
        return this.rowPositionInfo.getRowType() == flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILL;
    }

    /**
     *
     * @return {Boolean}
     */
    getIsDataRow() {
        return !this.getIsChromeRow();
    }

    /**
     * Returns true if we are a data cell or if we area a chrome cell
     * at a nest depth of greater than 1. These cells are all drawn in the
     * content area, hence have to scroll
     @return {Boolean|*|Boolean}
     */
    getIsContentArea() {
        return (this.getIsDataRow() || (this.getIsChromeRow() && (this.rowPositionInfo.getLevelNestDepth() > 1)));
    }

    /**
     * Returns true for Header, footer, Pager and renderer rows.
     @return {Boolean}
     */
    getIsChromeRow() {
        const rt = this.rowPositionInfo.getRowType();
        //return isHeaderRow || isFooterRow  || isPagerRow || isRendererRow || isFilterRow || isColumnGroupRow;
        //1 2 3 4 7 8
        return (rt != flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA && rt != flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILL)
    }

    /**
     * Returns true for Header, Data and Footer rows,where cells map directly to columns
     @return {Boolean}
     */

    getIsColumnBased() {
        const rt = this.rowPositionInfo.getRowType();
        //return isHeaderRow || isFooterRow  || isDataRow || isColumnGroupRow|| isFilterRow;
        return rt != flexiciousNmsp.RowPositionInfo.ROW_TYPE_PAGER && rt != flexiciousNmsp.RowPositionInfo.ROW_TYPE_RENDERER
    }

    getName() {
        return this.getData() ? this.getData().name : "";
    }

    /**
     *
     * @param other
     * @return {Boolean}
     */
    isSimilarTo(other) {
        const similarRowTypes = ([flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FOOTER,
        flexiciousNmsp.RowPositionInfo.ROW_TYPE_HEADER, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILTER, flexiciousNmsp.RowPositionInfo.ROW_TYPE_FILL]);

        if (this.rowPositionInfo.getLevelNestDepth() != other.rowPositionInfo.getLevelNestDepth())
            return false;
        else if ((similarRowTypes.includes(this.rowPositionInfo.getRowType()) &&
            similarRowTypes.includes(other.rowPositionInfo.getRowType())) && this.rowPositionInfo.getLevelNestDepth() == other.rowPositionInfo.getLevelNestDepth())
            return true;
        else
            return false;
    }

    /**
     *
     * @return {Boolean}
     */
    paddingExists() {
        return this.hasNestPad;
    }

    /**
     *
     * @return {Boolean}
     */
    scrollbarPadExists() {
        return this.hasScrollbarPad;
    }

    /**
     *
     * @return {Array}
     */
    getInnerCells() {
        const result = [];

        for (const cell of this.cells) {
            result.push(cell.component);
        }

        return result;
    }

    /**
     *
     * @return {Boolean}
     */
    disclosureExists() {
        return this.hasDisclosure;
    }

    /**
     *
     * @return {Boolean}
     */
    rightLockedExists() {
        if (this.hasRightLockPad) return true;

        for (const cell of this.cells) {
            const comp = cell.component;
            if (comp && comp.getIsRightLocked())
                return true;
        }

        return false;
    }

    /**
     *
     * @param col
     * @return {Boolean}
     */
    columnCellExists(col) {
        return this.getCellForColumn(col) != null;
    }

    /**
     *
     * @param col
     * @return {Boolean}
     */
    columnGroupCellExists(col) {
        return this.getCellForColumnGroup(col) != null;
    }

    /**
     * Gets the component info object for the specified column
     * @param col
     * @return {*}
     */
    getCellForColumnGroup(col) {
        for (const cell of this.cells) {
            if (cell.component.hasOwnProperty("columnGroup") && cell.component.columnGroup == col)
                return cell;
        }

        return null;
    }

    /**
     * Gets the first IExpandCollapseComponent
     * @return {*}
     */

    getExpandCollapseCell() {
        for (const cell of this.cells) {
            if (cell.component.getIsExpandCollapseCell())
                return cell.getIExpandCollapseComponent();
        }

        return null;
    }

    /**
     * Gets the component info object for the specified column
     * @param col
     * @return {*}
     */
    getCellForColumn(col) {
        for (const cell of this.cells) {
            if (cell.component.getColumn() == col)
                return cell;
        }

        return null;
    }

    /**
     *
     * @return {Array}
     */
    getLockedCells() {
        const lockedCells = [];

        for (const cell of this.cells) {
            if (cell.component.getIsLocked())
                lockedCells.push(cell.component);
        }

        return lockedCells;
    }

    /**
     *
     * @param component
     * @param x
     * @return {flexiciousNmsp.ComponentInfo}
     */
    addCell(component, x) {
        if (component && component.hasOwnProperty("scrollBarPad")) {
            if (component.scrollBarPad)
                this.hasScrollbarPad = true;

            else if (component.forceRightLock)
                this.hasRightLockPad = true;

            else {
                this.hasNestPad = true;
                this.paddingCell = component;
            }
        }
        else if (component.implementsOrExtends('FlexDataGridExpandCollapseCell')) {
            this.hasDisclosure = true;
            this.disclosureCell = component;
        }
        const comp = new flexiciousNmsp.ComponentInfo(component, x, this);
        this.cells.push(comp);
        return comp;
    }

    /**
     *
     * @param comp
     */
    removeCell(comp) {
        if (comp.component.implementsOrExtends('FlexDataGridPaddingCell')) {
            if (comp.component.scrollBarPad)
                this.hasScrollbarPad = false;
            else if (comp.component.forceRightLock)
                this.hasRightLockPad = false;
            else {
                this.hasNestPad = false;
                this.paddingCell = null;
            }
        }
        else if (comp.component.implementsOrExtends('FlexDataGridExpandCollapseCell')) {
            this.hasDisclosure = false;
            this.disclosureCell = null;
        }
        if (this.cells.includes(comp))
            this.cells.splice(comp.row.cells.indexOf(comp), 1);

    }

    /**
     * Reuse an existing row.
     * @param rowPos
     * @param setHt
     */

    setRowPositionInfo(rowPos, setHt) {
        this.rowPositionInfo = rowPos;
        this.height = rowPos.getRowHeight();
        this.y = rowPos.getVerticalPosition();

        for (const cell of this.cells) {
            const comp = cell.component;
            if (comp)
                comp.refreshCell();
            if (setHt)
                comp.setActualSize(comp.width, rowPos.getRowHeight());
        }
    }

    forceUpdate(){
        for (const cell of this.cells) {
            if(cell.component.reactComponent){
                cell.component.reactComponent.forceUpdate();
            }
        }
    }
    /**
     *
     * @param showHide
     */
    showHide(showHide) {
        for (const cell of this.cells) {
            cell.component.setVisible(showHide);
        }

        this.visible = showHide;
    }

    /**
     *  handle invalidatecells
     */
    invalidateCells() {
        for (const cell of this.cells) {
            const cellComp = cell.component;
            if (cellComp)
                cellComp.invalidateBackground();
        }
    }

    /**
     *
     * @return {int}
     */
    getRowIndex() {
        return this.rowPositionInfo.getRowIndex();
    }

    /**
     *
     * @param grid
     * @return {int}
     */
    getColumnGroupDepth(grid) {
        if (this.rowPositionInfo.getLevelNestDepth() == 1)
            return this.rowPositionInfo.getRowIndex() + 1;
        else
            //- not supported for now. Column groups are only supported at the top level.
            return grid.getBodyContainer().getColumnGroupDepth(this);

    }

    /**
     *
     * @param col
     * @return {Number}
     */
    getMaxCellHeight(col) {
        if (typeof col == "undefined") col = null;

        if (!this.getIsDataRow()) return this.height;
        if (this.getIsFillRow()) return this.height;
        let ht = this.height;

        for (const cell of this.cells) {
            const cellComp = cell.component;
            if (col != null && (cellComp.getColumn() != col)) continue;
            if (cellComp && (cellComp.height > ht))
                ht = cellComp.height;
        }

        return ht;
    }

    //filter calls
    /**
     *
     * @return {ArrayCollection}
     */
    getFilterArguments() {
        return this.filterContainerInterface.getFilterArguments();

    }

    /**
     * Registers the IFilter control in our list.
     * @param iFilterControl     A control that implements IFilterControl
     */
    registerIFilterControl(iFilterControl) {
        this.filterContainerInterface.registerIFilterControl(iFilterControl);

    }

    /**
     * Registers the IFilter control in our list.
     * @param iFilterControl     A control that implements IFilterControl
     */
    unRegisterIFilterControl(iFilterControl) {
        this.filterContainerInterface.unRegisterIFilterControl(iFilterControl);

    }

    /**
     * Clears all the filters
     */

    clearFilter() {
        this.filterContainerInterface.clearFilter();

    }

    /**
     * process all the filters.
     */
    processFilter() {
        this.filterContainerInterface.processFilter(null);
    }

    /**
     * Used to set the initial value of the filter.
     * @param column        The column to set filter for
     * @param value            The value to set
     */
    setFilterValue(column, value) {
        this.filterContainerInterface.setFilterValue(column, value);

    }

    /**
     * @param fld    The string to match the searchField property of the filter control with.
     * @return         True if focus was set, false if otherwise.
     * Focus may not be set if the given search field does not exist, <br/>
     * or if the filter control for the given search field does not  <br/>
     * implement IFocusManagerComponent.
     */
    setFilterFocus(fld) {
        return this.filterContainerInterface.setFilterFocus(fld);

    }

    /**
     * Used to set the initial value of the filter.
     * @param column
     * @return {Object}
     */

    getFilterValue(column) {
        return this.filterContainerInterface.getFilterValue(column);

    }

    /**
     * Returns true if the container has a filter control
     * that is bound to the specified column.
     * @param column
     * @return {Boolean}
     */

    hasField(column) {
        return this.filterContainerInterface.hasField(column);

    }

    /**
     * handle kill
     */
    kill() {
        for (const cell of this.cells) {
            cell.component.kill();
        }

        this.cells = [];
        this.grid = null;
        if (this.rowPositionInfo)
            this.rowPositionInfo.kill();
        this.rowPositionInfo = null;
        if (this.fillerRow) {
            this.fillerRow.kill();
        }
        this.filterContainerInterface.kill();
    }

    /**
     *
     * @return {Number}
     */
    getWidth() {
        return this.width;
    }

    /**
     *
     * @param val
     * @return {Number}
     */
    setWidth(val) {
        if (!val && val !== 0) return val;
        this.width = val;
    }

    /**
     *
     * @return {Number}
     */
    getHeight() {
        return this.height;
    }

    /**
     *
     * @param val
     * @return {Number}
     */
    setHeight(val) {
        if (!val && val !== 0) return val;
        this.height = val;
        return val;
    }

    /**
     *
     * @return {Number}
     */
    getY() {
        return this.y;
    }

    /**
     *
     * @param val
     * @return {Number}
     */
    setY(val) {
        if (!val && val !== 0) return val;
        this.y = val;
        return val;
    }
}

/**
 *
 * @type {Function}
 * @property RowInfo
 * @default RowInfo
 */
flexiciousNmsp.RowInfo = RowInfo;