/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'
/**
 *
 * Internal class used to keep track of the overall grid rows. (Not just the visible ones).
 * @param rowData
 * @param rowIndex
 * @param verticalPosition
 * @param rowHeight
 * @param level
 * @param rowType
 * @constructor
 * @class RowPositionInfo
 */

export default class RowPositionInfo extends TypedObject {
    constructor(rowData, rowIndex, verticalPosition, rowHeight, level, rowType) {
        super();
        /**
             *
             * @type {Object}
             * @property  rowData
             * @default null
             */
        this.rowData = null;
        /**
         *
         * @type {VirtualScrollLoadInfo}
         * @property  virtualScrollLoadInfo
         * @default null
         */
        this.virtualScrollLoadInfo = null;
        /**
         *
         * @type {String}
         * @property  dataString
         * @default null
         */
        this.dataString = "";

        this.rowData = rowData;
        this.dataString = `${rowIndex}:${verticalPosition}:${rowHeight}:${rowType}:${level.getNestDepth()}`;
        super();
        this.printReduced = false;
        this.rowSpan = 1;

    }

    /**
     *
     * @return {String}
     */
    getClassNames() {
        return "RowPositionInfo";
    }

    kill() {
        this.dataString = null;
        this.rowData = null;
        this.virtualScrollLoadInfo = null;
    }

    /**
     *
     * @param grid
     * @return {*}
     */
    getLevel(grid) {
        const depth = this.getLevelNestDepth();
        if (depth < 1)
            return grid.getColumnLevel();
        let level = grid.getColumnLevel();
        while (depth != level.getNestDepth())
            level = level.nextLevel;
        return level;
    }

    /**
     *
     * @return {Number}
     */
    getRowIndex() {
        return parseInt(this.dataString.split(":")[0]);
    }

    /**
     *
     * @param val
     */
    setRowIndex(val) {
        this.dataString = `${val}:${this.getVerticalPosition()}:${this.getRowHeight()}:${this.getRowType()}:${this.getLevelNestDepth()}`;
    }

    /**
     *
     * @param val
     */
    setRowHeight(val) {
        this.dataString = `${this.getRowIndex()}:${this.getVerticalPosition()}:${val}:${this.getRowType()}:${this.getLevelNestDepth()}`;
    }

    /**
     *
     * @return {Number}
     */
    getVerticalPosition() {
        return parseFloat(this.dataString.split(":")[1]);
    }

    /**
     *
     * @param val
     */
    setVerticalPosition(val) {
        this.dataString = `${this.getRowIndex()}:${val}:${this.getRowHeight()}:${this.getRowType()}:${this.getLevelNestDepth()}`;
    }

    /**
     *
     * @return {Number}
     */
    getRowHeight() {
        return parseInt(this.dataString.split(":")[2]);
    }

    /**
     *
     * @return {Number}
     */
    getRowType() {
        return parseInt(this.dataString.split(":")[3])
    }

    /**
     *
     * @return {Number}
     */
    getLevelNestDepth() {
        return parseInt(this.dataString.split(":")[4])
    }

    /**
     *
     * @return {*}
     */
    getRowNestlevel() {
        return this.getRowType() == RowPositionInfo.ROW_TYPE_RENDERER ? this.getLevelNestDepth() + 1 : this.getLevelNestDepth();
    }

    /**
     *
     * @return {Boolean}
     */
    getIsRendererRow() {
        return this.getRowType() == RowPositionInfo.ROW_TYPE_RENDERER
    }

    /**
     *
     * @return {Boolean}
     */
    getIsDataRow() {
        return this.getRowType() == RowPositionInfo.ROW_TYPE_DATA
    }

    /**
     *
     * @return {Boolean}
     */
    getIsHeaderRow() {
        return this.getRowType() == RowPositionInfo.ROW_TYPE_HEADER
    }

    /**
     *
     * @return {Number}
     */
    getVerticalPositionPlusHeight() {
        return this.getVerticalPosition() + this.getRowHeight();
    }
}

RowPositionInfo.ROW_TYPE_HEADER = 1;
RowPositionInfo.ROW_TYPE_FOOTER = 2;
RowPositionInfo.ROW_TYPE_PAGER = 3;
RowPositionInfo.ROW_TYPE_FILTER = 4;
RowPositionInfo.ROW_TYPE_DATA = 5;
RowPositionInfo.ROW_TYPE_FILL = 6;
RowPositionInfo.ROW_TYPE_RENDERER = 7;
RowPositionInfo.ROW_TYPE_COLUMN_GROUP = 8;


flexiciousNmsp.RowPositionInfo = RowPositionInfo;