/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'
import Constants from '../../utils/Constants'
/**
 * A class that represents a custom toolbar action.
 *
 * @param [name]     Name of the toolbar action
 * @param [level]    Nest Depth of the level to enable the toolbar action for. Nest Depths Start at 1. Use -1 for all levels.
 * @param [code]    The Code associated with the tool bar action, not required.
 * @param [tooltip]    The hover over tooltip when the user hovers over the image specified by the icon url
 * @param [iconUrl]    The image to use for the toolbar action.
 * @param [seperatorBefore]    Whether or not to draw a separator before the toolbar action
 * @param [seperatorAfter]    Whether or not to draw a separator after the toolbar action
 *
 * @class ToolbarAction
 * @constructor
 */
export default class ToolbarAction extends TypedObject {
    constructor(
        name,
        level,
        code,
        tooltip,
        iconUrl,
        seperatorBefore,
        seperatorAfter,
        requiresSelection,
        requiresSingleSelection) {
            super();
        /**
     * The actual button associated with the toolbar action.
     */
        this.trigger = null;
        /**
         * To support runtime enable/disable of toolbar actions.
         */
        this.enabled = true;
        /**
         * Flag to control whether or not to draw a seperator before the action icon.
         * Deperecated. Please use the code='separator' instead
         */
        this.seperatorBefore = false;
        /**
         * Flag to control whether or not to draw a seperator after the action icon
         * Deperecated. Please use the code='separator' instead
         */
        this.seperatorAfter = false;
        /**
         * Name of the action
         */
        this.name = "";
        /**
         * Code of the action, if not specified, defaults to the name
         */
        this.code = null;
        /**
         * Tooltip for the icon. If not specified, defaults to the name
         */
        this.tooltip = null;
        /**
         * Icon for the image button. If not specified, defaults to the custom action icon.
         */
        this.iconUrl = null;
        /**
         * Icon for the image button. If not specified, defaults to the custom action icon.
         */
        this.disabledIconUrl = null;
        /**
         * This action will be disabled if a selection is required and there is nothing selected in the grid.
         */
        this.requiresSelection = null;
        /**
         * This action will be disabled if a more than one item or no items are selected in the grid.
         */
        this.requiresSingleSelection = null;
        /**
         * Level at which the action is to be applied.
         * If not specified, defaults to 1. If you set this to -1, the action will appear in the tool bar at all levels.
         * Defaults to -1
         */
        this.level = -1;
        /**
         * A list of ToolbarAction objects that get converted into dropdown menu buttons.
         */
        this.subActions = [];

        /**
         * This means the action itself is only a container for subactions, clicking on it does nothing.
         */
        this.dropdownOnly = false;

        this.name = name;
        if (!code) code = name
        this.code = code;
        if (!tooltip) tooltip = name
        this.tooltip = tooltip;
        if (!iconUrl) iconUrl = "";
        this.iconUrl = iconUrl;
        this.level = level;
        this.seperatorBefore = seperatorBefore;
        this.seperatorAfter = seperatorAfter;
        this.requiresSelection = requiresSelection;
        this.requiresSingleSelection = requiresSingleSelection; 

    }

    getClassNames() {
        return "ToolbarAction";
    }

    /**
     * Returns true if type!=seperator and does not have subActions
     */
    getIsRegularAction() {
        return (this.name != 'separator') && (this.subActions.length == 0)
    }

    /**
     * Returns true if type!=seperator and has subActions
     */
    getIsDropdownAction() {
        return (this.name != 'separator') && (this.subActions.length > 0)
    }

    /**
     * Returns true if type==seperator
     */
    getIsSeparator() {
        return (this.name == 'separator')
    }

    /**
     * For hierarchical menu support
     * @return
     */
    getChildren() {
        return this.subActions.length > 0 ? this.subActions : undefined;
    }

    /**
     *
     */
    getLabel() {
        return this.name;
    }

    static create(name, callback, closeDialog){
        let action = new ToolbarAction();
        action.name = name;
        action.callback = callback;
        action.closeDialog = closeDialog;
        return action;
    }
}
ToolbarAction.DEFAULT_ICON = `${Constants.IMAGE_PATH}/customAction.png`;
flexiciousNmsp.ToolbarAction = ToolbarAction;
