/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils' 
import TypedObject from '../../core/TypedObject'

/**
 * When grid.enableVirtualScroll=true, the grid dispatches a virtualScroll event when the grid scrolls.
 * This event contains a recordsToLoad array, which contains an series of VirtualScrollLoadInfo records.
 * This provides the application a list of record indexes to load, and which levels to load them at.
 *
 * In flat grids, the FlexDataGridVirtualScrollEvent.recordsToLoad will always contain a list of VirtualScrollLoadInfo records were level=1.
 * For nested grids, depending on where the user scrolls and which items are open, you could get a list of records from multiple levels,
 * for example. Records 1-3 from level 1, All children of record 3 from level 2, and then Record 4-10 from level 1.
 *
 * Setting enableVirtualScroll requires that you specify childrenCountField as well as selectedKeyField.
 * Additionally, enableVirtualScroll also requires that you set filterPageSortMode=server and
 * itemLoadMode=server. Finally, virtual scroll does not work with variableRowHeights.
 * @class VirtualScrollLoadInfo
 * @constructor
 */
export default class VirtualScrollLoadInfo extends TypedObject {
    constructor(
        level,
        recordIndex,
        verticalScrollPosition,
        itemHeight,
        item,
        parent,
        rowType) {

        /**
         * FlexDataGridColumnLevel that this item belongs to
         */
        this.level = null;
        /**
         * The record index.
         * */
        this.recordIndex = null;
        /**
         * The verticalScrollPosition that this record appears at
         */
        this.verticalScrollPosition = null;
        /**
         * The "area" covered by this item. Includes any children.
         */
        this.itemHeight = null;
        /**
         * Pointer to the item. Should be populated before calling setVirtualScrollData.
         */
        this.item = null;
        /**
         * Pointer to the parent item.
         */
        this.parent = null;
        /**
         * Type of the row.
         */
        this.rowType = null;

        this.level = level;
        this.recordIndex = recordIndex;
        this.verticalScrollPosition = verticalScrollPosition;
        this.item = item;
        this.itemHeight = itemHeight;
        this.parent = parent;
        if (rowType == -1)
            this.rowType = flexiciousNmsp.RowPositionInfo.ROW_TYPE_DATA;
        else
            this.rowType = rowType;
        flexiciousNmsp.TypedObject.apply(this, []);

    }

    getClassNames() {
        return "VirtualScrollLoadInfo";
    }

    getRowType() {
        return this.rowType;
    }

    /**
     * The nest depth of the level
     */
    getLevelNestDepth() {
        return this.level.getNestDepth();
    }

    toString() {
        return `Id:${this.item && this.item.hasOwnProperty("Id") ? this.item["Id"] : "__"} NestDepth:${this.level.getNestDepth()} V pos:${this.verticalScrollPosition} Item Height::${this.itemHeight} Record Index::${this.recordIndex} Parent::${this.parent && this.parent.hasOwnProperty("Id") ? this.parent["Id"] : "__"}\n`;
    }

    getItemArea() {
        return this.level.getRowHeight() + this.itemHeight;
    }

    isParentOf(child) {
        return (child.verticalScrollPosition + child.getItemArea() > this.verticalScrollPosition)
            && (child.verticalScrollPosition < this.verticalScrollPosition + this.getItemArea())
            && child.levelNestDepth > this.levelNestDepth;
    }

    getVerticalScrollPosition() {
        return this.verticalScrollPosition;
    }

    setVerticalScrollPosition(val) {
        this.verticalScrollPosition = val;
    }

    getRowType() {
        return this.rowType;
    }

    setRowType(val) {
        this.rowType = val;
    }
}

flexiciousNmsp.VirtualScrollLoadInfo = VirtualScrollLoadInfo;