/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import React from 'react'
import TypedObject from '../core/TypedObject'
import ReactDataGridColumn from './ReactDataGridColumn'
import ReactDataGridColumnGroup from './ReactDataGridColumnGroup'
import FlexDataGridColumnLevel from '../flexgrid/grids/FlexDataGridColumnLevel'


/**
 * Configuration for the ReactDataGridColumnLevel - this is not a visual component, just container
 * for properties for the corresponding {@link FlexDataGridColumnLevel} object.
 * 
 * For more details, refer to the documentation of the {@link ReactDataGrid} class.
 */
export default class ReactDataGridColumnLevel extends React.Component {

    static process(grid, parentLevel, props) {
        //the grid has already called will mount, so we have the grid.
        let level = grid.getColumnLevel(); //we parse out all the columns and store it - we dont
        if (parentLevel) {
            level =  props.columnLevelFactory ? new props.columnLevelFactory(grid) :new FlexDataGridColumnLevel(grid);
            parentLevel.nextLevel = (level);
            parentLevel.setParentLevel();
            
        }
        //want to touch the original level until we are all done, because if we do that, it will
        //try to render that level, which is not complete yet.
        for (var prop in props) {
            //here we apply all the attributes that were set on us from the calling component
            if (prop == "children" || prop == "grid") continue;
            grid.applyAttribute(level, prop, props[prop], true);
        }
        let hasGroups = false;
        var childrenWithProps = React.Children.map(props.children, (child) => {
            if (child.type.typeName == "ReactDataGridColumn") {
                ReactDataGridColumn.process(grid, level, child.props);
            } else if (child.type.typeName == "ReactDataGridColumnGroup") {
                ReactDataGridColumnGroup.process(grid, level, child.props);
                hasGroups = true;
            } else if (child.type.typeName == "ReactDataGridColumnLevel") {
                ReactDataGridColumnLevel.process(grid, level, child.props);
            } else {
                console.error("Invalid Child type "+child.type.typeName+" specified for ReactDataGridColumnLevel - must be one of ReactDataGridColumn|ReactDataGridColumnGroup|ReactDataGridColumnLevel")
            }
        });
        if (level._tempCols) {
            if (hasGroups) {
                level.setGroupedColumns(level._tempCols);
            } else {
                level.setColumns(level._tempCols);
            }

        }
    }

}

flexiciousNmsp.ReactDataGridColumnLevel = ReactDataGridColumnLevel; //add to name space
ReactDataGridColumnLevel.prototype.typeName = ReactDataGridColumnLevel.typeName = 'ReactDataGridColumnLevel';//for quick inspection
