/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import ReactDataGrid from '../ReactDataGrid'
import ReactDataGridPsuedoScrollBodyContainer from './ReactDataGridPsuedoScrollBodyContainer'
import Constants from '../../utils/Constants'
import UIUtils from '../../utils/UIUtils'

/**
 * Specialized extension of the FlexDataGrid that is used for psuedo scroll. Hides the
 * vertical scrollbar, and shows only what can be displayed on the page given the
 * height constraints of the page being printed to.
 * @constructor
 * @namespace flexiciousNmsp.print
 * @extends FlexDataGrid
 */
export default class ReactDataGridPsuedoScroll extends ReactDataGrid {
    constructor(props, arg1, arg2) {
        super(props, arg1, arg2)
        this.mouseWheelDeltaMultiplier = 10;
    }
    getClassNames() {
        return ["ReactDataGridPsuedoScroll ", "FlexDataGrid", "NdgBase", "UIComponent", "IPrintDatagrid",
            "IExtendedFlexDataGrid", "IExtendedDataGrid", "ISpinnerOwner"];
    }

    /**
     * We create a specialized body container
     * @return
        *
     */
    createBodyContainer() {

        return new ReactDataGridPsuedoScrollBodyContainer(this);


    }

    synchronizeLockedVerticalScroll() {
    }

    /**
     * @method placeComponent
     */

    placeComponent(comp) {
        let v = 0;

        if (comp.component.parent == this._treeDataGridContainer
            || comp.component.parent == this._leftLockedContent
            || comp.component.parent == this._rightLockedContent) {
            v = this._treeDataGridContainer.rows.indexOf(comp.row) * comp.row.height;// Math.max(0,comp.row.getY() -this.getVerticalScrollPosition())
        } else {
            v = (comp.row && !comp.inCornerAreas ? comp.row.getY() -
                ((comp.row.getIsHeaderRow() || comp.row.getIsColumnGroupRow()) ?
                    comp.component.getHeight() - comp.row.getHeight() : 0) : this.getCornerY(comp));

        }
        const compx = comp.component.implementsOrExtends("FlexDataGridColumnGroupCell") ? comp.component.getX() : comp.getX();

        comp.component.move(compx, v);
        if (this.cellPlacementFunction) {
            this.cellPlacementFunction(comp.component);
        }
    }

    /**
    * Transfers mouse wheel events over the locked left and right sections to scroll events on the body container,
    * so we scroll when the user mouse wheels over the left or right locked containers.
    * @method onMouseWheel
    * @param event
    */
    onMouseWheel(event) {
        var e = event.triggerEvent;
        if (e.preventDefault)
            e.preventDefault();
        e.returnValue = false; 

        let scrollpos = this.getBodyContainer().externalScrollContainer.getVerticalScrollPosition()|| 0;
        scrollpos -= (this.mouseWheelDeltaMultiplier * UIUtils.getMouseWheelDelta(event.triggerEvent));
        const scrollEvent = this.buildScrollEvent(scrollpos);
        this.getBodyContainer().externalScrollContainer.setVerticalScrollPosition(scrollpos);
        this.getBodyContainer().onExternalContainerScroll(scrollEvent, true);
    }

    buildScrollEvent(scrollpos) {
        const scrollEvent = new flexiciousNmsp.FlexDataGridEvent(Constants.EVENT_SCROLL);
        if (scrollpos > this.getBodyContainer().getMaxVerticalScrollPosition() + this.getRowHeight()) {
            scrollpos = this.getBodyContainer().getMaxVerticalScrollPosition() + this.getRowHeight();
        } else if (scrollpos < 0) {
            scrollpos = 0;
        }
        scrollEvent.scrollTop = scrollpos;
        scrollEvent.type = Constants.EVENT_SCROLL;
        return scrollEvent;
    }
}

flexiciousNmsp.ReactDataGridPsuedoScroll  = ReactDataGridPsuedoScroll ; //add to name space
ReactDataGridPsuedoScroll.prototype.typeName = ReactDataGridPsuedoScroll.typeName = 'ReactDataGridPsuedoScroll';//for quick inspection
