/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import Timer from '../../utils/Timer'
import UIComponent from '../../core/UIComponent'
import ReactDataGridCellContainer from '../../core/ReactDataGridCellContainer'
import Constants from '../../utils/Constants'
import FlexDataGridBodyContainer from '../../flexgrid/grids/FlexDataGridBodyContainer'
import FlexDataGridVirtualBodyContainer from '../../flexgrid/grids/FlexDataGridVirtualBodyContainer'
import UIUtils from '../../utils/UIUtils'
/**
 * An object that represents information about a cell.
 * @constructor
 * @class CellInfo
 * @namespace flexiciousNmsp
 * @extends TypedObject
 */
export default class ReactDataGridPsuedoScrollBodyContainer extends FlexDataGridVirtualBodyContainer {
    constructor(grid) {
        super(grid);
        this.setStyleAttribute("overflowY", "hidden");
        this.setStyleAttribute("position", "absolute");

    }
    componentDidMountCustom(){
        this.addEventListener(this, Constants.EVENT_MOUSE_WHEEL, this.grid.onMouseWheel.bind(this.grid));
    }
    /**
     * @return {Array}
     */
    getClassNames() {
        return ["UIComponent", "ReactDataGridPsuedoScrollBodyContainer", "FlexDataGridVirtualBodyContainer", "FlexDataGridBodyContainer"];
    }

    setVerticalScrollPolicy(policy) {

    }

    setVerticalScrollPosition(pos) {
        this.verticalScrollPosition = pos;
    }

    getVerticalScrollPosition() {
        return this.verticalScrollPosition || 0;
    }

    checkScrollChange(prevCalc) {
    }

    onExternalContainerScroll(evt, noThrottle) {
        if (noThrottle) {
        } else {
            if (!this.callIndex) {
                this.callIndex = 1;
            }
            this.callIndex++;
            if (this.callIndex % 3 != 0) {
                return;
            }

        }
        this.setVerticalScrollPosition(evt.scrollTop);
        this.grid.setupScrollTimer(evt);
        this.grid.dispatchEvent(evt);

        let delta;
        if (evt.scrollTop > this.getMaxVerticalScrollPosition()) {
            evt.scrollTop = this.getMaxVerticalScrollPosition();
        }
        if (evt.scrollTop < 0) {
            evt.scrollTop = 0;
        }

        if (this.grid.lastVScroll != evt.scrollTop) {

            delta = evt.scrollTop - this.grid.lastVScroll;
            if ((this.grid.lastVScroll + delta) > this.getMaxVerticalScrollPosition()) {
                //this.synchronizeLockedVerticalScroll();
                return;
            }

            this.grid.lastVScroll = evt.scrollTop;
            const seedHeight = this.getVerticalScrollPosition();
            if (seedHeight >= 0 && this.setCurrentRowAtScrollPosition(seedHeight)) {
                this.recycleNoScroll(this.grid.getColumnLevel(), delta > 0, delta);//only recycle if our seed actually changed..
            }
        }
    }

    recycleNoScroll(level, scrollDown, scrollDelta, scrolling) {
        if (typeof scrollDelta == "undefined") scrollDelta = 0;
        if (typeof scrolling == "undefined") scrolling = true;

        if ((this.getVerticalScrollPosition() + scrollDelta) > this.getMaxVerticalScrollPosition()) {
            this.setVerticalScrollPosition(this.getMaxVerticalScrollPosition());
        }
        if ((this.getVerticalScrollPosition() + scrollDelta) < 0) {
            this.setVerticalScrollPosition(0);
        }
        this.setVisibleRange();
        const start = this._visibleRange[0];
        const end = this._visibleRange[1];
        let rowIndex = 0;
        for (let posIndex = start.getRowIndex(); posIndex <= end.getRowIndex(); posIndex++) {
            const seed = this.itemVerticalPositions[posIndex];
            const row = this.rows[rowIndex++];
            if (row) {
                row.setRowPositionInfo(seed);
            } else {
                break;
            }
        }
    }

    removeMatchSticks() {
        if (this.externalScrollContainer) {
            this.externalScrollContainer.removeEventListener(Constants.EVENT_SCROLL, this.onExternalContainerScroll);
            UIUtils.removeChild(this.grid, this.externalScrollContainer);
            this.externalScrollContainer = null;
        }
        flexiciousNmsp.FlexDataGridBodyContainer.prototype.removeMatchSticks.apply(this, []);
    }

    addMatchSticks(p) {
        if (!this.externalScrollContainer) {
            this.externalScrollContainer = new UIComponent("div");
            this.externalScrollContainer.setStyleAttribute("overflowY", "auto");
            this.externalScrollContainer.setStyleAttribute("position", "absolute");
            this.externalScrollContainer.setStyleAttribute("height", `${this.getHeight()}px`);
            this.externalScrollContainer.setStyleAttribute("right", "0px");
            this.externalScrollContainer.setStyleAttribute("top", `${this.getY()}px`);
            this.externalScrollContainer.setStyleAttribute("width", "20px");
            this.externalScrollContainer.setStyleAttribute("border", "solid 1px red");
            this.externalScrollContainer.setStyleAttribute("zIndex", 9999);
            this.externalScrollContainer.addEventListener(this, Constants.EVENT_SCROLL, this.onExternalContainerScroll);
            this.grid.addChild(this.externalScrollContainer);

            this.externalScrollContainer.matchStick = new UIComponent("div");
            this.externalScrollContainer.matchStick.attachClass("matchStick");
            this.externalScrollContainer.addChild(this.externalScrollContainer.matchStick);
        }
        this.externalScrollContainer.setVerticalScrollPosition(0)
        FlexDataGridBodyContainer.prototype.addMatchSticks.apply(this, [p]);
    }

    addMatchStick(p) {
        FlexDataGridBodyContainer.prototype.addMatchStick.apply(this, [p]);
        this.externalScrollContainer.matchStick.setHeight(this._calculatedTotalHeight);

    }

    /**
     * Gets the scrollLeft
     */
    getMaxVerticalScrollPosition() {
        return this._calculatedTotalHeight - this.getHeight() - 55;
    }

    /**
     * Scrolls to a row that is already drawn.
     */
    scrollToExistingRow(vsp, scrollDown) {
        if ((this.getVerticalScrollPosition() != vsp) && (vsp <= (this.getMaxVerticalScrollPosition() + this.grid.getColumnLevel().getRowHeight()))) {
            this.setVerticalScrollPosition(vsp);
            this.externalScrollContainer.setVerticalScrollPosition(vsp);
            this.grid.traceValue = vsp;
            this.recycleNoScroll(this.grid.getColumnLevel(), scrollDown, 0, false);
            this.grid.synchronizeLockedVerticalScroll();
        }

    }

    recycle(level, scrollDown, scrollDelta, scrolling) {
        this.recycleNoScroll(level, scrollDown, scrollDelta, scrolling)
    }

    drawRows(forceFiller) {
        if (this.rows.length == 0) {
            FlexDataGridBodyContainer.prototype.drawRows.apply(this, [forceFiller]);
        }
    }

    handleArrowKey(cell, keyCode, triggerEvent) {
        if (keyCode == Constants.KEYBOARD_UP || keyCode == Constants.KEYBOARD_DOWN) {
            if (this.isYOutOfVisibleArea(cell.rowInfo.getY() - cell.getHeight(), cell.getHeight()) && keyCode == Constants.KEYBOARD_UP) {
                var scrollpos = this.externalScrollContainer.getVerticalScrollPosition() - this.grid.getRowHeight();
                var scrollEvent = this.grid.buildScrollEvent(scrollpos);
                scrollEvent.triggerEvent = triggerEvent;
                this.externalScrollContainer.setVerticalScrollPosition(scrollEvent.scrollTop);
                this.onExternalContainerScroll(scrollEvent, true);

            } else if (this.isYOutOfVisibleArea(cell.rowInfo.getY() + cell.getHeight(), cell.getHeight()) && keyCode == Constants.KEYBOARD_DOWN) {
                var scrollpos = this.externalScrollContainer.getVerticalScrollPosition() + this.grid.getRowHeight();
                var scrollEvent = this.grid.buildScrollEvent(scrollpos);
                scrollEvent.triggerEvent = triggerEvent;
                this.externalScrollContainer.setVerticalScrollPosition(scrollEvent.scrollTop);
                this.onExternalContainerScroll(scrollEvent, true);
            }
            else {
                return FlexDataGridBodyContainer.prototype.handleArrowKey.apply(this, [cell, keyCode, triggerEvent]);
            }
        } else {
            return FlexDataGridBodyContainer.prototype.handleArrowKey.apply(this, [cell, keyCode, triggerEvent]);
        }

    }
}

flexiciousNmsp.ReactDataGridPsuedoScrollBodyContainer = ReactDataGridPsuedoScrollBodyContainer; //add to name space
ReactDataGridPsuedoScrollBodyContainer.prototype.typeName = ReactDataGridPsuedoScrollBodyContainer.typeName = 'ReactDataGridPsuedoScrollBodyContainer';//for quick inspection