/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */
import TypedObject from '../core/TypedObject'
import React from 'react'
/**
 * A generator class that instantiates new classes of type classConstruct.
 * @constructor
 * @class ClassFactory
 * @namespace flexiciousNmsp
 * 
 * @param classConstruct
 * @param [props]
 * @param [passPropertiesToConstructor]
 *
 */
export default class ClassFactory extends TypedObject {
    constructor(classConstruct, props, passPropertiesToConstructor) {
        super();
        if (!classConstruct) {
            throw new Error("Class Construct cannot be null");
        }
        /**
         * The constructor to instantiate
         * @type Function
         */
        this.classConstruct = classConstruct;
        /**
         * The properties to apply to this constructors
         */
        this.properties = props;
        /**
         * If true, passes the properties to constructor, if false, loops through properties and
         * sets them individually on the bean
         */
        this.passPropertiesToConstructor = passPropertiesToConstructor;

    }

    getClassNames() { //for support of "is" keyword
        return ["TypedObject", this.typeName];
    }

    /**
     * Creates a new instance of the object specified by the class construct
     * @return {*}
     */
    newInstance(props, passPropertiesIfReactComponent) {
        let obj;

        if (this.passPropertiesToConstructor) {
            obj = new this.classConstruct(this.properties);
        } else {
            if (this.classConstruct.typeName) {
                obj = new this.classConstruct();
                if (passPropertiesIfReactComponent) {
                    for (const reactProp in props) {
                        obj[reactProp] = props[reactProp];
                    }

                }
            } else {
                obj = React.createElement(this.classConstruct, props);
            }
            if (this.properties) {
                for (const prop in this.properties) {
                    obj[prop] = this.properties[prop];
                }
            }

        }
        return obj;
    }
}

flexiciousNmsp.ClassFactory = ClassFactory; //add to name space
ClassFactory.prototype.typeName = ClassFactory.typeName = "ClassFactory";//for quick inspection