/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */

import Constants from './Constants'
import UIUtils from './UIUtils' 
import TypedObject from '../core/TypedObject'
/**
     * Given a date range, sets the startDate and endDate values to the appropraite values.
     * @param dateRangeType	One of the DATE_RANGE constants
     * @param startDate		Start Date of the custom date range. Only required where dateRangeType=DateRange.DATE_RANGE_CUSTOM
     * @param endDate		End Date of the custom date range. Only required where dateRangeType=DateRange.DATE_RANGE_CUSTOM
     * @constructor
     * @class DateRange
     */
export default class DateRange extends TypedObject {

    constructor(dateRangeType, startDate, endDate) {
        super();
        if (typeof startDate == "undefined") startDate = null;
        if (typeof endDate == "undefined") endDate = null;

        const now = new Date();
        if (dateRangeType == DateRange.DATE_RANGE_CUSTOM) {
            this.startDate = startDate;
            this.endDate = endDate;
        }
        else if (dateRangeType == DateRange.DATE_RANGE_THISHOUR) {
            this.startDate = new Date(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), 0, 0);
            this.endDate = new Date(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), 59, 59);
        }
        else if (dateRangeType == DateRange.DATE_RANGE_LAST_SIXTY_MINTUES) {
            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.MINUTES, -60, new Date());
            this.endDate = new Date();
        }
        else if (dateRangeType == DateRange.DATE_RANGE_LAST_12_HOURS) {
            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -12, new Date());
            this.endDate = new Date();
        }
        else if (dateRangeType == DateRange.DATE_RANGE_LAST_24_HOURS) {
            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -24, new Date());
            this.endDate = new Date();
        }
        else if (dateRangeType == DateRange.DATE_RANGE_LAST_7_DAYS) {
            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.DAY_OF_MONTH, -7, new Date());
            this.endDate = new Date();
        }
        else if (dateRangeType == DateRange.DATE_RANGE_LASTHOUR) {

            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -1, new Date(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), 0, 0));
            this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -1, new Date(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), 59, 59));
        }
        else if (dateRangeType == DateRange.DATE_RANGE_NEXTHOUR) {

            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 1, new Date(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), 0, 0));
            this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 1, new Date(now.getFullYear(), now.getMonth(), now.getDate(), now.getHours(), 59, 59));
        }
        else if (dateRangeType == DateRange.DATE_RANGE_TODAY) {

            this.startDate = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0);
            this.endDate = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 23, 59, 59);
        }
        else if (dateRangeType == DateRange.DATE_RANGE_YESTERDAY) {

            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -24, new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0));
            this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -24, new Date(now.getFullYear(), now.getMonth(), now.getDate(), 23, 59, 59));
        }
        else if (dateRangeType == DateRange.DATE_RANGE_TOMORROW) {

            this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 24, new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0));
            this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 24, new Date(now.getFullYear(), now.getMonth(), now.getDate(), 23, 59, 59));
        }
        else if (dateRangeType == DateRange.DATE_RANGE_THISWEEK || dateRangeType == DateRange.DATE_RANGE_LASTWEEK || dateRangeType == DateRange.DATE_RANGE_NEXTWEEK) {
            this.startDate = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0);
            this.endDate = new Date(now.getFullYear(), now.getMonth(), now.getDate(), 0, 0, 0);
            while (this.startDate.getDay() != 1)
                this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -24, this.startDate);
            while (this.endDate.getDay() != 0)
                this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 24, this.endDate);

            this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.MILLISECONDS, -1, flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 24, this.endDate));

            if (dateRangeType == DateRange.DATE_RANGE_LASTWEEK) {
                this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -24 * 7, this.startDate);
                this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, -24 * 7, this.endDate);
            }
            else if (dateRangeType == DateRange.DATE_RANGE_NEXTWEEK) {
                this.startDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 24 * 7, this.startDate);
                this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS, 24 * 7, this.endDate);
            }
        }
        else if (dateRangeType == DateRange.DATE_RANGE_THISMONTH || dateRangeType == DateRange.DATE_RANGE_LASTMONTH || dateRangeType == DateRange.DATE_RANGE_NEXTMONTH) {
            this.startDate = new Date(now.getFullYear(), now.getMonth(), 1, 0, 0, 0);
            this.endDate = new Date(now.getFullYear(), now.getMonth() + 1, 0, 23, 59, 59);
            //this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS,-24,this.endDate);

            if (dateRangeType == DateRange.DATE_RANGE_LASTMONTH) {

                this.startDate = new Date(now.getFullYear(), now.getMonth() - 1, 1, 0, 0, 0);
                this.endDate = new Date(now.getFullYear(), now.getMonth(), 0, 23, 59, 59);

            }
            else if (dateRangeType == DateRange.DATE_RANGE_NEXTMONTH) {
                this.startDate = new Date(now.getFullYear(), now.getMonth() + 1, 1, 0, 0, 0);
                this.endDate = new Date(now.getFullYear(), now.getMonth() + 2, 0, 23, 59, 59);
            }
        }
        else if (dateRangeType == DateRange.DATE_RANGE_THISQUARTER || dateRangeType == DateRange.DATE_RANGE_LASTQUARTER || dateRangeType == DateRange.DATE_RANGE_NEXTQUARTER) {
            const quarter = Math.floor((now.getMonth()) / 3) + 1;
            this.startDate = DateRange.getStartOfQuarter(now.getFullYear(), quarter);
            this.endDate = DateRange.getEndOfQuarter(now.getFullYear(), quarter);
            //this.endDate = flexiciousNmsp.DateUtils.dateAdd(flexiciousNmsp.DateUtils.HOURS,-24,this.endDate);

            if (dateRangeType == DateRange.DATE_RANGE_LASTQUARTER) {

                this.startDate = DateRange.getStartOfQuarter(now.getFullYear(), quarter - 1);
                this.endDate = DateRange.getEndOfQuarter(now.getFullYear(), quarter - 1);

            }
            else if (dateRangeType == DateRange.DATE_RANGE_NEXTQUARTER) {
                this.startDate = DateRange.getStartOfQuarter(now.getFullYear(), quarter + 1);
                this.endDate = DateRange.getEndOfQuarter(now.getFullYear(), quarter + 1);
            }
        }

        else if (dateRangeType == DateRange.DATE_RANGE_THISYEAR || dateRangeType == DateRange.DATE_RANGE_LASTYEAR || dateRangeType == DateRange.DATE_RANGE_NEXTYEAR) {
            this.startDate = new Date(now.getFullYear(), 0, 1, 0, 0, 0);
            this.endDate = new Date(now.getFullYear(), 11, 31, 23, 59, 59);
            if (dateRangeType == DateRange.DATE_RANGE_LASTYEAR) {
                this.startDate = new Date(now.getFullYear() - 1, 0, 1, 0, 0, 0);
                this.endDate = new Date(now.getFullYear() - 1, 11, 31, 23, 59, 59);
            }
            else if (dateRangeType == DateRange.DATE_RANGE_NEXTYEAR) {
                this.startDate = new Date(now.getFullYear() + 1, 0, 1, 0, 0, 0);
                this.endDate = new Date(now.getFullYear() + 1, 11, 31, 23, 59, 59);
            }
        }
        else if (UIUtils.doesArrayContainObjectValue(DateRange.customDateRanges, "name", dateRangeType)) {
            const drObj = UIUtils.doesArrayContainObjectValue(DateRange.customDateRanges, "name", dateRangeType);
            drObj.initFunction(this);//this should populate start and end dates
        }
        else
            throw new Error(`Invalid date range type${dateRangeType}`);
        this.dateRangeType = dateRangeType;


    }

    getClassNames() {
        return "DateRange";
    }

    /**
     * A Function that takes a date range code, and a initialization function.
     * The code is what shows up in the list of dropdowns. The initFunction
     * is resposible for initializing the date range object, which should populate
     * its start and end dates. The initFunction should take a dateRange object,
     * and return that object with its start and end date populated.
     * 
     * @param {String} rangeName
     * @param {Function} initFunction
     */
    static registerDateRange(rangeName, initFunction) {
        DateRange.customDateRanges.push({ 'name': rangeName, 'initFunction': initFunction });
    }

    /**
     * Given a quarter, returns the date at the start of the quarter
     * @param {number} year
     * @param {number} quarter
     */
    static getStartOfQuarter(year, quarter) {
        if (quarter < 1) {
            quarter = 4;
            year--;
        }
        if (quarter > 4) {
            quarter = 1;
            year++;
        }
        if (quarter == 1)
            return new Date(year, 0, 1, 0, 0, 0, 0);
        else if (quarter == 2)
            return new Date(year, 3, 1, 0, 0, 0, 0);
        else if (quarter == 3)
            return new Date(year, 6, 1, 0, 0, 0, 0);
        else return new Date(year, 9, 1, 0, 0, 0, 0);

    }

    /**
     * Given a quarter, returns the date at the end of the quarter
     * @param {number} year
     * @param {number} quarter
     */
    static getEndOfQuarter(year, quarter) {

        if (quarter < 1) {
            quarter = 4;
            year--;
        }
        if (quarter > 4) {
            quarter = 1;
            year++;
        }
        if (quarter == 1)
            return new Date(year, 2,
                flexiciousNmsp.DateUtils.daysInMonth(new Date(year, 2)), 0, 0, 0, 0);
        else if (quarter == 2)
            return new Date(year, 5,
                30, 23, 59, 59, 999);
        else if (quarter == 3)
            return new Date(year, 8,
                30, 23, 59, 59, 999);
        else
            return new Date(year, 11,
                31, 23, 59, 59, 999);

    }
}

DateRange.DATE_RANGE_LAST_SIXTY_MINTUES = "Last 60 Minutes";
DateRange.DATE_RANGE_LAST_12_HOURS = "Last 12 Hours";
DateRange.DATE_RANGE_LAST_24_HOURS = "Last 24 Hours";
DateRange.DATE_RANGE_LAST_7_DAYS = "Last 7 Days";
DateRange.DATE_RANGE_THISHOUR = "This Hour";
DateRange.DATE_RANGE_LASTHOUR = "Last Hour";
DateRange.DATE_RANGE_NEXTHOUR = "Next Hour";
DateRange.DATE_RANGE_TODAY = "Today";
DateRange.DATE_RANGE_YESTERDAY = "Yesterday";
DateRange.DATE_RANGE_TOMORROW = "Tomorrow";
DateRange.DATE_RANGE_THISWEEK = "This Week";
DateRange.DATE_RANGE_LASTWEEK = "Last Week";
DateRange.DATE_RANGE_NEXTWEEK = "Next Week";
DateRange.DATE_RANGE_THISMONTH = "This Month";
DateRange.DATE_RANGE_LASTMONTH = "Last Month";
DateRange.DATE_RANGE_NEXTMONTH = "Next Month";
DateRange.DATE_RANGE_THISYEAR = "This Year";
DateRange.DATE_RANGE_LASTYEAR = "Last Year";
DateRange.DATE_RANGE_NEXTYEAR = "Next Year";
DateRange.DATE_RANGE_CUSTOM = "Custom";
DateRange.DATE_RANGE_THISQUARTER = "This Quarter";
DateRange.DATE_RANGE_NEXTQUARTER = "Next Quarter";
DateRange.DATE_RANGE_LASTQUARTER = "Last Quarter";
DateRange.customDateRanges = [];
flexiciousNmsp.DateRange = DateRange;