import EventDispatcher from '../core/EventDispatcher'
import Constants from '../utils/Constants'

/*eslint-disable */
/**
 * Flexicious
 * Copyright 2011, Flexicious LLC
 */


/**
 *
 * @param delay
 * @param [repeatCount]
 * @constructor
 * @class Timer
 */
export default class Timer extends EventDispatcher {
    constructor(delay, repeatCount) {
        super();
        if (typeof repeatCount == "undefined")repeatCount = 0;

        /*
         * The total number of times the timer has fired since it started at zero. If the timer has been reset, only the fires since the reset are counted.
         */
        this.currentCount = 0;
        /*
         * The delay, in milliseconds, between timer events
         */
        this.delay = 1000;
        /*
         * The total number of times the timer is set to run. If the repeat count is set to 0, the timer continues forever or
         * until the stop() method is invoked or the program stops. If the repeat count is nonzero, the timer runs the specified number of times.
         * If repeatCount is set to a total that is the same or less then currentCount the timer stops and will not fire again.
         */
        this.repeatCount = 0;
        /*
         * The timer's current state; true if the timer is running, otherwise false.
         */
        this.running = false;

        this.timeoutObject = null;
        this.delay = delay;
        this.repeatCount = repeatCount;
    }

    getClassNames() {
        return "Timer";
    }

    reset() {
        this.currentCount = 0;
    }

    start() {
        if (this.running) {
            this.stop();
        }
        this.running = true;
        const timer = this;
        this.timeoutObject = window.setInterval(() => {
            timer.onTimer();
        }, this.delay);
        return this;
    }

    onTimer() {
        this.currentCount++;
        this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_TIMER));
        if (this.repeatCount == 0) {
            //continue
        } else if (this.currentCount >= this.repeatCount) {
            this.stop();
            this.dispatchEvent(new flexiciousNmsp.BaseEvent(Constants.EVENT_TIMER_COMPLETE));
        }
    }

    stop() {
        this.running = false;
        if (this.timeoutObject) {
            window.clearInterval(this.timeoutObject);
            this.timeoutObject = null;
        }
    }

    kill() {
        flexiciousNmsp.EventDispatcher.prototype.kill.apply(this);
        this.stop();
    }
}

/**
 *
 * @type {Function}
 */

flexiciousNmsp.Timer = Timer; //add to name space

/**
 *
 * @type {String}
 */
Timer.prototype.typeName = Timer.typeName = 'Timer';//for quick inspection
